/*-------------------------------------------------------------------------
 *
 * itup.h--
 *    POSTGRES index tuple definitions.
 *
 *
 * Copyright (c) 1994, Regents of the University of California
 *
 * $Id: itup.h,v 0.6 1996/06/07 01:39:55 am Exp $
 *
 *-------------------------------------------------------------------------
 */
#ifndef ITUP_H
#define ITUP_H

#include "c.h"
#include "access/ibit.h"
#include "access/tupdesc.h"	/* for TupleDesc */
#include "storage/itemptr.h"

#define MaxIndexAttributeNumber	7

typedef struct IndexTupleData {
    ItemPointerData		t_tid; /* reference TID to base tuple */

    /*
     * t_info is layed out in the following fashion:
     *
	 * [Mariposa]
     * first (leftmost) bit: "has nulls" bit
     * second bit: "has varlenas" bit
     * third bit: "has rules" bit - (removed ay 11/94)
     * fourth-16th bit: size of tuple.
	 *[Postgres95]
     * 15th (leftmost) bit: "has nulls" bit
     * 14th bit: "has varlenas" bit
     * 13th bit: "has rules" bit - (removed ay 11/94)
     * bits 12-0 bit: size of tuple.
     */

    unsigned short		t_info; /* various info about tuple */

    /*
     * please make sure sizeof(IndexTupleData) is MAXALIGN'ed.
     * See IndexInfoFindDataOffset() for the reason.
     */
    
} IndexTupleData;		/* MORE DATA FOLLOWS AT END OF STRUCT */

typedef IndexTupleData	*IndexTuple;


typedef struct InsertIndexResultData {
    ItemPointerData	pointerData;
} InsertIndexResultData;

typedef InsertIndexResultData *InsertIndexResult;


typedef struct RetrieveIndexResultData {
    ItemPointerData	index_iptr;
    ItemPointerData	heap_iptr;
} RetrieveIndexResultData;

typedef RetrieveIndexResultData	*RetrieveIndexResult;


/*-----------------
 * PredInfo -
 *    used for partial indices
 *-----------------
 */
typedef struct PredInfo {
    Node		*pred;
    Node		*oldPred;
} PredInfo;


#ifndef P95
/* ----------------
 *	soon to be obsolete index result stuff
 * ----------------
 */

/*
 * GeneralInsertIndexResultIsValid --
 *	True iff general index insertion result is valid.
 */
#define	GeneralInsertIndexResultIsValid(result)		PointerIsValid(result)

/*
 * InsertIndexResultIsValid --
 *	True iff (specific) index insertion result is valid.
 */
#define	InsertIndexResultIsValid(result)		PointerIsValid(result)

/*
 * GeneralRetrieveIndexResultIsValid --
 *	True iff general index retrieval result is valid.
 */
#define	GeneralRetrieveIndexResultIsValid(result)	PointerIsValid(result)

/*
 * RetrieveIndexResultIsValid --
 *	True iff (specific) index retrieval result is valid.
 */
#define	RetrieveIndexResultIsValid(result)	PointerIsValid(result)

/*
 * GeneralRetrieveIndexResultGetHeapItemPointer --
 *	Returns heap item pointer associated with a general index retrieval.
 *
 * Note:
 *	Assumes general index retrieval result is valid.
 */
#define GeneralRetrieveIndexResultGetHeapItemPointer(result) \
    (AssertMacro(GeneralRetrieveIndexResultIsValid(result)) ? \
     ((ItemPointer) (&(result)->heapItemData)) : (ItemPointer) 0)

/*
 * RetrieveIndexResultGetIndexItemPointer --
 *	Returns index item pointer associated with a (specific) index retrieval
 *
 * Note:
 *	Assumes (specific) index retrieval result is valid.
 */
#define RetrieveIndexResultGetIndexItemPointer(result) \
    (AssertMacro(RetrieveIndexResultIsValid(result)) ? \
     ((ItemPointer) (&(result)->indexItemData)) : (ItemPointer) 0)

/*
 * RetrieveIndexResultGetHeapItemPointer --
 *	Returns heap item pointer associated with a (specific) index retrieval.
 *
 * Note:
 *	Assumes (specific) index retrieval result is valid.
 */
#define RetrieveIndexResultGetHeapItemPointer(result) \
    (AssertMacro(RetrieveIndexResultIsValid(result)) ? \
     ((ItemPointer) (&(result)->heapItemData)) : (ItemPointer) 0)



#endif /* P95 */
/* ----------------
 *	externs 
 * ----------------
 */

#define INDEX_SIZE_MASK 0x1FFF
#define INDEX_NULL_MASK 0x8000
#define INDEX_VAR_MASK  0x4000

#define IndexTupleSize(itup)       (((IndexTuple) (itup))->t_info & 0x1FFF)
#define IndexTupleDSize(itup)                      ((itup).t_info & 0x1FFF)
#define IndexTupleNoNulls(itup)  (!(((IndexTuple) (itup))->t_info & 0x8000))
#define IndexTupleAllFixed(itup) (!(((IndexTuple) (itup))->t_info & 0x4000))

#define IndexTupleHasMinHeader(itup) (IndexTupleNoNulls(itup))


/* indextuple.h */
#ifndef P95
extern IndexTuple index_formtuple(int numberOfAttributes,
	TupleDesc tupleDescriptor, Datum value[], char null[]);
#else /* P95 */
extern IndexTuple index_formtuple(TupleDesc tupleDescriptor,
				  Datum value[], char null[]);
#endif /* P95 */
extern char *fastgetiattr(IndexTuple tup, int attnum,
#ifndef P95
	AttributeTupleForm att[], bool *isnull);
extern Datum index_getsysattr(IndexTuple tuple, int attributeNumber);
#else /* P95 */
	TupleDesc att, bool *isnull);
#endif /* P95 */
extern Datum index_getattr(IndexTuple tuple, AttrNumber attNum,
	TupleDesc tupDesc, bool *isNullOutP);
#ifndef P95

#endif /* P95 */
extern RetrieveIndexResult
FormRetrieveIndexResult(ItemPointer indexItemPointer,
			ItemPointer heapItemPointer);
#ifndef P95

#endif /* P95 */
extern void CopyIndexTuple(IndexTuple source, IndexTuple *target);


#endif	/* ITUP_H */

