/*-------------------------------------------------------------------------
 *
 * bootstrap.h--
 *    include file for the bootstrapping code
 *
 *
 * Copyright (c) 1994, Regents of the University of California
 *
 * $Id: bootstrap.h,v 0.4 1996/06/07 04:54:36 am Exp $
 *
 *-------------------------------------------------------------------------
 */
#ifndef BOOTSTRAP_H
#define BOOTSTRAP_H

#include <sys/file.h>
#include <stdio.h>
#include <string.h>
#include <signal.h>
#include <ctype.h>

#include "access/htup.h"
#include "access/itup.h"
#include "access/relscan.h"
#include "access/skey.h"
#include "utils/tqual.h"
#include "storage/buf.h"
#include "storage/bufmgr.h"	/* for BufferManagerFlush */
#include "utils/portal.h"
#include "utils/elog.h"
#include "utils/rel.h"

#ifndef P95
#define ALLOC(t, c)	(t *)calloc((unsigned)(c), sizeof(t))

#define FIRST_TYPE_OID 16	/* OID of the first type */
#endif /* P95 */
#define	MAXATTR 40		/* max. number of attributes in a relation */

#ifndef P95
/* ami_lexer.l */
#define STRTABLESIZE	10000
#define HASHTABLESIZE	503

/* Hash function numbers */
#define NUM	23
#define	NUMSQR	529
#define	NUMCUBE	12167

#endif /* P95 */
typedef struct hashnode {
    int		strnum;		/* Index into string table */
    struct hashnode	*next;
} hashnode;

#ifndef P95
typedef struct mcro {
    char *s;
    int  mderef;
} macro;

#endif /* P95 */
#define EMITPROMPT printf("> ")

extern Relation reldesc;
extern AttributeTupleForm attrtypes[MAXATTR];
extern int numattr;
extern int DebugMode;

extern int BootstrapMain(int ac, char *av[]);
#ifndef P95
extern void index_register(Name heap,
			   Name ind,
#else /* P95 */
extern void index_register(char *heap,
			   char *ind,
#endif /* P95 */
			   int natts,
			   AttrNumber *attnos,
			   uint16 nparams,
			   Datum *params,
			   FuncIndexInfo *finfo,
			   PredInfo *predInfo);

extern void err(void);
extern void InsertOneTuple(Oid objectid);
#ifndef P95
extern void createrel(char *name);
#endif /* P95 */
extern void closerel(char *name);
extern void boot_openrel(char *name);
extern char *LexIDStr(int ident_num);

#ifndef P95
extern void showtup(HeapTuple tuple, Buffer buffer, Relation relation);
extern void showtime(AbsoluteTime time);
#endif /* P95 */
extern void DefineAttr(char *name, char *type, int attnum);
extern void InsertOneValue(Oid objectid, char *value, int i);
extern void InsertOneNull(int i);
#ifndef P95
extern void defineindex(char *heapName, char *indexName,
			char *accessMethodName, List *attributeList);
#endif /* P95 */
extern bool BootstrapAlreadySeen(Oid id);
extern void cleanup(void);
extern int gettype(char *type);
extern AttributeTupleForm AllocateAttribute(void);
#ifndef P95
extern unsigned char MapEscape(char **s);
#else /* P95 */
extern char* MapArrayTypeName(char *s);
extern char* CleanUpStr(char *s);
#endif /* P95 */
extern int EnterString (char *str);
extern int CompHash (char *str, int len);
extern hashnode *FindStr (char *str, int length, hashnode *mderef);
extern hashnode *AddStr(char *str, int strlength, int mderef);
#ifndef P95
extern void printhashtable(void);
extern void printstrtable(void);
extern char *emalloc (unsigned size);
extern int LookUpMacro(char *xmacro);
extern void DefineMacro(int indx1, int indx2);
extern void printmacros(void);
#endif /* P95 */
extern void build_indices(void);
#ifndef P95
extern void printrel();
#endif /* P95 */

#endif /* BOOTSTRAP_H */
