/*-------------------------------------------------------------------------
 *
 * pg_access.h--
 *    definition of the system "access" relation (pg_access)
 *    along with the relation's initial contents.
 *
 *    The ACCESS relation is a shared relation that is used to keep track
 *    of the usage statistics for remote fragments by the local site.
 *
 * Copyright (c) 1994, 1995 Regents of the University of California
 *
 * $Id:
 *
 * NOTES
 *    the genbki.sh script reads this file and generates .bki
 *    information from the DATA() statements.
 *
 *-------------------------------------------------------------------------
 */
#ifndef PG_ACCESS_H
#define PG_ACCESS_H

/* ----------------
 *	postgres.h contains the system type definintions and the
 *	CATALOG(), BOOTSTRAP and DATA() sugar words so this file
 *	can be read by both genbki.sh and the C compiler.
 * ----------------
 */
#include "postgres.h"
#include "utils/nabstime.h"

/* ----------------
 *	pg_database definition.  cpp turns this into
 *	typedef struct FormData_pg_database
 * ----------------
 */ 
CATALOG(pg_access) BOOTSTRAP {
    char16 	 datname;          /* XXX - The type should be NameData */
    Oid 	 fralogicalid;
    int4         fralocation;
    float8       fradebits;
    float8       fracredits;
    float8       fraexpprice;
    int4         fraaccesses;
    dt           fraarrtime;       /* XXX - Not used */
    dt           fralastacctime;
} FormData_pg_access;

/* ----------------
 *	Form_pg_access corresponds to a pointer to a tuple with
 *	the format of pg_access relation.
 * ----------------
 */
typedef FormData_pg_access	*Form_pg_access;

/* ----------------
 *	compiler constants for pg_database
 * ----------------
 */
extern Name Name_pg_access;

#define Natts_pg_access		        9
#define Anum_pg_access_datname	        1
#define Anum_pg_access_fralogicalid	2
#define Anum_pg_access_fralocation      3
#define Anum_pg_access_fradebits        4
#define Anum_pg_access_fracredits       5
#define Anum_pg_access_fraexpprice      6
#define Anum_pg_access_fraaccesses      7
#define Anum_pg_access_fraarrtime       8
#define Anum_pg_access_fralastacctime   9

#endif /* ! PG_ACCESS_H */
