/*-------------------------------------------------------------------------
 *
 * pg_attribute.h--
 *    definition of the system "attribute" relation (pg_attribute)
 *    along with the relation's initial contents.
 *
 *
 * Copyright (c) 1994, Regents of the University of California
 *
 * $Id: pg_attribute.h,v 0.19 1996/06/07 04:54:52 am Exp $
 *
 * NOTES
 *    the genbki.sh script reads this file and generates .bki
 *    information from the DATA() statements.
 *
 *    utils/cache/relcache.c requires some hard-coded tuple descriptors
 *    for some of the system catalogs so if the schema for any of
 *    these changes, be sure and change the appropriate Schema_xxx
 *    macros!  -cim 2/5/91
 *
 *    fastgetattr() now uses attcacheoff to cache byte offsets of
 *    attributes in heap tuples.  The data actually stored in 
 *    pg_attribute (-1) indicates no cached value.  But when we copy
 *    these tuples into a tuple descriptor, we may then update attcacheoff
 *    in the copies.  This speeds up the attribute walking process.
 *
 *-------------------------------------------------------------------------
 */
#ifndef PG_ATTRIBUTE_H
#define PG_ATTRIBUTE_H
   
/* ----------------
 *	postgres.h contains the system type definintions and the
 *	CATALOG(), BOOTSTRAP and DATA() sugar words so this file
 *	can be read by both genbki.sh and the C compiler.
 * ----------------
 */
#include "postgres.h"
#include "access/attnum.h"

/* ----------------
 *	pg_attribute definition.  cpp turns this into
 *	typedef struct FormData_pg_attribute
 *
 *      If you change the following, make sure you change the structs for
 *      system attributes in heap.c and index.c also.
 * ----------------
 */
CATALOG(pg_attribute) BOOTSTRAP {
#ifndef P95
    Oid  	attrelid;
    char16  	attname;
#else /* P95 */
    Oid  	attrelid;      
    NameData  	attname;
#endif /* P95 */
    Oid  	atttypid;
    Oid  	attdefrel;
    int4  	attnvals;
    Oid  	atttyparg;	/* type arg for arrays/spquel/procs */
    int2 	attlen;
    int2  	attnum;
    int2 	attbound;
    bool  	attbyval;
    bool 	attcanindex;
    Oid 	attproc;	/* spquel? */
    int4	attnelems;
    int4	attcacheoff;
    bool        attisset;
    char	attalign;	/* alignment (c=char, s=short, i=int, d=double) */
} FormData_pg_attribute;

/*
 * someone should figure out how to do this properly. (The problem is
 * the size of the C struct is not the same as the size of the tuple.)
 */
#define ATTRIBUTE_TUPLE_SIZE \
    (offsetof(FormData_pg_attribute,attalign) + sizeof(char))

/* ----------------
 *	Form_pg_attribute corresponds to a pointer to a tuple with
 *	the format of pg_attribute relation.
 * ----------------
 */
typedef FormData_pg_attribute	*AttributeTupleForm;

/* ----------------
 *	compiler constants for pg_attribute
 * ----------------
 */
#ifndef P95
extern Name Name_pg_attribute;
#endif /* P95 */

#define Natts_pg_attribute		16
#define Anum_pg_attribute_attrelid	1
#define Anum_pg_attribute_attname	2
#define Anum_pg_attribute_atttypid	3
#define Anum_pg_attribute_attdefrel	4
#define Anum_pg_attribute_attnvals	5
#define Anum_pg_attribute_atttyparg	6
#define Anum_pg_attribute_attlen	7
#define Anum_pg_attribute_attnum	8
#define Anum_pg_attribute_attbound	9
#define Anum_pg_attribute_attbyval	10
#define Anum_pg_attribute_attcanindex	11
#define Anum_pg_attribute_attproc	12
#define Anum_pg_attribute_attnelems	13
#define Anum_pg_attribute_attcacheoff	14
#define Anum_pg_attribute_attisset      15
#define Anum_pg_attribute_attalign      16


/* ----------------
 *	SCHEMA_ macros for declaring hardcoded tuple descriptors.
 *	these are used in utils/cache/relcache.c
 * ----------------
 */
#ifndef P95
#define SCHEMA_NAME(x) CppConcat(Name_,x)
#define SCHEMA_DESC(x) CppConcat(Desc_,x)
#define SCHEMA_NATTS(x) CppConcat(Natts_,x)
#define SCHEMA_DEF(x) \
    FormData_pg_attribute \
    SCHEMA_DESC(x) [ SCHEMA_NATTS(x) ] = \
    { \
	CppConcat(Schema_,x) \
    }
#endif /* P95 */
#ifdef P95
 /* above items commented out */
#endif /* P95 */

/* ----------------
 *	initial contents of pg_attribute
 * ----------------
 */

/* ----------------
 *	pg_type schema
 * ----------------
 */
#ifndef P95
#define Schema_pg_type \
{ 71l, "typname",      19l, 71l, 0l, 0l, 16,  1, 0,   '\0', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 71l, "typowner",     26l, 71l, 0l, 0l,  4,  2, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 71l, "typlen",       21l, 71l, 0l, 0l,  2,  3, 0, '\001', '\001', 0l, 0l, -1l, '\0', 's' }, \
{ 71l, "typprtlen",    21l, 71l, 0l, 0l,  2,  4, 0, '\001', '\001', 0l, 0l, -1l, '\0', 's' }, \
{ 71l, "typbyval",     16l, 71l, 0l, 0l,  1,  5, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'c' }, \
{ 71l, "typtype",      18l, 71l, 0l, 0l,  1,  6, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'c' }, \
{ 71l, "typisdefined", 16l, 71l, 0l, 0l,  1,  7, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'c' }, \
{ 71l, "typdelim",     18l, 71l, 0l, 0l,  1,  8, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'c' }, \
{ 71l, "typrelid",     26l, 71l, 0l, 0l,  4,  9, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 71l, "typelem",      26l, 71l, 0l, 0l,  4, 10, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 71l, "typinput",     24l, 71l, 0l, 0l,  4, 11, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 71l, "typoutput",    24l, 71l, 0l, 0l,  4, 12, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 71l, "typreceive",   24l, 71l, 0l, 0l,  4, 13, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 71l, "typsend",      24l, 71l, 0l, 0l,  4, 14, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 71l, "typalign",     18l, 71l, 0l, 0l,  1, 15, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'c' }, \
{ 71l, "typdefault",   25l, 71l, 0l, 0l, -1, 16, 0,   '\0', '\001', 0l, 0l, -1l, '\0', 'i' }
#else /* P95 */
#define Schema_pg_type \
{ 71l, {"typname"},      19l, 71l, 0l, 0l, NAMEDATALEN,  1, 0,   '\0', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 71l, {"typowner"},     26l, 71l, 0l, 0l,  4,  2, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 71l, {"typlen"},       21l, 71l, 0l, 0l,  2,  3, 0, '\001', '\001', 0l, 0l, -1l, '\0', 's' }, \
{ 71l, {"typprtlen"},    21l, 71l, 0l, 0l,  2,  4, 0, '\001', '\001', 0l, 0l, -1l, '\0', 's' }, \
{ 71l, {"typbyval"},     16l, 71l, 0l, 0l,  1,  5, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'c' }, \
{ 71l, {"typtype"},      18l, 71l, 0l, 0l,  1,  6, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'c' }, \
{ 71l, {"typisdefined"}, 16l, 71l, 0l, 0l,  1,  7, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'c' }, \
{ 71l, {"typdelim"},     18l, 71l, 0l, 0l,  1,  8, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'c' }, \
{ 71l, {"typrelid"},     26l, 71l, 0l, 0l,  4,  9, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 71l, {"typelem"},      26l, 71l, 0l, 0l,  4, 10, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 71l, {"typinput"},     24l, 71l, 0l, 0l,  4, 11, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 71l, {"typoutput"},    24l, 71l, 0l, 0l,  4, 12, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 71l, {"typreceive"},   24l, 71l, 0l, 0l,  4, 13, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 71l, {"typsend"},      24l, 71l, 0l, 0l,  4, 14, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 71l, {"typalign"},     18l, 71l, 0l, 0l,  1, 15, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'c' }, \
{ 71l, {"typdefault"},   25l, 71l, 0l, 0l, -1, 16, 0,   '\0', '\001', 0l, 0l, -1l, '\0', 'i' }
#endif /* P95 */

#ifndef P95
DATA(insert OID = 0 (  71 typname          19 0 0 0 16   1 0 f t 0 0 -1 f i));
#else /* P95 */
DATA(insert OID = 0 (  71 typname          19 0 0 0 NAMEDATALEN   1 0 f t 0 0 -1 f i));
#endif /* P95 */
DATA(insert OID = 0 (  71 typowner         26 0 0 0  4   2 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  71 typlen           21 0 0 0  2   3 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  71 typprtlen        21 0 0 0  2   4 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  71 typbyval         16 0 0 0  1   5 0 t t 0 0 -1 f c));
DATA(insert OID = 0 (  71 typtype          18 0 0 0  1   6 0 t t 0 0 -1 f c));
DATA(insert OID = 0 (  71 typisdefined     16 0 0 0  1   7 0 t t 0 0 -1 f c));
DATA(insert OID = 0 (  71 typdelim         18 0 0 0  1   8 0 t t 0 0 -1 f c));
DATA(insert OID = 0 (  71 typrelid         26 0 0 0  4   9 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  71 typelem          26 0 0 0  4  10 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  71 typinput         26 0 0 0  4  11 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  71 typoutput        26 0 0 0  4  12 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  71 typreceive       26 0 0 0  4  13 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  71 typsend          26 0 0 0  4  14 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  71 typalign         18 0 0 0  1  15 0 t t 0 0 -1 f c));
DATA(insert OID = 0 (  71 typdefault       25 0 0 0 -1  16 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  71 ctid             27 0 0 0  6  -1 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  71 oid              26 0 0 0  4  -2 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  71 xmin             28 0 0 0  4  -3 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  71 cmin             29 0 0 0  2  -4 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  71 xmax             28 0 0 0  4  -5 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  71 cmax             29 0 0 0  2  -6 0 t t 0 0 -1 f s));
#ifndef P95
DATA(insert OID = 0 (  71 globalxmin       28 0 0 0  4  -7 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  71 globalxmax       28 0 0 0  4  -8 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  71 chain            27 0 0 0  6  -9 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  71 anchor           27 0 0 0  6 -10 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  71 tmax            702 0 0 0  4 -11 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  71 tmin            702 0 0 0  4 -12 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  71 vtype            18 0 0 0  1 -13 0 t t 0 0 -1 f c));
#else /* P95 */
DATA(insert OID = 0 (  71 chain            27 0 0 0  6  -7 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  71 anchor           27 0 0 0  6  -8 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  71 tmax            702 0 0 0  4  -9 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  71 tmin            702 0 0 0  4 -10 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  71 vtype            18 0 0 0  1 -11 0 t t 0 0 -1 f c));
#endif /* P95 */

/* ----------------
 *	pg_database
 * ----------------
 */
#ifndef P95
DATA(insert OID = 0 (  88 datname          19 0 0 0 16   1 0 f t 0 0 -1 f i));
#else /* P95 */
DATA(insert OID = 0 (  88 datname          19 0 0 0 NAMEDATALEN   1 0 f t 0 0 -1 f i));
#endif /* P95 */
DATA(insert OID = 0 (  88 datdba           26 0 0 0  4   2 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  88 datpath          25 0 0 0 -1   3 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  88 ctid             27 0 0 0  6  -1 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  88 oid              26 0 0 0  4  -2 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  88 xmin             28 0 0 0  4  -3 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  88 cmin             29 0 0 0  2  -4 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  88 xmax             28 0 0 0  4  -5 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  88 cmax             29 0 0 0  2  -6 0 t t 0 0 -1 f s));
#ifndef P95
DATA(insert OID = 0 (  88 globalxmin       28 0 0 0  4  -7 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  88 globalxmax       28 0 0 0  4  -8 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  88 chain            27 0 0 0  6  -9 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  88 anchor           27 0 0 0  6 -10 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  88 tmax            702 0 0 0  4 -11 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  88 tmin            702 0 0 0  4 -12 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  88 vtype            18 0 0 0  1 -13 0 t t 0 0 -1 f c));
#else /* P95 */
DATA(insert OID = 0 (  88 chain            27 0 0 0  6  -7 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  88 anchor           27 0 0 0  6  -8 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  88 tmax            702 0 0 0  4  -9 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  88 tmin            702 0 0 0  4 -10 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  88 vtype            18 0 0 0  1 -11 0 t t 0 0 -1 f c));
#endif /* P95 */
    
/* ----------------
 *	pg_demon
 * ----------------
 */
DATA(insert OID = 0 (  76 demserid         26 0 0 0  4   1 0 t t 0 0 -1 f i));
#ifndef P95
DATA(insert OID = 0 (  76 demname          19 0 0 0 16   2 0 f t 0 0 -1 f i));
#else /* P95 */
DATA(insert OID = 0 (  76 demname          19 0 0 0 NAMEDATALEN   2 0 f t 0 0 -1 f i));
#endif /* P95 */
DATA(insert OID = 0 (  76 demowner         26 0 0 0  4   3 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  76 demcode          24 0 0 0  4   4 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  76 ctid             27 0 0 0  6  -1 0 f t 0 0 -1 f i));
#ifdef P95

#endif /* P95 */
DATA(insert OID = 0 (  76 oid              26 0 0 0  4  -2 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  76 xmin             28 0 0 0  4  -3 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  76 cmin             29 0 0 0  2  -4 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  76 xmax             28 0 0 0  4  -5 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  76 cmax             29 0 0 0  2  -6 0 t t 0 0 -1 f s));
#ifndef P95
DATA(insert OID = 0 (  76 globalxmin       28 0 0 0  4  -7 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  76 globalxmax       28 0 0 0  4  -8 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  76 chain            27 0 0 0  6  -9 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  76 anchor           27 0 0 0  6 -10 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  76 tmax            702 0 0 0  4 -11 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  76 tmin            702 0 0 0  4 -12 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  76 vtype            18 0 0 0  1 -13 0 t t 0 0 -1 f c));

#else /* P95 */
DATA(insert OID = 0 (  76 chain            27 0 0 0  6  -7 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  76 anchor           27 0 0 0  6  -8 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  76 tmax            702 0 0 0  4  -9 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  76 tmin            702 0 0 0  4 -10 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  76 vtype            18 0 0 0  1 -11 0 t t 0 0 -1 f c));
    
#endif /* P95 */
#ifndef P95
/* ----------------
 *	pg_fragment
 * ----------------
 */
DATA(insert OID = 0 (  109 fraclassid      26 0 0 0  4   1 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  109 frastoreid	   26 0 0 0  4   2 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  109 fralogicalid	   26 0 0 0  4   3 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  109 fralocation     23 0 0 0  4   4 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  109 frapages        23 0 0 0  4   5 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  109 fratuples       23 0 0 0  4   6 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  109 fralokey        25 0 0 0 -1   7 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  109 frahikey        25 0 0 0 -1   8 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  109 ctid            27 0 0 0  6  -1 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  109 oid             26 0 0 0  4  -2 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  109 xmin            28 0 0 0  5  -3 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  109 cmin            29 0 0 0  1  -4 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  109 xmax            28 0 0 0  5  -5 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  109 cmax            29 0 0 0  1  -6 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  109 globalxmin      28 0 0 0  4  -7 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  109 globalxmax      28 0 0 0  4  -8 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  109 chain           27 0 0 0  6  -9 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  109 anchor          27 0 0 0  6 -10 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  109 tmax           702 0 0 0  4 -11 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  109 tmin           702 0 0 0  4 -12 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  109 vtype           18 0 0 0  1 -13 0 t t 0 0 -1 f c));

/* ----------------
 *	pg_has
 * ----------------
 */
DATA(insert OID = 0 (  112 datname         19 0 0 0 16   1 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  112 fralogicalid	   26 0 0 0  4   2 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  112 fralocation	   23 0 0 0  4   3 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  112 fradebits      701 0 0 0  8   4 0 f t 0 0 -1 f d));
DATA(insert OID = 0 (  112 fracredits     701 0 0 0  8   5 0 f t 0 0 -1 f d));
DATA(insert OID = 0 (  112 fraexpprice    701 0 0 0  8   6 0 f t 0 0 -1 f d));
DATA(insert OID = 0 (  112 fraaccesses	   23 0 0 0  4   7 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  112 fraarrtime     702 0 0 0  4   8 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  112 fralastacctime 702 0 0 0  4   9 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  112 ctid            27 0 0 0  6  -1 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  112 oid             26 0 0 0  4  -2 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  112 xmin            28 0 0 0  5  -3 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  112 cmin            29 0 0 0  1  -4 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  112 xmax            28 0 0 0  5  -5 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  112 cmax            29 0 0 0  1  -6 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  112 globalxmin      28 0 0 0  4  -7 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  112 globalxmax      28 0 0 0  4  -8 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  112 chain           27 0 0 0  6  -9 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  112 anchor          27 0 0 0  6 -10 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  112 tmax           702 0 0 0  4 -11 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  112 tmin           702 0 0 0  4 -12 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  112 vtype           18 0 0 0  1 -13 0 t t 0 0 -1 f c));

/* ----------------
 *      pg_access
 * ----------------
 */
DATA(insert OID = 0 (  113 datname         19 0 0 0 16   1 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  113 fralogicalid	   26 0 0 0  4   2 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  113 fralocation	   23 0 0 0  4   3 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  113 fradebits      701 0 0 0  4   4 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  113 fracredits     701 0 0 0  4   5 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  113 fraexpprice    701 0 0 0  4   6 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  113 fraaccesses	   23 0 0 0  4   7 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  113 fraarrtime     702 0 0 0  4   8 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  113 fralastacctime 702 0 0 0  4   9 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  113 ctid            27 0 0 0  6  -1 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  113 oid             26 0 0 0  4  -2 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  113 xmin            28 0 0 0  5  -3 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  113 cmin            29 0 0 0  1  -4 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  113 xmax            28 0 0 0  5  -5 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  113 cmax            29 0 0 0  1  -6 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  113 globalxmin      28 0 0 0  4  -7 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  113 globalxmax      28 0 0 0  4  -8 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  113 chain           27 0 0 0  6  -9 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  113 anchor          27 0 0 0  6 -10 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  113 tmax           702 0 0 0  4 -11 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  113 tmin           702 0 0 0  4 -12 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  113 vtype           18 0 0 0  1 -13 0 t t 0 0 -1 f c));

/* ----------------
 *	pg_host
 * ----------------
 */
DATA(insert OID = 0 (  110 hostid          23 0 0 0  4   1 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  110 address	   23 0 0 0  4   2 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  110 ctid            27 0 0 0  6  -1 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  110 oid             26 0 0 0  4  -2 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  110 xmin            28 0 0 0  5  -3 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  110 cmin            29 0 0 0  1  -4 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  110 xmax            28 0 0 0  5  -5 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  110 cmax            29 0 0 0  1  -6 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  110 globalxmin      28 0 0 0  4  -7 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  110 globalxmax      28 0 0 0  4  -8 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  110 chain           27 0 0 0  6  -9 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  110 anchor          27 0 0 0  6 -10 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  110 tmax           702 0 0 0  4 -11 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  110 tmin           702 0 0 0  4 -12 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  110 vtype           18 0 0 0  1 -13 0 t t 0 0 -1 f c));

/* ----------------
 *	pg_contract
 * ----------------
 */
DATA(insert OID = 0 (  111 contractid      26 0 0 0  4   1 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  111 sendfrastoreid  26 0 0 0  4   2 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  111 recvfrastoreid  26 0 0 0  4   3 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  111 sidefrastoreid  26 0 0 0  4   4 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  111 lastupdcommit   23 0 0 0  4   5 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  111 otherloc        23 0 0 0  4   6 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  111 period          23 0 0 0  4   7 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  111 typemask        18 0 0 0  1   8 0 t t 0 0 -1 f c));
DATA(insert OID = 0 (  111 ctid            27 0 0 0  6  -1 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  111 oid             26 0 0 0  4  -2 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  111 xmin            28 0 0 0  5  -3 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  111 cmin            29 0 0 0  1  -4 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  111 xmax            28 0 0 0  5  -5 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  111 cmax            29 0 0 0  1  -6 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  111 globalxmin      28 0 0 0  4  -7 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  111 globalxmax      28 0 0 0  4  -8 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  111 chain           27 0 0 0  6  -9 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  111 anchor          27 0 0 0  6 -10 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  111 tmax           702 0 0 0  4 -11 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  111 tmin           702 0 0 0  4 -12 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  111 vtype           18 0 0 0  1 -13 0 t t 0 0 -1 f c));

#endif /* P95 */
/* ----------------
 *	pg_proc
 * ----------------
 */
#ifndef P95

#define Schema_pg_proc \
{ 81l, "proname",       19l, 81l, 0l, 0l, 16,  1, 0,   '\0', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 81l, "proowner",      26l, 81l, 0l, 0l,  4,  2, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 81l, "prolang",       26l, 81l, 0l, 0l,  4,  3, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 81l, "proisinh",      16l, 81l, 0l, 0l,  1,  4, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'c' }, \
{ 81l, "proistrusted",  16l, 81l, 0l, 0l,  1,  5, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'c' }, \
{ 81l, "proiscachable", 16l, 81l, 0l, 0l,  1,  6, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'c' }, \
{ 81l, "pronargs",      21l, 81l, 0l, 0l,  2,  7, 0, '\001', '\001', 0l, 0l, -1l, '\0', 's' }, \
{ 81l, "proretset",     16l, 81l, 0l, 0l,  1,  8, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'c' }, \
{ 81l, "prorettype",    26l, 81l, 0l, 0l,  4,  9, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 81l, "proargtypes",   30l, 81l, 0l, 0l, 32, 10, 0,   '\0', '\001', 0l, 0l, \
      -1l, '\0', 'i' }, \
{ 81l, "probyte_pct",   23l, 81l, 0l, 0l,  4, 11, 0, '\001', '\001', 0l, 0l, \
      -1l, '\0', 'i' }, \
{ 81l, "properbyte_cpu",   23l, 81l, 0l, 0l,  4, 12, 0, '\001', '\001', 0l, 0l,      -1l, '\0', 'i' }, \
{ 81l, "propercall_cpu",   23l, 81l, 0l, 0l,  4, 13, 0, '\001', '\001', 0l, 0l,      -1l, '\0', 'i' }, \
{ 81l, "prooutin_ratio",   23l, 81l, 0l, 0l,  4, 14, 0, '\001', '\001', 0l, 0l,      -1l, '\0', 'i' }, \
{ 81l, "prosrc",        25l, 81l, 0l, 0l, -1,  15, 0,   '\0', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 81l, "probin",        17l, 81l, 0l, 0l, -1,  16, 0,   '\0', '\001', 0l, 0l, -1l, '\0', 'i' }
#else /* P95 */

#define Schema_pg_proc \
{ 81l, {"proname"},       19l, 81l, 0l, 0l, NAMEDATALEN,  1, 0,   '\0', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 81l, {"proowner"},      26l, 81l, 0l, 0l,  4,  2, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 81l, {"prolang"},       26l, 81l, 0l, 0l,  4,  3, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 81l, {"proisinh"},      16l, 81l, 0l, 0l,  1,  4, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'c' }, \
{ 81l, {"proistrusted"},  16l, 81l, 0l, 0l,  1,  5, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'c' }, \
{ 81l, {"proiscachable"}, 16l, 81l, 0l, 0l,  1,  6, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'c' }, \
{ 81l, {"pronargs"},      21l, 81l, 0l, 0l,  2,  7, 0, '\001', '\001', 0l, 0l, -1l, '\0', 's' }, \
{ 81l, {"proretset"},     16l, 81l, 0l, 0l,  1,  8, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'c' }, \
{ 81l, {"prorettype"},    26l, 81l, 0l, 0l,  4,  9, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 81l, {"proargtypes"},   30l, 81l, 0l, 0l, 32, 10, 0,   '\0', '\001', 0l, 0l, \
      -1l, '\0', 'i' }, \
{ 81l, {"probyte_pct"},   23l, 81l, 0l, 0l,  4, 11, 0, '\001', '\001', 0l, 0l, \
      -1l, '\0', 'i' }, \
{ 81l, {"properbyte_cpu"},   23l, 81l, 0l, 0l,  4, 12, 0, '\001', '\001', 0l, 0l,      -1l, '\0', 'i' }, \
{ 81l, {"propercall_cpu"},   23l, 81l, 0l, 0l,  4, 13, 0, '\001', '\001', 0l, 0l,      -1l, '\0', 'i' }, \
{ 81l, {"prooutin_ratio"},   23l, 81l, 0l, 0l,  4, 14, 0, '\001', '\001', 0l, 0l,      -1l, '\0', 'i' }, \
{ 81l, {"prosrc"},        25l, 81l, 0l, 0l, -1,  15, 0,   '\0', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 81l, {"probin"},        17l, 81l, 0l, 0l, -1,  16, 0,   '\0', '\001', 0l, 0l, -1l, '\0', 'i' }
#endif /* P95 */

#ifndef P95
DATA(insert OID = 0 (  81 proname          19 0 0 0 16   1 0 f t 0 0 -1 f i));
#else /* P95 */
DATA(insert OID = 0 (  81 proname          19 0 0 0 NAMEDATALEN   1 0 f t 0 0 -1 f i));
#endif /* P95 */
DATA(insert OID = 0 (  81 proowner         26 0 0 0  4   2 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  81 prolang          26 0 0 0  4   3 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  81 proisinh         16 0 0 0  1   4 0 t t 0 0 -1 f c));
DATA(insert OID = 0 (  81 proistrusted     16 0 0 0  1   5 0 t t 0 0 -1 f c));
DATA(insert OID = 0 (  81 proiscachable    16 0 0 0  1   6 0 t t 0 0 -1 f c));
DATA(insert OID = 0 (  81 pronargs         21 0 0 0  2   7 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  81 proretset        16 0 0 0  1   8 0 t t 0 0 -1 f c));
DATA(insert OID = 0 (  81 prorettype       26 0 0 0  4   9 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  81 proargtypes      30 0 0 0 32  10 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  81 probyte_pct      23 0 0 0  4  11 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  81 properbyte_cpu   23 0 0 0  4  12 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  81 propercall_cpu   23 0 0 0  4  13 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  81 prooutin_ratio   23 0 0 0  4  14 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  81 prosrc           25 0 0 0 -1  15 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  81 probin           17 0 0 0 -1  16 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  81 ctid             27 0 0 0  6  -1 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  81 oid              26 0 0 0  4  -2 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  81 xmin             28 0 0 0  4  -3 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  81 cmin             29 0 0 0  2  -4 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  81 xmax             28 0 0 0  4  -5 0 f t 0 0 -1 f i));
#ifndef P95
DATA(insert OID = 0 (  81 cmax             29 0 0 0  2  -6 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  81 globalxmin       28 0 0 0  4  -7 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  81 globalxmax       28 0 0 0  4  -8 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  81 chain            27 0 0 0  6  -9 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  81 anchor           27 0 0 0  6 -10 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  81 tmax            702 0 0 0  4 -11 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  81 tmin            702 0 0 0  4 -12 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  81 vtype            18 0 0 0  1 -13 0 t t 0 0 -1 f c));
#else /* P95 */
DATA(insert OID = 0 (  81 cmax             29 0 0 0  2 -6 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  81 chain            27 0 0 0  6  -7 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  81 anchor           27 0 0 0  6  -8 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  81 tmax            702 0 0 0  4  -9 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  81 tmin            702 0 0 0  4 -10 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  81 vtype            18 0 0 0  1 -11 0 t t 0 0 -1 f c));
#endif /* P95 */
    
/* ----------------
 *	pg_server
 * ----------------
 */
#ifndef P95
DATA(insert OID = 0 (  82 sername          19 0 0 0 16   1 0 f t 0 0 -1 f i));
#else /* P95 */
DATA(insert OID = 0 (  82 sername          19 0 0 0 NAMEDATALEN   1 0 f t 0 0 -1 f i));
#endif /* P95 */
DATA(insert OID = 0 (  82 serpid           21 0 0 0  2   2 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  82 serport          21 0 0 0  2   3 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  82 ctid             27 0 0 0  6  -1 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  82 oid              26 0 0 0  4  -2 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  82 xmin             28 0 0 0  4  -3 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  82 cmin             29 0 0 0  2  -4 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  82 xmax             28 0 0 0  4  -5 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  82 cmax             29 0 0 0  2  -6 0 t t 0 0 -1 f s));
#ifndef P95
DATA(insert OID = 0 (  82 globalxmin       28 0 0 0  4  -7 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  82 globalxmax       28 0 0 0  4  -8 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  82 chain            27 0 0 0  6  -9 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  82 anchor           27 0 0 0  6 -10 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  82 tmax            702 0 0 0  4 -11 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  82 tmin            702 0 0 0  4 -12 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  82 vtype            18 0 0 0  1 -13 0 t t 0 0 -1 f c));
#else /* P95 */
DATA(insert OID = 0 (  82 chain            27 0 0 0  6  -7 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  82 anchor           27 0 0 0  6  -8 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  82 tmax            702 0 0 0  4  -9 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  82 tmin            702 0 0 0  4 -10 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  82 vtype            18 0 0 0  1 -11 0 t t 0 0 -1 f c));
#endif /* P95 */
    
/* ----------------
 *	pg_user
 * ----------------
 */
#ifndef P95
DATA(insert OID = 0 (  86 usename          19 0 0 0 16   1 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  86 usesysid         21 0 0 0  2   2 0 t t 0 0 -1 f s));
#else /* P95 */
DATA(insert OID = 0 (  86 usename          19 0 0 0 NAMEDATALEN   1 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  86 usesysid         23 0 0 0  4   2 0 t t 0 0 -1 f s));
#endif /* P95 */
DATA(insert OID = 0 (  86 usecreatedb      16 0 0 0  1   3 0 t t 0 0 -1 f c));
DATA(insert OID = 0 (  86 usetrace         16 0 0 0  1   4 0 t t 0 0 -1 f c));
DATA(insert OID = 0 (  86 usesuper         16 0 0 0  1   5 0 t t 0 0 -1 f c));
DATA(insert OID = 0 (  86 usecatupd        16 0 0 0  1   6 0 t t 0 0 -1 f c));
DATA(insert OID = 0 (  86 ctid             27 0 0 0  6  -1 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  86 oid              26 0 0 0  4  -2 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  86 xmin             28 0 0 0  4  -3 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  86 cmin             29 0 0 0  2  -4 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  86 xmax             28 0 0 0  4  -5 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  86 cmax             29 0 0 0  2  -6 0 t t 0 0 -1 f s));
#ifndef P95
DATA(insert OID = 0 (  86 globalxmin       28 0 0 0  4  -7 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  86 globalxmax       28 0 0 0  4  -8 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  86 chain            27 0 0 0  6  -9 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  86 anchor           27 0 0 0  6 -10 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  86 tmax            702 0 0 0  4 -11 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  86 tmin            702 0 0 0  4 -12 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  86 vtype            18 0 0 0  1 -13 0 t t 0 0 -1 f c));
#else /* P95 */
DATA(insert OID = 0 (  86 chain            27 0 0 0  6  -7 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  86 anchor           27 0 0 0  6  -8 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  86 tmax            702 0 0 0  4  -9 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  86 tmin            702 0 0 0  4 -10 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  86 vtype            18 0 0 0  1 -11 0 t t 0 0 -1 f c));
#endif /* P95 */

/* ----------------
 *	pg_group
 * ----------------
 */
#ifndef P95
DATA(insert OID = 0 (  87 groname          19 0 0 0 16   1 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  87 grosysid         21 0 0 0  2   2 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  87 grolist        1005 0 0 0 -1   3 0 f t 0 0 -1 f i));
#else /* P95 */
DATA(insert OID = 0 (  87 groname          19 0 0 0 NAMEDATALEN   1 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  87 grosysid         23 0 0 0  4   2 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  87 grolist        1007 0 0 0 -1   3 0 f t 0 0 -1 f i));
#endif /* P95 */
DATA(insert OID = 0 (  87 ctid             27 0 0 0  6  -1 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  87 oid              26 0 0 0  4  -2 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  87 xmin             28 0 0 0  4  -3 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  87 cmin             29 0 0 0  2  -4 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  87 xmax             28 0 0 0  4  -5 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  87 cmax             29 0 0 0  2  -6 0 t t 0 0 -1 f s));
#ifndef P95
DATA(insert OID = 0 (  87 globalxmin       28 0 0 0  4  -7 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  87 globalxmax       28 0 0 0  4  -8 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  87 chain            27 0 0 0  6  -9 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  87 anchor           27 0 0 0  6 -10 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  87 tmax            702 0 0 0  4 -11 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  87 tmin            702 0 0 0  4 -12 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  87 vtype            18 0 0 0  1 -13 0 t t 0 0 -1 f c));

#else /* P95 */
DATA(insert OID = 0 (  87 chain            27 0 0 0  6  -7 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  87 anchor           27 0 0 0  6  -8 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  87 tmax            702 0 0 0  4  -9 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  87 tmin            702 0 0 0  4 -10 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  87 vtype            18 0 0 0  1 -11 0 t t 0 0 -1 f c));
    
#endif /* P95 */
/* ----------------
 *	pg_attribute
 * ----------------
 */
#ifndef P95
#define Schema_pg_attribute \
{ 75l, {"attrelid"},    26l, 75l, 0l, 0l,  4,  1, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 75l, {"attname"},     19l, 75l, 0l, 0l, 16,  2, 0,   '\0', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 75l, {"atttypid"},    26l, 75l, 0l, 0l,  4,  3, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 75l, {"attdefrel"},   26l, 75l, 0l, 0l,  4,  4, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 75l, {"attnvals"},    23l, 75l, 0l, 0l,  4,  5, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 75l, {"atttyparg"},   26l, 75l, 0l, 0l,  4,  6, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 75l, {"attlen"},      21l, 75l, 0l, 0l,  2,  7, 0, '\001', '\001', 0l, 0l, -1l, '\0', 's' }, \
{ 75l, {"attnum"},      21l, 75l, 0l, 0l,  2,  8, 0, '\001', '\001', 0l, 0l, -1l, '\0', 's' }, \
{ 75l, {"attbound"},    21l, 75l, 0l, 0l,  2,  9, 0, '\001', '\001', 0l, 0l, -1l, '\0', 's' }, \
{ 75l, {"attbyval"},    16l, 75l, 0l, 0l,  1, 10, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'c' }, \
{ 75l, {"attcanindex"}, 16l, 75l, 0l, 0l,  1, 11, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'c' }, \
{ 75l, {"attproc"},     26l, 75l, 0l, 0l,  4, 12, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 75l, {"attnelems"},   23l, 75l, 0l, 0l,  4, 13, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 75l, {"attcacheoff"}, 23l, 75l, 0l, 0l,  4, 14, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 75l, {"attisset"},    16l, 75l, 0l, 0l,  1, 15, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'c' }, \
{ 75l, {"attalign"},    18l, 75l, 0l, 0l,  1, 16, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'c' }
#else /* P95 */
#define Schema_pg_attribute \
{ 75l, {"attrelid"},    26l, 75l, 0l, 0l,  4,  1, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 75l, {"attname"},     19l, 75l, 0l, 0l, NAMEDATALEN,  2, 0,   '\0', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 75l, {"atttypid"},    26l, 75l, 0l, 0l,  4,  3, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 75l, {"attdefrel"},   26l, 75l, 0l, 0l,  4,  4, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 75l, {"attnvals"},    23l, 75l, 0l, 0l,  4,  5, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 75l, {"atttyparg"},   26l, 75l, 0l, 0l,  4,  6, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 75l, {"attlen"},      21l, 75l, 0l, 0l,  2,  7, 0, '\001', '\001', 0l, 0l, -1l, '\0', 's' }, \
{ 75l, {"attnum"},      21l, 75l, 0l, 0l,  2,  8, 0, '\001', '\001', 0l, 0l, -1l, '\0', 's' }, \
{ 75l, {"attbound"},    21l, 75l, 0l, 0l,  2,  9, 0, '\001', '\001', 0l, 0l, -1l, '\0', 's' }, \
{ 75l, {"attbyval"},    16l, 75l, 0l, 0l,  1, 10, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'c' }, \
{ 75l, {"attcanindex"}, 16l, 75l, 0l, 0l,  1, 11, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'c' }, \
{ 75l, {"attproc"},     26l, 75l, 0l, 0l,  4, 12, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 75l, {"attnelems"},   23l, 75l, 0l, 0l,  4, 13, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 75l, {"attcacheoff"}, 23l, 75l, 0l, 0l,  4, 14, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 75l, {"attisset"},    16l, 75l, 0l, 0l,  1, 15, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'c' }, \
{ 75l, {"attalign"},    18l, 75l, 0l, 0l,  1, 16, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'c' }
#endif /* P95 */

DATA(insert OID = 0 (  75 attrelid         26 0 0 0  4   1 0 t t 0 0 -1 f i));
#ifndef P95
DATA(insert OID = 0 (  75 attname          19 0 0 0 16   2 0 f t 0 0 -1 f i));
#else /* P95 */
DATA(insert OID = 0 (  75 attname          19 0 0 0 NAMEDATALEN   2 0 f t 0 0 -1 f i));
#endif /* P95 */
DATA(insert OID = 0 (  75 atttypid         26 0 0 0  4   3 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  75 attdefrel        26 0 0 0  4   4 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  75 attnvals         23 0 0 0  4   5 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  75 atttyparg        26 0 0 0  4   6 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  75 attlen           21 0 0 0  2   7 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  75 attnum           21 0 0 0  2   8 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  75 attbound         21 0 0 0  2   9 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  75 attbyval         16 0 0 0  1  10 0 t t 0 0 -1 f c));
DATA(insert OID = 0 (  75 attcanindex      16 0 0 0  1  11 0 t t 0 0 -1 f c));
DATA(insert OID = 0 (  75 attproc          26 0 0 0  4  12 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  75 attnelems        23 0 0 0  4  13 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  75 attcacheoff      23 0 0 0  4  14 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  75 attisset         16 0 0 0  1  15 0 t t 0 0 -1 f c));
DATA(insert OID = 0 (  75 attalign         18 0 0 0  1  16 0 t t 0 0 -1 f c));
DATA(insert OID = 0 (  75 ctid             27 0 0 0  6  -1 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  75 oid              26 0 0 0  4  -2 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  75 xmin             28 0 0 0  4  -3 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  75 cmin             29 0 0 0  2  -4 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  75 xmax             28 0 0 0  4  -5 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  75 cmax             29 0 0 0  2  -6 0 t t 0 0 -1 f s));
#ifndef P95
DATA(insert OID = 0 (  75 globalxmin       28 0 0 0  4  -7 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  75 globalxmax       28 0 0 0  4  -8 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  75 chain            27 0 0 0  6  -9 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  75 anchor           27 0 0 0  6 -10 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  75 tmax            702 0 0 0  4 -11 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  75 tmin            702 0 0 0  4 -12 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  75 vtype            18 0 0 0  1 -13 0 t t 0 0 -1 f c));

#else /* P95 */
DATA(insert OID = 0 (  75 chain            27 0 0 0  6  -7 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  75 anchor           27 0 0 0  6  -8 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  75 tmax            702 0 0 0  4  -9 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  75 tmin            702 0 0 0  4 -10 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  75 vtype            18 0 0 0  1 -11 0 t t 0 0 -1 f c));
    
#endif /* P95 */
/* ----------------
 *	pg_class
 * ----------------
 */
#ifndef P95

#define Schema_pg_class \
{ 83l, {"relname"},      19l, 83l, 0l, 0l, 16,  1, 0,   '\0', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 83l, {"relowner"},     26l, 83l, 0l, 0l,  4,  2, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 83l, {"relam"},        26l, 83l, 0l, 0l,  4,  3, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 83l, {"relpages"},     23,  83l, 0l, 0l,  4,  4, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 83l, {"reltuples"},    23,  83l, 0l, 0l,  4,  5, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 83l, {"relexpires"},   702,  83l, 0l, 0l,  4,  6, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 83l, {"relpreserved"}, 703,  83l, 0l, 0l,  4,  7, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 83l, {"relhasindex"},  16,  83l, 0l, 0l,  1,  8, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'c' }, \
{ 83l, {"relisshared"},  16,  83l, 0l, 0l,  1,  9, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'c' }, \
{ 83l, {"relkind"},      18,  83l, 0l, 0l,  1, 10, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'c' }, \
{ 83l, {"relarch"},      18,  83l, 0l, 0l,  1, 11, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'c' }, \
{ 83l, {"relnatts"},     21,  83l, 0l, 0l,  2, 12, 0, '\001', '\001', 0l, 0l, -1l, '\0', 's' }, \
{ 83l, {"relsmgr"},      210l,  83l, 0l, 0l,  2, 13, 0, '\001', '\001', 0l, 0l, -1l, '\0', 's' }, \
{ 83l, {"relkey"},       22,  83l, 0l, 0l, 16, 14, 0,   '\0', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 83l, {"relkeyop"},     30,  83l, 0l, 0l, 32, 15, 0,   '\0', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 83l, {"relpartcmp"},   24l, 83l, 0l, 0l,  4, 16, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 83l, {"relparthash"},  24l, 83l, 0l, 0l,  4, 17, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 83l, {"relpartatt"},   21l, 83l, 0l, 0l,  2, 18, 0, '\001', '\001', 0l, 0l, -1l, '\0', 's' }, \
{ 83l, {"relpartmode"},  18,  83l, 0l, 0l,  1, 19, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'c' }, \
{ 83l, {"relhasrules"},  16,  83l, 0l, 0l,  1, 20, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'c' }, \
{ 83l, {"relacl"},     1034l, 83l, 0l, 0l, -1, 21, 0,   '\0', '\001', 0l, 0l, -1l, '\0', 'i' }
#else /* P95 */


#define Schema_pg_class \
{ 83l, {"relname"},      19l, 83l, 0l, 0l, NAMEDATALEN,  1, 0,   '\0', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 83l, {"reltype"},     26l, 83l, 0l, 0l,  4,  2, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 83l, {"relowner"},     26l, 83l, 0l, 0l,  4,  2, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 83l, {"relam"},        26l, 83l, 0l, 0l,  4,  3, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 83l, {"relpages"},     23,  83l, 0l, 0l,  4,  4, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 83l, {"reltuples"},    23,  83l, 0l, 0l,  4,  5, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 83l, {"relexpires"},   702,  83l, 0l, 0l,  4,  6, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 83l, {"relpreserved"}, 703,  83l, 0l, 0l,  4,  7, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 83l, {"relhasindex"},  16,  83l, 0l, 0l,  1,  8, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'c' }, \
{ 83l, {"relisshared"},  16,  83l, 0l, 0l,  1,  9, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'c' }, \
{ 83l, {"relkind"},      18,  83l, 0l, 0l,  1, 10, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'c' }, \
{ 83l, {"relarch"},      18,  83l, 0l, 0l,  1, 11, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'c' }, \
{ 83l, {"relnatts"},     21,  83l, 0l, 0l,  2, 12, 0, '\001', '\001', 0l, 0l, -1l, '\0', 's' }, \
{ 83l, {"relsmgr"},      210l,  83l, 0l, 0l,  2, 13, 0, '\001', '\001', 0l, 0l, -1l, '\0', 's' }, \
{ 83l, {"relkey"},       22,  83l, 0l, 0l, 16, 14, 0,   '\0', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 83l, {"relkeyop"},     30,  83l, 0l, 0l, 32, 15, 0,   '\0', '\001', 0l, 0l, -1l, '\0', 'i' }, \
{ 83l, {"relhasrules"},  16,  83l, 0l, 0l,  1, 16, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'c' }, \
{ 83l, {"relacl"},     1034l, 83l, 0l, 0l, -1, 17, 0,   '\0', '\001', 0l, 0l, -1l, '\0', 'i' }
#endif /* P95 */

#ifndef P95
DATA(insert OID = 0 (  83 relname          19 0 0 0 16   1 0 f t 0 0 -1 f i));
#else /* P95 */
DATA(insert OID = 0 (  83 relname          19 0 0 0 NAMEDATALEN   1 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  83 reltype          26 0 0 0  4   2 0 t t 0 0 -1 f i));
#endif /* P95 */
DATA(insert OID = 0 (  83 relowner         26 0 0 0  4   2 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  83 relam            26 0 0 0  4   3 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  83 relpages         23 0 0 0  4   4 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  83 reltuples        23 0 0 0  4   5 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  83 relexpires      702 0 0 0  4   6 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  83 relpreserved    702 0 0 0  4   7 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  83 relhasindex      16 0 0 0  1   8 0 t t 0 0 -1 f c));
DATA(insert OID = 0 (  83 relisshared      16 0 0 0  1   9 0 t t 0 0 -1 f c));
DATA(insert OID = 0 (  83 relkind          18 0 0 0  1  10 0 t t 0 0 -1 f c));
DATA(insert OID = 0 (  83 relarch          18 0 0 0  1  11 0 t t 0 0 -1 f c));
DATA(insert OID = 0 (  83 relnatts         21 0 0 0  2  12 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  83 relsmgr         210 0 0 0  2  13 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  83 relkey           22 0 0 0 16  14 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  83 relkeyop         30 0 0 0 32  15 0 f t 0 0 -1 f i));
#ifndef P95
DATA(insert OID = 0 (  83 relpartcmp       26 0 0 0  4  16 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  83 relparthash      26 0 0 0  4  17 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  83 relpartatt       21 0 0 0  2  18 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  83 relpartmode      18 0 0 0  1  19 0 t t 0 0 -1 f c));
DATA(insert OID = 0 (  83 relhasrules      16 0 0 0  1  20 0 t t 0 0 -1 f c));
DATA(insert OID = 0 (  83 relacl         1034 0 0 0 -1  21 0 f t 0 0 -1 f i));
#else /* P95 */
DATA(insert OID = 0 (  83 relhasrules      16 0 0 0  1  16 0 t t 0 0 -1 f c));
DATA(insert OID = 0 (  83 relacl         1034 0 0 0 -1  17 0 f t 0 0 -1 f i));
#endif /* P95 */
DATA(insert OID = 0 (  83 ctid             27 0 0 0  6  -1 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  83 oid              26 0 0 0  4  -2 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  83 xmin             28 0 0 0  4  -3 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  83 cmin             29 0 0 0  2  -4 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  83 xmax             28 0 0 0  4  -5 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  83 cmax             29 0 0 0  2  -6 0 t t 0 0 -1 f s));
#ifndef P95
DATA(insert OID = 0 (  83 globalxmin       28 0 0 0  4  -7 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  83 globalxmax       28 0 0 0  4  -8 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  83 chain            27 0 0 0  6  -9 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  83 anchor           27 0 0 0  6 -10 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  83 tmax            702 0 0 0  4 -11 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  83 tmin            702 0 0 0  4 -12 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  83 vtype            18 0 0 0  1 -13 0 t t 0 0 -1 f c));

#else /* P95 */
DATA(insert OID = 0 (  83 chain            27 0 0 0  6  -7 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  83 anchor           27 0 0 0  6  -8 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  83 tmax            702 0 0 0  4  -9 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  83 tmin            702 0 0 0  4 -10 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  83 vtype            18 0 0 0  1 -11 0 t t 0 0 -1 f c));
    
#endif /* P95 */
/* ----------------
 *	pg_magic
 * ----------------
 */
#ifndef P95
DATA(insert OID = 0 (  80 magname          19 0 0 0 16   1 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  80 magvalue         19 0 0 0 16   2 0 f t 0 0 -1 f i));
#else /* P95 */
DATA(insert OID = 0 (  80 magname          19 0 0 0 NAMEDATALEN   1 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  80 magvalue         19 0 0 0 NAMEDATALEN   2 0 f t 0 0 -1 f i));
#endif /* P95 */
DATA(insert OID = 0 (  80 ctid             27 0 0 0  6  -1 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  80 oid              26 0 0 0  4  -2 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  80 xmin             28 0 0 0  4  -3 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  80 cmin             29 0 0 0  2  -4 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  80 xmax             28 0 0 0  4  -5 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  80 cmax             29 0 0 0  2  -6 0 t t 0 0 -1 f s));
#ifndef P95
DATA(insert OID = 0 (  80 globalxmin       28 0 0 0  4  -7 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  80 globalxmax       28 0 0 0  4  -8 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  80 chain            27 0 0 0  6  -9 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  80 anchor           27 0 0 0  6 -10 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  80 tmax            702 0 0 0  4 -11 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  80 tmin            702 0 0 0  4 -12 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  80 vtype            18 0 0 0  1 -13 0 t t 0 0 -1 f c));
#else /* P95 */
DATA(insert OID = 0 (  80 chain            27 0 0 0  6  -7 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  80 anchor           27 0 0 0  6  -8 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  80 tmax            702 0 0 0  4  -9 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  80 tmin            702 0 0 0  4 -10 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  80 vtype            18 0 0 0  1 -11 0 t t 0 0 -1 f c));
#endif /* P95 */
    
/* ----------------
 *	pg_defaults
 * ----------------
 */
#ifndef P95
DATA(insert OID = 0 (  89 defname          19 0 0 0 16   1 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  89 defvalue         19 0 0 0 16   2 0 f t 0 0 -1 f i));
#else /* P95 */
DATA(insert OID = 0 (  89 defname          19 0 0 0 NAMEDATALEN   1 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  89 defvalue         19 0 0 0 NAMEDATALEN   2 0 f t 0 0 -1 f i));
#endif /* P95 */
DATA(insert OID = 0 (  89 ctid             27 0 0 0  6  -1 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  89 oid              26 0 0 0  4  -2 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  89 xmin             28 0 0 0  4  -3 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  89 cmin             29 0 0 0  2  -4 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  89 xmax             28 0 0 0  4  -5 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  89 cmax             29 0 0 0  2  -6 0 t t 0 0 -1 f s));
#ifndef P95
DATA(insert OID = 0 (  89 globalxmin       28 0 0 0  4  -7 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  89 globalxmax       28 0 0 0  4  -8 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  89 chain            27 0 0 0  6  -9 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  89 anchor           27 0 0 0  6 -10 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  89 tmax            702 0 0 0  4 -11 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  89 tmin            702 0 0 0  4 -12 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  89 vtype            18 0 0 0  1 -13 0 t t 0 0 -1 f c));
#else /* P95 */
DATA(insert OID = 0 (  89 chain            27 0 0 0  6  -7 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  89 anchor           27 0 0 0  6  -8 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  89 tmax            702 0 0 0  4  -9 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  89 tmin            702 0 0 0  4 -10 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  89 vtype            18 0 0 0  1 -11 0 t t 0 0 -1 f c));
#endif /* P95 */
    
#ifdef P95

#endif /* P95 */
#ifdef P95
/* ----------------
 *	pg_hosts - this relation is used to store host based authentication
 *	           info
 *		  
 * ----------------
 */
DATA(insert OID = 0 (  101 dbName           19 0 0 0  NAMEDATALEN   1 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  101 address           25 0 0 0  -1   2 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  101 mask           25 0 0 0  -1   3 0 f t 0 0 -1 f i));

#endif /* P95 */
/* ----------------
 *	pg_variable - this relation is modified by special purpose access
 *	          method code.  The following is garbage but is needed
 *		  so that the reldesc code works properly.
 * ----------------
 */
#ifndef P95
#define Schema_pg_variable \
{ 90l, "varfoo",  26l, 90l, 0l, 0l, 4, 1, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }
#else /* P95 */
#define Schema_pg_variable \
{ 90l, {"varfoo"},  26l, 90l, 0l, 0l, 4, 1, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }
#endif /* P95 */
    
DATA(insert OID = 0 (  90 varfoo           26 0 0 0  4   1 0 t t 0 0 -1 f i));
    
/* ----------------
 *	pg_log - this relation is modified by special purpose access
 *	          method code.  The following is garbage but is needed
 *		  so that the reldesc code works properly.
 * ----------------
 */
#define Schema_pg_log \
{ 99l, {"logfoo"},  26l, 99l, 0l, 0l, 4, 1, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }

DATA(insert OID = 0 (  99 logfoo           26 0 0 0  4   1 0 t t 0 0 -1 f i));
    
/* ----------------
 *	pg_time - this relation is modified by special purpose access
 *	          method code.  The following is garbage but is needed
 *		  so that the reldesc code works properly.
 * ----------------
 */
#define Schema_pg_time \
{ 100l, {"timefoo"},  26l, 100l, 0l, 0l, 4, 1, 0, '\001', '\001', 0l, 0l, -1l, '\0', 'i' }

DATA(insert OID = 0 (  100 timefoo         26 0 0 0  4   1 0 t t 0 0 -1 f i));
#ifndef P95

/* ----------------
 *	pg_nsvcfrag
 * ----------------
 */
DATA(insert OID = 0 (  114 fraclassid      26 0 0 0  4   1 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  114 frastoreid	   26 0 0 0  4   2 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  114 fralogicalid	   26 0 0 0  4   3 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  114 fralocation     23 0 0 0  4   4 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  114 frapages        23 0 0 0  4   5 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  114 fratuples       23 0 0 0  4   6 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  114 fralokey        25 0 0 0 -1   7 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  114 frahikey        25 0 0 0 -1   8 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  114 ctid            27 0 0 0  6  -1 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  114 oid             26 0 0 0  4  -2 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  114 xmin            28 0 0 0  5  -3 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  114 cmin            29 0 0 0  1  -4 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  114 xmax            28 0 0 0  5  -5 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  114 cmax            29 0 0 0  1  -6 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  114 globalxmin      28 0 0 0  4  -7 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  114 globalxmax      28 0 0 0  4  -8 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  114 chain           27 0 0 0  6  -9 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  114 anchor          27 0 0 0  6 -10 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  114 tmax           702 0 0 0  4 -11 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  114 tmin           702 0 0 0  4 -12 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  114 vtype           18 0 0 0  1 -13 0 t t 0 0 -1 f c));

/* ----------------
 *	pg_nsvcattr
 * ----------------
 */
DATA(insert OID = 0 (  115 attrelid         26 0 0 0  4   1 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  115 attname          19 0 0 0 16   2 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  115 atttypid         26 0 0 0  4   3 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  115 attdefrel        26 0 0 0  4   4 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  115 attnvals         23 0 0 0  4   5 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  115 atttyparg        26 0 0 0  4   6 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  115 attlen           21 0 0 0  2   7 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  115 attnum           21 0 0 0  2   8 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  115 attbound         21 0 0 0  2   9 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  115 attbyval         16 0 0 0  1  10 0 t t 0 0 -1 f c));
DATA(insert OID = 0 (  115 attcanindex      16 0 0 0  1  11 0 t t 0 0 -1 f c));
DATA(insert OID = 0 (  115 attproc          26 0 0 0  4  12 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  115 attnelems        23 0 0 0  4  13 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  115 attcacheoff      23 0 0 0  4  14 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  115 attisset         16 0 0 0  1  15 0 t t 0 0 -1 f c));
DATA(insert OID = 0 (  115 attalign         18 0 0 0  1  16 0 t t 0 0 -1 f c));
DATA(insert OID = 0 (  115 ctid             27 0 0 0  6  -1 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  115 oid              26 0 0 0  4  -2 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  115 xmin             28 0 0 0  4  -3 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  115 cmin             29 0 0 0  2  -4 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  115 xmax             28 0 0 0  4  -5 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  115 cmax             29 0 0 0  2  -6 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  115 globalxmin       28 0 0 0  4  -7 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  115 globalxmax       28 0 0 0  4  -8 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  115 chain            27 0 0 0  6  -9 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  115 anchor           27 0 0 0  6 -10 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  115 tmax            702 0 0 0  4 -11 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  115 tmin            702 0 0 0  4 -12 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  115 vtype            18 0 0 0  1 -13 0 t t 0 0 -1 f c));

/* ----------------
 *	pg_nsvcclass
 * ----------------
 */
DATA(insert OID = 0 (  116 relname          19 0 0 0 16   1 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  116 relowner         26 0 0 0  4   2 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  116 relam            26 0 0 0  4   3 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  116 relpages         23 0 0 0  4   4 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  116 reltuples        23 0 0 0  4   5 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  116 relexpires      702 0 0 0  4   6 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  116 relpreserved    702 0 0 0  4   7 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  116 relhasindex      16 0 0 0  1   8 0 t t 0 0 -1 f c));
DATA(insert OID = 0 (  116 relisshared      16 0 0 0  1   9 0 t t 0 0 -1 f c));
DATA(insert OID = 0 (  116 relkind          18 0 0 0  1  10 0 t t 0 0 -1 f c));
DATA(insert OID = 0 (  116 relarch          18 0 0 0  1  11 0 t t 0 0 -1 f c));
DATA(insert OID = 0 (  116 relnatts         21 0 0 0  2  12 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  116 relsmgr         210 0 0 0  2  13 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  116 relkey           22 0 0 0 16  14 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  116 relkeyop         30 0 0 0 32  15 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  116 relpartcmp       26 0 0 0  4  16 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  116 relparthash      26 0 0 0  4  17 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  116 relpartatt       21 0 0 0  2  18 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  116 relpartmode      18 0 0 0  1  19 0 t t 0 0 -1 f c));
DATA(insert OID = 0 (  116 relhasrules      16 0 0 0  1  20 0 t t 0 0 -1 f c));
DATA(insert OID = 0 (  116 relacl         1034 0 0 0 -1  21 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  116 ctid             27 0 0 0  6  -1 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  116 oid              26 0 0 0  4  -2 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  116 xmin             28 0 0 0  4  -3 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  116 cmin             29 0 0 0  2  -4 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  116 xmax             28 0 0 0  4  -5 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  116 cmax             29 0 0 0  2  -6 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  116 globalxmin       28 0 0 0  4  -7 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  116 globalxmax       28 0 0 0  4  -8 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  116 chain            27 0 0 0  6  -9 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  116 anchor           27 0 0 0  6 -10 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  116 tmax            702 0 0 0  4 -11 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  116 tmin            702 0 0 0  4 -12 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  116 vtype            18 0 0 0  1 -13 0 t t 0 0 -1 f c));

/* ----------------
 *	pg_ledger
 * ----------------
 */
DATA(insert OID = 0 (  225 procsite         23 0 0 0  4   1 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  225 bidamount       701 0 0 0  8   2 0 f t 0 0 -1 f d));
DATA(insert OID = 0 (  225 recdamount      701 0 0 0  8   3 0 f t 0 0 -1 f d));
DATA(insert OID = 0 (  225 bidwon           16 0 0 0  1   4 0 t t 0 0 -1 f c));
DATA(insert OID = 0 (  225 planstr          25 0 0 0 -1   5 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  225 ctid             27 0 0 0  6  -1 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  225 oid              26 0 0 0  4  -2 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  225 xmin             28 0 0 0  4  -3 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  225 cmin             29 0 0 0  2  -4 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  225 xmax             28 0 0 0  4  -5 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  225 cmax             29 0 0 0  2  -6 0 t t 0 0 -1 f s));
DATA(insert OID = 0 (  225 globalxmin       28 0 0 0  4  -7 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  225 globalxmax       28 0 0 0  4  -8 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  225 chain            27 0 0 0  6  -9 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  225 anchor           27 0 0 0  6 -10 0 f t 0 0 -1 f i));
DATA(insert OID = 0 (  225 tmax            702 0 0 0  4 -11 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  225 tmin            702 0 0 0  4 -12 0 t t 0 0 -1 f i));
DATA(insert OID = 0 (  225 vtype            18 0 0 0  1 -13 0 t t 0 0 -1 f c));
#endif /* P95 */
#endif /* PG_ATTRIBUTE_H */
