/*-------------------------------------------------------------------------
 *
 * pg_class.h--
 *    definition of the system "relation" relation (pg_class)
 *    along with the relation's initial contents.
 *
 *
 * Copyright (c) 1994, Regents of the University of California
 *
 * $Id: pg_class.h,v 1.17 1996/06/07 04:54:53 am Exp $
 *
 * NOTES
 *    ``pg_relation'' is being replaced by ``pg_class''.  currently
 *    we are only changing the name in the catalogs but someday the
 *    code will be changed too. -cim 2/26/90
 *    [it finally happens.  -ay 11/5/94]
 *
 *    the genbki.sh script reads this file and generates .bki
 *    information from the DATA() statements.
 *
 *-------------------------------------------------------------------------
 */
/* [Mariposa] called this PG_CLASS_H in the header guards */

#ifndef PG_RELATION_H
#define PG_RELATION_H

/* ----------------
 *	postgres.h contains the system type definintions and the
 *	CATALOG(), BOOTSTRAP and DATA() sugar words so this file
 *	can be read by both genbki.sh and the C compiler.
 * ----------------
 */
#include "postgres.h"
#include "utils/nabstime.h"

/* ----------------
 *	pg_class definition.  cpp turns this into
 *	typedef struct FormData_pg_class
 *
 *	Note: the #if 0, #endif around the BKI_BEGIN.. END block
 *	      below keeps cpp from seeing what is meant for the
 *	      genbki script: pg_relation is now called pg_class, but
 *	      only in the catalogs -cim 2/26/90
 * ----------------
 */

/* ----------------
 *	This structure is actually variable-length (the last attribute is
 *	a POSTGRES array).  Hence, sizeof(FormData_pg_class) does not
 *	describe the fixed-length or actual size of the structure.
 *	FormData_pg_class.relacl may not be correctly aligned, either,
 *	if aclitem and struct varlena don't align together.  Hence,
 *	you MUST use heap_getattr() to get the relacl field.
 * ----------------
 */
CATALOG(pg_class) BOOTSTRAP {
#ifndef P95
     char16 	relname;
#else /* P95 */
     NameData 	relname;
     Oid        reltype;          
#endif /* P95 */
     Oid 	relowner;
     Oid 	relam;
     int4 	relpages;
     int4 	reltuples;
#ifndef P95
     dt 	relexpires;
     dt 	relpreserved;
#else /* P95 */
     int4 	relexpires; /* really used as a abstime, but fudge it for now*/
     int4	relpreserved;/*really used as a reltime, but fudge it for now*/
#endif /* P95 */
     bool 	relhasindex;
     bool 	relisshared;
     char 	relkind;
#ifndef P95
     char 	relarch;
#else /* P95 */
     char 	relarch; /* 'h' = heavy, 'l' = light, 'n' = no archival*/
#endif /* P95 */
     int2 	relnatts;
     int2	relsmgr;
     int28 	relkey;			/* not used */
     oid8	relkeyop;		/* not used */
#ifndef P95
     regproc	relpartcmp;		/* comparison function */
     regproc	relparthash;		/* hash partition function */
     int2	relpartatt;		/* partitioning attribute */
     char	relpartmode;		/* partitioning mode */
#endif /* P95 */
     bool	relhasrules;
     aclitem	relacl[1];		/* this is here for the catalog */
} FormData_pg_class;

#define CLASS_TUPLE_SIZE \
     (offsetof(FormData_pg_class,relhasrules) + sizeof(bool))

/* ----------------
 *	Form_pg_class corresponds to a pointer to a tuple with
 *	the format of pg_class relation.
 * ----------------
 */
typedef FormData_pg_class	*Form_pg_class;

/* ----------------
 *	compiler constants for pg_class
 * ----------------
 */
#ifndef P95
extern Name Name_pg_class;
#endif /* P95 */

/* ----------------
 *	Natts_pg_class_fixed is used to tell routines that insert new
 *	pg_class tuples (as opposed to replacing old ones) that there's no
 *	relacl field.
 * ----------------
 */
#ifndef P95
#define Natts_pg_class_fixed		20
#define Natts_pg_class			21
#else /* P95 */
#define Natts_pg_class_fixed		17
#define Natts_pg_class			18
#endif /* P95 */
#define Anum_pg_class_relname		1
#ifndef P95
#define Anum_pg_class_relowner		2
#define Anum_pg_class_relam		3
#define Anum_pg_class_relpages		4
#define Anum_pg_class_reltuples		5
#define Anum_pg_class_relexpires	6
#define Anum_pg_class_relpreserved	7
#define Anum_pg_class_relhasindex	8
#define Anum_pg_class_relisshared	9
#define Anum_pg_class_relkind		10
#define Anum_pg_class_relarch		11
#define Anum_pg_class_relnatts		12
#define Anum_pg_class_relsmgr		13
#define Anum_pg_class_relkey		14
#define Anum_pg_class_relkeyop		15
#define Anum_pg_class_relpartcmp	16
#define Anum_pg_class_relparthash	17
#define Anum_pg_class_relpartatt	18
#define Anum_pg_class_relpartmode	19
#define Anum_pg_class_relhasrules	20
#define Anum_pg_class_relacl		21
#else /* P95 */
#define Anum_pg_class_reltype           2
#define Anum_pg_class_relowner		3
#define Anum_pg_class_relam		4
#define Anum_pg_class_relpages		5
#define Anum_pg_class_reltuples		6
#define Anum_pg_class_relexpires	7
#define Anum_pg_class_relpreserved	8
#define Anum_pg_class_relhasindex	9
#define Anum_pg_class_relisshared	10
#define Anum_pg_class_relkind		11
#define Anum_pg_class_relarch		12
#define Anum_pg_class_relnatts		13
#define Anum_pg_class_relsmgr		14
#define Anum_pg_class_relkey		15
#define Anum_pg_class_relkeyop		16
#define Anum_pg_class_relhasrules	17
#define Anum_pg_class_relacl		18
#endif /* P95 */

/* ----------------
 *	initial contents of pg_class
 * ----------------
 */

#ifndef P95
DATA(insert OID =  71 (  pg_type           PGUID 0 0 0 0 0 f f r n 16 0 - - - - 0 R f _null_ ));
DATA(insert OID =  88 (  pg_database       PGUID 0 0 0 0 0 f t r n 3 0 - - - - 0 R f _null_ ));
DATA(insert OID =  76 (  pg_demon          PGUID 0 0 0 0 0 f t r n 4 0 - - - - 0 R f _null_ ));
DATA(insert OID = 109 (  pg_fragment       PGUID 0 0 0 0 0 f f r n 8 0 - - - - 0 R f _null_ ));
DATA(insert OID = 110 (  pg_host           PGUID 0 0 0 0 0 f t r n 2 0 - - - - 0 R f _null_ ));
DATA(insert OID = 111 (  pg_contract       PGUID 0 0 0 0 0 f t r n 8 0 - - - - 0 R f _null_ ));
DATA(insert OID =  81 (  pg_proc           PGUID 0 0 0 0 0 f f r n 16 0 - - - - 0 R f _null_ ));
DATA(insert OID =  82 (  pg_server         PGUID 0 0 0 0 0 f t r n 3 0 - - - - 0 R f _null_ ));
DATA(insert OID =  86 (  pg_user           PGUID 0 0 0 0 0 f t r n 6 0 - - - - 0 R f _null_ ));
DATA(insert OID =  75 (  pg_attribute      PGUID 0 0 0 0 0 f f r n 16 0 - - - - 0 R f _null_ ));
DATA(insert OID =  83 (  pg_class          PGUID 0 0 0 0 0 f f r n 20 0 - - - - 0 R f _null_ ));    
DATA(insert OID =  80 (  pg_magic          PGUID 0 0 0 0 0 f t r n 2 0 - - - - 0 R f _null_ ));
DATA(insert OID =  89 (  pg_defaults       PGUID 0 0 0 0 0 f t r n 2 0 - - - - 0 R f _null_ ));
DATA(insert OID =  90 (  pg_variable       PGUID 0 0 0 0 0 f t s n 2 0 - - - - 0 R f _null_ ));
DATA(insert OID =  99 (  pg_log            PGUID 0 0 0 0 0 f t s n 1 0 - - - - 0 R f _null_ ));
DATA(insert OID = 100 (  pg_time           PGUID 0 0 0 0 0 f t s n 1 0 - - - - 0 R f _null_ ));
DATA(insert OID =  87 (  pg_group          PGUID 0 0 0 0 0 f t r n 3 0 - - - - 0 R f _null_ ));
DATA(insert OID =  115 (  pg_nsvcattr      PGUID 0 0 0 0 0 f t r n 16 0 - - - - 0 R f _null_ ));
DATA(insert OID =  116 (  pg_nsvcclass     PGUID 0 0 0 0 0 f t r n 20 0 - - - - 0 R f _null_ ));    
DATA(insert OID =  225 (  pg_ledger        PGUID 0 0 0 0 0 f t r n 5 0 - - - - 0 R f _null_ ));
DATA(insert OID = 114 (  pg_nsvcfrag       PGUID 0 0 0 0 0 f t r n 8 0 - - - - 0 R f _null_ ));
DATA(insert OID = 112 (  pg_has            PGUID 0 0 0 0 0 f t r n 9 0 - - - - 0 R f _null_ ));
DATA(insert OID = 113 (  pg_access         PGUID 0 0 0 0 0 f t r n 9 0 - - - - 0 R f _null_ ));
#else /* P95 */
DATA(insert OID =  71 (  pg_type 71          PGUID 0 0 0 0 0 f f r n 16 0 - - f _null_ ));
DATA(insert OID =  75 (  pg_attribute 75      PGUID 0 0 0 0 0 f f r n 16 0 - - f _null_ ));
DATA(insert OID =  76 (  pg_demon 76          PGUID 0 0 0 0 0 f t r n 4 0 - - f _null_ ));
DATA(insert OID =  80 (  pg_magic 80         PGUID 0 0 0 0 0 f t r n 2 0 - - f _null_ ));
DATA(insert OID =  81 (  pg_proc 81          PGUID 0 0 0 0 0 f f r n 16 0 - - f _null_ ));
DATA(insert OID =  82 (  pg_server 82         PGUID 0 0 0 0 0 f t r n 3 0 - - f _null_ ));
DATA(insert OID =  83 (  pg_class 83         PGUID 0 0 0 0 0 f f r n 17 0 - - f _null_ ));    
DATA(insert OID =  86 (  pg_user 86          PGUID 0 0 0 0 0 f t r n 6 0 - - f _null_ ));
DATA(insert OID =  87 (  pg_group 87          PGUID 0 0 0 0 0 f t s n 3 0 - - f _null_ ));
DATA(insert OID =  88 (  pg_database 88      PGUID 0 0 0 0 0 f t r n 3 0 - - f _null_ ));
DATA(insert OID =  89 (  pg_defaults 89       PGUID 0 0 0 0 0 f t r n 2 0 - - f _null_ ));
DATA(insert OID =  90 (  pg_variable 90        PGUID 0 0 0 0 0 f t s n 2 0 - - f _null_ ));
DATA(insert OID =  99 (  pg_log  99           PGUID 0 0 0 0 0 f t s n 1 0 - - f _null_ ));
DATA(insert OID = 100 (  pg_time 100           PGUID 0 0 0 0 0 f t s n 1 0 - - f _null_ ));
DATA(insert OID = 101 (  pg_hosts 101           PGUID 0 0 0 0 0 f t s n 3 0 - - f _null_ ));
#endif /* P95 */

#define RelOid_pg_type		71
#ifndef P95
#define RelOid_pg_database    	88   
#endif /* P95 */
#define RelOid_pg_demon       	76   
#ifndef P95
#define RelOid_pg_fragment	109
#define RelOid_pg_host		110
#define RelOid_pg_contract	111
#define RelOid_pg_has   	112
#define RelOid_pg_access	113
#else /* P95 */
#define RelOid_pg_attribute  	75   
#define RelOid_pg_magic   	80      
#endif /* P95 */
#define RelOid_pg_proc       	81   
#define RelOid_pg_server     	82   
#ifdef P95
#define RelOid_pg_class   	83   
#endif /* P95 */
#define RelOid_pg_user       	86   
#define RelOid_pg_group       	87
#ifndef P95
#define RelOid_pg_attribute  	75   
#define RelOid_pg_class   	83   
#define RelOid_pg_magic   	80      
#else /* P95 */
#define RelOid_pg_database    	88   
#endif /* P95 */
#define RelOid_pg_defaults  	89    
#define RelOid_pg_variable   	90   
#define RelOid_pg_log   	99       
#define RelOid_pg_time   	100      
#ifndef P95
#define RelOid_pg_nsvcattr	115
#define RelOid_pg_nsvcclass	116
#define RelOid_pg_ledger	225
#define RelOid_pg_nsvcfrag	114
#else /* P95 */
#define RelOid_pg_hosts   	101      
    
#define MAX_SYSTEM_RELOID       101
#endif /* P95 */

#ifndef P95
#define	RELKIND_INDEX		'i'	/* secondary index */
#define	RELKIND_RELATION	'r'	/* cataloged heap */
#define	RELKIND_SPECIAL		's'	/* special (non-heap) */
#define	RELKIND_UNCATALOGED	'u'	/* temporary heap */
#else /* P95 */
#define       RELKIND_INDEX           'i'     /* secondary index */
#define       RELKIND_RELATION        'r'     /* cataloged heap */
#define       RELKIND_SPECIAL         's'     /* special (non-heap) */
#define       RELKIND_UNCATALOGED     'u'     /* temporary heap */
#endif /* P95 */

#ifndef P95
#define	PARTITIONMODE_NONE	'n'	/* none */
#define	PARTITIONMODE_RANDOM	'r'	/* all to one random */
#define	PARTITIONMODE_ROUND	'R'	/* round-robin */
#define	PARTITIONMODE_KEY	'k'	/* key-value range */
#define	PARTITIONMODE_HASH	'h'	/* hash-value range */
#endif /* P95 */
#endif /* PG_RELATION_H */
