/*-------------------------------------------------------------------------
 *
 * pg_fragment.h--
 *    definition of the system "fragment" relation (pg_fragment) along
 *    with the relation's initial contents.
 *
 * Copyright (c) 1994, Regents of the University of California
 *
 * $Id: pg_fragment.h,v 1.13 1996/05/31 17:13:05 jsidell Exp $
 *
 * NOTES
 *    the genbki.sh script reads this file and generates .bki
 *    information from the DATA() statements.
 *
 *-------------------------------------------------------------------------
 */
#ifndef PG_FRAGMENT_H
#define PG_FRAGMENT_H

/* ----------------
 *	postgres.h contains the system type definintions and the
 *	CATALOG(), BOOTSTRAP and DATA() sugar words so this file
 *	can be read by both genbki.sh and the C compiler.
 * ----------------
 */
#include "postgres.h"
#include "utils/nabstime.h"

/* ----------------
 *	pg_fragment definition.  cpp turns this into
 *	typedef struct FormData_pg_fragment
 *
 *      Note: Copies are distinguished by the frastoreid. The master copy
 *            has the same frastoreid as fralogicalid.
 * ----------------
 */
CATALOG(pg_fragment) BOOTSTRAP {
    Oid		fraclassid; 	/* global class this fragment belongs to */
    Oid		frastoreid; 	/* local relation this fragment is stored in */
    Oid		fralogicalid;	/* logical fragment number, shared by copies */
    int4	fralocation; 	/* unique hostid */
    int4 	frapages;	/* number of pages of this fragment */
    int4 	fratuples;	/* number of tuples of this fragment */
    text	fralokey;	/* range partition low key (external form) */
    text	frahikey;	/* range partition high key (external form) */
} FormData_pg_fragment;

/* ----------------
 *	Form_pg_fragment corresponds to a pointer to a tuple with
 *	the format of pg_fragment relation.
 * ----------------
 */
typedef FormData_pg_fragment	*Form_pg_fragment;

/* ----------------
 *	compiler constants for pg_fragment
 * ----------------
 */
extern Name Name_pg_fragment;
#define Natts_pg_fragment		8
#define Anum_pg_fragment_fraclassid	1
#define Anum_pg_fragment_frastoreid	2
#define Anum_pg_fragment_fralogicalid	3
#define Anum_pg_fragment_fralocation	4
#define Anum_pg_fragment_frapages	5
#define Anum_pg_fragment_fratuples	6
#define Anum_pg_fragment_fralokey	7
#define Anum_pg_fragment_frahikey	8

/* ----------------
 *	initial contents of pg_fragment
 * ----------------
 */

DATA(insert OID = 850 (   71  71  71 PGHOSTID 0 0 _null_ _null_ ));
DATA(insert OID = 851 (   88  88  88 PGHOSTID 0 0 _null_ _null_ ));
DATA(insert OID = 852 (   76  76  76 PGHOSTID 0 0 _null_ _null_ ));
DATA(insert OID = 853 (  109 109 109 PGHOSTID 0 0 _null_ _null_ ));
DATA(insert OID = 854 (  110 110 110 PGHOSTID 0 0 _null_ _null_ ));
DATA(insert OID = 855 (  111 111 111 PGHOSTID 0 0 _null_ _null_ ));
DATA(insert OID = 856 (   81  81  81 PGHOSTID 0 0 _null_ _null_ ));
DATA(insert OID = 857 (   82  82  82 PGHOSTID 0 0 _null_ _null_ ));
DATA(insert OID = 858 (   86  86  86 PGHOSTID 0 0 _null_ _null_ ));
DATA(insert OID = 859 (   75  75  75 PGHOSTID 0 0 _null_ _null_ ));
DATA(insert OID = 860 (   83  83  83 PGHOSTID 0 0 _null_ _null_ ));
DATA(insert OID = 861 (   80  80  80 PGHOSTID 0 0 _null_ _null_ ));
DATA(insert OID = 862 (   89  89  89 PGHOSTID 0 0 _null_ _null_ ));
DATA(insert OID = 863 (   90  90  90 PGHOSTID 0 0 _null_ _null_ ));
DATA(insert OID = 864 (   99  99  99 PGHOSTID 0 0 _null_ _null_ ));
DATA(insert OID = 865 (  100 100 100 PGHOSTID 0 0 _null_ _null_ ));
DATA(insert OID = 866 (   87  87  87 PGHOSTID 0 0 _null_ _null_ ));
DATA(insert OID = 867 (  112 112 112 PGHOSTID 0 0 _null_ _null_ ));
DATA(insert OID = 868 (  113 113 113 PGHOSTID 0 0 _null_ _null_ ));
DATA(insert OID = 869 (  114 114 114 PGHOSTID 0 0 _null_ _null_ ));
DATA(insert OID = 870 (  115 115 115 PGHOSTID 0 0 _null_ _null_ ));
DATA(insert OID = 871 (  116 116 116 PGHOSTID 0 0 _null_ _null_ ));
DATA(insert OID = 872 (  225 225 225 PGHOSTID 0 0 _null_ _null_ ));

typedef struct FragPartInfo {
    int		fpi_flags;
#define	FRAPART_NOKEY	0x0
#define	FRAPART_LOKEY	0x1
#define	FRAPART_HIKEY	0x2
    Oid		fpi_class;
    Datum	fpi_lokey;	/* if hashed, this is hash range */
    Datum	fpi_hikey;	/* if hashed, this is hash range */
    char	*fpi_lokeytext;
    char	*fpi_hikeytext;
} FragPartInfoData;
typedef FragPartInfoData *FragPartInfo;

#endif /* PG_FRAGMENT_H */
