/*-------------------------------------------------------------------------
 *
 * pg_has.h--
 *    definition of the system "has" relation (pg_has)
 *    along with the relation's initial contents.
 *
 *    The HAS relation is a shared relation that is used to keep track
 *    of the usage statistics for all fragments stored at the local site.
 *
 * Copyright (c) 1994, 1995 Regents of the University of California
 *
 * $Id:
 *
 * NOTES
 *    the genbki.sh script reads this file and generates .bki
 *    information from the DATA() statements.
 *
 *-------------------------------------------------------------------------
 */
#ifndef PG_HAS_H
#define PG_HAS_H

/* ----------------
 *	postgres.h contains the system type definintions and the
 *	CATALOG(), BOOTSTRAP and DATA() sugar words so this file
 *	can be read by both genbki.sh and the C compiler.
 * ----------------
 */
#include "postgres.h"
#include "utils/nabstime.h"

/* ----------------
 *	pg_database definition.  cpp turns this into
 *	typedef struct FormData_pg_database
 * ----------------
 */ 
CATALOG(pg_has) BOOTSTRAP {
    char16 	 datname;           /* XXX - The type should be NameData */
    Oid 	 fralogicalid;
    int4         fralocation;
    float8       fradebits;
    float8       fracredits;
    float8       fraexpprice;       /* XXX - Probably not applicable */
    int4         fraaccesses;
    dt           fraarrtime;
    dt           fralastacctime;
} FormData_pg_has;

/* ----------------
 *	Form_pg_has corresponds to a pointer to a tuple with
 *	the format of pg_has relation.
 * ----------------
 */
typedef FormData_pg_has	*Form_pg_has;

/* ----------------
 *	compiler constants for pg_database
 * ----------------
 */
extern Name Name_pg_has;

#define Natts_pg_has		        9
#define Anum_pg_has_datname	        1
#define Anum_pg_has_fralogicalid	2
#define Anum_pg_has_fralocation	        3
#define Anum_pg_has_fradebits           4
#define Anum_pg_has_fracredits          5
#define Anum_pg_has_fraexpprice         6
#define Anum_pg_has_fraaccesses	        7
#define Anum_pg_has_fraarrtime	        8
#define Anum_pg_has_fralastacctime      9


#endif /* ! PG_HAS_H */
