/*-------------------------------------------------------------------------
 *
 * pg_host.h--
 *    definition of the global system "host" relation (pg_host)
 *    Maps hostid to internet address.
 *
 * Copyright (c) 1994, Regents of the University of California
 *
 * $Id: pg_host.h,v 1.2 1995/09/05 09:29:24 aoki Exp $
 *
 * NOTES
 *    the genbki.sh script reads this file and generates .bki
 *    information from the DATA() statements.
 *
 *-------------------------------------------------------------------------
 */
#ifndef PG_HOST_H
#define PG_HOST_H

/* ----------------
 *	postgres.h contains the system type definintions and the
 *	CATALOG(), BOOTSTRAP and DATA() sugar words so this file
 *	can be read by both genbki.sh and the C compiler.
 * ----------------
 */
#include "postgres.h"
#include "utils/nabstime.h"

/* ----------------
 *	pg_host definition.  cpp turns this into
 *	typedef struct FormData_pg_host
 * ----------------
 */
CATALOG(pg_host) BOOTSTRAP {
    int4	hostid;		/* relation indexed on this */
    int4	address;	/* Internet address (unsigned int) */
} FormData_pg_host;

/* ----------------
 *	Form_pg_host corresponds to a pointer to a tuple with
 *	the format of pg_host relation.
 * ----------------
 */
typedef FormData_pg_host		*Form_pg_host;

/* ----------------
 *	compiler constants for pg_host
 * ----------------
 */
extern Name Name_pg_host;

#define Natts_pg_host			2
#define Anum_pg_host_hostid		1
#define Anum_pg_host_location		2

/* ----------------
 *	pg_host is initially empty
 * ----------------
 */

#endif /* PG_HOST_H */
