/*-------------------------------------------------------------------------
 *
 * pg_ledger.h--
 *    definition of the system ledger relation (pg_ledger)
 *    Records money spent and received for processing queries.
 *    Also records all bids sent out from this site, including
 *    losing bids.
 *
 * Copyright (c) 1995, Regents of the University of California
 *
 * $Id: pg_ledger.h,v 1.3 1996/06/11 17:13:40 am Exp $
 *
 * NOTES
 *    the genbki.sh script reads this file and generates .bki
 *    information from the DATA() statements.
 *
 *-------------------------------------------------------------------------
 */
#ifndef PG_LEDGER_H
#define PG_LEDGER_H

/* ----------------
 *	postgres.h contains the system type definitions and the
 *	CATALOG(), BOOTSTRAP and DATA() sugar words so this file
 *	can be read by both genbki.sh and the C compiler.
 * ----------------
 */
#include "postgres.h"
#include "utils/nabstime.h"

/* ----------------
 *	pg_ledger definition.  cpp turns this into
 *	typedef struct FormData_pg_ledger
 *
 * ----------------
 */

CATALOG(pg_ledger) BOOTSTRAP {
    int4	procsite;	/* hostid of site which processed the plan fragment */
    float8	bidamount; 	/* amount we bid or amount other site bid */
    float8	recdamount; 	/* amount we received (negative if we paid $$$ to another site) */
    bool	bidwon;		/* 't' if we bid and won for this plan fragment */
    text       planstring;	/* string representation of plan fragment */
} FormData_pg_ledger;

/* changed from (text *) to (text) to remove syntax error; */
/* Mariposa bugid 17 */

/* ----------------
 *	Form_pg_ledger corresponds to a pointer to a tuple with
 *	the format of pg_ledger relation.
 * ----------------
 */
typedef FormData_pg_ledger		*Form_pg_ledger;

/* ----------------
 *	compiler constants for pg_ledger
 * ----------------
 */
#define Name_pg_ledger				"pg_ledger" 
#define Natts_pg_ledger				5
#define Anum_pg_ledger_procsite			1
#define Anum_pg_ledger_bidamount		2
#define Anum_pg_ledger_recdamount		3
#define Anum_pg_ledger_bidwon			4
#define Anum_pg_ledger_planstring		5

/* ----------------
 *	pg_ledger is initially empty
 * ----------------
 */

/* ----------------
 *	Access methods for pg_ledger tuples.
 *	May want to add something here eventually.
 * ----------------
 */


#endif /* PG_LEDGER_H */
