/*-------------------------------------------------------------------------
 *
 * pg_nsvcattr.h--
 *    definition of the name server "nsvcattr" relation (pg_nsvcattr)
 *
 *
 * Copyright (c) 1994, Regents of the University of California
 *
 * $Id: pg_nsvcattr.h,v 1.1 1996/02/21 06:43:40 marcel Exp $
 *
 * NOTES
 *    the genbki.sh script reads this file and generates .bki
 *    information from the DATA() statements.
 *
 *-------------------------------------------------------------------------
 */
#ifndef PG_NSVCATTR_H
#define PG_NSVCATTR_H
   
/* ----------------
 *	postgres.h contains the system type definintions and the
 *	CATALOG(), BOOTSTRAP and DATA() sugar words so this file
 *	can be read by both genbki.sh and the C compiler.
 * ----------------
 */
#include "postgres.h"
#include "access/attnum.h"

/* ----------------
 *	pg_nsvcattr definition.  cpp turns this into
 *	typedef struct FormData_pg_nsvcattr
 *
 *      If you change the following, make sure you change the structs for
 *      system nsvcattrs in heap.c and index.c also.
 * ----------------
 */
CATALOG(pg_nsvcattr) BOOTSTRAP {
    Oid  	attrelid;
    char16  	attname;
    Oid  	atttypid;
    Oid  	attdefrel;
    int4  	attnvals;
    Oid  	atttyparg;	/* type arg for arrays/spquel/procs */
    int2 	attlen;
    int2  	attnum;
    int2 	attbound;
    bool  	attbyval;
    bool 	attcanindex;
    Oid 	attproc;	/* spquel? */
    int4	attnelems;
    int4	attcacheoff;
    bool        attisset;
    char	attalign;	/* alignment (c=char, s=short, i=int, d=double) */
} FormData_pg_nsvcattr;

/*
 * someone should figure out how to do this properly. (The problem is
 * the size of the C struct is not the same as the size of the tuple.)
 */
#define NSVCATTR_TUPLE_SIZE \
    (offsetof(FormData_pg_nsvcattr,attalign) + sizeof(char))

/* ----------------
 *	Form_pg_nsvcattr corresponds to a pointer to a tuple with
 *	the format of pg_nsvcattr relation.
 * ----------------
 */
typedef FormData_pg_nsvcattr	*NsvcAttrTupleForm;

/* ----------------
 *	compiler constants for pg_nsvcattr
 * ----------------
 */
extern Name Name_pg_nsvcattr;

#define Natts_pg_nsvcattr		16
#define Anum_pg_nsvcattr_attrelid	1
#define Anum_pg_nsvcattr_attname	2
#define Anum_pg_nsvcattr_atttypid	3
#define Anum_pg_nsvcattr_attdefrel	4
#define Anum_pg_nsvcattr_attnvals	5
#define Anum_pg_nsvcattr_atttyparg	6
#define Anum_pg_nsvcattr_attlen	7
#define Anum_pg_nsvcattr_attnum	8
#define Anum_pg_nsvcattr_attbound	9
#define Anum_pg_nsvcattr_attbyval	10
#define Anum_pg_nsvcattr_attcanindex	11
#define Anum_pg_nsvcattr_attproc	12
#define Anum_pg_nsvcattr_attnelems	13
#define Anum_pg_nsvcattr_attcacheoff	14
#define Anum_pg_nsvcattr_attisset      15
#define Anum_pg_nsvcattr_attalign      16

#endif /* PG_NSVCATTR_H */
