/*-------------------------------------------------------------------------
 *
 * pg_nsvcclass.h--
 *    definition of the name server "relation" relation (pg_nsvcclass)
 *
 *
 * Copyright (c) 1994, Regents of the University of California
 *
 * $Id: pg_nsvcclass.h,v 1.1 1996/02/21 06:43:41 marcel Exp $
 *
 * NOTES
 *    the genbki.sh script reads this file and generates .bki
 *    information from the DATA() statements.
 *
 *-------------------------------------------------------------------------
 */
#ifndef PG_NSVCCLASS_H
#define PG_NSVCCLASS_H

/* ----------------
 *	postgres.h contains the system type definintions and the
 *	CATALOG(), BOOTSTRAP and DATA() sugar words so this file
 *	can be read by both genbki.sh and the C compiler.
 * ----------------
 */
#include "postgres.h"
#include "utils/nabstime.h"

/* ----------------
 *	This structure is actually variable-length (the last attribute is
 *	a POSTGRES array).  Hence, sizeof(FormData_pg_nsvcclass) does not
 *	describe the fixed-length or actual size of the structure.
 *	FormData_pg_nsvcclass.relacl may not be correctly aligned, either,
 *	if aclitem and struct varlena don't align together.  Hence,
 *	you MUST use heap_getattr() to get the relacl field.
 * ----------------
 */
CATALOG(pg_nsvcclass) BOOTSTRAP {
     char16 	relname;
     Oid 	relowner;
     Oid 	relam;
     int4 	relpages;
     int4 	reltuples;
     dt 	relexpires;
     dt 	relpreserved;
     bool 	relhasindex;
     bool 	relisshared;
     char 	relkind;
     char 	relarch;
     int2 	relnatts;
     int2	relsmgr;
     int28 	relkey;			/* not used */
     oid8	relkeyop;		/* not used */
     regproc	relpartcmp;		/* comparison function */
     regproc	relparthash;		/* hash partition function */
     int2	relpartatt;		/* partitioning attribute */
     char	relpartmode;		/* partitioning mode */
     bool	relhasrules;
     aclitem	relacl[1];		/* this is here for the catalog */
} FormData_pg_nsvcclass;

#define NSVCCLASS_TUPLE_SIZE \
     (offsetof(FormData_pg_nsvcclass,relhasrules) + sizeof(bool))

/* ----------------
 *	Form_pg_nsvcclass corresponds to a pointer to a tuple with
 *	the format of pg_nsvcclass relation.
 * ----------------
 */
typedef FormData_pg_nsvcclass	*Form_pg_nsvcclass;

/* ----------------
 *	compiler constants for pg_nsvcclass
 * ----------------
 */
extern Name Name_pg_nsvcclass;

/* ----------------
 *	Natts_pg_nsvcclass_fixed is used to tell routines that insert new
 *	pg_nsvcclass tuples (as opposed to replacing old ones) that there's no
 *	relacl field.
 * ----------------
 */
#define Natts_pg_nsvcclass_fixed		20
#define Natts_pg_nsvcclass			21
#define Anum_pg_nsvcclass_relname		1
#define Anum_pg_nsvcclass_relowner		2
#define Anum_pg_nsvcclass_relam		3
#define Anum_pg_nsvcclass_relpages		4
#define Anum_pg_nsvcclass_reltuples		5
#define Anum_pg_nsvcclass_relexpires	6
#define Anum_pg_nsvcclass_relpreserved	7
#define Anum_pg_nsvcclass_relhasindex	8
#define Anum_pg_nsvcclass_relisshared	9
#define Anum_pg_nsvcclass_relkind		10
#define Anum_pg_nsvcclass_relarch		11
#define Anum_pg_nsvcclass_relnatts		12
#define Anum_pg_nsvcclass_relsmgr		13
#define Anum_pg_nsvcclass_relkey		14
#define Anum_pg_nsvcclass_relkeyop		15
#define Anum_pg_nsvcclass_relpartcmp	16
#define Anum_pg_nsvcclass_relparthash	17
#define Anum_pg_nsvcclass_relpartatt	18
#define Anum_pg_nsvcclass_relpartmode	19
#define Anum_pg_nsvcclass_relhasrules	20
#define Anum_pg_nsvcclass_relacl		21

#endif /* PG_NSVCCLASS_H */
