/*-------------------------------------------------------------------------
 *
 * pg_nsvcfrag.h--
 *    definition of the name server "nsvcfrag" relation (pg_nsvcfrag) 
 *
 * Copyright (c) 1994, Regents of the University of California
 *
 * $Id: pg_nsvcfrag.h,v 1.1 1996/02/21 06:43:42 marcel Exp $
 *
 * NOTES
 *    the genbki.sh script reads this file and generates .bki
 *    information from the DATA() statements.
 *
 *-------------------------------------------------------------------------
 */
#ifndef PG_NSVCFRAG_H
#define PG_NSVCFRAG_H

/* ----------------
 *	postgres.h contains the system type definintions and the
 *	CATALOG(), BOOTSTRAP and DATA() sugar words so this file
 *	can be read by both genbki.sh and the C compiler.
 * ----------------
 */
#include "postgres.h"
#include "utils/nabstime.h"

/* ----------------
 *	pg_nsvcfrag definition.  cpp turns this into
 *	typedef struct FormData_pg_nsvcfrag
 *
 *      Note: Copies are distinguished by the frastoreid. The master copy
 *            has the same frastoreid as fralogicalid.
 * ----------------
 */
CATALOG(pg_nsvcfrag) BOOTSTRAP {
    Oid		fraclassid; 	/* global class this nsvcfrag belongs to */
    Oid		frastoreid; 	/* local relation this nsvcfrag is stored in */
    Oid		fralogicalid;	/* logical nsvcfrag number, shared by copies */
    int4	fralocation; 	/* unique hostid */
    int4 	frapages;	/* number of pages of this nsvcfrag */
    int4 	fratuples;	/* number of tuples of this nsvcfrag */
    text	fralokey;	/* range partition low key (external form) */
    text	frahikey;	/* range partition high key (external form) */
} FormData_pg_nsvcfrag;

/* ----------------
 *	Form_pg_nsvcfrag corresponds to a pointer to a tuple with
 *	the format of pg_nsvcfrag relation.
 * ----------------
 */
typedef FormData_pg_nsvcfrag	*Form_pg_nsvcfrag;

/* ----------------
 *	compiler constants for pg_nsvcfrag
 * ----------------
 */
extern Name Name_pg_nsvcfrag;
#define Natts_pg_nsvcfrag		8
#define Anum_pg_nsvcfrag_fraclassid	1
#define Anum_pg_nsvcfrag_frastoreid	2
#define Anum_pg_nsvcfrag_fralogicalid	3
#define Anum_pg_nsvcfrag_fralocation	4
#define Anum_pg_nsvcfrag_frapages	5
#define Anum_pg_nsvcfrag_fratuples	6
#define Anum_pg_nsvcfrag_fralokey	7
#define Anum_pg_nsvcfrag_frahikey	8

#endif /* PG_NSVCFRAG_H */
