/*-------------------------------------------------------------------------
 *
 * copy.c--
 *
 * Copyright (c) 1994, Regents of the University of California
 *
 *
 * IDENTIFICATION
 *    $Header: /usr/local/devel/mariposa/cvs/src/backend/commands/copy.c,v 0.7 1996/06/07 04:55:28 am Exp $
 *
 *-------------------------------------------------------------------------
 */
#include <stdio.h>
#include <sys/types.h>	/* for mode_t */
#include <sys/stat.h>	/* for umask(2) prototype */

#include "postgres.h"
#include "miscadmin.h"
#include "utils/builtins.h"
#include "utils/syscache.h"
#include "catalog/pg_type.h"
#include "catalog/pg_index.h"
#include "catalog/index.h"

#include "access/heapam.h"
#include "access/htup.h"
#include "access/itup.h"
#include "access/relscan.h"
#include "access/funcindex.h"
#ifdef P95
#include "access/tupdesc.h"
#endif /* P95 */
#include "nodes/execnodes.h"
#include "nodes/plannodes.h"
#include "nodes/pg_list.h"
#include "executor/tuptable.h"
#include "executor/executor.h"
#include "utils/rel.h"
#include "utils/elog.h"
#include "utils/memutils.h"
#include "utils/palloc.h"
#ifndef P95
/* #include "parser/daemon.h"	 for what?? -AY */
#endif /* P95 */
#include "fmgr.h"
#include "machine.h"

/*
 * New copy code.
 * 
 * This code "knows" the following about tuples:
 * 
 */

static bool reading_from_input = false;

/* non-export function prototypes */
#ifndef P95
static void CopyTo(Relation rel, bool binary, FILE *fp);
static void CopyFrom(Relation rel, bool binary, FILE *fp);
#else /* P95 */
static void CopyTo(Relation rel, bool binary, FILE *fp, char *delim);
static void CopyFrom(Relation rel, bool binary, FILE *fp, char *delim);
#endif /* P95 */
static Oid GetOutputFunction(Oid type);
static Oid GetTypeElement(Oid type);
static Oid GetInputFunction(Oid type);
static Oid IsTypeByVal(Oid type);
static void GetIndexRelations(Oid main_relation_oid,
			      int *n_indices,
			      Relation **index_rels);
#ifndef P95
static char *CopyReadAttribute(int attno, FILE *fp, bool *isnull);
static void CopyAttributeOut(FILE *fp, char *string);
#else /* P95 */
static char *CopyReadAttribute(int attno, FILE *fp, bool *isnull, char *delim);
static void CopyAttributeOut(FILE *fp, char *string, char *delim);
#endif /* P95 */
static int CountTuples(Relation relation);

extern FILE *Pfout, *Pfin;

void
#ifndef P95
DoCopy(char *relname, bool binary, bool from, bool pipe, char *filename)
#else /* P95 */
DoCopy(char *relname, bool binary, bool from, bool pipe, char *filename, 
       char *delim)
#endif /* P95 */
{
    FILE *fp;
    Relation rel;
#ifndef P95
    NameData relnameData;
    
#endif /* P95 */
    reading_from_input = pipe;
    
#ifndef P95
    namestrcpy(&relnameData,relname);
    rel = heap_openr(&relnameData);
#else /* P95 */
    rel = heap_openr(relname);
#endif /* P95 */
    if (rel == NULL) elog(WARN, "Copy: class %s does not exist.", relname);
    
    if (from) {
	if (pipe && IsUnderPostmaster) ReceiveCopyBegin();
	if (IsUnderPostmaster) {
	    fp = pipe ? Pfin : fopen(filename, "r");
	}else {
	    fp = pipe ? stdin : fopen(filename, "r");
	}
	if (fp == NULL) {
	    elog(WARN, "COPY: file %s could not be open for reading", filename);
	}
#ifndef P95
	CopyFrom(rel, binary, fp);
#else /* P95 */
	CopyFrom(rel, binary, fp, delim);
#endif /* P95 */
    }else {
	
	mode_t oumask = umask((mode_t) 0);
	
	if (pipe && IsUnderPostmaster) SendCopyBegin();
	if (IsUnderPostmaster) {
	    fp = pipe ? Pfout : fopen(filename, "w");
	    
	}else {
	    fp = pipe ? stdout : fopen(filename, "w");
	}
	(void) umask(oumask);
	if (fp == NULL)  {
	    elog(WARN, "COPY: file %s could not be open for writing", filename);
	}
#ifndef P95
	CopyTo(rel, binary, fp);
#else /* P95 */
	CopyTo(rel, binary, fp, delim);
#endif /* P95 */
    }
    if (!pipe) {
	fclose(fp);
    }else if (!from && !binary) {
	fputs(".\n", fp);
	if (IsUnderPostmaster) fflush(Pfout);
    }
}

static void
#ifndef P95
CopyTo(Relation rel, bool binary, FILE *fp)
#else /* P95 */
CopyTo(Relation rel, bool binary, FILE *fp, char *delim)
#endif /* P95 */
{
    HeapTuple tuple;
    HeapScanDesc scandesc;
    
    int32 attr_count, i;
    AttributeTupleForm *attr;
    func_ptr *out_functions;
    int dummy;
    Oid out_func_oid;
    Oid *elements;
    Datum value;
    bool isnull = (bool) true;
    char *nulls;
    char *string;
    int32 ntuples;
#ifdef P95
    TupleDesc tupDesc;
#endif /* P95 */
    
    scandesc = heap_beginscan(rel, 0, NULL, 0, NULL);
#ifndef P95
    
    attr_count = rel->rd_rel->relnatts;
    attr = (AttributeTupleForm *) rel->rd_att;
    
#else /* P95 */

    attr_count = rel->rd_att->natts;
    attr = rel->rd_att->attrs;
    tupDesc = rel->rd_att;

#endif /* P95 */
    if (!binary) {
	out_functions = (func_ptr *)
	    palloc(attr_count * sizeof(func_ptr));
	elements = (Oid *) palloc(attr_count * sizeof(Oid));
	for (i = 0; i < attr_count; i++) {
	    out_func_oid = (Oid) GetOutputFunction(attr[i]->atttypid);
	    fmgr_info(out_func_oid, &out_functions[i], &dummy);
	    elements[i] = GetTypeElement(attr[i]->atttypid);
	}
    }else {
	nulls = (char *) palloc(attr_count);
	for (i = 0; i < attr_count; i++) nulls[i] = ' ';
	
	/* XXX expensive */
	
	ntuples = CountTuples(rel);
	fwrite(&ntuples, sizeof(int32), 1, fp);
    }
    
    for (tuple = heap_getnext(scandesc, 0, NULL);
         tuple != NULL; 
         tuple = heap_getnext(scandesc, 0, NULL)) {
	
	for (i = 0; i < attr_count; i++) {
	    value = (Datum) 
#ifndef P95
		heap_getattr(tuple, InvalidBuffer, i+1, attr, &isnull);
#else /* P95 */
		heap_getattr(tuple, InvalidBuffer, i+1, tupDesc, &isnull);
#endif /* P95 */
	    if (!binary) {
		if (!isnull) {
		    string = (char *) (out_functions[i]) (value, elements[i]);
#ifndef P95
		    CopyAttributeOut(fp, string);
#else /* P95 */
		    CopyAttributeOut(fp, string, delim);
#endif /* P95 */
		    pfree(string);
		}
		if (i == attr_count - 1) {
		    fputc('\n', fp);
		}else {
#ifndef P95
		    fputc('\t', fp);
#else /* P95 */
		    /* when copying out, only use the first char of the delim
		       string */
		    fputc(delim[0], fp);
#endif /* P95 */
		}
	    }else {
		/*
		 * only interesting thing heap_getattr tells us in this case
		 * is if we have a null attribute or not.
		 */
		if (isnull) nulls[i] = 'n';
	    }
	}
	
	if (binary) {
	    int32 null_ct = 0, length;
	    
	    for (i = 0; i < attr_count; i++) {
		if (nulls[i] == 'n') null_ct++;
	    }
	    
	    length = tuple->t_len - tuple->t_hoff;
	    fwrite(&length, sizeof(int32), 1, fp);
	    fwrite(&null_ct, sizeof(int32), 1, fp);
	    if (null_ct > 0) {
		for (i = 0; i < attr_count; i++) {
		    if (nulls[i] == 'n') {
			fwrite(&i, sizeof(int32), 1, fp);
			nulls[i] = ' ';
		    }
		}
	    }
	    fwrite((char *) tuple + tuple->t_hoff, length, 1, fp);
	}
    }
    
    heap_endscan(scandesc);
    if (binary) {
	pfree(nulls);
    }else {
	pfree(out_functions);
	pfree(elements);
    }
    
    heap_close(rel);
}

static void
#ifndef P95
CopyFrom(Relation rel, bool binary, FILE *fp)
#else /* P95 */
CopyFrom(Relation rel, bool binary, FILE *fp, char *delim)
#endif /* P95 */
{
    HeapTuple tuple;
    IndexTuple ituple;
    AttrNumber attr_count;
    AttributeTupleForm *attr;
    func_ptr *in_functions;
    int i, dummy;
    Oid in_func_oid;
    Datum *values;
    char *nulls, *index_nulls;
    bool *byval;
    bool isnull;
    bool has_index;
    int done = 0;
#ifndef P95
    char *string, *ptr, *CopyReadAttribute();
#else /* P95 */
    char *string, *ptr;
#endif /* P95 */
    Relation *index_rels;
    int32 len, null_ct, null_id;
    int32 ntuples, tuples_read = 0;
    bool reading_to_eof = true;
    Oid *elements;
    FuncIndexInfo *finfo, **finfoP;
#ifndef P95
    TupleDesc *itupdesc;
#else /* P95 */
    TupleDesc *itupdescArr;
#endif /* P95 */
    HeapTuple pgIndexTup;
    IndexTupleForm *pgIndexP;
    int *indexNatts;
    char *predString;
    Node **indexPred;
    TupleDesc rtupdesc;
    ExprContext *econtext;
    TupleTable tupleTable;
    TupleTableSlot *slot;
    int natts;
    AttrNumber *attnumP;
    Datum idatum;
    int n_indices;
    InsertIndexResult indexRes;
#ifndef P95
    Relation sideRelationDesc;
    
    /* ----------------
     *	In Mariposa, we keep track of inserts to fragments that are copied.
     *  So we check if this fragment has a copy, if so, enter the tuples
     *  into the side-file.
     * ----------------
     */
#else /* P95 */
    TupleDesc tupDesc;
#endif /* P95 */

#ifndef P95
    if (rel->rd_iscopied) {
	sideRelationDesc = heap_open(rel->rd_copysidef);
	if (!RelationIsValid(sideRelationDesc)) {
	    elog(WARN, "CopyFrom: Side-file open for %s failed.",
		 rel->rd_rel->relname.data);
	}
    }
    
    attr = (AttributeTupleForm *)rel->rd_att;
    
    attr_count = rel->rd_rel->relnatts;
    
#else /* P95 */
    tupDesc = RelationGetTupleDescriptor(rel);
    attr = tupDesc->attrs;
    attr_count = tupDesc->natts;

#endif /* P95 */
    has_index = false;
    
    /*
     *  This may be a scalar or a functional index.  We initialize all
     *  kinds of arrays here to avoid doing extra work at every tuple
     *  copy.
     */
    
    if (rel->rd_rel->relhasindex) {
        GetIndexRelations(rel->rd_id, &n_indices, &index_rels);
	if (n_indices > 0) {
	    has_index = true;
#ifndef P95
	    itupdesc =
#else /* P95 */
	    itupdescArr = 
#endif /* P95 */
		(TupleDesc *)palloc(n_indices * sizeof(TupleDesc));
	    pgIndexP =
		(IndexTupleForm *)palloc(n_indices * sizeof(IndexTupleForm));
	    indexNatts = (int *) palloc(n_indices * sizeof(int));
	    finfo = (FuncIndexInfo *) palloc(n_indices * sizeof(FuncIndexInfo));
	    finfoP = (FuncIndexInfo **) palloc(n_indices * sizeof(FuncIndexInfo *));
	    indexPred = (Node **) palloc(n_indices * sizeof(Node*));
	    econtext = NULL;
	    for (i = 0; i < n_indices; i++) {
#ifndef P95
		itupdesc[i] = RelationGetTupleDescriptor(index_rels[i]);
#else /* P95 */
		itupdescArr[i] = RelationGetTupleDescriptor(index_rels[i]);
#endif /* P95 */
		pgIndexTup =
		    SearchSysCacheTuple(INDEXRELID,
#ifndef P95
					(char *)
#endif /* P95 */
					ObjectIdGetDatum(index_rels[i]->rd_id),
#ifndef P95
					(char *) NULL,
					(char *) NULL,
					(char *) NULL);
#else /* P95 */
					0,0,0);
#endif /* P95 */
		Assert(pgIndexTup);
		pgIndexP[i] = (IndexTupleForm)GETSTRUCT(pgIndexTup);
		for (attnumP = &(pgIndexP[i]->indkey[0]), natts = 0;
		     *attnumP != InvalidAttrNumber;
		     attnumP++, natts++);
		if (pgIndexP[i]->indproc != InvalidOid) {
		    FIgetnArgs(&finfo[i]) = natts;
		    natts = 1;
		    FIgetProcOid(&finfo[i]) = pgIndexP[i]->indproc;
		    *(FIgetname(&finfo[i])) = '\0';
		    finfoP[i] = &finfo[i];
		} else
		    finfoP[i] = (FuncIndexInfo *) NULL;
		indexNatts[i] = natts;
		if (VARSIZE(&pgIndexP[i]->indpred) != 0) {
		    predString = fmgr(F_TEXTOUT, &pgIndexP[i]->indpred);
		    indexPred[i] = stringToNode(predString);
		    pfree(predString);
		    /* make dummy ExprContext for use by ExecQual */
		    if (econtext == NULL) {
#ifndef P95

			tupleTable = ExecCreateTupleTable(1); 
			slot =   ExecAllocTableSlot(tupleTable);
			econtext = makeNode(ExprContext);
			econtext->ecxt_scantuple = slot;
			rtupdesc = RelationGetTupleDescriptor(rel);
			slot->ttc_tupleDescriptor = rtupdesc;
			/*
			 * There's no buffer associated with heap tuples here,
			 * so I set the slot's buffer to NULL.  Currently, it
			 * appears that the only way a buffer could be needed
			 * would be if the partial index predicate referred to
			 * the "lock" system attribute.  If it did, then
			 * heap_getattr would call HeapTupleGetRuleLock, which
			 * uses the buffer's descriptor to get the relation id.
			 * Rather than try to fix this, I'll just disallow
			 * partial indexes on "lock", which wouldn't be useful
			 * anyway. --Nels, Nov '92
			 */
			/* SetSlotBuffer(slot, (Buffer) NULL); */
			/* SetSlotShouldFree(slot, false); */
			slot->ttc_buffer = (Buffer)NULL;
			slot->ttc_shouldFree = false;

#else /* P95 */

#ifndef OMIT_PARTIAL_INDEX
			tupleTable = ExecCreateTupleTable(1); 
			slot =   ExecAllocTableSlot(tupleTable);
			econtext = makeNode(ExprContext);
			econtext->ecxt_scantuple = slot;
			rtupdesc = RelationGetTupleDescriptor(rel);
			slot->ttc_tupleDescriptor = rtupdesc;
			/*
			 * There's no buffer associated with heap tuples here,
			 * so I set the slot's buffer to NULL.  Currently, it
			 * appears that the only way a buffer could be needed
			 * would be if the partial index predicate referred to
			 * the "lock" system attribute.  If it did, then
			 * heap_getattr would call HeapTupleGetRuleLock, which
			 * uses the buffer's descriptor to get the relation id.
			 * Rather than try to fix this, I'll just disallow
			 * partial indexes on "lock", which wouldn't be useful
			 * anyway. --Nels, Nov '92
			 */
			/* SetSlotBuffer(slot, (Buffer) NULL); */
			/* SetSlotShouldFree(slot, false); */
			slot->ttc_buffer = (Buffer)NULL;
			slot->ttc_shouldFree = false;
#endif /* OMIT_PARTIAL_INDEX */    

#endif /* P95 */
		    }
		} else {
		    indexPred[i] = NULL;
		}
	    }
	}
    }
    
    if (!binary)
	{
	    in_functions = (func_ptr *) palloc(attr_count * sizeof(func_ptr));
	    elements = (Oid *) palloc(attr_count * sizeof(Oid));
	    for (i = 0; i < attr_count; i++)
		{
		    in_func_oid = (Oid) GetInputFunction(attr[i]->atttypid);
		    fmgr_info(in_func_oid, &in_functions[i], &dummy);
		    elements[i] = GetTypeElement(attr[i]->atttypid);
		}
	}
    else
	{
	    fread(&ntuples, sizeof(int32), 1, fp);
	    if (ntuples != 0) reading_to_eof = false;
	}
    
    values       = (Datum *) palloc(sizeof(Datum) * attr_count);
    nulls        = (char *) palloc(attr_count);
    index_nulls  = (char *) palloc(attr_count);
    byval        = (bool *) palloc(attr_count * sizeof(bool));
    
    for (i = 0; i < attr_count; i++) {
	nulls[i] = ' ';
	index_nulls[i] = ' ';
	byval[i] = (bool) IsTypeByVal(attr[i]->atttypid);
    }
    
    while (!done) {
	if (!binary) {
	    for (i = 0; i < attr_count && !done; i++) {
#ifndef P95
		string = CopyReadAttribute(i, fp, &isnull);
#else /* P95 */
		string = CopyReadAttribute(i, fp, &isnull, delim);
#endif /* P95 */
		if (isnull) {
		    values[i] = PointerGetDatum(NULL);
		    nulls[i] = 'n';
		}else if (string == NULL) {
		    done = 1;
		}else {
#ifndef P95
		    values[i] = (Datum)(in_functions[i]) (string, elements[i]);
#else /* P95 */
		    values[i] =
			(Datum)(in_functions[i])(string,
						 elements[i],
						 attr[i]->attlen);
#endif /* P95 */
		    /*
		     * Sanity check - by reference attributes cannot return
		     * NULL
		     */
		    if (!PointerIsValid(values[i]) &&
#ifndef P95
			!(rel->rd_att[i]->attbyval)) {
#else /* P95 */
			!(rel->rd_att->attrs[i]->attbyval)) {
#endif /* P95 */
			elog(WARN, "copy from: Bad file format");
		    }
		}
	    }
	}else { /* binary */
	    fread(&len, sizeof(int32), 1, fp);
	    if (feof(fp)) {
		done = 1;
	    }else {
		fread(&null_ct, sizeof(int32), 1, fp);
		if (null_ct > 0) {
		    for (i = 0; i < null_ct; i++) {
			fread(&null_id, sizeof(int32), 1, fp);
			nulls[null_id] = 'n';
		    }
		}
		
		string = (char *) palloc(len);
		fread(string, len, 1, fp);
		
		ptr = string;
		
		for (i = 0; i < attr_count; i++) {
		    if (byval[i] && nulls[i] != 'n') {
			
			switch(attr[i]->attlen) {
			case sizeof(char):
			    values[i] = (Datum) *(unsigned char *) ptr;
			    ptr += sizeof(char);
			    break;
			case sizeof(short):
			    ptr = (char *) SHORTALIGN(ptr);
			    values[i] = (Datum) *(unsigned short *) ptr; 
			    ptr += sizeof(short);
			    break;
			case sizeof(int32):
			    ptr = (char *) INTALIGN(ptr);
			    values[i] = (Datum) *(uint32 *) ptr;
			    ptr += sizeof(int32);
			    break;
			default:
			    elog(WARN, "COPY BINARY: impossible size!");
			    break;
			}
		    }else if (nulls[i] != 'n') {
#ifndef P95
			switch (attr[i]->attlen == -1) {
#else /* P95 */
			switch (attr[i]->attlen) {
#endif /* P95 */
			case -1:
			    if (attr[i]->attalign == 'd') 
#ifndef P95
			        ptr = (char *)DOUBLEALIGN(ptr);
#else /* P95 */
				ptr = (char *)DOUBLEALIGN(ptr);
#endif /* P95 */
			    else
#ifndef P95
			        ptr = (char *)INTALIGN(ptr);
                            values[i] = (Datum) ptr;
                            ptr += * (uint32 *) ptr;
#else /* P95 */
				ptr = (char *)INTALIGN(ptr);
			    values[i] = (Datum) ptr;
			    ptr += * (uint32 *) ptr;
#endif /* P95 */
			    break;
			case sizeof(char):
			    values[i] = (Datum)ptr;
			    ptr += attr[i]->attlen;
			    break;
			case sizeof(short):
			    ptr = (char*)SHORTALIGN(ptr);
			    values[i] = (Datum)ptr;
			    ptr += attr[i]->attlen;
			    break;
			case sizeof(int32):
			    ptr = (char*)INTALIGN(ptr);
			    values[i] = (Datum)ptr;
			    ptr += attr[i]->attlen;
			    break;
			default:
			    if (attr[i]->attalign == 'd')
#ifndef P95
			        ptr = (char *)DOUBLEALIGN(ptr);
#else /* P95 */
				ptr = (char *)DOUBLEALIGN(ptr);
#endif /* P95 */
			    else 
#ifndef P95
			        ptr = (char *)LONGALIGN(ptr);
                            values[i] = (Datum) ptr;
                            ptr += attr[i]->attlen;
                        }
#else /* P95 */
				ptr = (char *)LONGALIGN(ptr);
			    values[i] = (Datum) ptr;
			    ptr += attr[i]->attlen;
			}
#endif /* P95 */
		    }
		}
	    }
	}
	if (done) continue;
	    
#ifndef P95
	tuple = heap_formtuple(attr_count, attr, values, nulls);
#else /* P95 */
	tupDesc = CreateTupleDesc(attr_count, attr);
	tuple = heap_formtuple(tupDesc, values, nulls);
#endif /* P95 */
	heap_insert(rel, tuple);
	    
#ifndef P95
	if (rel->rd_iscopied) {
	    UpdateTupleSetIsInsert(tuple);
	    heap_insert(sideRelationDesc, tuple);
	}

#endif /* P95 */
	if (has_index) {
	    for (i = 0; i < n_indices; i++) {
		if (indexPred[i] != NULL) {
#ifdef P95
#ifndef OMIT_PARTIAL_INDEX
#endif /* P95 */
		    /* if tuple doesn't satisfy predicate,
		     * don't update index
		     */
		    slot->val = tuple;
		    /*SetSlotContents(slot, tuple); */
		    if (ExecQual((List*)indexPred[i], econtext) == false)
			continue;
#ifdef P95
#endif /* OMIT_PARTIAL_INDEX */    	
#endif /* P95 */
		}
		FormIndexDatum(indexNatts[i],
			       (AttrNumber *)&(pgIndexP[i]->indkey[0]),
			       tuple,
#ifndef P95
			       attr,
#else /* P95 */
			       tupDesc,
#endif /* P95 */
			       InvalidBuffer,
			       &idatum,
			       index_nulls,
			       finfoP[i]);
#ifndef P95
		ituple = index_formtuple(1, itupdesc[i], &idatum, index_nulls);
#else /* P95 */
		ituple = index_formtuple(itupdescArr[i], &idatum, index_nulls);
#endif /* P95 */
		ituple->t_tid = tuple->t_ctid;
		indexRes = index_insert(index_rels[i], ituple);
		if (indexRes) pfree(indexRes);
		pfree(ituple);
	    }
	}
	    
	if (binary) pfree(string);
	    
	for (i = 0; i < attr_count; i++) {
	    if (!byval[i] && nulls[i] != 'n') {
		if (!binary) pfree((void*)values[i]);
	    }else if (nulls[i] == 'n') {
		nulls[i] = ' ';
	    }
	}
	    
	pfree(tuple);
	tuples_read++;
	    
	if (!reading_to_eof && ntuples == tuples_read) done = true;
    }
#ifndef P95

    if (rel->rd_iscopied) {
	heap_close(sideRelationDesc);
    }

#endif /* P95 */
    pfree(values);
    if (!binary) pfree(in_functions);
    pfree(nulls);
    pfree(byval);
    heap_close(rel);
}

static Oid
GetOutputFunction(Oid type)
{
    HeapTuple    typeTuple;
    
    typeTuple = SearchSysCacheTuple(TYPOID,
#ifndef P95
				    (char *) ObjectIdGetDatum(type),
				    (char *) NULL,
				    (char *) NULL,
				    (char *) NULL);
#else /* P95 */
				    ObjectIdGetDatum(type),
				    0,0,0);
#endif /* P95 */
    
    if (HeapTupleIsValid(typeTuple))
	return((int) ((TypeTupleForm) GETSTRUCT(typeTuple))->typoutput);
    
    elog(WARN, "GetOutputFunction: Cache lookup of type %d failed", type);
    return(InvalidOid);
}

static Oid
GetTypeElement(Oid type)
{
    HeapTuple    typeTuple;
    
    typeTuple = SearchSysCacheTuple(TYPOID,
#ifndef P95
				    (char *) ObjectIdGetDatum(type),
				    (char *) NULL,
				    (char *) NULL,
				    (char *) NULL);
#else /* P95 */
				    ObjectIdGetDatum(type),
				    0,0,0);

#endif /* P95 */
    
    if (HeapTupleIsValid(typeTuple))
	return((int) ((TypeTupleForm) GETSTRUCT(typeTuple))->typelem);
    
    elog(WARN, "GetOutputFunction: Cache lookup of type %d failed", type);
    return(InvalidOid);
}

static Oid
GetInputFunction(Oid type)
{
    HeapTuple    typeTuple;
    
    typeTuple = SearchSysCacheTuple(TYPOID,
#ifndef P95
				    (char *) ObjectIdGetDatum(type),
				    (char *) NULL,
				    (char *) NULL,
				    (char *) NULL);
#else /* P95 */
				    ObjectIdGetDatum(type),
				    0,0,0);
#endif /* P95 */
    
    if (HeapTupleIsValid(typeTuple))
	return((int) ((TypeTupleForm) GETSTRUCT(typeTuple))->typinput);
    
    elog(WARN, "GetInputFunction: Cache lookup of type %d failed", type);
    return(InvalidOid);
}

static Oid
IsTypeByVal(Oid type)
{
    HeapTuple    typeTuple;
    
    typeTuple = SearchSysCacheTuple(TYPOID,
#ifndef P95
				    (char *) ObjectIdGetDatum(type),
				    (char *) NULL,
				    (char *) NULL,
				    (char *) NULL);
#else /* P95 */
				    ObjectIdGetDatum(type),
				    0,0,0);
#endif /* P95 */
    
    if (HeapTupleIsValid(typeTuple))
        return((int) ((TypeTupleForm) GETSTRUCT(typeTuple))->typbyval);
    
    elog(WARN, "GetInputFunction: Cache lookup of type %d failed", type);
    
    return(InvalidOid);
}

/* 
 * Given the OID of a relation, return an array of index relation descriptors
 * and the number of index relations.  These relation descriptors are open
 * using heap_open().
 *
 * Space for the array itself is palloc'ed.
 */

typedef struct rel_list {
    Oid index_rel_oid;
    struct rel_list *next;
} RelationList;

static void
GetIndexRelations(Oid main_relation_oid,
		  int *n_indices,
		  Relation **index_rels)
{
    RelationList *head, *scan;
    Relation pg_index_rel;
    HeapScanDesc scandesc;
    Oid index_relation_oid;
    HeapTuple tuple;
#ifndef P95
    AttributeTupleForm *attr;
#else /* P95 */
    TupleDesc tupDesc;
#endif /* P95 */
    int i;
    bool isnull;
    
#ifndef P95
    pg_index_rel = heap_openr(Name_pg_index);
#else /* P95 */
    pg_index_rel = heap_openr(IndexRelationName);
#endif /* P95 */
    scandesc = heap_beginscan(pg_index_rel, 0, NULL, 0, NULL);
#ifndef P95
    attr = (AttributeTupleForm *) pg_index_rel->rd_att;
#else /* P95 */
    tupDesc = RelationGetTupleDescriptor(pg_index_rel);
#endif /* P95 */
    
    *n_indices = 0;
    
    head = (RelationList *) palloc(sizeof(RelationList));
    scan = head;
    head->next = NULL;
    
    for (tuple = heap_getnext(scandesc, 0, NULL);
         tuple != NULL; 
         tuple = heap_getnext(scandesc, 0, NULL)) {
	
	index_relation_oid =
	    (Oid) DatumGetInt32(heap_getattr(tuple, InvalidBuffer, 2,
#ifndef P95
						  attr, &isnull));
#else /* P95 */
					     tupDesc, &isnull));
#endif /* P95 */
	if (index_relation_oid == main_relation_oid) {
	    scan->index_rel_oid =
		(Oid) DatumGetInt32(heap_getattr(tuple, InvalidBuffer,
#ifndef P95
						      Anum_pg_index_indexrelid,
						      attr, &isnull));
#else /* P95 */
						 Anum_pg_index_indexrelid,
						 tupDesc, &isnull));
#endif /* P95 */
	    (*n_indices)++;
	    scan->next = (RelationList *) palloc(sizeof(RelationList));
	    scan = scan->next;
	}
    }
    
    heap_endscan(scandesc);
    heap_close(pg_index_rel);
    
    *index_rels = (Relation *) palloc(*n_indices * sizeof(Relation));
    
    for (i = 0, scan = head; i < *n_indices; i++, scan = scan->next) {
	(*index_rels)[i] = index_open(scan->index_rel_oid);
    }
    
    for (i = 0, scan = head; i < *n_indices + 1; i++) {
	scan = head->next;
	pfree(head);
	head = scan;
    }
}

#define EXT_ATTLEN 5*8192

#ifdef P95
/*
   returns 1 is c is in s
*/
static bool
inString(char c, char* s)
{
    int i;

    if (s) {
	i = 0;
	while (s[i] != '\0') {
	    if (s[i] == c)
		return 1;
	    i++;
	}
    }
    return 0;
}

#endif /* P95 */
/*
 * Reads input from fp until eof is seen.  If we are reading from standard
 * input, AND we see a dot on a line by itself (a dot followed immediately
 * by a newline), we exit as if we saw eof.  This is so that copy pipelines
 * can be used as standard input.
 */

static char *
#ifndef P95
CopyReadAttribute(int attno, FILE *fp, bool *isnull)
#else /* P95 */
CopyReadAttribute(int attno, FILE *fp, bool *isnull, char *delim)
#endif /* P95 */
{
    static char attribute[EXT_ATTLEN];
    char c;
    int done = 0;
    int i = 0;
    
    if (feof(fp)) {
	*isnull = (bool) false;
	return(NULL);
    }
    
    while (!done) {
	c = getc(fp);
	    
	if (feof(fp)) {
	    *isnull = (bool) false;
	    return(NULL);
	}else if (reading_from_input && attno == 0 && i == 0 && c == '.') {
	    attribute[0] = c;
	    c = getc(fp);
	    if (c == '\n') {
		*isnull = (bool) false;
		return(NULL);
#ifndef P95
	    }else if (c == '\t') {
#else /* P95 */
	    }else if (inString(c,delim)) {
#endif /* P95 */
		attribute[1] = 0;
		*isnull = (bool) false;
		return(&attribute[0]);
	    }else {
		attribute[1] = c;
		i = 2;
	    }
	}else if (c == '\\') {
	    c = getc(fp);
#ifndef P95
	}else if (c == '\t' || c == '\n') {
#else /* P95 */
	}else if (inString(c,delim) || c == '\n') {
#endif /* P95 */
	    done = 1;
	}
	if (!done) attribute[i++] = c;
	if (i == EXT_ATTLEN - 1)
	    elog(WARN, "CopyReadAttribute - attribute length too long");
    }
    attribute[i] = '\0';
    if (i == 0) {
	*isnull = (bool) true;
	return(NULL);
    }else {
	*isnull = (bool) false;
	return(&attribute[0]);
    }
}

static void
#ifndef P95
CopyAttributeOut(FILE *fp, char *string)
#else /* P95 */
CopyAttributeOut(FILE *fp, char *string, char *delim)
#endif /* P95 */
{
    int i;
    int len = strlen(string);
    
    for (i = 0; i < len; i++) {
#ifndef P95
	if (string[i] == '\t' || string[i] == '\\') {
#else /* P95 */
	if (string[i] == delim[0] || string[i] == '\n' || string[i] == '\\') {
#endif /* P95 */
	    fputc('\\', fp);
	}
	fputc(string[i], fp);
    }
}

/*
 * Returns the number of tuples in a relation.  Unfortunately, currently
 * must do a scan of the entire relation to determine this.
 *
 * relation is expected to be an open relation descriptor.
 */
static int
CountTuples(Relation relation)
{
    HeapScanDesc scandesc;
    HeapTuple tuple;
    
    int i;
    
    scandesc = heap_beginscan(relation, 0, NULL, 0, NULL);
    
    for (tuple = heap_getnext(scandesc, 0, NULL), i = 0;
         tuple != NULL; 
         tuple = heap_getnext(scandesc, 0, NULL), i++)
	;
    heap_endscan(scandesc);
    return(i);
}
