/*-------------------------------------------------------------------------
 *
 * namesvcsetup.c--
 *    Commands dealing with name service setup.
 *
 * Copyright (c) 1995, Regents of the University of California
 *
 *
 * IDENTIFICATION
 *    $Header: /usr/local/devel/mariposa/cvs/src/backend/commands/namesvcsetup.c,v 1.2 1996/03/01 19:27:50 marcel Exp $
 *
 *-------------------------------------------------------------------------
 */
#include <sys/param.h>
#include <sys/socket.h>
#include <rpc/types.h>
#include <pthread.h>
#include "postgres.h"
#include "mariposa_rpc.h"
#include "rpc/rpcutils.h"
#include "rpc/rexec.h"
#include "utils/elog.h"

#include "access/heapam.h"
#include "storage/bufmgr.h"	/* for access/transam.h */
#include "access/transam.h"	/* for GetNewObjectId */
#include "access/xact.h"
#include "catalog/catname.h"
#include "catalog/pg_class.h"
#include "catalog/pg_contract.h"
#include "catalog/pg_host.h"
#include "nodes/parsenodes.h"	/* for SubscribeStmt / UnsubscribeStmt */
#include "commands/fragments.h"	/* for FragmentCopy */
#include "commands/namesvcsetup.h"

extern unsigned short tcpPort;		/* set in ??? */

/* ----------------------------------------------------------------
 *	Subscribe
 *
 *	Calls FragmentCopy to setup the name service contracts that
 *	implement the subscription indicated by stmt.
 *
 * ----------------------------------------------------------------
 */
void
Subscribe(SubscribeStmt *stmt)
{
    CopyFragStmt copy = stmt->copyarg;
	    
    if (stmt->hostid != getMyHostId()) {
	/*
	 * This sets up the update contracts to put the subscription in effect.
	 */
	FragmentCopy(copy.fraclassid, copy.frastoreid, copy.sourceloc,
		     copy.period, copy.isreadonly, true,
		     stmt->isnsspecific);
    } else {
	elog(WARN, "Cannot subscribe to own host (you're on host %d).",
	    getMyHostId());
    }
}

/* ----------------------------------------------------------------
 *	Unsubscribe
 *
 *	Cancels the copy contracts, unschedules update streams to
 *	terminate the name service subscription.
 *
 * ----------------------------------------------------------------
 */
void
Unsubscribe(unsigned int hostid, bool isnsspecific)
{
    CancelContracts(isnsspecific, !isnsspecific, hostid, NULL);
}
