/*-------------------------------------------------------------------------
 *
 * setcmd.c--
 *    handles the utility command SET
 *
 * Copyright (c) 1994, Regents of the University of California
 *
 *
 * IDENTIFICATION
 *    $Header: /usr/local/devel/mariposa/cvs/src/backend/commands/setcmd.c,v 1.8 1996/05/31 17:13:14 jsidell Exp $
 *
 *-------------------------------------------------------------------------
 */
#include "postgres.h"
#include "commands/setcmd.h"
#include "utils/elog.h"
#include "bid.h"
#include "include/miscadmin.h"
#include "access/transam.h"

extern double bidCurveCost1, bidCurveDelay1, bidCurveCost2, bidCurveDelay2;
extern BidProtocol defaultProtocol;
extern int fragmentation;
extern int chunksize;
extern int demo;

void
setEnvironment(char *varname, char *value)
{
    if (!strcasecmp(varname, "bidcurve")) {
	sscanf(value, " %lg %lg %lg %lg ",
	    &bidCurveCost1, &bidCurveDelay1, &bidCurveCost2, &bidCurveDelay2);
    } else if (!strcasecmp(varname, "bidproto")) {
	
    defaultProtocol = (*value == 'l') ?
	LONG_PROCESS_QUERY : SHORT_PROCESS_QUERY;
	if (defaultProtocol == LONG_PROCESS_QUERY)
	    elog(NOTICE, "Using the long protocol for query processing.");
	else
	    elog(NOTICE, "Using the short protocol for query processing.");
    } else if (!strcasecmp(varname, "fragmentation")) {
	int temp;
	sscanf(value, " %i ", &temp);
	if (temp >= 0 && temp <= 100) {
	    fragmentation = temp;
	    elog(NOTICE, "Fragmentation factor set to %i out of 100.",
	        fragmentation);
	} else {
	    elog(WARN, "Fragmentation factor must be between 0 and 100.");
	}
    } else if (!strcasecmp(varname, "chunksize")) {
	int temp;
	sscanf(value, " %i ", &temp);
	if (temp >= 0 && temp <= 100) {
	    chunksize = temp;
	    elog(NOTICE, "chunksize set to %i out of 100.", chunksize);
	} else {
	    elog(WARN, "chunksize must be between 0 and 100.");
	}
    }  else if (!strcasecmp(varname, "demo")) {
	if (!strcasecmp(value, "on")) {
	    demo = TRUE;
	    elog(NOTICE, "Demo output ON.");
	} else if (!strcasecmp(value, "off")) {
	    demo = FALSE;
	    elog(NOTICE, "Demo output OFF.");
	} else {
	    elog(WARN, "Set demo on or off.");
	}
    } else if (!strcasecmp(varname, "nameserver")) {
	MyNameServerHostId = atoi(value);
	/* make this persistent */
	VariableRelationPutNameServerHostId(MyNameServerHostId);
	elog(NOTICE, "Name server set to %i.", MyNameServerHostId);
    } else if (!strcasecmp(varname, "hostid")) {
	MyHostId = atoi(value);
	/* make this persistent */
	VariableRelationPutMyHostId(MyHostId);
	elog(NOTICE, "Host id set to %i.", MyHostId);
    } else {
	elog(WARN, "SET %s: not recognized.",varname);
    }
}

void
showEnvironment(char *varname)
{
    /*
    if (!strcasecmp(varname, "bidcurve")) {
	sscanf(value, " %lg %lg %lg %lg ",
	    &bidCurveCost1, &bidCurveDelay1, &bidCurveCost2, &bidCurveDelay2);
    } else if (!strcasecmp(varname, "bidproto")) {
	defaultProtocol = (*value == 'l') ?
	    LONG_PROCESS_QUERY : SHORT_PROCESS_QUERY;
	if (defaultProtocol == LONG_PROCESS_QUERY)
	    elog(NOTICE, "Using the long protocol for query processing.");
	else
	    elog(NOTICE, "Using the short protocol for query processing.");
    } else if (!strcasecmp(varname, "fragmentation")) {
    elog(NOTICE, "Fragmentation factor set to %i out of 100.",
	        fragmentation);
	    elog(NOTICE, "chunksize set to %i out of 100.", chunksize);
    } else*/ if (!strcasecmp(varname, "nameserver")) {
	elog(NOTICE, "NAMESERVER: %d", MyNameServerHostId);
    } else {
	elog(WARN, "SHOW %s: not recognized.",varname);
    }
}
