/*-------------------------------------------------------------------------
 *
 * view.c--
 * [Postgres95]
 *    use rewrite rules to construct views
 *
 * Copyright (c) 1994, Regents of the University of California
 *
 *
 * IDENTIFICATION
 *    $Header: /usr/local/devel/mariposa/cvs/src/backend/commands/view.c,v 0.5 1996/06/07 04:55:44 am Exp $
 *
 *-------------------------------------------------------------------------
 */
#include <stdio.h>	/* for sprintf() */
#include "postgres.h"
#include "access/heapam.h"
#include "access/xact.h"
#include "utils/builtins.h"
#include "utils/syscache.h"
#include "utils/elog.h"
#include "utils/palloc.h"
#include "nodes/relation.h"
#include "nodes/primnodes.h"
#include "nodes/parsenodes.h"
#include "parser/catalog_utils.h"
#include "parser/parse_query.h"
#include "rewrite/rewriteDefine.h"
#include "rewrite/rewriteHandler.h"
#include "rewrite/rewriteManip.h"
#include "rewrite/rewriteRemove.h"
#include "commands/creatinh.h"

#ifndef P95
Name
attname(Name relname, int attnum)
{
    Relation relptr;
    HeapTuple atp;
    Oid reloid;
    Name varname;
    
    relptr = heap_openr (relname );
    reloid = RelationGetRelationId ( relptr );
    
    atp = SearchSysCacheTuple(ATTNUM,
			      (char *) ObjectIdGetDatum(reloid),
			      (char *) Int32GetDatum(attnum),
			      (char *) NULL, (char *) NULL);
    if (!HeapTupleIsValid(atp)) {
	elog(WARN, "getattnvals: no attribute tuple %d %d",
	     reloid, attnum);
	return(0);
    }
    varname = (Name)&( ((AttributeTupleForm) GETSTRUCT(atp))->attname );
    
    return ( varname );
}

#endif /* P95 */
/*---------------------------------------------------------------------
 * DefineVirtualRelation
 *
 * Create the "view" relation.
 * `DefineRelation' does all the work, we just provide the correct
 * arguments!
 *
 * If the relation already exists, then 'DefineRelation' will abort
 * the xact...
 *---------------------------------------------------------------------
 */
static void
#ifndef P95
DefineVirtualRelation(Name relname, List *tlist)
#else /* P95 */
DefineVirtualRelation(char *relname, List *tlist)
#endif /* P95 */
{
    CreateStmt createStmt;
    List *attrList, *t;
    TargetEntry *entry;
    Resdom  *res;
#ifndef P95
    Name resname;
    Name restypename;
#else /* P95 */
    char *resname;
    char *restypename;
#endif /* P95 */
    
    /*
     * create a list with one entry per attribute of this relation.
     * Each entry is a two element list. The first element is the
     * name of the attribute (a string) and the second the name of the type
     * (NOTE: a string, not a type id!).
     */
    attrList = NIL;
    if (tlist!=NIL) {
	foreach (t, tlist ) {
	    ColumnDef *def = makeNode(ColumnDef);
	    TypeName *typename;

	    /*
	     * find the names of the attribute & its type
	     */
	    entry = lfirst(t);
	    res   = entry->resdom;
	    resname = res->resname;
	    restypename = tname(get_id_type((long)res->restype));

	    typename = makeNode(TypeName);

#ifndef P95
	    /* XXX char * */
	    typename->name = (char*)makeName((char*)&restypename->data[0]); 
	    def->colname = (char*)makeName((char*)&resname->data[0]);
#else /* P95 */
	    typename->name = pstrdup(restypename);
	    def->colname = pstrdup(resname);
#endif /* P95 */

	    def->typename = typename;

	    attrList = lappend(attrList, def);
	}
    } else {
	elog ( WARN, "attempted to define virtual relation with no attrs");
    }
    
    /*
     * now create the parametesr for keys/inheritance etc.
     * All of them are nil...
     */
#ifndef P95
    createStmt.relname = (char*)relname;
#else /* P95 */
    createStmt.relname = relname;
#endif /* P95 */
    createStmt.tableElts = attrList;
#ifdef P95
/*    createStmt.tableType = NULL;*/
#endif /* P95 */
    createStmt.inhRelnames = NIL;
    createStmt.archiveType = ARCH_NONE;
    createStmt.location = -1;
    createStmt.archiveLoc = -1;

#ifndef P95
    /*
     * views do not have any meaningful partitioning.  we will say
     * they are random irrespective of the partitioning of the
     * underlying tables (which might all be different anyway).
     */
    createStmt.partattname = (char *) NULL;
    createStmt.partcmpfn = (char *) NULL;
    createStmt.parthashfn = (char *) NULL;
    createStmt.partmode = palloc(sizeof("random") + 1);
    (void) strcpy(createStmt.partmode, "random");

#endif /* P95 */
    /*
     * finally create the relation...
     */
    DefineRelation(&createStmt);
}    

/*------------------------------------------------------------------
 * makeViewRetrieveRuleName
 *
 * [Mariposa]
 * Given a view name, create the name for the 'on retrieve to "view"'
 * [Postgre95]
 * Given a view name, returns the name for the 'on retrieve to "view"'
 * rule.
 * This routine is called when defining/removing a view.
 *
 * NOTE: it quarantees that the name is at most 15 chars long
 *
 * XXX it also means view_name cannot be 16 chars long! - ay 11/94
 * XXX it also means viewName cannot be 16 chars long! - ay 11/94
 *------------------------------------------------------------------
 */
#ifndef P95
void
makeRetrieveViewRuleName(Name rule_name, Name view_name)
{
    char buf[100];
    
    /*
     * make sure that no non-null characters follow the
     * '\0' at the end of the string...
     */
    memset(buf, 0, sizeof(buf));
    sprintf(buf, "_RET%.*s", NAMEDATALEN, view_name->data);
    buf[15] = '\0';
    namestrcpy(rule_name, buf);
}

#else /* P95 */
char *
MakeRetrieveViewRuleName(char *viewName)
{
/*
    char buf[100];

    memset(buf, 0, sizeof(buf));
    sprintf(buf, "_RET%.*s", NAMEDATALEN, viewName->data);
    buf[15] = '\0';
    namestrcpy(rule_name, buf);
*/

    char *buf;
    buf = palloc(strlen(viewName) + 5);
    sprintf(buf, "_RET%s",viewName);
    return buf;
}

#endif /* P95 */


static RuleStmt *
#ifndef P95
FormViewRetrieveRule(Name view_name, Query *view_parse)
#else /* P95 */
FormViewRetrieveRule(char *viewName, Query *viewParse)
#endif /* P95 */
{
    RuleStmt *rule;
#ifndef P95
    char16 rname;
#else /* P95 */
    char *rname;
#endif /* P95 */
    Attr *attr;
    
    /*
     * Create a RuleStmt that corresponds to the suitable
     * rewrite rule args for DefineQueryRewrite();
     */
    rule = makeNode(RuleStmt);
#ifndef P95
    makeRetrieveViewRuleName(&rname, view_name);
#else /* P95 */
    rname = MakeRetrieveViewRuleName(viewName);
#endif /* P95 */

    attr = makeNode(Attr);
#ifndef P95
    attr->relname = (char*)makeName((char*)view_name);

    rule->rulename = (char*)makeName((char*)&rname);	/* XXX char * */
#else /* P95 */
    attr->relname = pstrdup(viewName);
/*    attr->refname = pstrdup(viewName);*/
    rule->rulename = pstrdup(rname);
#endif /* P95 */
    rule->whereClause = NULL;
    rule->event = CMD_SELECT;
    rule->object = attr;
    rule->instead = true;
#ifndef P95
    rule->actions = lcons(view_parse, NIL);
#else /* P95 */
    rule->actions = lcons(viewParse, NIL);
#endif /* P95 */
	
    return rule;
}

static void
#ifndef P95
DefineViewRules(Name view_name, Query *view_parse)
#else /* P95 */
DefineViewRules(char *viewName, Query *viewParse)
#endif /* P95 */
{
    RuleStmt *retrieve_rule	= NULL;
#ifdef NOTYET
    RuleStmt *replace_rule	= NULL;
    RuleStmt *append_rule	= NULL;
    RuleStmt *delete_rule	= NULL;
#endif
    
    retrieve_rule = 
#ifndef P95
	FormViewRetrieveRule(view_name, view_parse);
#else /* P95 */
	FormViewRetrieveRule(viewName, viewParse);
#endif /* P95 */
    
#ifdef NOTYET
    
    replace_rule =
#ifndef P95
	FormViewReplaceRule(view_name, view_parse);
#else /* P95 */
	FormViewReplaceRule(viewName, viewParse);
#endif /* P95 */
    append_rule = 
#ifndef P95
	FormViewAppendRule(view_name, view_parse);
#else /* P95 */
	FormViewAppendRule(viewName, viewParse);
#endif /* P95 */
    delete_rule = 
#ifndef P95
	FormViewDeleteRule(view_name, view_parse);
#else /* P95 */
	FormViewDeleteRule(viewName, viewParse);
#endif /* P95 */
    
#endif
    
    DefineQueryRewrite(retrieve_rule);

#ifdef NOTYET
    DefineQueryRewrite(replace_rule);
    DefineQueryRewrite(append_rule);
    DefineQueryRewrite(delete_rule);
#endif
    
}     

/*---------------------------------------------------------------
 * UpdateRangeTableOfViewParse
 *
 * Update the range table of the given parsetree.
 * This update consists of adding two new entries IN THE BEGINNING
 * of the range table (otherwise the rule system will die a slow,
 * horrible and painful death, and we do not want that now, do we?)
 * one for the CURRENT relation and one for the NEW one (both of
 * them refer in fact to the "view" relation).
 *
 * Of course we must also increase the 'varnos' of all the Var nodes
 * by 2...
 *
 * NOTE: these are destructive changes. It would be difficult to
 * make a complete copy of the parse tree and make the changes
 * in the copy.
 *---------------------------------------------------------------
 */
static void
#ifndef P95
UpdateRangeTableOfViewParse(Name view_name, Query *view_parse)
#else /* P95 */
UpdateRangeTableOfViewParse(char *viewName, Query *viewParse)
#endif /* P95 */
{
    List *old_rt;
    List *new_rt;
    RangeTblEntry *rt_entry1, *rt_entry2;
    
    /*
     * first offset all var nodes by 2
     */
#ifndef P95
    OffsetVarNodes((Node*)view_parse->targetList, 2);
    OffsetVarNodes(view_parse->qual, 2);
#else /* P95 */
    OffsetVarNodes((Node*)viewParse->targetList, 2);
    OffsetVarNodes(viewParse->qual, 2);
#endif /* P95 */
    
    /*
     * find the old range table...
     */
#ifndef P95
    old_rt = view_parse->rtable;
#else /* P95 */
    old_rt = viewParse->rtable;
#endif /* P95 */

    /*
     * create the 2 new range table entries and form the new
     * range table...
     * CURRENT first, then NEW....
     */
    rt_entry1 =
#ifndef P95
	makeRangeTableEntry((char*)view_name, FALSE, NULL, "*CURRENT*");
#else /* P95 */
	makeRangeTableEntry((char*)viewName, FALSE, NULL, "*CURRENT*");
#endif /* P95 */
    rt_entry2 =
#ifndef P95
	makeRangeTableEntry((char*)view_name, FALSE, NULL, "*NEW*");
#else /* P95 */
	makeRangeTableEntry((char*)viewName, FALSE, NULL, "*NEW*");
#endif /* P95 */
    new_rt = lcons(rt_entry2, old_rt);
    new_rt = lcons(rt_entry1, new_rt);
    
    /*
     * Now the tricky part....
     * Update the range table in place... Be careful here, or
     * hell breaks loooooooooooooOOOOOOOOOOOOOOOOOOSE!
     */
#ifndef P95
    view_parse->rtable = new_rt;
#else /* P95 */
    viewParse->rtable = new_rt;
#endif /* P95 */
}

/*-------------------------------------------------------------------
 * DefineView
 *
 *	- takes a "viewname", "parsetree" pair and then
 *	1)	construct the "virtual" relation 
 *	2)	commit the command but NOT the transaction,
 *		so that the relation exists
 *		before the rules are defined.
 *	2)	define the "n" rules specified in the PRS2 paper
 *		over the "virtual" relation
 *-------------------------------------------------------------------
 */
void
#ifndef P95
DefineView(Name view_name, Query *view_parse)
#else /* P95 */
DefineView(char *viewName, Query *viewParse)
#endif /* P95 */
{
#ifndef P95
    List *view_tlist;
#else /* P95 */
    List *viewTlist;
#endif /* P95 */

#ifndef P95
    view_parse = copyObject(view_parse);

    view_tlist = view_parse->targetList;
#else /* P95 */
    viewTlist = viewParse->targetList;
#endif /* P95 */
    
    /*
     * Create the "view" relation
     * NOTE: if it already exists, the xaxt will be aborted.
     */
#ifndef P95
    DefineVirtualRelation(view_name, view_tlist);
#else /* P95 */
    DefineVirtualRelation(viewName, viewTlist);
#endif /* P95 */
    
    /*
     * The relation we have just created is not visible
     * to any other commands running with the same transaction &
     * command id.
     * So, increment the command id counter (but do NOT pfree any
     * memory!!!!)
     */
    CommandCounterIncrement();
    
    /*
     * The range table of 'view_parse' does not contain entries
     * The range table of 'viewParse' does not contain entries
     * for the "CURRENT" and "NEW" relations.
     * So... add them!
     * NOTE: we make the update in place! After this call 'view_parse' 
     * NOTE: we make the update in place! After this call 'viewParse' 
     * will never be what it used to be...
     */
#ifndef P95
    UpdateRangeTableOfViewParse(view_name, view_parse);
    DefineViewRules(view_name, view_parse);
#else /* P95 */
    UpdateRangeTableOfViewParse(viewName, viewParse);
    DefineViewRules(viewName, viewParse);
#endif /* P95 */
}

/*------------------------------------------------------------------
 * RemoveView
 *
 * Remove a view given its name
 *------------------------------------------------------------------
 */
void
#ifndef P95
RemoveView(Name view_name)
#else /* P95 */
RemoveView(char *viewName)
#endif /* P95 */
{
#ifndef P95
    char16 rname;
#else /* P95 */
    char* rname;
#endif /* P95 */
    
    /*
     * first remove all the "view" rules...
     * Currently we only have one!
     */
#ifndef P95
    makeRetrieveViewRuleName(&rname, view_name);
    RemoveRewriteRule(&rname);
#else /* P95 */
    rname = MakeRetrieveViewRuleName(viewName);
    RemoveRewriteRule(rname);
#endif /* P95 */

    /*
     * we don't really need that, but just in case...
     */
    CommandCounterIncrement();
    
    /*
     * now remove the relation.
     */
#ifndef P95
    heap_destroy(view_name->data);
#else /* P95 */
    heap_destroy(viewName);
    pfree(rname);
#endif /* P95 */
}
