/*-------------------------------------------------------------------------
 *
 * execdesc.h--
 *    plan and query descriptor accessor macros used by the executor
 *    and related modules.
 *
 *
 * Copyright (c) 1994, Regents of the University of California
 *
 * $Id: execdesc.h,v 0.6 1996/06/07 04:56:03 am Exp $
 *
 *-------------------------------------------------------------------------
 */
#ifndef EXECDESC_H
#define EXECDESC_H

#include "nodes/parsenodes.h"
#include "nodes/plannodes.h"
#include "tcop/dest.h"

/* ----------------
 *	query descriptor 
 * ----------------
 * [Postgres95]
 *	query descriptor:
 *  a QueryDesc encapsulates everything that the executor
 *  needs to execute the query
 * ---------------------
 */
typedef struct QueryDesc {
#ifndef P95
    int			operation;
    int			protocol;	/* What protocol we're using - needed during subcontracting. */
    unsigned int	requestId;	/* Unique identifier for query. */
    Query		*parsetree;
    Plan		*plantree;	/* All of the plan tree, starting with the root,
					   which is processed at the local site.	*/
    List 		*remoteplans;	/* List of CoordPlan *'s of plans other than QueryDesc->plantree.
					   Each one contains a pointer to the original plan as well as a
					   pointer to its parent node. Each plan in this list is sent out
					   * to the processing site. */
    List		*distPlanList;	/* List of entire plan trees, including nodes which
					   are processed at remote sites.		*/
    List		*arglist;
    List		*typelist;
    int			nargs;
    CommandDest		dest;
    int			count;		/* was feature.Count */
#else /* P95 */
    CmdType		operation; /* CMD_SELECT, CMD_UPDATE, etc. */
    Query		*parsetree; 
    Plan		*plantree;
    CommandDest		dest;  /* the destination output of the execution */
#endif /* P95 */
} QueryDesc;
#ifdef P95

/* in pquery.c */
extern QueryDesc *CreateQueryDesc(Query *parsetree, Plan *plantree,
				  CommandDest dest);
#endif /* P95 */

#endif /*  EXECDESC_H  */
