/*-------------------------------------------------------------------------
 *
 * queue.h--
 *      Simple queue primitives.
 *      The elements of Queue are void*, so Queues can contain anything.
 *	The elements can bey keyed.
 *      Qelem can only be in one Queue at a time.
 *
 *	I wouldn't recommend doing non-keyed and keyed enqueueings to the
 *	same queue.
 *     
 * Copyright (c) 1995, Regents of the University of California
 *
 * $Id: queue.h,v 1.1 1995/07/04 07:38:24 searcher Exp $
 *
 *-------------------------------------------------------------------------
 */

#ifndef QUEUE_H
#define QUEUE_H

#include "c.h"

struct QElem;

typedef struct QElem {
    struct QElem *next;
    Oid key;
    void *item;
} QElem;

typedef struct Queue {
    QElem *front;
    QElem *back;
    int length;
} Queue;

extern	Queue*	QMakeQueue();			/* Constructor */
extern	bool	QIsEmpty(Queue *q);		/* Tests if queue empty */
extern	int	QLength(Queue *q);		/* Gives length of queue */
extern	void	QEnqueue(Queue *q, void *item);	/* Enqueues non-keyed item */
extern	void	*QDequeue(Queue *q);		/* Dequeues off front */
extern	void	*QFrontPeek(Queue *q);		/* Returns front element,
						 * but does not dequeue it */
extern	void	QPrint(Queue *q);		/* Prints out queue */

extern	void	QKeyEnqueue(Queue *q, void *item, Oid key);
                                                /* Non-sorted enqueue of
						 * keyed item */
extern	void	*QRemoveKey(Queue *q, Oid key); /* Remove keyed item */

#endif /* QUEUE_H */
