/*-------------------------------------------------------------------------
 *
 * usqueue.c--
 *      Update Stream Queue definitions. See usqueue.h for clarifications.
 *     
 * Copyright (c) 1995, Regents of the University of California
 *
 * $Id: usqueue.c,v 1.3 1996/05/31 17:13:28 jsidell Exp $
 *
 *-------------------------------------------------------------------------
 */

#include <stdio.h>
#include <stdlib.h>
#include "postgres.h"
#include "copies.h"
#include "lib/queue.h"
#include "lib/usqueue.h"
#include "utils/elog.h"
#include "utils/palloc.h"
#include "sitemgr/cpupdate.h"

USQueue* USQMakeUSQueue(int updateInterval)
{
    USQueue *newUSQueue = (USQueue *)palloc(sizeof(USQueue));
    if (newUSQueue == NULL) {
	fprintf(stderr, "USQMakeUSQueue: Out of memory!");
    }

    newUSQueue->updateInterval = updateInterval;
    newUSQueue->queue = QMakeQueue();

    return newUSQueue;
}

void USQInsert(USQueue *usq, USInfo *newUS)
{
    if (usq == NULL) {
	fprintf(stderr, "USQInsert: Null Update Stream Queue");
    }
    if (newUS == NULL) {
	fprintf(stderr, "USQInsert: Try to insert null update stream structure");
    }


    QKeyEnqueue(usq->queue, (void *)newUS, newUS->updateContract.contractid);
}

void USQCycle(USQueue *usq)
{
    USInfo *u;
    int j;

    if (usq == NULL) {
	fprintf(stderr, "USQCycle: Null Update Stream Queue");
    }

    for (j = 0; j < QLength(usq->queue); j++) {
	u = (USInfo *)QDequeue(usq->queue);
	if (u == NULL) {
	    fprintf(stderr, "USQCycle: Popped null update stream struct off queue");
	}

	u->timeRemaining -= usq->updateInterval;
	if (u->timeRemaining <= 0) {
	    u->timeRemaining = u->updateContract.period;

	    /*
	     * Defined in sitemgr/cpupdate.c. Processes outgoing update
	     * streams.
	     */
	    CopyUpdateProcess(&u->updateContract);
	}

	QKeyEnqueue(usq->queue, u, u->updateContract.contractid);
    }
}

USInfo *USQRemove(USQueue *usq, Oid key)
{
    if (usq == NULL) {
	fprintf(stderr, "USQRemove: Null Update Stream Queue");
    }

    return QRemoveKey(usq->queue, key);
}
