/*-------------------------------------------------------------------------
 *
 * usqueue.h--
 *      Update Stream Queue implementation, based on Queue (queue.h).
 *	Elements are type USInfo*, keyed on 'updateContract.contractid' member
 *	of the USInfo struct. A USInfo* can only be in one Update Stream Queue
 *	at a time.
 *     
 * Copyright (c) 1995, Regents of the University of California
 *
 * $Id: usqueue.h,v 1.2 1995/08/24 20:34:53 searcher Exp $
 *
 *-------------------------------------------------------------------------
 */

#ifndef USQUEUE_H
#define USQUEUE_H

#include "c.h"

/* Time period for waking up update stream module. Should be defined elsewhere,
 * and included here. Actaully... this should be a compile argument, like
 * DEADLOCK_TIMEOUT or something.
 */
#define UPDATE_INTERVAL	5

typedef struct USQueue {
    int updateInterval;
    Queue *queue;
} USQueue;

typedef struct USInfo {
    UpdateContractInfo updateContract;
    int timeRemaining;			/* Until update stream sent again */
} USInfo;

extern	USQueue	*USQMakeUSQueue(int updateInterval);    /* Constructor */
extern	void	USQInsert(USQueue *usq, USInfo *newUS);	/* Schedule a stream */
extern	void	USQCycle(USQueue *usq);			/* Cycle through
							 * queue, send out
							 * due updates */
extern	USInfo	*USQRemove(USQueue *usq, Oid key);	/* Un-schedule a
							 * stream */

#endif /* USQUEUE_H */
