/*-------------------------------------------------------------------------
 *
 * listmanip.c--
 *    more complicated list manipulation routines
 *
 * Copyright (c) 1994, Regents of the University of California
 *
 *
 * IDENTIFICATION
 *    $Header: /usr/local/devel/mariposa/cvs/src/backend/nodes/listmanip.c,v 1.1 1995/04/21 05:42:12 andrew Exp $
 *
 * NOTES
 *    XXX a few of the following functions are duplicated to handle
 *	  List of pointers and List of integers separately. Some day,
 *        someone should unify them. 		- ay 11/2/94
 *    This file needs cleanup.
 *
 *-------------------------------------------------------------------------
 */
#include <stdarg.h>
#include "postgres.h"
#include "nodes/pg_list.h"
#include "nodes/parsenodes.h"
#include "utils/elog.h"
#include "utils/palloc.h"

/*
 * below are for backwards compatibility
 */
List *
append(List *l1, List *l2)
{
    List *newlist, *newlist2, *p;

    if (l1==NIL)
	return copyObject(l2);

    newlist = copyObject(l1);
    newlist2 = copyObject(l2);

    for (p=newlist; lnext(p)!=NIL; p=lnext(p))
	;
    lnext(p) = newlist2;
    return newlist;
}

List *
intAppend(List *l1, List *l2)
{
    List *newlist, *newlist2, *p;

    if (l1==NIL)
	return listCopy(l2);

    newlist = listCopy(l1);
    newlist2 = listCopy(l2);

    for (p=newlist; lnext(p)!=NIL; p=lnext(p))
	;
    lnext(p) = newlist2;
    return newlist;
}

List *
nreverse(List *list)
{
    List *rlist = NIL;
    List *p = NIL;
    
    if(list==NULL)
	return(NIL);
    
    if (length(list) == 1)
	return(list);
    
    for (p = list; p!=NULL; p = lnext(p)) {
	rlist = lcons(lfirst(p),rlist);
    }
    
    lfirst(list) = lfirst(rlist);
    lnext(list)  = lnext(rlist);
    return(list);
}

/*    
 *    	same
 *    
 *    	Returns t if two lists contain the same elements.
 *       now defined in lispdep.c
 *
 * XXX only good for IntList	-ay
 */
bool
same(List *foo, List *bar)
{
    List *temp = NIL;
    
    if (foo == NULL)
	return (bar==NULL);
    if (bar == NULL)
	return (foo==NULL);
    if (length(foo) == length(bar)) {
	foreach (temp,foo) {
	    if (!intMember((int)lfirst(temp),bar))
		return(false);
	}
	return(true);
    }
    return(false);
    
}	  

List *
LispUnion(List *foo, List *bar)
{
    List *retval = NIL;
    List *i = NIL;
    List *j = NIL;
    
    if (foo==NIL)
	return(bar); /* XXX - should be copy of bar */
    
    if (bar==NIL)
	return(foo); /* XXX - should be copy of foo */
    
    foreach (i,foo) {
	foreach (j,bar) {
	    if (! equal(lfirst(i), lfirst(j))) {
		retval = lappend(retval,lfirst(i));
		break;
	    }
	}
    }
    foreach(i,bar) {
	retval = lappend(retval,lfirst(i));
    }
    
    return(retval);
}

List *
LispUnioni(List *foo, List *bar)
{
    List *retval = NIL;
    List *i = NIL;
    List *j = NIL;
    
    if (foo==NIL)
	return(bar); /* XXX - should be copy of bar */
    
    if (bar==NIL)
	return(foo); /* XXX - should be copy of foo */
    
    foreach (i,foo) {
	foreach (j,bar) {
	    if (lfirsti(i) != lfirsti(j)) {
		retval = lappendi(retval,lfirst(i));
		break;
	    }
	}
    }
    foreach(i,bar) {
	retval = lappendi(retval, lfirsti(i));
    }
    
    return(retval);
}

/*
 * member()
 * - nondestructive, returns t iff foo is a member of the list
 *   bar
 */
bool
member(void *foo, List *bar)
{
    List *i;
    foreach (i,bar)
	if (equal((Node*)(lfirst(i)),(Node*)foo))
	    return(true);
    return(false);
}

bool
intMember(int foo, List *bar)
{
    List *i;
    foreach (i,bar)
	if (foo == (int)lfirst(i))
	    return(true);
    return(false);
}

/*
 * lremove -
 *    only does pointer comparisons. Removes 'elem' from the the linked list.
 */
List *
lremove(void *elem, List *list)
{
    List *l;
    List *prev = NIL;
    List *result = list;

    foreach(l, list) {
	if (elem == lfirst(l))
	    break;
	prev = l;
    }
    if (l!=NULL) {
	if (prev == NIL) {
	    result = lnext(list);
	} else {
	    lnext(prev) = lnext(l);
	}
    }
    return result;
}
	
List *
LispRemove(void *elem, List *list)
{
    List *temp = NIL;
    List *prev = NIL;
    
    if (equal(elem, lfirst(list)))
	return lnext(list);

    temp = lnext(list);
    prev = list;
    while(temp!=NIL) {
	if (equal(elem, lfirst(temp))) {
	    lnext(prev) = lnext(temp);
	    break;
	}
	temp = lnext(temp);
	prev = lnext(prev);
    }
    return(list);
}

List *
intLispRemove(int elem, List *list)
{
    List *temp = NIL;
    List *prev = NIL;
    
    if (elem == (int)lfirst(list))
	return lnext(list);

    temp = lnext(list);
    prev = list;
    while(temp!=NIL) {
	if (elem == (int)lfirst(temp)) {
	    lnext(prev) = lnext(temp);
	    break;
	}
	temp = lnext(temp);
	prev = lnext(prev);
    }
    return(list);
}

List *
set_difference(List *list1, List *list2)
{
    List *temp1 = NIL;
    List *result = NIL;
    
    if (list2==NIL)
	return(list1); 
    
    foreach (temp1, list1) {
	if (!member(lfirst(temp1), list2))
	    result = lappend(result, lfirst(temp1));
    }
    return(result);
}

List *
set_differencei(List *list1, List *list2)
{
    List *temp1 = NIL;
    List *result = NIL;
    
    if (list2==NIL)
	return(list1); 
    
    foreach (temp1, list1) {
	if (!intMember(lfirsti(temp1), list2))
	    result = lappendi(result, lfirst(temp1));
    }
    return(result);
}
