/*-------------------------------------------------------------------------
 *
 * nodes.h--
 *    Definitions for tagged nodes.
 *
 *
 * Copyright (c) 1994, Regents of the University of California
 *
 * $Id: nodes.h,v 0.15 1996/06/07 04:56:56 am Exp $
 *
 *-------------------------------------------------------------------------
 */
#ifndef NODES_H
#define	NODES_H

#include "c.h"

/*
 * The first field of every node is NodeTag. Each node created (with makeNode)
 * will have one of the following tags as the value of its first field.
 *
 * Note that the number of the node tags are not contiguous. We left holes
 * here so that we can add more tags without changing the existing enum's.
 */
typedef enum NodeTag {
#ifndef P95
    T_Invalid,
#else /* P95 */
    T_Invalid = 0,
#endif /* P95 */

    /*---------------------
     * TAGS FOR PLAN NODES (plannodes.h)
     *---------------------
     */
#ifndef P95
    T_Plan,
#else /* P95 */
    T_Plan = 10,
#endif /* P95 */
    T_Existential,
    T_Result,
    T_Append,
    T_Scan,
    T_SeqScan,
#ifndef P95
    T_ExchangeIn,
#endif /* P95 */
    T_IndexScan,
    T_Join,
    T_NestLoop,
    T_MergeJoin,
    T_HashJoin,
    T_Temp,
    T_Material,
    T_Sort,
    T_Agg,
#ifndef P95
    T_Group,
#endif /* P95 */
    T_Unique,
    T_Hash,
    T_Choose,
#ifndef P95
    T_TupleBuffer,
    T_Merge,
#else /* P95 */
    T_Tee,
    T_Group,
#endif /* P95 */

    /*---------------------
     * TAGS FOR PRIMITIVE NODES (primnodes.h)
     *---------------------
     */
#ifndef P95
    T_Resdom,
#else /* P95 */
    T_Resdom = 100,
#endif /* P95 */
    T_Fjoin,
    T_Expr,
    T_Var,
    T_Oper,
    T_Const,
    T_Param,
    T_Aggreg,
    T_Func,
    T_Array,
    T_ArrayRef,

    /*---------------------
     * TAGS FOR INNER PLAN NODES (relation.h)
     *---------------------
     */
#ifndef P95
    T_Rel,
#else /* P95 */
    T_Rel = 200,
#endif /* P95 */
    T_Path,
    T_IndexPath,
    T_JoinPath,
    T_MergePath,
    T_HashPath,
    T_OrderKey,
    T_JoinKey,
    T_MergeOrder,
    T_CInfo,
    T_JoinMethod,
    T_HInfo,
    T_MInfo,
    T_JInfo,
    T_Iter,
    T_Stream,

    /*---------------------
     * TAGS FOR EXECUTOR NODES (execnodes.h)
     *---------------------
     */
#ifndef P95
    T_IndexInfo,
#else /* P95 */
    T_IndexInfo = 300,
#endif /* P95 */
    T_RelationInfo,
    T_TupleCount,
    T_TupleTableSlot,
    T_ExprContext,
    T_ProjectionInfo,
    T_JunkFilter,
    T_EState,
    T_BaseNode,
    T_CommonState,
    T_ResultState,
    T_AppendState,
    T_CommonScanState,
    T_ScanState,
    T_IndexScanState,
#ifndef P95
    T_ExchangeInState,
#endif /* P95 */
    T_JoinState,
    T_NestLoopState,
    T_MergeJoinState,
    T_HashJoinState,
    T_MaterialState,
    T_AggState,
    T_GroupState,
    T_SortState,
    T_UniqueState,
    T_HashState,
#ifndef P95
    T_TupleBufferState,
    T_MergeState,
    
#else /* P95 */
    T_TeeState,

#endif /* P95 */
    /*---------------------
     * TAGS FOR MEMORY NODES (memnodes.h)
     *---------------------
     */
#ifndef P95
    T_MemoryContext,
#else /* P95 */
    T_MemoryContext = 400,
#endif /* P95 */
    T_GlobalMemory,
    T_PortalMemoryContext,
    T_PortalVariableMemory,
    T_PortalHeapMemory,

    /*---------------------
     * TAGS FOR VALUE NODES (pg_list.h)
     *---------------------
     */
#ifndef P95
    T_Value,
#else /* P95 */
    T_Value = 500,
#endif /* P95 */
    T_List,
    T_Integer,
    T_Float,
    T_String,
    T_Null,
    
    /*---------------------
     * TAGS FOR PARSE TREE NODES (parsenode.h)
     *---------------------
     */
#ifndef P95
    T_Query,
#else /* P95 */
    T_Query = 600,
#endif /* P95 */
    T_AppendStmt,
    T_DeleteStmt,
    T_ReplaceStmt,
    T_CursorStmt,
    T_RetrieveStmt,
    T_AddAttrStmt,
    T_AggregateStmt,
    T_ChangeACLStmt,
    T_ClosePortalStmt,
    T_ClusterStmt,
    T_CopyStmt,
    T_CreateStmt,
    T_VersionStmt,
    T_DefineStmt,
    T_DestroyStmt,
    T_ExtendStmt,
    T_FetchStmt,
    T_IndexStmt,
    T_MoveStmt,
    T_ProcedureStmt,
    T_PurgeStmt,
#ifdef P95
    T_RecipeStmt,
#endif /* P95 */
    T_RemoveFuncStmt,
    T_RemoveOperStmt,
    T_RemoveStmt,
    T_RenameStmt,
    T_RuleStmt,
    T_NotifyStmt,
    T_ListenStmt,
    T_TransactionStmt,
    T_ViewStmt,
    T_LoadStmt,
    T_CreatedbStmt,
    T_DestroydbStmt,
    T_VacuumStmt,
#ifndef P95
    T_MoveFragStmt,
    T_CopyFragStmt,
    T_UpdateContractStmt,
    T_NotUpdateStmt,
    T_SplitFragStmt,
    T_SetEnvironStmt,
    T_ShowEnvironStmt,
    T_SubscribeStmt,
    T_UnsubscribeStmt,
    T_A_Expr,
#else /* P95 */
    T_ExplainStmt,

    T_A_Expr = 700,
#endif /* P95 */
    T_Attr,
    T_A_Const,
    T_ParamNo,
    T_Ident,
    T_FuncCall,
    T_A_Indices,
    T_ResTarget,
    T_ParamString,
    T_TimeRange,
    T_RelExpr,
    T_SortBy,
    T_RangeVar,
    T_TypeName,
    T_IndexElem,
    T_ColumnDef,
    T_DefElem,
    T_TargetEntry,
    T_RangeTblEntry,
#ifndef P95
    T_SiteAddrNode,
#endif /* P95 */
    T_SortClause,
#ifndef P95
    T_GroupClause,
    T_FragInfo
#else /* P95 */
    T_GroupClause
#endif /* P95 */
} NodeTag;

/*
 * The first field of a node of any type is gauranteed to be the NodeTag.
 * Hence the type of any node can be gotten by casting it to Node. Declaring
 * a variable to be of Node * (instead of void *) can also facilitate
 * debugging.
 */
typedef struct Node {
    NodeTag	type;	
} Node;

#define	nodeTag(_node_)		((Node*)_node_)->type

#define	makeNode(_node_)	(_node_*)newNode(sizeof(_node_),T_##_node_)
#define NodeSetTag(n, t)	((Node *)n)->type = t

#define IsA(_node_,_tag_)	(nodeTag(_node_) == T_##_tag_)

/* ----------------------------------------------------------------
 *		      IsA functions (no inheritence any more)
 * ----------------------------------------------------------------
 */
#define IsA_JoinPath(jp) \
    (nodeTag(jp)==T_JoinPath || nodeTag(jp)==T_MergePath || \
     nodeTag(jp)==T_HashPath)

#define IsA_Join(j) \
    (nodeTag(j)==T_Join || nodeTag(j)==T_NestLoop || \
     nodeTag(j)==T_MergeJoin || nodeTag(j)==T_HashJoin)

#define IsA_Temp(t) \
    (nodeTag(t)==T_Temp || nodeTag(t)==T_Material || nodeTag(t)==T_Sort || \
     nodeTag(t)==T_Unique)

/* ----------------------------------------------------------------
 *		      extern declarations follow
 * ----------------------------------------------------------------
 */

/*
 * nodes/nodes.c
 */
extern Node *newNode(Size size, NodeTag tag);

/*
 * nodes/{outfuncs.c,print.c}
 */
#define nodeDisplay	print

extern char *nodeToString(void *obj);
extern void print(void *obj);

/*
 * nodes/{readfuncs.c,read.c}
 */
extern void *stringToNode(char *str);

/*
 * nodes/copyfuncs.c
 */
extern void *copyObject(void *obj);

/*
 * nodes/equalfuncs.c
 */
extern bool equal(void *a, void *b);


/* ----------------
 *      I don't know why this is here.  Most likely a hack..
 *      -cim 6/3/90
 * ----------------
 */
typedef float Cost;

/*
 * CmdType -
 *    enums for type of operation to aid debugging
 *
 * ??? could have put this in parsenodes.h but many files not in the
 *    optimizer also need this...
 */
typedef enum CmdType {
    CMD_UNKNOWN,
    CMD_SELECT,		/* select stmt (formerly retrieve) */
    CMD_UPDATE,		/* update stmt (formerly replace) */
    CMD_INSERT,		/* insert stmt (formerly append) */
    CMD_DELETE,
#ifndef P95
    CMD_NOTIFY,
    CMD_UTILITY,	/* cmds like create, destroy, copy, vacuum, etc. */
    CMD_EXCHOUT		/* exchange out */
#else /* P95 */
    CMD_NOTIFY, 
    CMD_UTILITY /* cmds like create, destroy, copy, vacuum, etc. */
#endif /* P95 */
} CmdType;
    

#endif /* NODES_H */
