/*-------------------------------------------------------------------------
 *
 * xdrfuncs.c--
 *    Routines to handle XDR data conversion for primnodes.h, plannodes.h,
 *    parsenodes.h.
 *
 * Copyright (c) 1994, Regents of the University of California
 *
 * IDENTIFICATION
 *    $Header: /usr/local/devel/mariposa/cvs/src/backend/nodes/xdrfuncs.c,v 1.15 1996/03/04 09:07:09 aoki Exp $
 *
 *-------------------------------------------------------------------------
 */
#include <rpc/rpc.h>

#include "postgres.h"

#include "nodes/nodes.h"
#include "nodes/pg_list.h"
#include "nodes/plannodes.h"
#include "nodes/primnodes.h"
#include "nodes/parsenodes.h"
#include "nodes/relation.h"	/* for Iter */

#include "parser/parse_query.h"
#include "utils/tqual.h"

#include "nodes/pqa.h"

#include "xdrfuncs.h"

#include "rpctypes.h"

#include "bid.h"

#define MAX_STR_LENGTH 255
#define CHAR16_LEN 16

static bool_t xdr_AggregPtr(XDR *xdrs, Aggreg **objp);
static bool_t xdr_Iter(XDR *xdrs, Iter *objp);

/* @@@ This brought in from parse_query.c, so that parse_query.o does not have
 *     to be linked with the sitemgr. For now, don't know how to implement
 *     without sharing postgres libraries, so returns dummy TimeQual.
 *								-Rex 95.1.14
 */
TimeQual
make_xdr_tr(char *datestring1, char *datestring2, int timecode)
/* For timecode, 0 = snapshot, 1 = timerange */
{
/*  fprintf(stderr, "_MXTR"); */

    /* @@@ Hack for now */
    return (TimeQual)malloc(sizeof((unsigned)10));
}

/* Given NodeTag, returns size of the structure that it represents. */
size_t
get_xdr_size(NodeTag t)
{
    switch(t) {
	case T_List:
	    return sizeof(struct List);
	case T_Plan:
	    return sizeof(struct Plan);
	case T_Existential:
	    return sizeof(Existential);
	case T_Result:
	    return sizeof(struct Result);
	case T_Append:
	    return sizeof(struct Append);
	case T_Scan:
	    return sizeof(struct Scan);
	case T_SeqScan:
	    return sizeof(SeqScan);
	case T_ExchangeIn:
	    return sizeof(struct ExchangeIn);
	case T_IndexScan:
	    return sizeof(struct IndexScan);
	case T_Join:
	    return sizeof(Join);
	case T_NestLoop:
	    return sizeof(struct NestLoop);
	case T_MergeJoin:
	    return sizeof(struct MergeJoin);
	case T_HashJoin:
	    return sizeof(struct HashJoin);
	case T_Temp:
	    return sizeof(struct Temp);
	case T_Material:
	    return sizeof(struct Material);
	case T_Sort:
	    return sizeof(struct Sort);
	case T_Agg:
	    return sizeof(struct Agg);
	case T_Group:
	    return sizeof(struct Group);
	case T_Unique:
	    return sizeof(struct Unique);
	case T_Choose:
	    return sizeof(struct Choose);
	case T_Hash:
	    return sizeof(struct Hash);
	case T_TupleBuffer:
	    return sizeof(struct TupleBuffer);
	case T_Merge:
	    return sizeof(struct Merge);
	case T_Resdom:
	    return sizeof(struct Resdom);
	case T_Fjoin:
	    return sizeof(struct Fjoin);
	case T_Expr:
	    return sizeof(struct Expr);
	case T_Var:
	    return sizeof(struct Var);
	case T_Oper:
	    return sizeof(struct Oper);
	case T_Const:
	    return sizeof(struct Const);
	case T_Param:
	    return sizeof(struct Param);
	case T_Func:
	    return sizeof(struct Func);
	case T_Aggreg:
	    return sizeof(struct Aggreg);
	case T_Array:
	    return sizeof(struct Array);
	case T_ArrayRef:
	    return sizeof(struct ArrayRef);
	case T_TargetEntry:
	    return sizeof(struct TargetEntry);
	case T_RangeTblEntry:
	    return sizeof(struct RangeTblEntry);
	case T_FragInfo:
	    return sizeof(struct FragInfo);
	case T_SiteAddrNode:
	    return sizeof(struct SiteAddrNode);
	case T_SortClause:
	    return sizeof(struct SortClause);
	case T_Iter:
	    return sizeof(struct Iter);
	default:
	    fprintf(stderr, "*UNKNOWN NODE* (%d)\n", t);
	    return sizeof(int);
    }
}

/* Given NodeTag, returns pointer to xdr function that it represents. */
PFB
get_xdr_func(NodeTag t)
{
    switch(t) {
	case T_List:
	    return xdr_Node_node;
	case T_Plan:
	    return xdr_Plan;
	case T_Existential:
	    return xdr_Existential;
	case T_Result:
	    return xdr_Result;
	case T_Append:
	    return xdr_Append;
	case T_Scan:
	    return xdr_Scan;
	case T_SeqScan:
	    return xdr_SeqScan;
	case T_ExchangeIn:
	    return xdr_ExchangeIn;
	case T_IndexScan:
	    return xdr_IndexScan;
	case T_Join:
	    return xdr_Join;
	case T_NestLoop:
	    return xdr_NestLoop;
	case T_MergeJoin:
	    return xdr_MergeJoin;
	case T_HashJoin:
	    return xdr_HashJoin;
	case T_Temp:
	    return xdr_Temp;
	case T_Material:
	    return xdr_Material;
	case T_Sort:
	    return xdr_Sort;
	case T_Agg:
	    return xdr_Agg;
	case T_Group:
	    return xdr_Group;
	case T_Unique:
	    return xdr_Unique;
	case T_Hash:
	    return xdr_Hash;
	case T_Choose:
	    return xdr_Choose;
	case T_TupleBuffer:
	    return xdr_TupleBuffer;
	case T_Merge:
	    return xdr_Merge;
	case T_Resdom:
	    return xdr_Resdom;
	case T_Fjoin:
	    return xdr_Fjoin;
	case T_Expr:
	    return xdr_Expr;
	case T_Var:
	    return xdr_Var;
	case T_Oper:
	    return xdr_Oper;
	case T_Const:
	    return xdr_Const;
	case T_Param:
	    return xdr_Param;
	case T_Func:
	    return xdr_Func;
	case T_Aggreg:
	    return xdr_Aggreg;
	case T_Array:
	    return xdr_Array;
	case T_ArrayRef:
	    return xdr_ArrayRef;
	case T_TargetEntry:
	    return xdr_TargetEntry;
	case T_RangeTblEntry:
	    return xdr_RangeTblEntry;
	case T_FragInfo:
	    return xdr_FragInfo;
	case T_SiteAddrNode:
	    return xdr_SiteAddrNode;
	case T_SortClause:
	    return xdr_SortClause;
	case T_Iter:
	    return xdr_Iter;
	default:
	    fprintf(stderr, "*UNKNOWN NODE* (%d)\n", t);
	    return xdr_Plan;
    }
}

/* These two routines, xdr_PQA_Plan_tree and xdr_PQA_Int_list, are used to
 * convert the Plan * and List * within a ProcessQueryArg struct.
 */
bool_t
xdr_PQA_Plan_tree(XDR *xdrs, PQA_Plan_tree *objp)
{
/*  fprintf(stderr, "_PQAP"); */

    return xdr_Plan_tree(xdrs, (Node **)objp);
}

bool_t
xdr_PQA_Int_list(XDR *xdrs, PQA_Int_list *objp)
{
/*  fprintf(stderr, "_PQAI"); */

    return xdr_Int_list(xdrs, (List **)objp);
}

bool_t
xdr_PQA_Node_list(XDR *xdrs, PQA_Node_list *objp)
{
    return xdr_Node_list(xdrs, (List **)objp);
}

bool_t
xdr_Str(XDR *xdrs, char *objp)
{
/*  fprintf(stderr, "__STR"); */

    if (!xdr_string(xdrs, (char **)&objp, MAX_STR_LENGTH))
	return FALSE;

    return TRUE;
}

bool_t
xdr_SiteAddrNode(xdrs, objp)
	XDR *xdrs;
	SiteAddrNode *objp;
{
    if (!xdr_NodeTag(xdrs, &objp->type)) {
	return (FALSE);
    }
	
    if (!xdr_u_int(xdrs, &objp->port)) {
	return (FALSE);
    }
    if (!xdr_u_int(xdrs, &objp->addr)) {
	return (FALSE);
    }
    if (!xdr_u_int(xdrs, &objp->hostid)) {
	return (FALSE);
    }
    return (TRUE);
}

#ifdef 0

bool_t
xdr_SiteAddrNode_node(XDR *xdrs, List *objp) {
    if (!xdr_NodeTag(xdrs, &objp->type))
	return FALSE;
    if (!xdr_SiteAddrNode(xdrs, (SiteAddrNode *)objp->elem))
	 return FALSE;
    if (!xdr_SiteAddrNode_list(xdrs, &objp->next))
	 return FALSE;

    return TRUE;
    
}

bool_t
xdr_SiteAddrNode_list(XDR *xdrs, List **objp) {

    if (xdrs->x_op == XDR_DECODE)
	*objp = NULL;


    return xdr_pointer(xdrs, (char **)objp, sizeof(struct List),
		       xdr_SiteAddrNode_node);
}

#endif

bool_t
xdr_bool_node(XDR *xdrs, List *objp) {
    if (!xdr_NodeTag(xdrs, &objp->type))
	return FALSE;
    if (!xdr_char(xdrs, (bool *)&objp->elem))
	 return FALSE;
    if (!xdr_bool_list(xdrs, &objp->next))
	 return FALSE;

    return TRUE;
    
}

bool_t
xdr_bool_list(XDR *xdrs, List **objp) {

    if (xdrs->x_op == XDR_DECODE)
	*objp = NULL;


    return xdr_pointer(xdrs, (char **)objp, sizeof(struct List),
		       xdr_bool_node);
}


bool_t
xdr_Int_node(XDR *xdrs, List *objp)
{
/*  fprintf(stderr, "_INTN"); */

    if (!xdr_NodeTag(xdrs, &objp->type))
	return FALSE;
    if (!xdr_int(xdrs, (int *)&objp->elem))
	 return FALSE;
    if (!xdr_Int_list(xdrs, &objp->next))
	 return FALSE;

    return TRUE;
}

bool_t
xdr_Int_list(XDR *xdrs, List **objp)
{
/*  fprintf(stderr, "_INTL"); */

    if (xdrs->x_op == XDR_DECODE)
	*objp = NULL;

    return xdr_pointer(xdrs, (char **)objp, sizeof(struct List), xdr_Int_node);
}

bool_t
xdr_Long_node(XDR *xdrs, List *objp)
{
/*  fprintf(stderr, "_LONN"); */

    if (!xdr_NodeTag(xdrs, &objp->type))
	return FALSE;
    if (!xdr_long(xdrs, (long *)objp->elem))
	return FALSE;
    if (!xdr_Long_list(xdrs, &objp->next))
	return FALSE;

    return TRUE;
}

bool_t
xdr_Long_list(XDR *xdrs, List **objp)
{
/*  fprintf(stderr, "_LONL"); */

    if (xdrs->x_op == XDR_DECODE)
	*objp = NULL;

    return xdr_pointer(xdrs, (char **)objp, sizeof(struct List), xdr_Long_node);
}

bool_t
xdr_Node_node(XDR *xdrs, List *objp)
{
    NodeTag t_save1 = T_Plan;

/*  fprintf(stderr, "_NODN"); */

    if (xdrs->x_op == XDR_DECODE)
	objp->elem = NULL;

    if (!xdr_NodeTag(xdrs, &objp->type))
	return FALSE;

    if ((xdrs->x_op == XDR_ENCODE || xdrs->x_op == XDR_FREE) &&
	objp->elem != NULL)
	t_save1 = nodeTag((Node *)objp->elem);
    if (!xdr_NodeTag(xdrs, &t_save1))
	return FALSE;
    if (!xdr_pointer(xdrs, (char **)&objp->elem, get_xdr_size(t_save1),
		     get_xdr_func(t_save1)))
	return FALSE;

    if (!xdr_Node_list(xdrs, &objp->next))
	return FALSE;

    return TRUE;
}

bool_t
xdr_Node_list(XDR *xdrs, List **objp)
{
/*  fprintf(stderr, "_NODL"); */

    if (xdrs->x_op == XDR_DECODE)
	*objp = NULL;

    return xdr_pointer(xdrs, (char **)objp, sizeof(struct List),
		       xdr_Node_node);
}

bool_t
xdr_NodeTag(XDR *xdrs, NodeTag *objp)
{
/*  fprintf(stderr, "_NODT"); */

    if (!xdr_enum(xdrs, (enum_t *)objp))
	return FALSE;

    return TRUE;
}

bool_t
xdr_CmdType(XDR *xdrs, CmdType *objp)
{
/*  fprintf(stderr, "_CMDT"); */

    if (!xdr_enum(xdrs, (enum_t *)objp))
	return FALSE;

    return TRUE;
}

bool_t
xdr_NameData(XDR *xdrs, NameData *objp)
{
/*  fprintf(stderr, "_NMDT"); */

    if (!xdr_vector(xdrs, objp->data, CHAR16_LEN, sizeof(char), xdr_char))
	return FALSE;

    return TRUE;
}

bool_t
xdr_TimeRange(XDR *xdrs, TimeRange *objp)
{
/*  fprintf(stderr, "_TMRN"); */

    if (xdrs->x_op == XDR_DECODE) {
	objp->startDate = NULL;
	objp->endDate = NULL;
    }

    if (!xdr_NodeTag(xdrs, &objp->type))
	return FALSE;
    if (!xdr_pointer(xdrs, (char **)&objp->startDate, sizeof(char *), xdr_Str))
	return FALSE;
    if (!xdr_pointer(xdrs, (char **)&objp->endDate, sizeof(char *), xdr_Str))
	return FALSE;

    return TRUE;
}

#if 0
bool_t
xdr_Budget(XDR *xdrs, Budget *objp) {
    if (!xdr_float(xdrs, (float *)&objp->cost))
	return FALSE;
    if (!xdr_float(xdrs, (float *)&objp->delay))
	return FALSE;
    if (!xdr_float(xdrs, (float *)&objp->staleness))
	return FALSE;
    if (!xdr_float(xdrs, (float *)&objp->accuracy))
	return FALSE;

    return TRUE;
}

#endif

bool_t
xdr_Plan(XDR *xdrs, Plan *objp)
{
    NodeTag t_save1 = T_Plan, t_save2 = T_Plan;

/*  fprintf(stderr, "_PLAN"); */

    if (xdrs->x_op == XDR_DECODE) {
	objp->targetlist = NULL;
	objp->qual = NULL;
	objp->lefttree = NULL;
	objp->righttree = NULL;
    }

    if (!xdr_NodeTag(xdrs, &objp->type))
	return FALSE;
    if (!xdr_float(xdrs, (float *)&objp->cost))
	return FALSE;
    if (!xdr_Budget(xdrs, &objp->budget))
	return FALSE;
    if (!xdr_SiteAddrNode(xdrs, &objp->processingSite))
	return FALSE;
    if (!xdr_int(xdrs, &objp->contract))
	return FALSE;
    if (!xdr_int(xdrs, &objp->qepFragId))
	return FALSE;
    if (!xdr_int(xdrs, &objp->plan_size))
	return FALSE;
    if (!xdr_int(xdrs, &objp->plan_width))
	return FALSE;
    if (!xdr_int(xdrs, &objp->plan_tupperpage))
	return FALSE;
    if (!xdr_long(xdrs, (long *)&objp->state))
	return FALSE;
    if ((xdrs->x_op == XDR_ENCODE || xdrs->x_op == XDR_FREE) &&
	objp->targetlist != NULL)
	t_save1 = nodeTag((Node *)objp->targetlist);
    if (!xdr_NodeTag(xdrs, &t_save1))
	return FALSE;
    if (!xdr_pointer(xdrs, (char **)&objp->targetlist, get_xdr_size(t_save1),
		     get_xdr_func(t_save1)))
	return FALSE;

    if ((xdrs->x_op == XDR_ENCODE || xdrs->x_op == XDR_FREE) &&
	objp->qual != NULL)
	t_save2 = nodeTag((Node *)objp->qual);
    if (!xdr_NodeTag(xdrs, &t_save2))
	return FALSE;
    if (!xdr_pointer(xdrs, (char **)&objp->qual, get_xdr_size(t_save2),
		     get_xdr_func(t_save2)))
	return FALSE;

    if (!xdr_Plan_tree(xdrs, (Node **)&objp->lefttree))
	return FALSE;
    if (!xdr_Plan_tree(xdrs, (Node **)&objp->righttree))
	return FALSE;

    return TRUE;
}

bool_t
xdr_Plan_tree(XDR *xdrs, Node **objp)
{
    /* Assigning it T_Plan, so that if *objp is null, then you don't get
     * funny UNKNOWN NODE error messages.
     */
    NodeTag t_save1 = T_Plan;

/*  fprintf(stderr, "_PLTR"); */

    if ((xdrs->x_op == XDR_ENCODE || xdrs->x_op == XDR_FREE) &&
	*objp != NULL)
	t_save1 = nodeTag((Node *)*objp);
    if (!xdr_NodeTag(xdrs, &t_save1))
	return FALSE;
    return (xdr_pointer(xdrs, (char **)objp, get_xdr_size(t_save1),
			get_xdr_func(t_save1)));
}

bool_t
xdr_Existential(XDR *xdrs, Existential *objp)
{
/*  fprintf(stderr, "_EXIS"); */

    if (!xdr_Plan(xdrs, (Plan *)objp))
	return FALSE;

    return TRUE;
}

bool_t
xdr_Result(XDR *xdrs, Result *objp)
{
    NodeTag t_save1 = T_Plan, t_save2 = T_Plan;

/*  fprintf(stderr, "_RSLT"); */

    if (xdrs->x_op == XDR_DECODE) {
	objp->resconstantqual = NULL;
    }

    if (!xdr_Plan(xdrs, &objp->plan))
	return FALSE;

    if ((xdrs->x_op == XDR_ENCODE || xdrs->x_op == XDR_FREE) &&
	objp->resconstantqual != NULL)
	t_save2 = nodeTag((Node *)objp->resconstantqual);
    if (!xdr_NodeTag(xdrs, &t_save2))
	return FALSE;
    if (!xdr_pointer(xdrs, (char **)&objp->resconstantqual,
		     get_xdr_size(t_save2), get_xdr_func(t_save2)))
	return FALSE;

    return TRUE;
}

bool_t
xdr_Append(XDR *xdrs, Append *objp)
{
    NodeTag t_save1 = T_Plan, t_save2 = T_Plan;

/*  fprintf(stderr, "_APPD"); */

    if (xdrs->x_op == XDR_DECODE) {
	objp->unionplans = NULL;
	objp->unionrtentries = NULL;
    }

    if (!xdr_Plan(xdrs, &objp->plan))
	return FALSE;

    if ((xdrs->x_op == XDR_ENCODE || xdrs->x_op == XDR_FREE) &&
	objp->unionplans != NULL)
	t_save1 = nodeTag((Node *)objp->unionplans);
    if (!xdr_NodeTag(xdrs, &t_save1))
	return FALSE;
    if (!xdr_pointer(xdrs, (char **)&objp->unionplans, get_xdr_size(t_save1),
		     get_xdr_func(t_save1)))
	return FALSE;

    if (!xdr_u_int(xdrs, (unsigned int *)&objp->unionrelid))
	return FALSE;

    if ((xdrs->x_op == XDR_ENCODE || xdrs->x_op == XDR_FREE) &&
	objp->unionrtentries != NULL)
	t_save1 = nodeTag((Node *)objp->unionrtentries);
    if (!xdr_NodeTag(xdrs, &t_save1))
	return FALSE;
    if (!xdr_pointer(xdrs, (char **)&objp->unionrtentries,
		     get_xdr_size(t_save1), get_xdr_func(t_save1)))
	return FALSE;

    return TRUE;
}

bool_t
xdr_Scan(XDR *xdrs, Scan *objp)
{
/*  fprintf(stderr, "_SCAN"); */

    if (!xdr_Plan(xdrs, &objp->plan))
	return FALSE;
    if (!xdr_u_int(xdrs, (unsigned int *)&objp->scanrelid))
	return FALSE;
    if (!xdr_u_int(xdrs, (unsigned int *)&objp->scanfragid))
	return FALSE;
    
    return TRUE;
}

bool_t
xdr_SeqScan(XDR *xdrs, SeqScan *objp)
{
/*  fprintf(stderr, "_SEQS"); */

    if (!xdr_Scan(xdrs, (Scan *)objp))
	return FALSE;

    return TRUE;
}

bool_t
xdr_ExchangeIn(XDR *xdrs, ExchangeIn *objp)
{
/*  fprintf(stderr, "_EXIN"); */

    if (!xdr_Plan(xdrs, &objp->plan))
	return FALSE;
    if (!xdr_Node_list(xdrs, &objp->exchsendingSites))
	return FALSE;
    if (!xdr_Int_list(xdrs, &objp->exchsendingQepFragIds))
	return FALSE;

    return TRUE;
}

bool_t
xdr_IndexScan(XDR *xdrs, IndexScan *objp)
{
/*  fprintf(stderr, "_IXSC"); */

    if (xdrs->x_op == XDR_DECODE) {
	objp->indxid = NULL;
	objp->indxqual = NULL;
    }

    if (!xdr_Scan(xdrs, &objp->scan))
	return FALSE;

    if (!xdr_Int_list(xdrs, &objp->indxid))
	return FALSE;

    if (!xdr_Node_list(xdrs, &objp->indxqual))
	return FALSE;

    return TRUE;
}

bool_t
xdr_Join(XDR *xdrs, Join *objp)
{
/*  fprintf(stderr, "_JOIN"); */

    if (!xdr_Plan(xdrs, (Plan *)objp))
	return FALSE;

    return TRUE;
}

bool_t
xdr_NestLoop(XDR *xdrs, NestLoop *objp)
{
/*  fprintf(stderr, "_NSLP"); */

    if (!xdr_Join(xdrs, &objp->join))
	return FALSE;

    return TRUE;
}

bool_t
xdr_MergeJoin(XDR *xdrs, MergeJoin *objp)
{
    NodeTag t_save1 = T_Plan;

/*  fprintf(stderr, "_MRJN"); */

    if (xdrs->x_op == XDR_DECODE)
	objp->mergeclauses = NULL;

    if (!xdr_Join(xdrs, &objp->join))
	return FALSE;

    if ((xdrs->x_op == XDR_ENCODE || xdrs->x_op == XDR_FREE) &&
	objp->mergeclauses != NULL)
	t_save1 = nodeTag((Node *)objp->mergeclauses);
    if (!xdr_NodeTag(xdrs, &t_save1))
	return FALSE;
    if (!xdr_pointer(xdrs, (char **)&objp->mergeclauses, get_xdr_size(t_save1),
		     get_xdr_func(t_save1)))
	return FALSE;

    if (!xdr_u_int(xdrs, (unsigned int *)&objp->mergesortop))
	return FALSE;

    if (!xdr_pointer(xdrs, (char **)&objp->mergerightorder,
		     sizeof(unsigned int), xdr_u_int))
	return FALSE;
    if (!xdr_pointer(xdrs, (char **)&objp->mergeleftorder,
		     sizeof(unsigned int), xdr_u_int))
	return FALSE;

    return TRUE;
}

bool_t
xdr_HashJoin(XDR *xdrs, HashJoin *objp)
{
    NodeTag t_save1 = T_Plan;

/*  fprintf(stderr, "_HAJN"); */

    if (xdrs->x_op == XDR_DECODE)
	objp->hashclauses = NULL;

    if (!xdr_Join(xdrs, &objp->join))
	return FALSE;

    if ((xdrs->x_op == XDR_ENCODE || xdrs->x_op == XDR_FREE) &&
	objp->hashclauses != NULL)
	t_save1 = nodeTag((Node *)objp->hashclauses);
    if (!xdr_NodeTag(xdrs, &t_save1))
	return FALSE;
    if (!xdr_pointer(xdrs, (char **)&objp->hashclauses, get_xdr_size(t_save1),
		     get_xdr_func(t_save1)))
	return FALSE;

    if (!xdr_u_int(xdrs, (unsigned int *)&objp->hashjoinop))
	return FALSE;
    if (!xdr_u_int(xdrs, (unsigned int *)&objp->hashjointablekey))
	return FALSE;
    if (!xdr_int(xdrs, &objp->hashjointablesize))
	return FALSE;
    if (!xdr_char(xdrs, &objp->hashdone))
	return FALSE;

    return (TRUE);
}

bool_t
xdr_Temp(XDR *xdrs, Temp *objp)
{
/*  fprintf(stderr, "_TEMP"); */

    if (!xdr_Plan(xdrs, &objp->plan))
	return FALSE;
    if (!xdr_u_int(xdrs, (unsigned int *)&objp->tempid))
	return FALSE;
    if (!xdr_int(xdrs, &objp->keycount))
	return FALSE;

    return TRUE;
}

bool_t
xdr_Material(XDR *xdrs, Material *objp)
{
/*  fprintf(stderr, "_MATL"); */

    if (!xdr_Plan(xdrs, &objp->plan))
	return FALSE;
    if (!xdr_u_int(xdrs, (unsigned int *)&objp->tempid))
	return FALSE;
    if (!xdr_int(xdrs, &objp->keycount))
	return FALSE;

    return TRUE;
}

bool_t
xdr_Sort(XDR *xdrs, Sort *objp)
{
/*  fprintf(stderr, "_SORT"); */

    if (!xdr_Plan(xdrs, &objp->plan))
	return FALSE;
    if (!xdr_u_int(xdrs, (unsigned int *)&objp->tempid))
	return FALSE;
    if (!xdr_int(xdrs, &objp->keycount))
	return FALSE;

    return TRUE;
}

bool_t
xdr_Agg(XDR *xdrs, Agg *objp)
{
/*  fprintf(stderr, "__AGG"); */

    if (xdrs->x_op == XDR_DECODE)
	objp->aggs = NULL;

    if (!xdr_Plan(xdrs, &objp->plan))
	return FALSE;
    if (!xdr_int(xdrs, &objp->numAgg))
	return FALSE;
    if (!xdr_array(xdrs, &objp->aggs, &objp->numAgg, objp->numAgg,
		   sizeof(Aggreg *), xdr_AggregPtr))
	return FALSE;

    return TRUE;
}

bool_t
xdr_Group(XDR *xdrs, Group *objp)
{
/*  fprintf(stderr, "__GRP"); */

    if (xdrs->x_op == XDR_DECODE)
	objp->grpColIdx = NULL;

    if (!xdr_Plan(xdrs, &objp->plan))
	return FALSE;
    if (!xdr_char(xdrs, &objp->tuplePerGroup))
	return FALSE;
    if (!xdr_int(xdrs, &objp->numCols))
	return FALSE;
    if (!xdr_array(xdrs, &objp->grpColIdx, &objp->numCols, objp->numCols,
		   sizeof(u_short), xdr_u_short))
	return FALSE;

    return TRUE;
}

bool_t
xdr_Unique(XDR *xdrs, Unique *objp)
{
/*  fprintf(stderr, "_UNQE"); */

    if (!xdr_Plan(xdrs, &objp->plan))
	return FALSE;

    if (!xdr_u_int(xdrs, (unsigned int *)&objp->tempid))
	return FALSE;
    if (!xdr_int(xdrs, &objp->keycount))
	return FALSE;

    return TRUE;
}

bool_t
xdr_Choose(XDR *xdrs, Choose *objp)
{
    NodeTag t_save1 = T_Plan;

/*  fprintf(stderr, "_CHOO"); */

    if (xdrs->x_op == XDR_DECODE)
	objp->chooseplanlist = NULL;

    if (!xdr_Plan(xdrs, &objp->plan))
	return FALSE;

    if ((xdrs->x_op == XDR_ENCODE || xdrs->x_op == XDR_FREE) &&
	objp->chooseplanlist != NULL)
	t_save1 = nodeTag((Node *)objp->chooseplanlist);
    if (!xdr_NodeTag(xdrs, &t_save1))
	return FALSE;
    if (!xdr_pointer(xdrs, (char **)&objp->chooseplanlist,
		     get_xdr_size(t_save1), get_xdr_func(t_save1)))
	return FALSE;

    return TRUE;
}

bool_t
xdr_Hash(XDR *xdrs, Hash *objp)
{
    NodeTag t_save1 = T_Plan;

/*  fprintf(stderr, "_HASH"); */

    if (xdrs->x_op == XDR_DECODE)
	objp->hashkey = NULL;

    if (!xdr_Plan(xdrs, &objp->plan))
	return FALSE;

    if ((xdrs->x_op == XDR_ENCODE || xdrs->x_op == XDR_FREE) &&
	objp->hashkey != NULL)
	t_save1 = nodeTag((Node *)objp->hashkey);
    if (!xdr_NodeTag(xdrs, &t_save1))
	return FALSE;
    if (!xdr_pointer(xdrs, (char **)&objp->hashkey, get_xdr_size(t_save1),
		     get_xdr_func(t_save1)))
	return FALSE;

    /* Not copying &objp->hashtable, it is a HashTableData * */
    if (!xdr_u_int(xdrs, (unsigned int *)&objp->hashtablekey))
	return FALSE;
    if (!xdr_int(xdrs, &objp->hashtablesize))
	return FALSE;

    return TRUE;
}

bool_t
xdr_TupleBuffer(XDR *xdrs, TupleBuffer *objp)
{
/*  fprintf(stderr, "_TPBF"); */

    if (!xdr_Plan(xdrs, &objp->plan))
	return FALSE;

    return TRUE;
}

bool_t
xdr_Merge(XDR *xdrs, Merge *objp)
{
    NodeTag t_save1 = T_Plan;

/*  fprintf(stderr, "_MERG"); */

    if (xdrs->x_op == XDR_DECODE)
	objp->mergeplans = NULL;

    if (!xdr_Plan(xdrs, &objp->plan))
	return FALSE;
    if (!xdr_u_int(xdrs, (unsigned int *)&objp->tempid))
	return FALSE;
    if (!xdr_int(xdrs, &objp->keycount))
	return FALSE;

    if ((xdrs->x_op == XDR_ENCODE || xdrs->x_op == XDR_FREE) &&
	objp->mergeplans != NULL)
	t_save1 = nodeTag((Node *)objp->mergeplans);
    if (!xdr_NodeTag(xdrs, &t_save1))
	return FALSE;
    if (!xdr_pointer(xdrs, (char **)&objp->mergeplans, get_xdr_size(t_save1),
		     get_xdr_func(t_save1)))
	return FALSE;

    return TRUE;
}

bool_t
xdr_Resdom(XDR *xdrs, Resdom *objp)
{
/*  fprintf(stderr, "_RSDM"); */

    if (xdrs->x_op == XDR_DECODE)
	objp->resname = NULL;

    if (!xdr_NodeTag(xdrs, &objp->type))
	return FALSE;
    if (!xdr_short(xdrs, (short *)&objp->resno))
	return FALSE;
    if (!xdr_u_int(xdrs, (unsigned int *)&objp->restype))
	return FALSE;
    if (!xdr_int(xdrs, &objp->reslen))
	return FALSE;
    if (!xdr_pointer(xdrs, (char **)&objp->resname, sizeof(struct char16),
		     xdr_NameData))
	return FALSE;
    if (!xdr_u_int(xdrs, (unsigned int *)&objp->reskey))
	return FALSE;
    if (!xdr_u_int(xdrs, (long int *)&objp->reskeyop))
	return FALSE;
    if (!xdr_int(xdrs, &objp->resjunk))
	return FALSE;

    return TRUE;
}

bool_t
xdr_Fjoin(XDR *xdrs, Fjoin *objp)
{
    NodeTag t_save1 = T_Plan;

/*  fprintf(stderr, "_FJOI"); */

    if (xdrs->x_op == XDR_DECODE) {
	objp->fj_innerNode = NULL;
	objp->fj_alwaysDone = NULL;
    }

    if (!xdr_NodeTag(xdrs, &objp->type))
	return FALSE;
    if (!xdr_char(xdrs, &objp->fj_initialized))
	return FALSE;
    if (!xdr_int(xdrs, &objp->fj_nNodes))
	return FALSE;

    if ((xdrs->x_op == XDR_ENCODE || xdrs->x_op == XDR_FREE) &&
	objp->fj_innerNode != NULL)
	t_save1 = nodeTag((Node *)objp->fj_innerNode);
    if (!xdr_NodeTag(xdrs, &t_save1))
	return FALSE;
    if (!xdr_pointer(xdrs, (char **)&objp->fj_innerNode, get_xdr_size(t_save1),
		     get_xdr_func(t_save1)))
	return FALSE;

    /* Skipping DatumPtr objp->fj_results, because outfuncs.c prints out
       the address! */
    if (!xdr_bytes(xdrs, (char **)&objp->fj_alwaysDone, &objp->fj_nNodes,
		   objp->fj_nNodes))
	return FALSE;

    return TRUE;
}

bool_t
xdr_OpType(XDR *xdrs, OpType *objp)
{
/*  fprintf(stderr, "_OPTP"); */

    if (!xdr_enum(xdrs, (enum_t *)objp))
	return FALSE;

    return TRUE;
}

bool_t
xdr_Expr(XDR *xdrs, Expr *objp)
{
    NodeTag t_save1 = T_Plan, t_save2 = T_Plan;

/*  fprintf(stderr, "_EXPR"); */

    if (xdrs->x_op == XDR_DECODE) {
	objp->oper = NULL;
	objp->args = NULL;
    }

    if (!xdr_NodeTag(xdrs, &objp->type))
	return FALSE;
    if (!xdr_u_int(xdrs, (unsigned int *)&objp->typeOid))
	return FALSE;
    if (!xdr_OpType(xdrs, &objp->opType))
	return FALSE;

    if ((xdrs->x_op == XDR_ENCODE || xdrs->x_op == XDR_FREE) &&
	objp->oper != NULL)
	t_save1 = nodeTag((Node *)objp->oper);
    if (!xdr_NodeTag(xdrs, &t_save1))
	return FALSE;
    if (!xdr_pointer(xdrs, (char **)&objp->oper, get_xdr_size(t_save1),
		     get_xdr_func(t_save1)))
	return FALSE;

    if ((xdrs->x_op == XDR_ENCODE || xdrs->x_op == XDR_FREE) &&
	objp->args != NULL)
	t_save2 = nodeTag((Node *)objp->args);
    if (!xdr_NodeTag(xdrs, &t_save2))
	return FALSE;
    if (!xdr_pointer(xdrs, (char **)&objp->args, get_xdr_size(t_save2),
		     get_xdr_func(t_save2)))
	return FALSE;

    return TRUE;
}

bool_t
xdr_Var(XDR *xdrs, Var *objp)
{
/*  fprintf(stderr, "__VAR"); */

    if (!xdr_NodeTag(xdrs, &objp->type))
	return FALSE;
    if (!xdr_u_int(xdrs, (unsigned int *)&objp->varno))
	return FALSE;
    if (!xdr_short(xdrs, (short *)&objp->varattno))
	return FALSE;
    if (!xdr_u_int(xdrs, (unsigned int *)&objp->vartype))
	return FALSE;
    if (!xdr_u_int(xdrs, (unsigned int *)&objp->varnoold))
	return FALSE;
    if (!xdr_short(xdrs, (short *)&objp->varoattno))
	return FALSE;

    return TRUE;
}

bool_t
xdr_Oper(XDR *xdrs, Oper *objp)
{
/*  fprintf(stderr, "_OPER"); */

    if (!xdr_NodeTag(xdrs, &objp->type))
	return FALSE;
    if (!xdr_u_int(xdrs, (unsigned int *)&objp->opno))
	return FALSE;
    if (!xdr_u_int(xdrs, (unsigned int *)&objp->opid))
	return FALSE;
    if (!xdr_u_int(xdrs, (unsigned int *)&objp->opresulttype))
	return FALSE;
    if (!xdr_int(xdrs, &objp->opsize))
	return FALSE;

    return TRUE;
}

bool_t
xdr_Const(XDR *xdrs, Const *objp)
{
    bool pointerValid;
    Size length;

/*  fprintf(stderr, "_CNST"); */

    if (!xdr_NodeTag(xdrs, &objp->type))
	return FALSE;
    if (!xdr_u_int(xdrs, (unsigned int *)&objp->consttype))
	return FALSE;
    if (!xdr_u_int(xdrs, (unsigned int *)&objp->constlen))
	return FALSE;
    if (!xdr_char(xdrs, &objp->constisnull))
	return FALSE;
    if (!xdr_char(xdrs, &objp->constbyval))
	return FALSE;
    if (!xdr_char(xdrs, &objp->constisset))
	return FALSE;

    if (xdrs->x_op == XDR_ENCODE || xdrs->x_op == XDR_FREE) {
	length = objp->constlen;
	if (((int)length <= -1))
	    length = VARSIZE((struct varlena *) objp->constvalue);
    }
    if (!xdr_u_int(xdrs, (unsigned int *)&length))
	return FALSE;

    if (objp->constbyval) {
	if (!xdr_u_long(xdrs, (unsigned long *)&objp->constvalue))
	    return FALSE;
    } else {
	if (xdrs->x_op == XDR_DECODE)
	    objp->constvalue = NULL;

	if (xdrs->x_op == XDR_ENCODE || xdrs->x_op == XDR_FREE)
	    pointerValid = PointerIsValid((char *)objp->constvalue);
	if (!xdr_char(xdrs, &pointerValid))
	    return FALSE;

	if (pointerValid) {
	    if (!xdr_bytes(xdrs, (char **)&objp->constvalue, &length, length))
		return FALSE;
	}
    }

    return TRUE;
}

bool_t
xdr_Param(XDR *xdrs, Param *objp)
{
    NodeTag t_save1 = T_Plan;

/*  fprintf(stderr, "_PARM"); */

    if (xdrs->x_op == XDR_DECODE) {
	objp->paramname = NULL;
	objp->param_tlist = NULL;
    }

    if (!xdr_NodeTag(xdrs, &objp->type))
	return FALSE;
    if (!xdr_int(xdrs, &objp->paramkind))
	return FALSE;
    if (!xdr_short(xdrs, (short *)&objp->paramid))
	return FALSE;
    if (!xdr_pointer(xdrs, (char **)&objp->paramname, sizeof(struct char16),
		     xdr_NameData))
	return FALSE;
    if (!xdr_u_int(xdrs, (unsigned int *)&objp->paramtype))
	return FALSE;

    if ((xdrs->x_op == XDR_ENCODE || xdrs->x_op == XDR_FREE) &&
	objp->param_tlist != NULL)
	t_save1 = nodeTag((Node *)objp->param_tlist);
    if (!xdr_NodeTag(xdrs, &t_save1))
	return FALSE;
    if (!xdr_pointer(xdrs, (char **)&objp->param_tlist, get_xdr_size(t_save1),
		     get_xdr_func(t_save1)))
	return FALSE;

    return TRUE;
}

bool_t
xdr_Func(XDR *xdrs, Func *objp)
{
    NodeTag t_save1 = T_Plan, t_save2 = T_Plan;

/*  fprintf(stderr, "_FUNC"); */

    if (xdrs->x_op == XDR_DECODE) {
	objp->func_tlist = NULL;
	objp->func_planlist = NULL;
    }

    if (!xdr_NodeTag(xdrs, &objp->type))
	return FALSE;
    if (!xdr_u_int(xdrs, (unsigned int *)&objp->funcid))
	return FALSE;
    if (!xdr_u_int(xdrs, (unsigned int *)&objp->functype))
	return FALSE;
    if (!xdr_char(xdrs, &objp->funcisindex))
	return FALSE;
    if (!xdr_int(xdrs, &objp->funcsize))
	return FALSE;
    /* I am skipping func_fcache, because outfuncs.c simply prints out a
       pointer value */

    if ((xdrs->x_op == XDR_ENCODE || xdrs->x_op == XDR_FREE) &&
	objp->func_tlist != NULL)
	t_save1 = nodeTag((Node *)objp->func_tlist);
    if (!xdr_NodeTag(xdrs, &t_save1))
	return FALSE;
    if (!xdr_pointer(xdrs, (char **)&objp->func_tlist, get_xdr_size(t_save1),
		     get_xdr_func(t_save1)))
	return FALSE;

    if ((xdrs->x_op == XDR_ENCODE || xdrs->x_op == XDR_FREE) &&
	objp->func_planlist != NULL)
	t_save2 = nodeTag((Node *)objp->func_planlist);
    if (!xdr_NodeTag(xdrs, &t_save2))
	return FALSE;
    if (!xdr_pointer(xdrs, (char **)&objp->func_planlist,
		     get_xdr_size(t_save2), get_xdr_func(t_save2)))
	return FALSE;

    return TRUE;
}

static
bool_t
xdr_AggregPtr(XDR *xdrs, Aggreg **objp)
{
    if (xdrs->x_op == XDR_DECODE) {
	*objp = NULL;
    }

    if (!xdr_pointer(xdrs, objp, sizeof(Aggreg), xdr_Aggreg))
	return(FALSE);
    return(TRUE);
}

bool_t
xdr_Aggreg(XDR *xdrs, Aggreg *objp)
{
    NodeTag t_save1 = T_Plan;

/*  fprintf(stderr, "_AGGR"); */

    if (xdrs->x_op == XDR_DECODE) {
	objp->aggname = NULL;
	objp->target = NULL;
    }

    if (!xdr_NodeTag(xdrs, &objp->type))
	return FALSE;
    if (!xdr_pointer(xdrs, (char **)&objp->aggname, sizeof(struct char16),
		     xdr_NameData))
	return FALSE;
    if (!xdr_u_int(xdrs, (unsigned int *)&objp->aggtype))
	return FALSE;
    if ((xdrs->x_op == XDR_ENCODE || xdrs->x_op == XDR_FREE) &&
	objp->target != NULL)
	t_save1 = nodeTag((Node *)objp->target);
    if (!xdr_NodeTag(xdrs, &t_save1))
	return FALSE;
    if (!xdr_pointer(xdrs, (char **)&objp->target, get_xdr_size(t_save1),
		     get_xdr_func(t_save1)))
	return FALSE;
    if (!xdr_int(xdrs, &objp->aggno))
	return FALSE;

    return TRUE;
}

bool_t
xdr_Array(XDR *xdrs, Array *objp)
{
/*  fprintf(stderr, "_ARRY"); */

    if (!xdr_NodeTag(xdrs, &objp->type))
	return FALSE;
    if (!xdr_u_int(xdrs, (unsigned int *)&objp->arrayelemtype))
	return FALSE;
    if (!xdr_int(xdrs, &objp->arrayelemlength))
	return FALSE;
    if (!xdr_char(xdrs, &objp->arrayelembyval))
	return FALSE;
    if (!xdr_int(xdrs, &objp->arrayndim))
	return FALSE;
    if (!xdr_array(xdrs, objp->arraylow.indx, &objp->arrayndim,
		   objp->arrayndim, sizeof(int), xdr_int))
	return FALSE;
    if (!xdr_array(xdrs, objp->arrayhigh.indx, &objp->arrayndim,
		   objp->arrayndim, sizeof(int), xdr_int))
	return FALSE;
    if (!xdr_int(xdrs, &objp->arraylen))
	return FALSE;

    return (TRUE);
}

bool_t
xdr_ArrayRef(XDR *xdrs, ArrayRef *objp)
{
    NodeTag t_save1 = T_Plan, t_save2 = T_Plan, t_save3 = T_Plan,
	    t_save4 = T_Plan;

/*  fprintf(stderr, "_ARRF"); */

    if (xdrs->x_op == XDR_DECODE) {
	objp->refupperindexpr = NULL;
	objp->reflowerindexpr = NULL;
	objp->refexpr = NULL;
	objp->refassgnexpr = NULL;
    }

    if (!xdr_NodeTag(xdrs, &objp->type))
	return FALSE;
    if (!xdr_int(xdrs, &objp->refattrlength))
	return FALSE;
    if (!xdr_int(xdrs, &objp->refelemlength))
	return FALSE;
    if (!xdr_u_int(xdrs, (unsigned int *)&objp->refelemtype))
	return FALSE;
    if (!xdr_char(xdrs, &objp->refelembyval))
	return FALSE;

    if ((xdrs->x_op == XDR_ENCODE || xdrs->x_op == XDR_FREE) &&
	objp->refupperindexpr != NULL)
	t_save1 = nodeTag((Node *)objp->refupperindexpr);
    if (!xdr_NodeTag(xdrs, &t_save1))
	return FALSE;
    if (!xdr_pointer(xdrs, (char **)&objp->refupperindexpr,
		     get_xdr_size(t_save1), get_xdr_func(t_save1)))
	return FALSE;

    if ((xdrs->x_op == XDR_ENCODE || xdrs->x_op == XDR_FREE) &&
	objp->reflowerindexpr != NULL)
	t_save2 = nodeTag((Node *)objp->reflowerindexpr);
    if (!xdr_NodeTag(xdrs, &t_save2))
	return FALSE;
    if (!xdr_pointer(xdrs, (char **)&objp->reflowerindexpr,
		     get_xdr_size(t_save2), get_xdr_func(t_save2)))
	return FALSE;

    if ((xdrs->x_op == XDR_ENCODE || xdrs->x_op == XDR_FREE) &&
	objp->refexpr != NULL)
	t_save3 = nodeTag((Node *)objp->refexpr);
    if (!xdr_NodeTag(xdrs, &t_save3))
	return FALSE;
    if (!xdr_pointer(xdrs, (char **)&objp->refexpr, get_xdr_size(t_save3),
		     get_xdr_func(t_save3)))
	return FALSE;

    if ((xdrs->x_op == XDR_ENCODE || xdrs->x_op == XDR_FREE) &&
	objp->refassgnexpr != NULL)
	t_save4 = nodeTag((Node *)objp->refassgnexpr);
    if (!xdr_NodeTag(xdrs, &t_save4))
	return FALSE;
    if (!xdr_pointer(xdrs, (char **)&objp->refassgnexpr, get_xdr_size(t_save4),
		     get_xdr_func(t_save4)))
	return FALSE;

    return TRUE;
}

bool_t
xdr_Query(XDR *xdrs, Query *objp)
{
    NodeTag t_save1 = T_Plan, t_save2 = T_Plan, t_save3 = T_Plan,
	    t_save4 = T_Plan;

/*  fprintf(stderr, "_QUER"); */

    if (xdrs->x_op == XDR_DECODE) {
	objp->into = NULL;
	objp->sortClause = NULL;
	objp->rtable = NULL;
	objp->targetList = NULL;
	objp->qual = NULL;
    }

    if (!xdr_NodeTag(xdrs, &objp->type))
	return FALSE;
    if (!xdr_CmdType(xdrs, &objp->commandType))
	return FALSE;
    if (!xdr_int(xdrs, &objp->resultRelation))
	return FALSE;
    if (!xdr_int(xdrs, &objp->resultFragment))
	return FALSE;
    if (!xdr_pointer(xdrs, (char **)&objp->into, sizeof(char *), xdr_Str))
	return FALSE;
    if (!xdr_char(xdrs, &objp->isPortal))
	return FALSE;
    if (!xdr_char(xdrs, &objp->isBinary))
	return FALSE;
    if (!xdr_pointer(xdrs, (char **)&objp->uniqueFlag, sizeof(char *),
		     xdr_Str))
	return FALSE;
    
    if ((xdrs->x_op == XDR_ENCODE || xdrs->x_op == XDR_FREE) &&
	objp->sortClause != NULL)
	t_save1 = nodeTag((Node *)objp->sortClause);
    if (!xdr_NodeTag(xdrs, &t_save1))
	return FALSE;
    if (!xdr_pointer(xdrs, (char **)&objp->sortClause, get_xdr_size(t_save1),
		     get_xdr_func(t_save1)))
	return FALSE;

    if ((xdrs->x_op == XDR_ENCODE || xdrs->x_op == XDR_FREE) &&
	objp->rtable != NULL)
	t_save2 = nodeTag((Node *)objp->rtable);
    if (!xdr_NodeTag(xdrs, &t_save2))
	return FALSE;
    if (!xdr_pointer(xdrs, (char **)&objp->rtable, get_xdr_size(t_save2),
		     get_xdr_func(t_save2)))
	return FALSE;

    if ((xdrs->x_op == XDR_ENCODE || xdrs->x_op == XDR_FREE) &&
	objp->targetList != NULL)
	t_save3 = nodeTag((Node *)objp->targetList);
    if (!xdr_NodeTag(xdrs, &t_save3))
	return FALSE;
    if (!xdr_pointer(xdrs, (char **)&objp->targetList, get_xdr_size(t_save3),
		     get_xdr_func(t_save3)))
	return FALSE;

    if ((xdrs->x_op == XDR_ENCODE || xdrs->x_op == XDR_FREE) &&
	objp->qual != NULL)
	t_save4 = nodeTag((Node *)objp->qual);
    if (!xdr_NodeTag(xdrs, &t_save4))
	return FALSE;
    if (!xdr_pointer(xdrs, (char **)&objp->qual, get_xdr_size(t_save4),
		     get_xdr_func(t_save4)))
	return FALSE;

    return TRUE;
}

bool_t
xdr_TargetEntry(XDR *xdrs, TargetEntry *objp)
{
    NodeTag t_save1 = T_Plan, t_save2 = T_Plan;

/*  fprintf(stderr, "_TRGN"); */

    if (xdrs->x_op == XDR_DECODE) {
	objp->resdom = NULL;
	objp->expr = NULL;
    }

    if (!xdr_NodeTag(xdrs, &objp->type))
	return FALSE;

    if ((xdrs->x_op == XDR_ENCODE || xdrs->x_op == XDR_FREE) &&
	objp->resdom != NULL)
	t_save1 = nodeTag((Node *)objp->resdom);
    if (!xdr_NodeTag(xdrs, &t_save1))
	return FALSE;
    if (!xdr_pointer(xdrs, (char **)&objp->resdom, get_xdr_size(t_save1),
		     get_xdr_func(t_save1)))
	return FALSE;

    if ((xdrs->x_op == XDR_ENCODE || xdrs->x_op == XDR_FREE) &&
	objp->expr != NULL)
	t_save2 = nodeTag((Node *)objp->expr);
    if (!xdr_NodeTag(xdrs, &t_save2))
	return FALSE;
    if (!xdr_pointer(xdrs, (char **)&objp->expr, get_xdr_size(t_save2),
		     get_xdr_func(t_save2)))
	return FALSE;

    return TRUE;
}

bool_t
xdr_FragInfo(XDR *xdrs, FragInfo *objp) {
    
    if (!xdr_NodeTag(xdrs, &objp->type))
	return FALSE;
    
    if (!xdr_u_int(xdrs, (unsigned int *)&objp->fralogicalid))
	return FALSE;

    if (!xdr_u_int(xdrs, (unsigned int *)&objp->frastoreid))
	return FALSE;
    
    if (!xdr_Node_list(xdrs, &objp->storageSites))
	return FALSE;

    if (!xdr_bool_list(xdrs, &objp->processYN))
	return FALSE;

    if (!xdr_int(xdrs, &objp->frapages))
	return FALSE;

    if (!xdr_int(xdrs, &objp->fratuples))
	return FALSE;
    if (!xdr_char(xdrs, &objp->usedLocally))
	return FALSE;
    
    return TRUE;
    
}

bool_t
xdr_RangeTblEntry(XDR *xdrs, RangeTblEntry *objp)
{
/*  fprintf(stderr, "__RTE"); */

    if (xdrs->x_op == XDR_DECODE) {
	objp->relname = NULL;
	objp->timeRange = NULL;
	objp->refname = NULL;
	objp->timeQual = NULL;
    }

    if (!xdr_NodeTag(xdrs, &objp->type))
	return FALSE;
    if (!xdr_pointer(xdrs, (char **)&objp->relname, sizeof(struct char16),
		     xdr_NameData))
	return FALSE;
    if (!xdr_pointer(xdrs, (char **)&objp->timeRange,
		     sizeof(struct TimeRange), xdr_TimeRange))
	return FALSE;
    if (!xdr_pointer(xdrs, (char **)&objp->refname, sizeof(char *), xdr_Str))
	return FALSE;
    if (!xdr_u_int(xdrs, (unsigned int *)&objp->relid))
	return FALSE;
    if (!xdr_char(xdrs, &objp->inh))
	return FALSE;
    if (!xdr_char(xdrs, &objp->archive))
	return FALSE;
    if (!xdr_char(xdrs, &objp->inFromCl))
	return FALSE;

    if (xdrs->x_op == XDR_DECODE && objp->timeRange != NULL)
	objp->timeQual = make_xdr_tr(objp->timeRange->startDate,
				     objp->timeRange->endDate,
				     (objp->timeRange->endDate == NULL) ? 0:1);

    if (!xdr_Node_list(xdrs, &objp->fragInfo))
	return FALSE;

    if (!xdr_char(xdrs, &objp->partitionMode))
	return FALSE;
    
    return TRUE;
}

bool_t
xdr_SortClause(XDR *xdrs, SortClause *objp)
{
/*  fprintf(stderr, "_SRCL"); */

    if (xdrs->x_op == XDR_DECODE)
	objp->resdom = NULL;

    if (!xdr_NodeTag(xdrs, &objp->type))
	return FALSE;
    if (!xdr_pointer(xdrs, (char **)&objp->resdom, sizeof(struct Resdom),
		     xdr_Resdom))
	return FALSE;
    if (!xdr_u_int(xdrs, (unsigned int *)&objp->opoid))
	return FALSE;

    return TRUE;
}

static
bool_t
xdr_Iter(XDR *xdrs, struct Iter *objp)
{
    NodeTag t_save1 = T_Plan;

/*  fprintf(stderr, "_ITER"); */

    if (xdrs->x_op == XDR_DECODE)
	objp->iterexpr = NULL;
    if (!xdr_NodeTag(xdrs, &objp->type))
	return FALSE;
    if ((xdrs->x_op == XDR_ENCODE || xdrs->x_op == XDR_FREE) &&
	objp->iterexpr != NULL)
	t_save1 = nodeTag(objp->iterexpr);
    if (!xdr_NodeTag(xdrs, &t_save1))
	return FALSE;
    if (!xdr_pointer(xdrs, (char **)&objp->iterexpr, get_xdr_size(t_save1),
		     get_xdr_func(t_save1)))
	return FALSE;
    if (!xdr_u_int(xdrs, (unsigned int *)&objp->itertype))
	return FALSE;

    return TRUE;
}
