/*-------------------------------------------------------------------------
 *
 * xdrfuncs.h--
 *    XDR data conversion routines for types in nodes/plannodes.h,
 *    nodes/primnodes.h, nodes/parsenodes.h. Includes extra routines to
 *    help out with the conversions.
 *
 *
 * Copyright (c) 1994, Regents of the University of California
 *
 * $Id: xdrfuncs.h,v 1.5 1996/02/19 22:50:51 aoki Exp $
 *
 *-------------------------------------------------------------------------
 */
#ifndef XDRFUNCS_H
#define XDRFUNCS_H

typedef bool_t (*PFB)();

extern size_t get_xdr_size(NodeTag t);
extern PFB get_xdr_func(NodeTag t);

extern bool_t xdr_PQA_Plan_tree(XDR *xdrs, PQA_Plan_tree *objp);
extern bool_t xdr_PQA_Int_list(XDR *xdrs, PQA_Int_list *objp);

extern bool_t xdr_Str(XDR *xdrs, char *objp);

extern bool_t xdr_Int_node(XDR *xdrs, List *objp);
extern bool_t xdr_Int_list(XDR *xdrs, List **objp);
extern bool_t xdr_Long_node(XDR *xdrs, List *objp);
extern bool_t xdr_Long_list(XDR *xdrs, List **objp);
extern bool_t xdr_Node_node(XDR *xdrs, List *objp);
extern bool_t xdr_Node_list(XDR *xdrs, List **objp);

extern bool_t xdr_NodeTag(XDR *xdrs, NodeTag *objp);
extern bool_t xdr_CmdType(XDR *xdrs, CmdType *objp);

extern bool_t xdr_NameData(XDR *xdrs, NameData *objp);
extern bool_t xdr_TimeRange(XDR *xdrs, TimeRange *objp);

/* From nodes/plannodes.h */
extern bool_t xdr_Plan(XDR *xdrs, Plan *objp);
extern bool_t xdr_Plan_tree(XDR *xdrs, Node **objp);
extern bool_t xdr_Existential(XDR *xdrs, Existential *objp);
extern bool_t xdr_Result(XDR *xdrs, Result *objp);
extern bool_t xdr_Append(XDR *xdrs, Append *objp);
extern bool_t xdr_Scan(XDR *xdrs, Scan *objp);
extern bool_t xdr_SeqScan(XDR *xdrs, SeqScan *objp);
extern bool_t xdr_ExchangeIn(XDR *xdrs, ExchangeIn *objp);
extern bool_t xdr_IndexScan(XDR *xdrs, IndexScan *objp);
extern bool_t xdr_Join(XDR *xdrs, Join *objp);
extern bool_t xdr_NestLoop(XDR *xdrs, NestLoop *objp);
extern bool_t xdr_MergeJoin(XDR *xdrs, MergeJoin *objp);
extern bool_t xdr_HashJoin(XDR *xdrs, HashJoin *objp);
extern bool_t xdr_Temp(XDR *xdrs, Temp *objp);
extern bool_t xdr_Material(XDR *xdrs, Material *objp);
extern bool_t xdr_Sort(XDR *xdrs, Sort *objp);
extern bool_t xdr_Agg(XDR *xdrs, Agg *objp);
extern bool_t xdr_Group(XDR *xdrs, Group *objp);
extern bool_t xdr_Unique(XDR *xdrs, Unique *objp);
extern bool_t xdr_Choose(XDR *xdrs, Choose *objp);
extern bool_t xdr_Hash(XDR *xdrs, Hash *objp);
extern bool_t xdr_TupleBuffer(XDR *xdrs, TupleBuffer *objp);
extern bool_t xdr_Merge(XDR *xdrs, Merge *objp);

/* From nodes/primnodes.h */
extern bool_t xdr_Resdom(XDR *xdrs, Resdom *objp);
extern bool_t xdr_Fjoin(XDR *xdrs, Fjoin *objp);
extern bool_t xdr_OpType(XDR *xdrs, OpType *objp);
extern bool_t xdr_Expr(XDR *xdrs, Expr *objp);
extern bool_t xdr_Var(XDR *xdrs, Var *objp);
extern bool_t xdr_Oper(XDR *xdrs, Oper *objp);
extern bool_t xdr_Const(XDR *xdrs, Const *objp);
extern bool_t xdr_Param(XDR *xdrs, Param *objp);
extern bool_t xdr_Func(XDR *xdrs, Func *objp);
extern bool_t xdr_Aggreg(XDR *xdrs, Aggreg *objp);
extern bool_t xdr_Array(XDR *xdrs, Array *objp);
extern bool_t xdr_ArrayRef(XDR *xdrs, ArrayRef *objp);

/* From nodes/parsenodes.h */
extern bool_t xdr_Query(XDR *xdrs, Query *objp);
extern bool_t xdr_TargetEntry(XDR *xdrs, TargetEntry *objp);
extern bool_t xdr_FragInfo(XDR *xdrs, FragInfo *objp);
extern bool_t xdr_RangeTblEntry(XDR *xdrs, RangeTblEntry *objp);
extern bool_t xdr_SiteAddrNode(XDR *xdrs, SiteAddrNode *objp);
extern bool_t xdr_SortClause(XDR *xdrs, SortClause *objp);

#endif
