/*-------------------------------------------------------------------------
 *
 * archive.c--
 *    Support for planning scans on archived relations
 *
 * Copyright (c) 1994, Regents of the University of California
 *
 *
 * IDENTIFICATION
 *    $Header: /usr/local/devel/mariposa/cvs/src/backend/optimizer/prep/archive.c,v 1.6 1996/06/07 04:57:30 am Exp $
 *
 *-------------------------------------------------------------------------
 */
#include <stdio.h>		/* for sprintf() */
#include <sys/types.h>		/* for u_int in relcache.h */
#include "postgres.h"

#include "utils/rel.h"
#include "utils/elog.h"
#include "utils/palloc.h"
#include "utils/relcache.h"
#include "catalog/pg_class.h"
#include "nodes/pg_list.h"
#include "nodes/parsenodes.h"
#include "optimizer/prep.h"
#ifdef P95
#include "commands/creatinh.h"
#endif /* P95 */

#ifndef P95
/*
 * plan_archive -- Traverse range table 'rt', setting the 'archive'
 *		   field of each range table entry if a time range
 *		   has been specified and the underlying table is
 *		   archived.
 *
 * It is of course possible to have time ranges specified on an
 * unarchived table; however, the intent here is to identify cases
 * where we must use two scans (i.e., the normal table and the
 * archival table) instead of one.
 */
#endif /* P95 */
void
plan_archive(List *rt)
{
    List *rtitem;
    RangeTblEntry *rte;
    TimeRange *trange;
#ifndef P95
    Oid reloid;
#endif /* P95 */
    Relation r;
#ifdef P95
    Oid reloid;
#endif /* P95 */

    foreach(rtitem, rt) {
	rte = lfirst(rtitem);
	trange = rte->timeRange;
#ifndef P95

	if (trange != (TimeRange *) NULL &&
	    trange->startDate != (char *) NULL) {
#else /* P95 */
	if (trange) {
#endif /* P95 */
	    reloid = rte->relid;
	    r = RelationIdGetRelation(reloid);
	    if (r->rd_rel->relarch != 'n') {
		rte->archive = true;
	    }
	}
    }
}


/*
 *  find_archive_rels -- Given a particular relid, find the archive
 *			 relation's relid.
 */
List *
find_archive_rels(Oid relid)
{
    Relation arel;
#ifdef P95
    char *arelName;
#endif /* P95 */

#ifndef P95
    arel = (Relation) OpenArchiveRelation(relid);
#else /* P95 */
    arelName = MakeArchiveName(relid);
    arel = RelationNameGetRelation(arelName);
    pfree(arelName);
#endif /* P95 */

#ifndef P95
    if (!RelationIsValid(arel)) {
	elog(WARN, "find_archive_rels: Couldn't open archive relation for relation %d",
	     relid);
    }

    return lconsi(arel->rd_id,
		  lconsi(relid, NIL));
#else /* P95 */
    return lconsi(arel->rd_id, lconsi(relid, NIL));
#endif /* P95 */
}
