/*-------------------------------------------------------------------------
 *
 * postmaster_stubs.c--
 *    stub routines to account for those in the backend but not linked in
 *    with the postmaster.
 *
 * Copyright (c) 1994-5, Regents of the University of California
 *
 *
 * IDENTIFICATION
 *    $Header: /usr/local/devel/mariposa/cvs/src/backend/postmaster/postmaster_stubs.c,v 1.3 1995/04/24 18:58:38 andrew Exp $
 *
 *-------------------------------------------------------------------------
 */
#include <stdio.h>
#include "postgres.h"

/*
 * elog_warn_exit
 * elog_fatal_exit-
 *    those in the backend (utils/error/elog_exit.c) do more sophisticated
 *    things like long jumps and locks cleanup.
 */
void
elog_warn_exit()
{
    exitpg(0);
}

void
elog_fatal_exit()
{
    exitpg(0);
}

void
die()
{
    exitpg(0);
}

/*
 * we don't do memory management as in the backend.
 */
void *
palloc(int size)
{
    return malloc(size);
}

void *
pfree(void *pointer)
{
    free(pointer);
}

void *
repalloc(void *pointer, int size)
{
    return realloc(pointer, size);
}

/*
 * only works when doesn't need access to pg_host.
 */
int
getAddrFromHostId(int hostid)
{
    if (hostid == getMyHostId()) {
	return getMyAddr();
    }

    fprintf(stderr, "Cannot call getAddrFromHostId inside sitemgr.");
    exitpg(0);
}
