/*-------------------------------------------------------------------------
 *
 * bid.x--
 *    data types for bidding
 *
 * Copyright (c) 1994, Regents of the University of California
 *
 * $Id: bid.x,v 1.7 1996/02/26 16:21:44 jsidell Exp $
 *
 *-------------------------------------------------------------------------
 */
#ifdef RPC_HDR
%#ifndef BID_H
%#include "postgres.h"
%#include "rpctypes.h"
%#include "sitemgr/execs.h"
%#include "queries.h"
%#include "datafrag.h"
%#define BID_H
#endif


enum BidProtocol {
    SHORT_PROCESS_QUERY,
    LONG_PROCESS_QUERY,
    SHORT_BUY_FRAGMENT,
    LONG_BUY_FRAGMENT,
    SHORT_SELL_FRAGMENT,
    LONG_SELL_FRAGMENT,
    SHORT_COPY_FRAGMENT
};

enum BidderResponse {
    REFUSETOBID,
    BID
};

struct Budget {
    BidderResponse response;
    float cost;
    float delay;
    float staleness;
    float accuracy;
};

union OfferDetails switch (BidProtocol protocol) {
    case SHORT_PROCESS_QUERY:
	ProcessQueryArg shortQuery;
    case LONG_PROCESS_QUERY:
	ProcessQueryArg longQuery;
    case SHORT_BUY_FRAGMENT:
	FragXArg shortBuy;
    case LONG_BUY_FRAGMENT:
	FragXArg longBuy;
    case SHORT_SELL_FRAGMENT:
	FragXArg shortSell;
    case LONG_SELL_FRAGMENT:
	FragXArg longSell;
    case SHORT_COPY_FRAGMENT:
	FragXArg shortCopy;
    default:
        void;
};


struct OfferBuffer {
    int requestIndex;
    unsigned int requestId;
    int whichChunk;
    int whichBid;
    bool isSubContract;
    SiteAddr requester;
    ContractId contract;
    BidProtocol protocol;
    Budget budget;
    OfferDetails details;
};

struct WinBuffer {
    SiteAddr recvSite;
    ContractId contract;
};

struct BidBuffer {
    int requestIndex;     	/* Index into requests[] */
    unsigned int requestId;    	/* Unique identifier of bid request */
    int whichChunk;		/* Which plan chunk this bid was for */
    int whichBid;		/* Which bid this was of all bids requested for the plan chunk. */
    ContractId contract;  	/* Unique contract identifier generated by sitemgr */
    QepFragmentId qepFragId;
    Budget bid;
    bool subContracted;		/* True if this bid was subcontracted out and we want the broker to know. */
    PQA_Plan_tree subContractedPlan;   /* If subContracted is true, the plan with the real processing sites. */
};

struct BackendBuffer {
    int pid;
    SiteAddr udpPort;
    SiteAddr tcpPort;
};

#ifdef RPC_HDR
%#endif	/* BID_H */
#endif
