/*-------------------------------------------------------------------------
 *
 * copies.x--
 *    Data types for copy contracting.
 *
 * Copyright (c) 1995, Regents of the University of California
 *
 * $Id: copies.x,v 1.6 1996/05/31 17:14:10 jsidell Exp $
 *
 *-------------------------------------------------------------------------
 */
#ifdef RPC_HDR
%#ifndef COPIES_H
%#define COPIES_H
%#include "postgres.h"
%#include "rpctypes.h"
%#include "exchange.h"		/* for QepFragmentId */
%
#endif

/*
 * During copy creation, these structs are passed around containing info
 * for each site, concerning them on the new copy so they act accordingly
 * (e.g. set up the right contracts, create a side-file.)
 */
struct CopySetupInfo {
    int contractid;
    bool ambuyer;
    bool isreadonly;
    bool isnamesvc; 
    bool isnsspecific;
    unsigned int hereloc;
    unsigned int thereloc;
    int ourcopystoreid;
    int theircopystoreid;
    int period;
};

/*
 * Since we need to pass update contracts around via RPC, we have this struct
 * stuck here to generate xdr routines for Form_pg_contract's. It's the same
 * as Form_pg_contract, so whenever Form_pg_contract changes you need to
 * change this, too.
 *
 * @@@ Note! typemask must always be initialized to 64-- that is, when creating
 *     new tuples to be inserted into pg_contract. Existing pg_contract tuples
 *     are ok, so if you're playing around with these you shouldn't have to
 *     worry about it. But, just so you know.
 */
struct UpdateContractInfo {
    int contractid;
    int sendfrastoreid;
    int recvfrastoreid;
    int sidefrastoreid;
    int lastupdcommit;
    unsigned int otherloc;
    int period;
    char typemask;
};

/*
 * Used to send tuples in the update stream.
 */
struct TransferDataBuffer {
    UpdateContractInfo transferInfo; /* information about transferred data */
    char transferStatus;	/* encodes transfer status information */
    int numTuples;		/* number of tuples passed */
    Data data;			/* the tuples */
    bool nsvcTransferFinished; /* true if this is the last update stream */
                               /* packet for nameservice catalogs */
};

%
%/*
% * Transfer status values.
% */
%#define INITIAL_PACKET		0
%#define MIDDLE_PACKET		1
%#define FINAL_PACKET		2
%#define ONLY_PACKET		3
%

struct SendUpdateStreamResult {
    int error;
    int committime;		/* when the incoming update xact committed */
};

#ifdef RPC_HDR
%
%/* Backend routines */
%extern void handleCall_ESTABLISHCOPY(void *arg);
%extern void handleCall_CP_SENDUPDATESTREAM(void *arg);
%extern void handleCall_UPDATETRANSFERSETUP(void *arg);
%extern void handleCall_CP_PUTUPDATE(void *arg);
%extern void handleCall_CP_INITIALIZEUPDATESTREAMS(void *arg);
%extern void handleCall_CANCELCONTRACT(void *arg);
%extern void CancelContracts(bool isnsspecific, bool isnsremote,
%			     unsigned int hostid, Oid relid);
%
%/* Sitemgr routines */
%extern void Cp_Toggle(void *arg);
%extern void Cp_ScheduleStream(void *arg);
%extern void Cp_UnscheduleStream(void *arg);
%extern void CallCancelContract(void *arg);
%
%#endif	/* COPIES_H */
#endif
