/*-------------------------------------------------------------------------
 *
 * copies_sm.c--
 *    copies related RPC calls in the sitemgr
 *
 * Copyright (c) 1995, Regents of the University of California
 *
 *
 * IDENTIFICATION
 *    $Header: /usr/local/devel/mariposa/cvs/src/backend/rpc/copies_sm.c,v 1.4 1996/03/04 09:07:12 aoki Exp $
 *
 *-------------------------------------------------------------------------
 */
#include <pthread.h>
#include <rpc/types.h>
#include "utils/elog.h"
#include "mariposa_rpc.h" /* for procedure constants */
#include "rpc/rpcutils.h"
#include "sitemgr/cpupdate.h"
#include "sitemgr/sitemgr.h"
#include "sitemgr/rpcerror.h"

/* ----------------------------------------------------------------
 *    	Cp_Toggle
 *
 *	Switches update system on or off.
 *	For debugging purposes only!!
 *
 * ----------------------------------------------------------------
 */
void
Cp_Toggle(void *args)
{
    char **argv = (char **)args;
    SVCXPRT *transp = (SVCXPRT *)argv[1];

    CopyUpdateToggle();

    if (!svc_sendreply(transp, xdr_void, NULL)) {
  	sitemgrDebug(0, "Cp_Toggle: Can't send reply.\n");
    }
}

/* ----------------------------------------------------------------
 *    	Cp_ScheduleStream
 *
 *	Inserts the contractid and time period into the update stream
 *	scheduling queue.
 *
 * ----------------------------------------------------------------
 */
void
Cp_ScheduleStream(void *args)
{
    char **argv = (char **)args;
    UpdateContractInfo *arg = (UpdateContractInfo *)argv[0];
    SVCXPRT *transp = (SVCXPRT *)argv[1];
    RPCErrorCode result;

    CopyUpdateScheduleStream(arg);

    result = SUCCESS;
    if (!svc_sendreply(transp, xdr_RPCErrorCode, &result)) {
	sitemgrDebug(0, "Cp_ScheduleStream: svc_sendreply failed\n");
    }
    else {
	sitemgrDebug(2,
		     "Cp_ScheduleStream: svc_sendreply succeeded, result = %d\n",
		     result);
    }
}

/* ----------------------------------------------------------------
 *    	Cp_UnscheduleStream
 *
 *	Deschedules stream with contractid 'arg' from update stream
 *	scheduling queue.
 *
 * ----------------------------------------------------------------
 */
void
Cp_UnscheduleStream(void *args)
{
    char **argv = (char **)args;
    int *arg = (int *)argv[0];
    SVCXPRT *transp = (SVCXPRT *)argv[1];
    RPCErrorCode result;

    CopyUpdateUnscheduleStream(*arg);

    result = SUCCESS;
    if (!svc_sendreply(transp, xdr_RPCErrorCode, &result)) {
	sitemgrDebug(0, "Cp_UnscheduleStream: svc_sendreply failed\n");
    }
    else {
	sitemgrDebug(2,
		     "Cp_UnscheduleStream: svc_sendreply succeeded, result = %d\n",
		     result);
    }
}

/* ----------------------------------------------------------------
 *    	CallCancelContract
 *
 *	Cancels contract with contractid 'arg'.
 *
 * ----------------------------------------------------------------
 */
void
CallCancelContract(void *args)
{
    char **argv = (char **)args;
    int *arg = (int *)argv[0];
    SVCXPRT *transp = (SVCXPRT *)argv[1];
    RPCErrorCode result = SUCCESS;

    /*
     * this cast is ok -- invokeExec returns different types depending
     * on the first argument, and in the case of CANCELCONTRACT, it
     * returns RPCErrorCode.
     */
    result = (RPCErrorCode)
	invokeExec(CANCELCONTRACT, (void *)arg, -1, "CallCancelContract");

    if (!svc_sendreply(transp, xdr_RPCErrorCode, &result)) {
	sitemgrDebug(0, "CallCancelContract: svc_sendreply failed\n");
    }
    else {
	sitemgrDebug(2,
		     "CallCancelContract: svc_sendreply succeeded, result = %d\n",
		     result);
    }
}
