/*-------------------------------------------------------------------------
 *
 * datafrag.x--
 *    RPC protocol for handling data fragments (eg. movement)
 *
 * Copyright (c) 1994, Regents of the University of California
 *
 * $Id: datafrag.x,v 1.12 1996/01/28 00:35:39 searcher Exp $
 *
 *-------------------------------------------------------------------------
 */
#ifdef RPC_HDR
%#ifndef DATAFRAG_H
%#define DATAFRAG_H
%
%#include <rpc/rpc.h>
%#include <rpc/types.h>
%#include <sys/socket.h>
%#include <netdb.h>
%#include "exchange.h"
%#include "rpctypes.h"
#endif

/*
 * Classifies type of fragment transfer.
 */
enum FragXType {
    BUYFRAGMENT,		/* Fragment move to */
    SELLFRAGMENT,		/* Fragment take from */
    COPYFRAGMENT,		/* Fragment copy from */
    CP_UPDATETRANSFER		/* Copy update stream send to */
};

/*
 * Specifies which part of fragment, archive or current part.
 */
enum FragRelType {
    ARCHIVE_PART,
    CURRENT_PART
};

struct FragXArg {
    FragXType fragxtype;	/* Type of fragment transfer */
    QepFragmentId qepFragId;	/* Identify the request */
    SiteAddr sender;		/* Sender address */
    SiteAddr receiver;		/* Receiver address */
    int fraclassid;		/* Oid of class */
    int	frastoreid;		/* Oid of fragment to be moved */
    unsigned int remotexactid;	/* Transaction id at called site */
};

struct FragCatalogInfo {
    FragXArg transferInfo;	/* information about fragment moved/copied */
    Tuple classTuple;		/* tuple in pg_class for the class */
    Tuple fragTuple;		/* tuple in pg_class for the fragment */
    Tuple pgfragTuple;		/* tuple in pg_fragment for the fragment */
    TupleArray attrTuples;	/* tuples in pg_attribute */

    RPCErrorCode error;		/* Since this can be returned via RPC call */
};

struct FragDataBuffer {
    FragXArg transferInfo;	/* information about fragment moved/copied */
    FragRelType which;		/* which part of the fragment */
    int numTuples;		/* number of tuples to insert */
    Data data;			/* the tuples */

    RPCErrorCode error;		/* Since this can be returned via RPC call */
};

struct SchedFragmentArg {
    int	frastoreid;		/* Oid of fragment to be moved */
    char type;			/* fragment cache or uncache query */  
};

#ifdef RPC_HDR
%
%extern void handleCall_FRAGXSETUP(void *arg);
%extern void handleCall_EXTRACTPREPARATIONS(void *arg);
%extern void handleCall_FRAGPUTCAT(void *arg);
%extern void handleCall_FRAGPUTDATA(void *arg);
%extern void handleCall_FRAGGETCAT(void *arg);
%extern void handleCall_FRAGGETDATA(void *arg);
%extern void handleCall_SCHEDFRAGMENT(void *arg);
%extern void CopyFragXArg(FragXArg *source, FragXArg *dest);
%
%#endif	/* DATAFRAG_H */
#endif
