/*-------------------------------------------------------------------------
 *
 * nameserver.c--
 *    Routines for the global NAME SERVER module in Mariposa
 *
 * Copyright (c) 1994, Regents of the University of California
 *
 *
 * IDENTIFICATION
 *    $Header: /usr/local/devel/mariposa/cvs/src/backend/rpc/nameserver.c,v 1.4 1995/09/29 15:45:03 jsidell Exp $
 *
 *-------------------------------------------------------------------------
 */

#include <stdio.h>
#include <stdlib.h>
#include "utils/elog.h"
#include "mariposa_rpc.h"
#include "rpc/rpcutils.h"

#include "namesvc.h"
#include "postgres.h"
#include "rpc/nameserver.h"
#include "utils/hsearch.h"
#include "sitemgr/rpcerror.h"
#include "catalog/pg_class.h"
#include "catalog/pg_fragment.h"

char NSVC_SiteListFileName[128]; 
char **NSVC_SiteNames; 
int *NSVC_SiteAddrs; 
int NSVC_NumSites;

HTAB *NSVC_ClassCache;

pthread_mutex_t namesvrWakeupLock;
pthread_cond_t namesvrWakeupMainThread;

void 
discoverClass(Oid classOid, NSVCProbeResult *result)
{
    NSVC_ClassEntry *classentry;
    bool found;

    TupleArray *classinfo;
    TupleArray *fraginfo;
/*  TupleArray *attrinfo; */
    Tuple *classtup;
    Tuple *fragtup;
/*  Tuple *attrtup; */
    int len, nfrags;
    int i;

    char *hashkey_addr;
    char *hashvalue_addr;
    char searchkey[sizeof(long)];

    memmove(searchkey, &classOid, sizeof(Oid));
    memset(searchkey + sizeof(Oid), 0, sizeof(long) - sizeof(Oid));
    
    hashkey_addr = (char *)
	hash_search(NSVC_ClassCache, searchkey, HASH_FIND, &found);
    hashvalue_addr = hashkey_addr + sizeof(long);

    if (! found) {
	result->error = CLASS_UNKNOWN;
	result->classes.TupleArrayArray_len = 0;
	return;
    }

    classentry = *(NSVC_ClassEntry **)hashvalue_addr;

    result->classes.TupleArrayArray_len = 2;
    result->classes.TupleArrayArray_val = (TupleArray *)
	palloc(2 * sizeof(TupleArray));
    classinfo = &(result->classes.TupleArrayArray_val[0]);
    fraginfo = &(result->classes.TupleArrayArray_val[1]);
/*  attrinfo = result->classes.TupleArrayArray_val[2]; */

    classinfo->TupleArray_len = 1;
    classinfo->TupleArray_val = (Tuple *)
	palloc(sizeof(Tuple));

    classtup = &(classinfo->TupleArray_val[0]);
    classtup->Tuple_len = len = classentry->class->t_len;
    classtup->Tuple_val = (char *)palloc(len);
    memmove(classtup->Tuple_val, classentry->class, len);

    fraginfo->TupleArray_len = nfrags = classentry->nfrags;

    if (nfrags) {
	fraginfo->TupleArray_val = (Tuple *)palloc(nfrags * sizeof(Tuple));

	for (i=0; i<nfrags; i++) {
	    fragtup = &(fraginfo->TupleArray_val[i]);
	    fragtup->Tuple_len = len = classentry->frags[i]->t_len;
	    fragtup->Tuple_val = (char *)palloc(len);
	    memmove(fragtup->Tuple_val, classentry->frags[i], len);
	}
    }

    else {
	fraginfo->TupleArray_val = NULL;
    }

    result->error = RPC_SUCCESS;

    return;
}

void 
updateNamesvr(Oid classOid, HeapTuple item, RPCErrorCode *result)
{
    /* classOid in this case is the Oid of the catalog to update.
       right now we handle pg_class and pg_fragment */
    bool found;
    int i;

    switch(classOid) {
    case RelOid_pg_class: {
	char *hashkey_addr;
	char *hashvalue_addr;
	char searchkey[sizeof(long)];

	memmove(searchkey, &(item->t_oid), sizeof(Oid));
	memset(searchkey + sizeof(Oid), 0, sizeof(long) - sizeof(Oid));
    
	hashkey_addr = (char *)
	    hash_search(NSVC_ClassCache, searchkey, HASH_ENTER, &found);
	hashvalue_addr = hashkey_addr + sizeof(long);

	if (! found) {
	    /* this is a new class - let's find out about its fragments */
	    /* NOT IMPLEMENTED YET */
	}

	if (found) {
	    /* class info may have changed through add_attr */
	    /* NOT IMPLEMENTED YET */
	}

	*result = NOT_YET_IMPLEMENTED;

	return;
    }
	
    case RelOid_pg_fragment: {
	/* The fragment information is stored twice:  once in the entry
	   for its class, and once in its own entry.  We must update
	   both of them. */

	Form_pg_fragment fragment = (Form_pg_fragment)GETSTRUCT(item);
	NSVC_ClassEntry *classentry;

	char *hashkey_addr;
	char *hashvalue_addr;
	char searchkey[sizeof(long)];

	/* update entry for the fragment */
	memmove(searchkey, &(fragment->frastoreid), sizeof(Oid));
	memset(searchkey + sizeof(Oid), 0, sizeof(long) - sizeof(Oid));
    
	hashkey_addr = (char *)
	    hash_search(NSVC_ClassCache, searchkey, HASH_FIND, &found);
	hashvalue_addr = hashkey_addr + sizeof(long);

	if (!found) {
	    /* this should only happen when split/coalesce is implemented */
	    *result = NOT_YET_IMPLEMENTED;
	    return;
	}

	classentry = *(NSVC_ClassEntry **)hashvalue_addr;

	classentry->class = (HeapTuple)repalloc(classentry->class,
						item->t_len);
	memmove(classentry->class, item, item->t_len);
	
	/* update entry for the class */
	memmove(searchkey, &(fragment->fraclassid), sizeof(Oid));
	memset(searchkey + sizeof(Oid), 0, sizeof(long) - sizeof(Oid));
    
	hashkey_addr = (char *)
	    hash_search(NSVC_ClassCache, searchkey, HASH_FIND, &found);
	hashvalue_addr = hashkey_addr + sizeof(long);

	if (!found) {
	    *result = CLASS_UNKNOWN;
	    return;
	}

	classentry = *(NSVC_ClassEntry **)hashvalue_addr;

	/* check if we already know about the fragment */
	for (i=0; i<classentry->nfrags; i++) {
	    Form_pg_fragment oldfrag = (Form_pg_fragment)
		            GETSTRUCT(classentry->frags[i]);
	    if (fragment->frastoreid == oldfrag->frastoreid) {
		/* this is the catalog entry for the same fragment; 
		   replace it */
		classentry->frags[i] = (HeapTuple)
		    realloc(classentry->frags[i], item->t_len);
		memmove(classentry->frags[i], item, item->t_len);
		*result = SUCCESS;
		return;
	    }
	}

	/* this is a new fragment */
	/* SHOULD ONLY HAPPEN WHEN SPLIT/COALESCE IS IMPLEMENTED */

	*result = NOT_YET_IMPLEMENTED;
	return;
    }	

    default:
	*result = ARGUMENT_OUTOFRANGE;
	return;
    }
}

void
BootstrapNamesvr(void)
{
    FILE *siteListFile;
    char siteName[128];
    struct hostent *siteInfo;
    HASHCTL hashinfo;

    int nsites;
    int maxsites;
    int i, j, k, l;

    /* initialize locks */
    pthread_mutex_init(&namesvrWakeupLock, pthread_mutexattr_default);
    pthread_cond_init(&namesvrWakeupMainThread, pthread_condattr_default);

    /* initialize cache - key is class oid, value is pointer to 
       NSVC_ClassEntry; key length should be sizeof(oid), but keys
       must be padded and long aligned. */
    hashinfo.keysize = sizeof(long);
    hashinfo.datasize = sizeof(NSVC_ClassEntry *);
    NSVC_ClassCache = hash_create(1001, &hashinfo, HASH_ELEM);

    /* get information on sites from file */
    nsites = 0;
    maxsites = 8;
    NSVC_SiteNames = (char **)palloc(maxsites * sizeof(char *));
    NSVC_SiteAddrs = (int *)palloc(maxsites * sizeof(int));

    siteListFile = fopen(NSVC_SiteListFileName, "r");
    if (siteListFile == NULL) {
	elog(FATAL, "Cannot open site list file %s", NSVC_SiteListFileName);
    }

    while (fscanf(siteListFile, "%s", siteName) != EOF) {
	if (nsites == maxsites) {
	    maxsites *=2;
	    NSVC_SiteNames = (char **)
		repalloc(NSVC_SiteNames, maxsites * sizeof(char *));
	    NSVC_SiteAddrs = (int *)
		repalloc(NSVC_SiteAddrs, maxsites * sizeof(int));
	}

	NSVC_SiteNames[nsites] = (char *)malloc(strlen(siteName) + 1);
	strcpy(NSVC_SiteNames[nsites], siteName);
	siteInfo = gethostbyname(siteName);
	(void) memmove((char *) &NSVC_SiteAddrs[nsites], 
		      siteInfo->h_addr, siteInfo->h_length);
	nsites++;
    }
    
    NSVC_NumSites = nsites;

    /* poll sitemgrs for class information */
    for (i=0; i<nsites; i++) {
	NSVCPollList pollArg;
	TupleArrayArray pollResult;
	RpcConn *rpcc;
	
	pollArg.NSVCPollList_len = 1;
	pollArg.NSVCPollList_val = (NSVCPollArg *)palloc(sizeof(NSVCPollArg));
	pollArg.NSVCPollList_val[0].classId = RelOid_pg_class;

	rpcc = Rpc_connectSitemgr(NSVC_SiteAddrs[i],
				  0,
				  "BootstrapNamesvr",
				  RPC_ONERR_RETURN);
	if (rpcc == NULL) {
	    elog(NOTICE, "BootstrapNamesvr: couldn't create client to poll %s",
		 NSVC_SiteNames[i]);
	    continue;
	}

	/* WARNING: we need to initialize the array_val field
	   to NULL to tell xdr_array to allocate space for the result! */
	pollResult.TupleArrayArray_val = NULL; 

	if (!Rpc_call(rpcc, RPC_NSVC_POLL, &pollArg, &pollResult)) {
	    elog(NOTICE, "BootstrapNamesvr: couldn't poll %s", 
		 NSVC_SiteNames[i]); 
	    continue;
	}

	elog(DEBUG, "BootstrapNamesvr: processing results from %s",
	     NSVC_SiteNames[i]);

	for (j=0; j < pollResult.TupleArrayArray_len; j++) {
	    /* look at the information for a single class, and insert it 
	       into the cache */

	    TupleArray classinfo = pollResult.TupleArrayArray_val[j];

	    /* classinfo's first element is the class tuple;
	       the rest are fragment tuples */
	    HeapTuple classtup = (HeapTuple)
		classinfo.TupleArray_val[0].Tuple_val;

	    int nfrags = classinfo.TupleArray_len - 1;
	    int oldnfrags,  newfragno;
	    bool found;

	    char *hashkey_addr;
	    char *hashvalue_addr;
	    char searchkey[sizeof(long)];

	    memmove(searchkey, &(classtup->t_oid), sizeof(Oid));
	    memset(searchkey + sizeof(Oid), 0, sizeof(long) - sizeof(Oid));
    
	    hashkey_addr = (char *)
		hash_search(NSVC_ClassCache, searchkey, HASH_ENTER, &found);
	    hashvalue_addr = hashkey_addr + sizeof(long);

	    if (found) {
		/* some other site already told us about this class,
		   but we may have found out about some more fragments */
		NSVC_ClassEntry **hashptr = (NSVC_ClassEntry **)hashvalue_addr;
		NSVC_ClassEntry *classentry = *hashptr;

		/* sanity check should go here to make sure the
		   class tuple should look the same */

		/* make space for the new fragment entries */
		newfragno = oldnfrags = classentry->nfrags;

		if (nfrags) {
		    classentry->frags = (HeapTuple *)
			repalloc(classentry->frags, 
				 (nfrags + oldnfrags) * sizeof(HeapTuple));
		}

		/* check each new fragment to see if it is really new */
		for (k = 1; k <= nfrags; k++) {
		    HeapTuple fragtup = (HeapTuple)
			classinfo.TupleArray_val[k].Tuple_val;

		    Form_pg_fragment newfrag = (Form_pg_fragment)
			                GETSTRUCT(fragtup);

		    Oid frastoreid = newfrag->frastoreid;

		    for (l = 0; l < oldnfrags; l++) {
			Form_pg_fragment oldfrag = (Form_pg_fragment)
			                GETSTRUCT(classentry->frags[l]);

			if (frastoreid == oldfrag->frastoreid) {
			    /* already know about this fragment */
			    break;
			}
		    }

		    if (l == oldnfrags) {
			/* the fragment is a new one, insert it */
			int fraglen = fragtup->t_len;
			
			classentry->frags[newfragno] = (HeapTuple)
			    palloc(fraglen);
			memmove(classentry[newfragno], fragtup, fraglen);

			newfragno++;
		    }
		}		

		classentry->nfrags = newfragno;
	    }

	    else /* not found */ {
		/* create the cache entry for this class */
		NSVC_ClassEntry *classentry = (NSVC_ClassEntry *)
		    palloc(sizeof(NSVC_ClassEntry));
		NSVC_ClassEntry **hashptr = (NSVC_ClassEntry **)hashvalue_addr;

		*hashptr = classentry;

/*		memmove(hashvalue_addr, &classentry, 
			sizeof(NSVC_ClassEntry *));
*/
		classentry->class = (HeapTuple)palloc(classtup->t_len);
		memmove(classentry->class, classtup, classtup->t_len);
		
		classentry->nfrags = nfrags;

		if (nfrags) {
		    classentry->frags = (HeapTuple *)
			palloc(nfrags * sizeof(HeapTuple));

		    /* insert info for each fragment */
		    for (k=1; k<=nfrags; k++) {
			HeapTuple fragtup = (HeapTuple)
			    classinfo.TupleArray_val[k].Tuple_val;
			classentry->frags[k-1] = (HeapTuple)
			    palloc(fragtup->t_len);
			memmove(classentry->frags[k-1], fragtup, 
				fragtup->t_len);
		    }
		}

		else {
		    classentry->frags = NULL;
		}
	    }
	}
    }

    elog(DEBUG, "BootstrapNamesvr completes");
}
