/*-------------------------------------------------------------------------
 *
 * optimizerRPC.c--
 *    optimizer-related RPC calls to the exec./backend
 *
 * Copyright (c) 1994, Regents of the University of California
 *
 *
 * IDENTIFICATION
 *    $Header: /usr/local/devel/mariposa/cvs/src/backend/rpc/optimizerRPC.c,v 1.4 1995/04/24 18:58:58 andrew Exp $
 *
 *-------------------------------------------------------------------------
 */

#include <rpc/types.h>
#include <pthread.h>
#include "utils/elog.h"
#include "mariposa_rpc.h" /* for procedure constants */
#include "rpc/rpcutils.h"

#include "rpc/call_exec.h" /* backend calling facility */

#include "access/heapam.h"
#include "catalog/catname.h"
#include "catalog/pg_class.h"
#include "catalog/pg_attribute.h"
#include "catalog/indexing.h"
#include "utils/catcache.h"
#include "utils/mcxt.h"
#include "nodes/plannodes.h"
#include "nodes/nodes.h"
#include "nodes/relation.h"
#include "optimizer/cost.h"
#include "optimizerRPC.h"

#define DEBUG_OPTRPC 1
	
float planCost(Plan *plan, List *rtrelids);
	
/*****************************************************************************
 *  RPC handler (server side)
 *****************************************************************************/

/*
 * handleCall --
 *	thread start routine; does synchronous call to backend to process query
 */
void
handleCall_OPT_GETCOST(void *args)
{
    char **argv = (char **)args;
    OptimizerArg *rpcarg = (OptimizerArg *)argv[0];
    SVCXPRT *transp = (SVCXPRT *)argv[1];
    OptimizerResult *result;
    
    mainThreadExec((thdproc_t)getQueryCost, rpcarg, &result);
    fprintf(stderr, "POSTGRES: OPT_GETCOST, result->cost = %f\n",
	    result->cost);
    
    if (!svc_sendreply(transp, xdr_OptimizerResult, result)) {
  	errorExit("handleCall: can't send reply\n");
    }

    free(rpcarg);
    free(args);
}

OptimizerResult *
getQueryCost(OptimizerArg *arg)
{
    OptimizerResult *result;
    Plan *plan;
    List *rtrelids;

    result = (OptimizerResult *)malloc(sizeof(OptimizerResult));
    memset(result, 0, sizeof(OptimizerResult));
    
    plan = stringToNode(arg->planStr);
    rtrelids = stringToNode(arg->relidStr);
    
    result->cost = planCost(plan, rtrelids);
    fprintf(stderr, "GETQUERYCOST: result->cost = %f\n", result->cost);
    return result;
}

static float
planCost(plan, rtrelids)
     Plan *plan;
     List *rtrelids;
{
    float cost = 0.0;
    
    switch (plan->type) {
	
    case T_SeqScan: {
	Scan *s = (Scan *)plan;
	Oid *relid = (Oid *)(rtrelids->elem);
	Relation r = heap_open(*relid);
	cost += cost_seqscan(*relid, r->rd_rel->relpages, r->rd_rel->reltuples);
	heap_close(r);
	
    }
	break;
	
    default: {
	errorExit("planCost: unexpected plan type\n");
    }
    }

    if (plan->lefttree != NULL) {
	cost += planCost(plan->lefttree, rtrelids);
    }

    if (plan->righttree != NULL) {
	cost += planCost(plan->righttree, rtrelids);
    }

    return cost;
}
