/*-------------------------------------------------------------------------
 *
 * queries.c--
 *    query-related calls to the exec./backend
 *
 * Copyright (c) 1994, Regents of the University of California
 *
 *
 * IDENTIFICATION
 *    $Header: /usr/local/devel/mariposa/cvs/src/backend/rpc/queries.c,v 1.15 1995/04/24 18:59:01 andrew Exp $
 *
 *-------------------------------------------------------------------------
 */
#include <rpc/types.h>
#include <pthread.h>
#include "utils/elog.h"
#include "mariposa_rpc.h" /* for procedure constants */
#include "rpc/rpcutils.h"

#include "rpc/call_exec.h" /* backend calling facility */
#include "queries.h"

/*
 * sitemgrBackendDone --
 *	wrapper for RPC call
 *
 *	creates and destroys client handle
 */
void 
sitemgrBackendDone(int pid)
{
    RPCErrorCode error;
    RpcConn *rpcc;

    rpcc = Rpc_connectSitemgr(getMyHostId(),
			      0,
			      "sitemgrBackendDone",
			      RPC_ONERR_WARN);

    Rpc_call(rpcc, RPC_BACKENDDONE, &pid, &error);

    Rpc_disconnect(rpcc);
}

/*
 * sitemgrNameSvcSitemgrUpdate --
 *	wrapper for RPC call
 *
 *	creates and destroys client handle
 */
void 
sitemgrNameSvcSitemgrUpdate(int action, /*Oid*/int frastoreid)
{
    RpcConn *rpcc;
    NSVCUpdate arg;

    arg.action = action;
    arg.frastoreid = (int)frastoreid;

    rpcc =
	Rpc_connectSitemgr(getMyHostId(),
			   0,
			   "sitemgrNameSvcSitemgrUpdate",
			   RPC_ONERR_RETURN);
    if (rpcc == NULL) {
	/* don't use WARN, we want running postgres stand-alone to work
	   also */
	elog(NOTICE, "sitemgrNameSvcSitemgrUpdate: can't talk to sitemgr\n");
	return;
    }
    Rpc_call(rpcc, RPC_NSVC_SITEMGRUPDATE, &arg, NULL);

    Rpc_disconnect(rpcc);
}

/*
 * handleCall --
 *	thread start routine; does synchronous call to backend to process query
 */
void
handleCall(void *args)
{
    char **argv = (char **)args;
    ProcessQueryArg *rpcarg = (ProcessQueryArg *)argv[0];
    SVCXPRT *transp = (SVCXPRT *)argv[1];
    int pid;

    /* don't keep site mgr waiting */
    if (!svc_sendreply(transp, xdr_void, NULL)) {
  	errorExit("handleCall: can't send reply\n");
    }

    mainThreadExec((thdproc_t)ProcessRemoteQuery, rpcarg, NULL);

    /* now tell the sitemgr we're done */
    pid = getpid();
    sitemgrBackendDone(pid);

/*
 *   if (!svc_freeargs(arg->transportHandle, xdr_ProcessQueryArg, arg->arg)) {
 *	errorExit("handleCall: can't free arguments\n");
 *    }
 */

    free(rpcarg);
    free(args);
}

