/*-------------------------------------------------------------------------
 *
 * queries.x--
 *    
 *
 * Copyright (c) 1994, Regents of the University of California
 *
 * $Id: queries.x,v 1.13 1996/05/31 17:14:15 jsidell Exp $
 *
 *-------------------------------------------------------------------------
 */
#ifdef RPC_HDR
%#ifndef QUERIES_H
%#define QUERIES_H
%
%#include <rpc/rpc.h>
%#include <rpc/types.h>
%#include <sys/socket.h>
%#include <netdb.h>
%#include "exchange.h"
%#include "rpctypes.h"
%#include "nodes/pqa.h"
#endif

typedef SiteAddr SiteAddrList<>;
typedef QepFragmentId QepFragIdList<>;

/*
 * ProcessQueryArg
 * 
 * Passed to the Broker (from the originating front-end, or from a Bidder
 * during subcontracting).  Also passed to the Bidder (from a Broker).
 * 
 */
struct ProcessQueryArg {
    /* PQA_Plan_tree and PQA_Node_list are defined in pqa.h. Their xdr functions
     * are in nodes/xdrfuncs.c. They are really of type Plan * and List *,
     * but it makes a problem to include nodes/plannodes.h, etc here!
     *								-Rex 95.1.16
     */
    int operation;			/* Same as QueryDesc->operation.  We need this    */
					/* to tell the difference between an INSERT, 	  */
					/* UPDATE, DELETE, which all have T_Result as	  */
					/* the top node in the plan tree.		  */

    int protocol;			/* The protocol used by the Query Broker for this */
					/* query - needed during subcontracting.	  */

    PQA_Plan_tree plan;			/* The query plan produced by the fragmenter. 	  */

    PQA_Node_list rtable;		/* The range table with fragment info filled in   */
					/* the original rtable. 			  */

    SiteAddr homeSite;			/* The site where the query originated. 	  */

    PQA_Node_list recvSites;		/* The sites which will receive the data from.    */
					/* this query.	This is a list of SiteAddrNode's  */

    PQA_Int_list recvQepFragIds; 	/* List of qepFragIds for query fragments         */
					/* expecting to receive output from this fragment */
    unsigned int requestId;		/* Unique identifier generated at home site	  */
    int resultRelation;			/* Index into range table for result relation     */
    int resultFragment;			/* Ditto for fragment. 				  */
    char partitionMode;			/* For CMD_EXCHOUT plans - how to partition 	  */
				   	/* data sent out. 				  */
};

#ifdef RPC_HDR
%
%void
%execProcessQuery(SiteAddr *addr, ProcessQueryArg *arg);
%
%extern void sitemgrBackendDone(int pid);
%
%extern void handleCall(void *arg);
%
%/* in tcop/pquery.c */
%extern void ProcessRemoteQuery(ProcessQueryArg *queryArg);
%
%#endif
#endif


