/*-------------------------------------------------------------------------
 *
 * rpcutils.h--
 *    prototypes for RPC utility functions
 *
 * Copyright (c) 1994-5, Regents of the University of California
 *
 * $Id: rpcutils.h,v 1.8 1996/02/21 06:52:08 marcel Exp $
 *
 *-------------------------------------------------------------------------
 */
#ifndef RPCUTILS_H
#define RPCUTILS_H

#include "rpccalls.h"	/* generated by rpc/Gen_rpccalls.sh */

typedef struct RpcConn {
    int		rpc_prog;		/* RPC program id */
    CLIENT	*rpc_clientHandle;	/* established client handle */
    int		rpc_sock;		/* socket of the connection */
    char	*rpc_caller_name;	/* function name of the caller */
    int		rpc_onerr;		/* what to do on error */
    int		rpc_hostid;		/* hostid (for debugging) */
} RpcConn;


/*
 * error handling code
 */
#define RPC_ONERR_WARN		1	/* call elog with WARN */
#define RPC_ONERR_RETURN	2	/* do nothing and returns */

#define RPC_ONERR_DEFAULT	RPC_ONERR_WARN


#define Rpc_setCallerName(rpcd, funcname) rpcd->rpc_caller_name = funcname
#define Rpc_setOnerr(rpcd, code) rpcd->rpc_onerr = code


extern RpcConn *Rpc_connectSitemgr(int hostid, int addr, char *funcname,
				   int onerr);
extern RpcConn *Rpc_connectExec(int addr, int port, char *funcname, int onerr);
extern RpcConn *Rpc_connectLocalBroker(char *funcname, int onerr);
extern RpcConn *Rpc_connectRemoteBroker(int addr, int port, int hostid, char *funcname, int onerr);
extern RpcConn *Rpc_connectNamesvr(int addr, char *funcname, int onerr);

extern int Rpc_call(RpcConn *rpcc, RpcCallDesc *callDesc,
		    void *arg, void *retval);

extern void Rpc_freeres(RpcConn *rpcc, RpcCallDesc *callDesc, void *retval);

extern void Rpc_disconnect(RpcConn *rpcc);


extern void rpcAsyncHandleRequest(SVCXPRT *transp, xdrproc_t xdrproc,
				  int argsize, void threadfunc(),
				  char *reqname);
extern void rpcSyncHandleRequest(SVCXPRT *transp, xdrproc_t xdrproc,
				  int argsize, void threadfunc(),
				  char *reqname);
extern void RPCThreadStart(void *args, SVCXPRT *transp, void threadfunc());


#define errorExit(string) \
    {fprintf(stderr, string); exit(1);}
#define checkAndExit(status, string) \
    if ((status) == -1) {errorExit(string);}
#define checkAndElog(status, string) \
    if ((status) == -1) {elog(WARN, string);}

typedef void *(*thdproc_t)(void *);
     
extern void mainThreadExec(thdproc_t func, void *arg, void *retval);
extern void mainThreadExecInXact(thdproc_t func, void *arg, void *retval);

extern int getMyAddr(void);
extern int getMyHostId(void);
extern int getMyNameServerHostId(void);

#endif /* RPCUTILS_H */
