/*-------------------------------------------------------------------------
 *
 * syncFrag.c--
 *    synchronous fragment transfer
 *
 * Copyright (c) 1994, Regents of the University of California
 *
 *
 * IDENTIFICATION
 *    $Header: /usr/local/devel/mariposa/cvs/src/backend/rpc/syncFrag.c,v 1.5 1995/09/23 06:50:11 searcher Exp $
 *
 *-------------------------------------------------------------------------
 */

#include <rpc/types.h>
#include <pthread.h>
#include "utils/elog.h"
#include "mariposa_rpc.h" /* for procedure constants */
#include "rpc/rpcutils.h"
#include "sitemgr/rpcerror.h"

#include "rpc/call_exec.h" /* backend calling facility */

#include "access/heapam.h"
#include "catalog/catname.h"
#include "catalog/pg_class.h"
#include "catalog/pg_attribute.h"
#include "catalog/indexing.h"
#include "utils/catcache.h"
#include "utils/mcxt.h"
#include "syncFrag.h"

#define DEBUG_OPTRPC 1
	
/*****************************************************************************
 *  RPC handler (server side)
 *****************************************************************************/

static void *
syncPutFragment(SyncFragmentArg *putArg)
{
    ContractId contract = putArg->contract;
    int subBid;
    int frastoreid = putArg->frastoreid;
    int fraclassid = putArg->fraclassid;
    int newloc = putArg->receiver.hostid;
    
    elog(NOTICE, "SYNCPUTFRAGMENT");
    
    fprintf(stderr, "fraclassid, frastoreid, newloc: %i, %i, %i\n",
	    fraclassid, frastoreid, newloc);
    
    StartTransactionCommand();
    FragmentMove(fraclassid, frastoreid, newloc);
    CommitTransactionCommand();
    
    /*
     * Signal SubmitBid() thread so that it will call
     * contractWon() on the storage site.
     */
    /*
       subBid = contractsMade[contract].subBids[0];
       bidsRequested[subBid].bidAwarded = FALSE;
       pthread_cond_signal(&(bidsRequested[subBid].awardCond));
       */
}

/*
 * handleCall --
 *	thread start routine; does synchronous call to backend to process query
 */
void
handleCall_SYNCPUTFRAGMENT(void *args)
{
    char **argv = (char **)args;
    SyncFragmentArg *rpcarg = (SyncFragmentArg *)argv[0];
    SVCXPRT *transp = (SVCXPRT *)argv[1];
    SyncFragmentResult result;	/* XXX set me! */
    
    result.error = SUCCESS;

    mainThreadExec((thdproc_t)syncPutFragment, rpcarg, NULL);

    if (!svc_sendreply(transp, xdr_SyncFragmentResult, &result)) {
  	errorExit("handleCall: can't send reply\n");
    }

    free(rpcarg);
    free(args);
}

static void *
syncGetFragment(SyncFragmentArg *sellArg)
{
    ContractId contract = sellArg->contract;
    int subBid;
    int frastoreid = sellArg->frastoreid;
    int fraclassid = sellArg->fraclassid;
    int sourceloc = sellArg->sender.hostid;
    
    elog(NOTICE, "SYNCGETFRAGMENT");
    
    fprintf(stderr, "fraclassid, frastoreid, sourceloc: %i, %i, %i\n",
	    fraclassid, frastoreid, sourceloc);
    
    StartTransactionCommand();
    FragmentTake(fraclassid, frastoreid, sourceloc);
    CommitTransactionCommand();
    
    /*
     * Signal SubmitBid() thread so that it will call
     * contractWon() on the storage site.
     */
    /*
       subBid = contractsMade[contract].subBids[0];
       bidsRequested[subBid].bidAwarded = FALSE;
       pthread_cond_signal(&(bidsRequested[subBid].awardCond));
       */
}

/*
 * handleCall --
 *	thread start routine; does synchronous call to backend to process query
 */
void
handleCall_SYNCGETFRAGMENT(void *args)
{
    char **argv = (char **)args;
    SyncFragmentArg *rpcarg = (SyncFragmentArg *)argv[0];
    SVCXPRT *transp = (SVCXPRT *)argv[1];
    SyncFragmentResult result;	/* XXX set me! */

    result.error = SUCCESS;
    
    mainThreadExec((thdproc_t)syncGetFragment, rpcarg, NULL);

    if (!svc_sendreply(transp, xdr_SyncFragmentResult, &result)) {
  	errorExit("handleCall: can't send reply\n");
    }

    free(rpcarg);
    free(args);
}
