#include <stdio.h>
#include "parameter.h"
#include "listSched.h"
#include "pg-interface.h"

typedef struct Token
{
    char * next;
    int event;
    float time;
    char * attr;
} TokenType;

ListX eventList;
ListX freeTokenList;

float simTime;

/*
 * routines declared in this file
 */
void InitSim PROTO((void));
void EventSchedule PROTO((int event, char * attr, float interTime));
void NextEvent PROTO((int *event, char ** attr));
float MyTime PROTO(());
float TokenCost PROTO((char * token));

TokenType * GetFreeToken PROTO(());
void TokenReleased  PROTO((TokenType * tokenPtr));

#include <sys/types.h>
#include <sys/time.h>
/* #include <values.h>
#include <sys/param.h>
*/
#define MAX_TIME 100000


extern int errno;

void InitMyTime()
{
    struct itimerval again;
    int retval;
    
    again.it_interval.tv_sec = MAX_TIME;
    again.it_interval.tv_usec = 0;
    again.it_value.tv_sec = MAX_TIME;
    again.it_value.tv_usec = 0;
    retval = setitimer(ITIMER_REAL, &again, (struct itimerval*) 0);
    if (retval < 0) {
	printf ("%d %d ", errno, again.it_value.tv_sec);
	PrintError("Timer error\n");
    }
    retval = getitimer(ITIMER_REAL, &again);
    if (retval < 0) {
	PrintError("Timer error 2\n");
    }
    return;
}

float MyTime()
{
    int retval;
    struct itimerval thistime;
    
    retval = getitimer ( ITIMER_REAL, &thistime );
    return (float) (MAX_TIME - thistime.it_value.tv_sec) -
	1e-6*(float)thistime.it_value.tv_usec;
}

void
InitSim()
{
    InitMyTime();
    InitList(&eventList);
    InitList(&freeTokenList);
}

void
NextEvent(event, attr)
int * event;
char ** attr;
{
    TokenType * tokenPtr;

    pthread_mutex_lock(&schedLock);
    if (IsEmpty(eventList)) {
	pthread_cond_wait(&schedEventAvail, &schedLock);
    }
    tokenPtr = 	(TokenType *) RemoveElfromList(&eventList);

    if (tokenPtr != NULL) {
	simTime = tokenPtr->time;
	*event = tokenPtr->event;
	*attr = tokenPtr->attr;
	TokenReleased (tokenPtr);
    } else {
	*event = -1;
    }
    pthread_mutex_unlock(&schedLock);

    if (is_debug("a")) {
	printf ("Dequeued: event %d, attr %x, time %f\n", 
		*event, *attr, simTime);
    }

}

void
EventSchedule(event, attr, readyTime)
int event;
char * attr;
float readyTime;
{
    TokenType * tokenPtr;

    pthread_mutex_lock(&schedLock);
    tokenPtr = GetFreeToken();
    tokenPtr->time = MyTime() + readyTime;
    tokenPtr->attr = attr;
    tokenPtr->event = event;
    InsertSorted (&eventList, (ElemType *) tokenPtr, TokenCost);

    pthread_cond_signal(&schedEventAvail);
    pthread_mutex_unlock(&schedLock);

    if (is_debug("a")) {
	printf ("Enqueued: event %d, attr %x, time %f\n", 
		event, attr, tokenPtr->time);
    }
}

float
TokenCost(tokenPtr)
char * tokenPtr;
{
    TokenType * temp;

    temp = (TokenType *) tokenPtr;
    return temp->time;
}

void
PrintTokenPtr(tokenPtr)
TokenType * tokenPtr;
{
    printf ("%2d %7.2f %x\n", tokenPtr->event, tokenPtr->time, 
	    tokenPtr->attr);
}

TokenType *
GetFreeToken()
{
    TokenType * tokenPtr;

    tokenPtr = (TokenType *) RemoveElfromList(&freeTokenList);

    if (tokenPtr == NULL) {
	tokenPtr = (TokenType *) MallocErrorCheck (sizeof(TokenType));
    }
    return (tokenPtr);
}

void
TokenReleased(tokenPtr)
TokenType * tokenPtr;
{
    AddEltoList (&freeTokenList, (ElemType *) tokenPtr);
}


