/*
 * list.c --
 *  routines to provide the list interface
 */

#include <stdio.h>
#include "listSched.h"


/*-------------------------------------------
 * initialize a List structure
 */
void
InitList(list)
ListX *list;
{
	list->head = NULL;
	list->tail = NULL;
}

/*----------------------------------------------
 * append list2 to list1
 */
void
AppendList (list1, list2)
ListX *list1, *list2;
{
	
    if (IsEmpty(*list2)) 
	return;
    if (list1->head == NULL) 
    {
	list1->head = list2->head;
    }
    else 
    {
	list1->tail->next = list2->head;
    }
    list1->tail = list2->tail;
    list2->head = list2->tail = NULL;
}

/*-----------------------------------------------
 * add an element to the end of the list
 */

void
AddEltoList(list, e)
ListX * list;
Element * e;
{
    e->next = NULL;
    
    if (list->head == NULL) {
	list->head = e;
    }
    else {
	list->tail->next = e;
    }
    list->tail = e;
}

/*--------------------------------------------------
 * remove first element from the list
 */
Element *
RemoveElfromList(list)
ListX * list;
{
    Element * e = NULL;
    
    if (list->head != NULL) {
	e = list->head;
	list->head = list->head->next;
	if (list->head == NULL) {
	    list->tail = NULL;
	}
    }
    if (e != NULL) {
	e->next = NULL;
    }
    return(e);
}

/*--------------------------------------------------
 * remove given element from the list
 */
void
RemoveGivenElfromList(list, el)
ListX * list;
Element *el;
{
    Element * e, *prevE = NULL;
    
    for (e = list->head; e != NULL; prevE = e, e = e->next) {
	if (e == el) break;
    }
    if (e == NULL) return;
    if (list->tail == e) {
	list->tail = prevE;
    }
    if (list->head == e) {
	list->head = e->next;
    }
    if (prevE) {
	prevE->next = e->next;
    }
}
	
/*-----------------------------------------------------
 * is the list empty?
 */
int
IsEmpty(list)
ListX list;
{
	if (list.head == NULL)
		return (1);
	return (0);
}


/*-----------------------------------------------
 * insert an elment to the list in the cost sorted order
 */

void
InsertSorted(list, e, costFunc)
ListX * list;
Element * e;
float costFunc();
{
    Element * templist, * prevEl;

    e->next = NULL;
    
    if (list->head == NULL) {
	list->head = e;
	list->tail = e;
	return;
    }
    for (templist = list->head, prevEl = NULL; templist != NULL; 
	 prevEl = templist, templist = templist->next ) {

	ElemType * tempEl;

	tempEl = GetData(templist);

	/* element already enqueued */
	if (tempEl == e) return;

	if (costFunc(e) < costFunc(tempEl)) {
	    /* 
	     * insert element at beginning of templist 
	     */
	    e->next = templist;
	    if (prevEl == NULL) {
		list->head = e;
		return;
	    }
	    prevEl->next = e;
	    return;
	}
    }
    /* 
     * insert at end of list
     */
    list->tail->next = e;
    list->tail = e;
}
    

ListLength(list)
ListX list;
{
    Element * el;
    int num = 0;

    ForEach (el, list) {
	num++;
    }
    return (num);
}

/*--------------------------------------------------
 * check if an element belongs to a list or not
 */
int
IsListElement(list, el)
ListX * list;
Element *el;
{
    Element * e;
    
    ForEach (e, (*list)) {
	if (el == e) {
	    return 1;
	}
    }
    return 0;
}

PrintList(list, PrintElement)
ListX list;
void PrintElement();
{
    Element * el;

    ForEach (el, list) {
	PrintElement(el);
    }
}
