#define INFTY 500000000

#define MAX_USERS   30     /* degree of multiprogramming */
 
#define MAX_QUERY  (1*MAX_USERS)

#define TIME_TO_NEXT_QUERY  10.0


/* device characteristics */
#define SWITCH_TIME 10.0
#define MAX_PLAT    100
#define MAX_DEVICE  20
#define PLAT_CAPACITY 5000        /* capacity of a platter in MB */
/* data base characterization */

#define A_MBYTE   1048576
#define SIZE_UNIT A_MBYTE          /* unit in which we measure size */
#define NUM_RELS  50              /* number of relation tables  */
#define MIN_SIZE  50               /* min size of relation in MB */ 
#define MAX_SIZE  5000             /* max size of relation in MB */

/* cache characterization */

#define CACHE_SIZE 8 /* size of cache in megabytes */

#define EVENT__SUBMIT_QUERY       1
#define EVENT__PROCESS_QUERY      2
#define EVENT__CACHE_REL          3
#define EVENT__UNCACHE_REL        4
#define EVENT__GET_NEXT_QUERY     5
#define EVENT__DATA_FETCH         6

extern int cacheSize;
extern int saveWkld;
extern int readWkld;
extern FILE * wkldFile;
extern int cachePercent;

extern int numUsers, maxQueries;
extern int startMeasurement, endMeasurement;
extern int batchMode;
extern int detectMean;


extern int numPlats, maxPlats, platCapacity, numDevice, limitPlats;

extern float tertWeight, tertFixedCost, switchTime;
extern float costWeight;
extern float joinWeight;
extern float fixedCost;
extern int tertNum;

extern float timeToNextQuery;
extern int joinPercent;
extern int seqScanPercent;

extern int numRels;
extern int numLogicalRels;
extern int maxSize, minSize;
extern int maxFragmentSize;
extern int maxPartSize;
extern int avgFragSize;

extern int numPerGrp;
extern int ageFactor;

extern int deadlock;


/*
 * prototypes
 */

#if __STDC__
#  define PROTO(x) x
#else
#  define PROTO(x) ()
#endif

/*
 * from tert.c
 */
void InitTertiaryMem PROTO((char * hardwareFileName, int num));
float DataFetchSubmit PROTO((char * reqPtr, int rel));
int ConvertToBlocks PROTO((int size));
float DataFetchCost PROTO((int rel));
float EstimateLatency PROTO((int rel));
float EstimateFetchCost PROTO((int rel));

/*
 * from event.c
 */
void InitSim PROTO((void));
float MyTime PROTO(());
float TokenCost PROTO((char * token));
void PrintEventStruct PROTO((void));
void NextEvent PROTO((int *event, char ** attr));
void EventSchedule PROTO((int event, char * attr, float interTime));

/*
 * from utils.c
 */
extern PrintError  PROTO((char * str));
void * MallocErrorCheck PROTO((int size));
int IntCeil PROTO((int p, int q));
int Max PROTO((int p, int q));
int Min PROTO((int p, int q));
int Abs PROTO((int p));
void StringAppend PROTO((char *s1, char *s2));
int IsAnElementOf PROTO((int el, int *array, int len));
float SS_QueryCost();
float ThresholdTime();

/*
 * from sim.c
 */
void MainLoop PROTO((void));
/* void SS_SubmitQuery  PROTO((char * query, float cost));
*/ void ScheduleIO  PROTO((char * req, float time));
void ScheduleCPU  PROTO((char * query, float time));

/*
 * from main.c
 */
void CPUJobDone PROTO((char * token));
void IOJobDone PROTO((char * token));
void SubmitJobs PROTO((void));

/*
 * from plan.c
 */
extern void InitLocalPlan();
extern float ThresholdTime PROTO((void));

/*
 * from pg-interface.c
 */
extern void PGinit PROTO((char * relFile));
/* extern void PGRunQuery PROTO((QueryPtr query));
*/
extern void PGCacheRel(char * token, int rel);
extern void PGUncacheRel(int rel);
void NextPGEvent PROTO((int *event, char ** attr));
#include "stat.h"


struct RelInfo
{
    int relid;
    int size;              /* relation size */
    int plat;              /* platter on which the relation is stored*/
    int startBlock;        /* starting position on platter */
    int numBlocks;         /* size in number of blocks     */
};
typedef struct RelInfo RelData;
typedef struct RelInfo * RelPtr;

extern RelData relList[NUM_RELS];
int RelSize (int i);
int RelPlatName (int rel);
int RelNumBlocks (int rel);
int RelStartBlock (int rel);





