#include <stdio.h>
#include "parameter.h"
#include "listSched.h"
#include "querySched.h"

#define MIPS 50 
#define INIT_INS 50000
#define INS_PER_TUPLE 400
#define SEEK_TIME 20e-3
#define DATA_XFER_RATE 5.0       /* in MB/sec */
#define TUPLE_SIZE 256
#define PAGE_SIZE 8196

/* percentage of queries that are large */
extern int fractionLargeQueries;   
extern int avgSel;

float tertWeight = 0.1;
float tertFixedCost = 1.0;

float costWeight;
float joinWeight;
float fixedCost = INIT_INS*1e-6/MIPS;

/*
 * routines declared in this file
 */
void InitPlan ();
float ThresholdTime (void);
void SetSS_QueryCost (QueryPtr query);

/* -----------------------------------------------------------
 * variable cost per tuple = cpu cost + I/O cost for single rel queries
 * I/O cost = time to fetch 1 disk page/number of tuples per page
 * for joins -- assuming grace hash join variable cost is 3*i/o cost
 * + 2*cpuCost (for two hash table constructions)
 */
void
InitLocalPlan()
{
    float cpuCost;
    float ioCost;

    cpuCost = INS_PER_TUPLE*1e-6/MIPS;
    /* 
     * the above is the per tuple cost, assuming a tuple size of
     * TUPLE_SIZE bytes we get the per SIZE_UNIT cost as
     */
    cpuCost = cpuCost*SIZE_UNIT/TUPLE_SIZE;

    ioCost  = SIZE_UNIT/(DATA_XFER_RATE*A_MBYTE);
    if (ioCost > tertWeight) {
	ioCost = tertWeight;
    }
    ioCost += SEEK_TIME;

    costWeight = cpuCost + ioCost;
    
    joinWeight = ioCost + 2*cpuCost;
}


void
SetSS_QueryCost(query)
QueryPtr query;
{
    if (SS_JoinQuery(query)) {
	query->cost = fixedCost + 
	    (RelSize(query->rel[0]) + RelSize(query->rel[1]))*joinWeight;
    } else {
	query->cost = fixedCost + RelSize(query->rel[0])*costWeight;
    }
}

/*---------------------------------------------------
 * this routines returns the maximum time that a query has to wait.
 */
float
ThresholdTime()
{
   float avgCost = fixedCost + SWITCH_TIME + avgSel * (maxSize + minSize) * costWeight/2.0;
   return ((float)ageFactor * (float)numUsers*avgCost);
}



    
    


