#include <stdio.h>
#include "parameter.h"
#include "listSched.h"
#include "querySched.h"
#include "schedInterface.h"

/*----------------------------------------------------------------
 * schedule jobs on a first come first serve basis
 */
QueryPtr
FcFs()
{
    QueryPtr query;
    static ListX subQueryList;

    if (IsEmpty(subQueryList)) {
	query = ((QueryPtr) RemoveElfromList(&scheduleList));
	if (query != NULL) {
	    subQueryList = query->subQueryList;
	}
    }
    return (QueryPtr) RemoveElfromList (&subQueryList);
}

/* -------------------------------------------------------------
 *
 */
QueryPtr
QueryOrientedPolicies()
{
    QueryPtr selQuery = NULL;
    static ListX subQueryList;
    Element * queryList;
    int merit;
    int maxMerit = -INFTY;
    
    if (IsEmpty(subQueryList)) {

	ForEach (queryList, scheduleList) {
	    QueryPtr query = (QueryPtr) GetData(queryList);
	
	    merit = GetQueryMerit(query);
	    if (merit > maxMerit) {
		selQuery = query;
		maxMerit = merit;
	    }
	}
	if (selQuery != NULL) {
	    RemoveGivenElfromList(&scheduleList, (ElemType *) selQuery);
	    subQueryList = selQuery->subQueryList;
	}
    }
    return (QueryPtr) RemoveElfromList (&subQueryList);
}


GetQueryMerit(parentQuery)
QueryPtr parentQuery;
{
    float cost = 0.0;
    Element * queryList;
    QueryPtr query;

    ForEach (queryList, parentQuery->subQueryList) {
	query = (QueryPtr) GetData(queryList);

	switch (schedulingPolicy) {
	case MINACCESSCOSTQ:
	    /* Minimum cost of fetching each relation */
	    
	    if (! RelCached(query->rel[0])) {
		cost += EstimateFetchCost(query->rel[0]);
	    }	
	    if (SS_JoinQuery(query)) {
		if (! RelCached(query->rel[1])) {
		    cost +=  EstimateFetchCost(query->rel[1]);
		}
	    }
	    break;

	case MINLATENCYQ:
	    /* Minimum latency of accessing first byte of each relation */
	    if (!RelCached(query->rel[0])) {
		cost += EstimateLatency (query->rel[0]);
	    }
	    if (SS_JoinQuery(query)) {
		if (! RelCached(query->rel[1])) {
		    cost +=  EstimateLatency(query->rel[1]);
		}
	    }
	    break;

	case MINCLQ:
	    /* Minimum amount of fresh data that has to be put in cache */
	    if (!RelCached(query->rel[0])) {
		cost += RelSize (query->rel[0]);
	    }
	    if (SS_JoinQuery(query)) {
		if (! RelCached(query->rel[1])) {
		    cost +=  RelSize (query->rel[1]);
		}
	    }
	}
    } /* end for */
    return (- ((int) cost));
}

