#include <stdio.h>
#include <string.h>

/*
 * routines exported from this file
 */

extern PrintError (char * str);
void * MallocErrorCheck(int size);
int IntCeil (int p, int q);
int Max (int p, int q);
int Min (int p, int q);
int Abs (int p);
void StringAppend (char *s1, char *s2);
int IsAnElementOf (int el, int *array, int len);



/*------------------------------------------
 * MallocErrorCheck () --
 *   malloc requested number of bytes and return pointer to
 *   requested block if successful. If malloc failed print error
 *   message and exit.
 */

extern char * malloc();

void *
MallocErrorCheck (nbytes)
int nbytes;
{
  char * ptr;
  
  if (nbytes == 0) {
      printf("Requesting 0 bytes\n");
      return NULL;
  }
  ptr = (char *) malloc (nbytes);
  if (ptr != NULL) return (ptr);
  printf ("Malloc Failure\n");
  exit (0);
}

IntCeil (p, q)
int p, q;
{
    return ( (p + q - 1)/q);
} 

Max(p,q)
int p,q;
{
    return ((p > q)?p:q);
}


Min(p,q)
int p,q;
{
    return ((p < q)?p:q);
}

Abs(i)
int i;
{
    return ((i >= 0)? i:-i);
}

PrintError(str)
char * str;
{
    printf("Error: %s\n", str);
    exit(0);
}

void
StringAppend(s1, s2)
char *s1, *s2;
{
    char temp[200];
    strcpy ((char *)temp, s2);
    strcpy(s2, s1);
    strcat(s2, (char *)temp);
}

int
IsAnElementOf(el, array, len)
int el, *array, len;
{
    int i;
    
    for (i = 0; i < len; i++) {
	if (array[i] == el) {
	    return (1);
	}
    }
    return (0);
}


char *
_AdvanceBy1word(str, word)
char *str, **word;
{
    char *retstr, *space;

    *word = NULL;
    if (str == NULL) return str;
    while (isspace(*str)) str++;
    *word = str;
    if ((space = (char *)index(str, ' ')) != (char *) NULL) {
        retstr = space + 1;
        *space = '\0';
    }
    else
        retstr = NULL;
    return retstr;
}

CountWordsInStr(str)
char * str;
{
    int numWords = 1;
    char * space;

    if (str == NULL) return 0;

    while (isspace(*str)) str++;
    while ((space = (char *)index(str, ' ')) != (char *) NULL) {
    str = space + 1;
    while (isspace(*str)) str++;
    if (str != '\0') numWords++;
    }
    return numWords;
}

char **
StringToArgv(str, argc)
char * str;
int * argc;
{
    char ** argv;
    int i;

    *argc = CountWordsInStr(str);
    argv = (char **) MallocErrorCheck(sizeof(char *)*(*argc));
    for (i = 0; i < *argc; i++) {
    str = _AdvanceBy1word(str, &(argv[i]));
    }
    return argv;
}


static	char	dbg_flags[128];

extern	void
debug_init ( )
{
    int	i;

    for (i=0; i<128; i++ ) dbg_flags[i]=0;
    return;
}


extern	void
debug_set ( flags )
char	*flags;
{
    char	*p;
    int		i;
    int		dec_flag = 0;

    if ( flags == 0 ) return;
    p = flags;

    if ( ( *p != '+' ) && ( *p != '-' )) debug_init();

    while ( *p != 0 ) {
	switch ( *p ) {
	    case '+': 
		break;
	    case '-': 
		dec_flag++;
		break;
	    case '*':
		for (i=0; i<128; i++) dbg_flags[i] = 1;
		break;
	    default:
		if (dec_flag) dbg_flags[*p] = 0;
		else dbg_flags[*p] = 1;
		break;
	}
	p++;
    }
}

extern	int
is_debug ( flags )
char	*flags;
{
    char	*p;
    for (p = flags; *p != 0; p++ ) {
	if (dbg_flags[*p]) return(1);
    }
    return(0);
}


