/*-------------------------------------------------------------------------
 *
 * bidder.c--
 *    Bidder functions.
 *
 * Copyright (c) 1994, Regents of the University of California
 *
 *
 * IDENTIFICATION
 *    $Header: /usr/local/devel/mariposa/cvs/src/backend/sitemgr/bidder.c,v 1.22 1996/05/31 17:14:23 jsidell Exp $
 *
 *-------------------------------------------------------------------------
 */

#include <string.h>

#include "sitemgr/bidder.h"
#include "sitemgr/sitemgr.h"
#include "datafrag.h"
#include "bid.h"
#include "utils/elog.h"
#include "tcl.h"

Budget *EmptyBudget();
     
extern Contract *contractsMade;

/*
 *-------------------------------------------------------------------------
 * PlanBid--
 *
 * Sets up the Tcl global variables plan, rtable and contract and then calls
 * the Tcl procedure GetQueryBid.
 *-------------------------------------------------------------------------
 */

Budget *
PlanBid(Plan *plan, List *rtable, ContractId contract) {
    Budget *bid = EmptyBudget();
    char *rtableStr;
    char planStr[1024];
    char *tclStmt = (char *)malloc(1024 * sizeof(char));
    char *tclResult = (char *)malloc(1024 * sizeof(char));
    int tclReturn;
    char *spacePos;
    int space = ' ';
    int n = 0;
    
    char *responseStr, *priceStr, *delayStr, *staleStr, *accStr;
    
    /*
     * Put the plan into its Tcl list format.
     */
    planStr[0] = '\0';
    planString(planStr, plan, rtable, &n, TRUE, FALSE);
    /*
     * Do the same for the range table
     */
    rtableStr = rtableString(rtable);
    
    /*
     * Now expose the range table and the contract ID to the Tcl interpreter
     */
    
    strcpy(tclStmt, "set rtable ");
    strcat(tclStmt, rtableStr);
    tclReturn = tclEvaluate(tclStmt, &tclResult);
    
    sitemgrDebug(1, "%s\n", tclStmt);

    sprintf(tclStmt, "set contract %i", contract);
    tclReturn = tclEvaluate(tclStmt, &tclResult);
    
    sitemgrDebug(1, "%s\n", tclStmt);
    
    /*
     * Call GetQueryBid
     */

    strcpy(tclStmt, "GetQueryBid ");
    strcat(tclStmt, planStr);
    
    sitemgrDebug(1, "%s\n", tclStmt);

    tclReturn = tclEvaluate(tclStmt, &tclResult);

    if (tclReturn == TCL_OK) {
	/*
	 * The result should be a string representing a Tcl list
	 * with five elements: bidYN, price, delay, staleness and accuracy.
	 */

	responseStr = tclResult;
	spacePos = strchr(tclResult, space);
	spacePos[0] = '\0';
    
	priceStr = ++spacePos;
	spacePos = strchr(priceStr, space);
	spacePos[0] = '\0';
	delayStr = ++spacePos;

	spacePos = strchr(delayStr, space);
	spacePos[0] = '\0';
	staleStr = ++spacePos;

	spacePos = strchr(staleStr, space);
	spacePos[0] = '\0';
	accStr = ++spacePos;

	bid->response = atoi(responseStr);
	bid->cost = atof(priceStr);
	bid->delay = atof(delayStr);
	bid->staleness = atof(staleStr);
	bid->accuracy = atof(accStr);
    } else {
	bid->response = REFUSETOBID;
	elog(NOTICE, "Error in bidder script.");
    }
    
    /*
     * Free up the malloc'd stuff.
     */

    free(tclResult);

    return bid;
    
}


/*
 *-------------------------------------------------------------------------
 * GetSellBid - send back cost of selling a fragment to another site.
 *
 * This means selling the fragment, not setting up a copy at another
 * site.
 *-------------------------------------------------------------------------
 */
void
GetSellBid(details, contract, bid)
    FragXArg details;
    int contract;
    Budget *bid;
{
    
    /* For fragment purchasing, we just concern ourselves with master copy for
     * now. @@@ Why not copies too?				-Rex 6.21.1995
     */
    if (isLocalFragment(details.frastoreid, MASTER)) {
	bid->delay = 2.0;
	bid->cost= 20.0;
    } else {
	bid->delay = 2000.0;
	bid->cost = 20000.0;
    }
    
    contractsMade[contract].subBids = FALSE;

    sitemgrDebug(3,
		 "GETSELLBID: Bidding (%f, %f) on fragment %i, contract %i.\n",
		 bid->cost, bid->delay, details.frastoreid, contract);

}

/*
 *-------------------------------------------------------------------------
 * GetBuyBid - send back cost of buying a fragment from another site.
 *
 * This means buying the fragment, not setting up a copy from another
 * site.
 *-------------------------------------------------------------------------
 */
void
GetBuyBid(details, contract, bid)
    FragXArg details;
    int contract;
    Budget *bid;
{
    
    /* For fragment purchasing, we just concern ourselves with master copy for
     * now. @@@ Why not copies too?				-Rex 6.21.1995
     */
    if (isLocalFragment(details.frastoreid, MASTER)) {
        /*
	 * This should never happen... - rp2y
	 */
        sitemgrDebug(0, "GETBUYBID: Bidding on a local fragment (%i)!\n",
                     details.frastoreid);
	bid->delay = 10000.0;
	bid->cost= 100000.0;
    } else {
        /*
	 * XXX - This will need to change once we get copies since we
	 *       may want to bid differently on a fragment that is
	 *       replicated at this site. - rp2y
	 */
	bid->delay = 2.0;   /* bogus */
	bid->cost = 20.0;   /* bogus */
    }
    
    contractsMade[contract].subBids = FALSE;

    sitemgrDebug(3,
		 "GETBUYBID: Bidding (%f, %f) on fragment %i, contract %i.\n",
		 bid->cost, bid->delay, details.frastoreid, contract);

}

/*
 *-------------------------------------------------------------------------
 * EmptyBudget--
 *
 *	Returns a budget with everything initialized to 0.
 *-------------------------------------------------------------------------
 */
Budget *
EmptyBudget() {

    Budget *bid = (Budget *)calloc(1, sizeof(Budget));

    bid->cost = 0.0;
    bid->delay = 0.0;
    bid->staleness = 0.0;
    bid->accuracy = 0.0;
    bid->response = BID;
    
    return bid;
}
