/*-------------------------------------------------------------------------
 *
 * bidder.h--
 *    Header file for the bidder functions
 *
 * Copyright (c) 1994, Regents of the University of California
 *
 * $Id: bidder.h,v 1.9 1996/02/26 16:22:53 jsidell Exp $
 *
 *-------------------------------------------------------------------------
 */

#ifndef BIDDER_H

#define BIDDER_H

#include "sitemgr/sitemgr.h"
#include "queries.h"
#include "executor/execdesc.h"
#include "bid.h"

/*-------------------------------------------------------------------------
 * Contract - main data structure for bidder.
 *
 * A Contract is created by the bidder routine ReceiveRequestForBid, in
 * sitemgr.c every time the bidder is requested to make a bid on a piece
 * of work.
 *
 * The contract contains the description of the work to be done, whether it's
 * selling a fragment or processing a query.  It also contains which bid
 * collection at the broker site the contract is for, and which bid request
 * for that bid collection this contract corresponds to.  This is mainly
 * for debugging, so that we can tell who a contract was made for.
 *
 * A contract also contains subBid information, since contracts can involve
 * sub-contracting out work to other sites.
 *
 *-------------------------------------------------------------------------
 */

typedef struct {
    Budget bid;                         /* What we're bidding for this work.                   */
    
    bool subBids;			/* TRUE if we contacted Broker to subcontract out work.*/
    
    unsigned int requestId;             /* Which bid collection on the broker side this        */
                                        /* contract is for.                                    */
    int requestIndex;			/* Index into requests[] in Broker process		*/

    int whichChunk;			/* Which plan chunk this bid is for (in Broker).	*/

    int whichBid;			/* Which bid for the plan chunk this is (in Broker).	*/
    
    BidProtocol protocol;		/* What we're doing.					*/

    SiteAddr requester;			/* Who asked us to perform this contract		*/
    SiteAddr homeSite;			/* The site at which the query originated		*/
    
    union {                             /* Details - query description for a
					   query-processing contract, etc. */
	ProcessQueryArg queryDetails;
	FragXArg moveDetails;
    } details_u;
    
} Contract;

extern Budget *
PlanBid(Plan *plan, List *rtable, ContractId contract);

extern void
GetSellBid(FragXArg details, int contract, Budget *bid);

#endif
