/*-------------------------------------------------------------------------
 *
 * execs.h--
 *    Header file for execs.c.
 *    Defines utilities for the sitemgr to maintain and keep track of
 *    running exec processes.
 *
 * Copyright (c) 1994, Regents of the University of California
 *
 * $Id: execs.h,v 1.13 1996/02/06 18:49:22 jsidell Exp $
 *
 *-------------------------------------------------------------------------
 */
#ifndef	EXECS_H
#define	EXECS_H

#include "exchange.h"
#include "queries.h"
#include "datafrag.h"
#include "sitemgr/rpcerror.h"

/*
 * ExecInfo -
 *    All the information the sitemgr needs to know about an exec.
 *    Add fields as needed.
 */

struct exec_info {
    bool alive;
    SiteAddr *udpPort;
    SiteAddr *tcpPort;
    int pid;
    QepFragmentId qepFragId;
    SiteAddr requester;
    
    int fragid;		/* (temporary) holds (logical|store) oid of fragment
			   being (processed|moved) (for sitemgrLogEvent) */
};

typedef struct exec_info ExecInfo;

#define MAX_OPEN_EXECS 20
#define BLOCK_IF_NO_EXECS 0

/* The following will become RUSH variables. */
extern int numAssignedExecs;
extern int numReadyExecs;
extern int numExecsNeeded;

extern ExecInfo *
PidGetExecInfo(int pid);

/* Note that currently we do not have transaction id's or command
 * id's, so we can only use the query fragment id to identify execs.
 * This should be changed when xact id's work to the following:
 *
 * ExecInfo *
 * QueryIdGetExecInfo(TransactionId globalXactId, CommandId cmdId, 
 * 		   QepFragmentId qepFragId);
 */

extern ExecInfo *
QueryIdGetExecInfo(QepFragmentId qepFragId);

extern int ExecStart(void);

extern RPCErrorCode
ExecFragX(FragXArg details, QepFragmentId qepFragId);

extern RPCErrorCode
ExecTwopcStart(SiteAddr *sender, QepFragmentId qepFragId);

/* As above, xact id's do not yet work.  When they do, use:
 *
 * void 
 * ExecProcessQuery(TransactionId globalXactId, CommandId cmdId,
 *		 QepFragmentId qepFragId, RPCOpaque plan);
 */

extern void
ExecProcessQuery(ProcessQueryArg details, QepFragmentId qepFragId);

extern void 
ExecFinishedQuery(int pid);

extern void
ExecReady(int pid);

extern void
ExecKill(void);

extern void 
ExecsInit(void);

extern SiteAddr *popReadyExec(int qepFragId, int *execIdPtr);
extern void pushDoneExec(int execId);

extern void ExecKilled(int pid);
#endif	EXECS_H
