###########################################################################
# rules implementation on top of DMT
#
puts "installing rules..."
trace debug off

set ruleID -1
set rulePrefix rule
proc rule-convert-idstr {idstr} {
    global rulePrefix conTable
    if {![regexp "${rulePrefix}(\[0-9\]+)" $idstr id id]} {
	error "illegal rule handle: $idstr"
    }
    if {![info exists conTable($id)]} {
	error "unknown (deleted?) rule handle: $idstr"
    }
    return $id
}

#mkrule expr {vars} <cond> <action>
proc mkrule {option args} {
    global rulePrefix ruleID conTable
    if {"$option"=="configure"} {
	set id [rule-convert-idstr [lindex $args 0]]
	set max [llength $args]
	if {$max==1} {
	    return
	}
	for {set i 1} {$i<$max} {incr i} {
	    set opt [lindex $args $i]
	    if {"$opt"=="-enable"} {
		incr i
		set enable [expr ([lindex $args $i]==1)]
		foreach tid $conTable($id) {
		    trace configure $tid -enabled $enable
		}
	    }
	}
	return
    }
    if {"$option"=="dbgprint"} {
	foreach idstr $args {
	    set id [rule-convert-idstr $idstr]
	    puts "\ntraces used in rule \"$idstr\":"
	    foreach tid $conTable($id) {
		puts "  [trace configure $tid]"
	    }
	}
	return
    }
    if {"$option"=="delete"} {
	foreach idstr $args {
	    if {![regexp "${rulePrefix}(\[0-9\]+)" $idstr id id]} {
		error "illegal rule handle: $idstr"
	    }
	    if [array exists conTable($id)] {
		trace delete $tid $conTable($id)
		unset $conTable($id)
	    }
	}
	return
    }
    if {"$option"=="expr"} {
	set id [incr ruleID]
	set conTable($id) {}
	set vars   [lindex $args 0]
	set expr   [lindex $args 1]
	set action [lindex $args 2]
	foreach var $vars {
	    regsub -all \\\$$var $expr %n checkStr
	    regsub -all \\\$~$var $checkStr \$%v checkStr
	    regsub -all \\\$~ $checkStr \$ checkStr
	    regsub -all \\\$$var $action %n actionStr
	    regsub -all \\\$~$var $actionStr \$%v actionStr
	    regsub -all \\\$~ $actionStr \$ actionStr
	    set tid [uplevel 1 "trace writes $var {
		if {$checkStr} {$actionStr} {set %v %n}
	    }"]
	    lappend conTable($id) $tid
	}
	return $rulePrefix$id
    }
    error "bad option \"$option\": should be configure, delete, expr"
}

#set id [rule expr {$a<7} a]
#rule configure $id -enable 1
#rule dbgprint $id

