/*-------------------------------------------------------------------------
 *
 * sitemgr.c--
 *    Main function for sitemgr
 *
 * Contains:
 *
 *   sitemgrMain()
 *   execProcessQuery()
 *   execFragX()
 *   execTwopcStart()
 *   OfferShortSellContract()
 *   invokeExec()
 *   isLocalFragment()
 *
 * Copyright (c) 1994, Regents of the University of California
 *
 *
 * IDENTIFICATION
 *    $Header: /usr/local/devel/mariposa/cvs/src/backend/sitemgr/sitemgr.c,v 1.52 1996/03/04 09:07:14 aoki Exp $
 *
 *-------------------------------------------------------------------------
 */
#include <stdio.h>
#include <string.h>
#include <sys/param.h>
#include <rpc/rpc.h>
#include <netinet/in.h>
#include <netdb.h>
#include "mariposa_rpc.h"
#include "sitemgr/sitemgr.h"
#include <pthread.h>
#include "queries.h"
#include "postgres.h"
#include "rpc/rpcutils.h"

#include "rpcerror.h"
#include "sitemgr/bidder.h"
#include "access/htup.h"	/* for sm_takeInventory */
#include "libpq/pqcomm.h"
#include "syncFrag.h"
#include "sitemgr/cpupdate.h"	/* for UpdateStreamSetup */
#include "nodes/parsenodes.h"

/* declare lock and signals */
pthread_mutex_t execsLock;
pthread_mutex_t bidderLock;
pthread_mutex_t bidLock;
pthread_mutex_t updateStreamQueueLock;
pthread_cond_t bidAvailable;
pthread_cond_t execAvailableCond;
pthread_cond_t execAssignedCond;

char *backendPath;

/* extern BidCollection *bidsRequested; */

/*
 * First approximation to CPU and I/O load.
 */
int loadAverage = 0;

#define MAXCONTRACTS 100
#define SENDBUFSIZE 7168		/* copied from exchange.c */

extern Contract *contractsMade;
static void sm_takeInventory();

extern bool EnableTcl;

/*
 * -------------------------------------------------------------------
 * sitemgrMain
 *
 *   main function for sitemgr
 *
 * -------------------------------------------------------------------
 */
int
sitemgrMain(int argc, char **argv)
{
    char *arg;
    char opt;
    int i;
    pthread_t pmLoopThread;
    pthread_t svcLoopThread;
    extern int ServerLoop();
    extern FILE *eventsFile;

    backendPath = malloc(MAXPATHLEN);
    FindBackend(backendPath, argv[0]);

    contractsMade = (Contract *)calloc(MAXCONTRACTS, sizeof(Contract));

    sitemgrDebug(1, "using backend: %s\n", backendPath);

    pthread_mutex_init(&execsLock, pthread_mutexattr_default);
    pthread_mutex_init(&bidderLock, pthread_mutexattr_default);
    pthread_mutex_init(&bidLock, pthread_mutexattr_default);
    pthread_mutex_init(&updateStreamQueueLock, pthread_mutexattr_default);
    pthread_cond_init(&bidAvailable, pthread_condattr_default);
    pthread_cond_init(&execAvailableCond, pthread_condattr_default);
    pthread_cond_init(&execAssignedCond, pthread_condattr_default);

    ExecsInit();
/*     BrokerStart(); */

    /*
     * Start up thread to handle coordinating of update stream sendoffs to
     * other sites.
     */
    RPCThreadStart(NULL, NULL, CopyUpdateSetup);

    /*
     * set up the RPC server loop and start svc_run in a thread. We need
     * the main thread for the Tcl rule engine (which doesn't work in a
     * thread just yet).
     */
    sitemgr_svc_setup();
    
    pthread_create(&svcLoopThread, pthread_attr_default,
		   (void*)svc_run, NULL);
    pthread_detach(&svcLoopThread);

    if (EnableTcl || scheduleQueries) {
	pthread_create(&pmLoopThread, pthread_attr_default,
		       (void*)ServerLoop, NULL);
	pthread_detach(&pmLoopThread);
    }
	
    /*
     * print out what fragments this site has (ie. mpFrag)
     */
    if (eventsFile != NULL) {
	sm_takeInventory();
    }

    /*
     * For each contract in pg_contract enqueue an update stream.
     */
    if (invokeExec(CP_INITIALIZEUPDATESTREAMS,
		   NULL, -1, "sitemgrMain") != SUCCESS)
    {
	sitemgrDebug(0, "*************************************************\n");
	sitemgrDebug(0, "*************************************************\n");
	sitemgrDebug(0, "***** INITIALIZE UPDATE STREAMS FAILED\n");
	sitemgrDebug(0, "***** \n");
	sitemgrDebug(0, "***** At least one contract failed to be\n");
	sitemgrDebug(0, "***** scheduled. Chances are, all failed. This\n");
	sitemgrDebug(0, "***** means that your site will not be\n");
	sitemgrDebug(0, "***** maintaining its contracts to update copies\n");
	sitemgrDebug(0, "***** at other sites. I suggest you shut this\n");
	sitemgrDebug(0, "***** site down and figure out the problem.\n");
	sitemgrDebug(0, "*************************************************\n");
	sitemgrDebug(0, "*************************************************\n");
    }

    if (scheduleQueries == true)
	/*
	 * not running mariposa -- never returns.
	 */
	InitSchedMain(argc, argv);
    else if (EnableTcl) {
	/*
	 * start running the rule engine. This should never return.
	 */
	tclRuleEngineRun();
    } else {
	/*
	 * the regular postmaster ServerLoop
	 */
	ServerLoop();
    }

    exit(1);
}

/*
 * -------------------------------------------------------------------
 * sm_takeInventory
 *
 *   Invokes the local name service inventory call.
 * -------------------------------------------------------------------
 */
static void
sm_takeInventory()
{
    NSVCProbeArg arg;
    NSVCProbeResult *info;

    int i;

    arg.action = NSVC_TAKEINVENTORY;

    info = (NSVCProbeResult *)
	invokeExec(NSVC_LOCALPROBE, (void *)&arg, -1, "sm_takeInventory");

    if (info->error != SUCCESS) {
	fprintf(stderr, "sm_takeInventory: NSVC_LOCALPROBE failed");
	return;
    }

    for (i=0; i < info->classes.TupleArrayArray_len; i++) {
	HeapTuple classtup = (HeapTuple)
	    info->classes.TupleArrayArray_val[i].TupleArray_val[0].Tuple_val;
	
	fprintf(eventsFile, "mpFrag %d\n", classtup->t_oid);
    }
    fflush(eventsFile);
    return;
}


/*****************************************************************************
 *  RPC wrappers (sitemgr -> exec)
 *****************************************************************************/

/*
 * -------------------------------------------------------------------
 * execProcessQuery
 *
 *   wrapper for RPC call
 *
 *   creates and destroys client handle
 * -------------------------------------------------------------------
 */
void 
execProcessQuery(SiteAddr *addr, ProcessQueryArg *arg)
{
    RpcConn *rpcc;

    rpcc =
	Rpc_connectExec(addr->addr, addr->port,
			"execProcessQuery", RPC_ONERR_RETURN);

    if (!Rpc_call(rpcc, RPC_PROCESSQUERY, arg, NULL)) {
	sitemgrDebug(0, "execProcessQuery: RPC error\n");
    }
    Rpc_disconnect(rpcc);
}

/*
 * execFragX --
 *	wrapper for RPC call
 *
 *	creates and destroys client handle
 *
 *	XXX judging by handleCall_TWOPCSTART, the final value of
 *	'result' is actually the exec's current TransactionId.
 */
RPCErrorCode 
execFragX(SiteAddr *addr, FragXArg *arg)
{
    RpcConn *rpcc;
    RPCErrorCode result;

    rpcc =
	Rpc_connectExec(addr->addr, addr->port,
			"execFragX", RPC_ONERR_RETURN);
    if (rpcc == NULL) {
	sitemgrDebug(0, "execFragX: can't create client handle\n");
	return EXECUTION_ERROR;
    }

    if (!Rpc_call(rpcc, RPC_FRAGXSETUP, arg, &result)) {
	sitemgrDebug(0, "execFragX: calling FRAGXSETUP: RPC error\n");
    }
    if (result != SUCCESS) {
	sitemgrDebug(0, "execFragX: FRAGXSETUP failed.");
	Rpc_disconnect(rpcc);
	return result;
    }
    if (!Rpc_call(rpcc, RPC_TWOPCSTART, &arg->sender, &result)) {
	sitemgrDebug(0, "execFragX: calling TWOPCSTART: RPC error\n");
	return EXECUTION_ERROR;
    }
    Rpc_disconnect(rpcc);

    return result;
}

/*
 * execTwopcStart --
 *	wrapper for RPC call
 *
 *	Connects to the backend and gets the transaction going.
 *
 *	XXX judging by handleCall_TWOPCSTART, the final value of
 *	'result' is actually the exec's current TransactionId.
 */
RPCErrorCode 
execTwopcStart(SiteAddr *addr, SiteAddr *sender)
{
    RpcConn *rpcc;
    RPCErrorCode result;

    rpcc =
	Rpc_connectExec(addr->addr, addr->port,
			"execTwopcStart", RPC_ONERR_RETURN);
    if (rpcc == NULL) {
	sitemgrDebug(0, "execTwopcStart: can't create client handle\n");
	return EXECUTION_ERROR;
    }

    if (!Rpc_call(rpcc, RPC_TWOPCSTART, sender, &result)) {
	sitemgrDebug(0, "execTwopcStart: calling TWOPCSTART: RPC error\n");
	return EXECUTION_ERROR;
    }
    Rpc_disconnect(rpcc);

    return result;
}

void *
invokeExec(int rpcproc, void *args, int qepFragId, char *funcName)
{
    SiteAddr *execAddr;
    RpcConn *rpcc;
    void *result = NULL;
    int execId;


    sitemgrDebug(2, "%s: acquiring execsLock...", funcName);
    pthread_mutex_lock(&execsLock);
    sitemgrDebug(2, "acquired\n");

    numExecsNeeded++;

    sitemgrDebug(2, "%d execs assigned, %d needed\n", 
		 numAssignedExecs, numExecsNeeded);

    if (numAssignedExecs < numExecsNeeded) {
	/* need to open a new backend */

	sitemgrDebug(1, "%s: Starting exec\n", funcName);

	if (ExecStart() == SERVICE_REFUSED) {
	    sitemgrDebug(1, "%s: Cannot start exec\n", funcName);
	    return NULL;
	}
    }

    /* before sending a query to an exec we must check
       that there is one ready to run a query */
    while (numReadyExecs == 0) { 
	/* wait for some exec to become available */
	sitemgrDebug(1, "%s: Waiting for ready backend\n", funcName);
	pthread_cond_wait(&execAvailableCond, &execsLock);
    } 

    sitemgrDebug(1, "%s: sending query to backend\n", funcName);

    execAddr = popReadyExec(qepFragId, &execId);

    /* notify waiting threads that an exec has been assigned to a query */
    pthread_cond_broadcast(&execAssignedCond);

    sitemgrDebug(2, "%s: Releasing execsLock...", funcName);
    pthread_mutex_unlock(&execsLock);
    sitemgrDebug(2, "released\n");

    /*
     * set up a handle for the RPC call
     */
    rpcc =
	Rpc_connectExec(execAddr->addr, execAddr->port,
			funcName, RPC_ONERR_RETURN);
    if (rpcc == NULL) {
	sitemgrDebug(0, "%s: can't create client handle\n", funcName);
	return NULL;
    }

    switch (rpcproc) {
    case OPT_GETCOST:
	{
	    result = malloc(sizeof(OptimizerResult));
	    memset(result, 0, sizeof(OptimizerResult));
	    if (!Rpc_call(rpcc, RPC_OPT_GETCOST, args, result)) {
		sitemgrDebug(0, "%s: RPC error\n", funcName);
	    }
	    break;
	}

    case FRAGIDS:
	{
	    result = malloc(sizeof(FragDataBuffer));
	    memset(result, 0, sizeof(FragDataBuffer));
	    
	    if (!Rpc_call(rpcc, RPC_FRAGIDS, args, result)) {
		sitemgrDebug(0, "%s: RPC error\n", funcName);
	    }
	    break;
	}

    case CLASSES:
	{
	    result = malloc(sizeof(FragDataBuffer));
	    memset(result, 0, sizeof(FragDataBuffer));
	    
	    if (!Rpc_call(rpcc, RPC_CLASSES, args, result)) {
		sitemgrDebug(0, "%s: RPC error\n", funcName);
	    }
	    break;
	}

    case SYNCPUTFRAGMENT:
	{
	    result = malloc(sizeof(SyncFragmentResult));
	    memset(result, 0, sizeof(SyncFragmentResult));
	    
	    if (!Rpc_call(rpcc, RPC_SYNCPUTFRAGMENT, args, result)) {
		sitemgrDebug(0, "%s: RPC error\n", funcName);
	    }
	    break;
	}

    case SYNCGETFRAGMENT:
	{
	    result = malloc(sizeof(SyncFragmentResult));
	    memset(result, 0, sizeof(SyncFragmentResult));
	    
	    if (!Rpc_call(rpcc, RPC_SYNCGETFRAGMENT, args, result)) {
		sitemgrDebug(0, "%s: RPC error\n", funcName);
	    }
	    break;
	}
	
    case NSVC_LOCALPROBE:
	{
	    result = malloc(sizeof(NSVCProbeResult));
	    memset(result, 0, sizeof(NSVCProbeResult));
	    if (!Rpc_call(rpcc, RPC_NSVC_LOCALPROBE, args, result)) {
		sitemgrDebug(0, "%s: RPC error\n", funcName);
	    }
	    break;
	}

    case NSVC_LOOKUPCLASS:
	{
	    result = malloc(sizeof(NsvcLookupClassResult));
	    memset(result, 0, sizeof(NsvcLookupClassResult));
	    if (!Rpc_call(rpcc, RPC_NSVC_LOOKUPCLASS, args, result)) {
		sitemgrDebug(0, "%s: RPC error\n", funcName);
	    }
	    break;
	}

    case NSVC_LOOKUPFRAGS:
	{
	    result = malloc(sizeof(NsvcLookupFragsResult));
	    memset(result, 0, sizeof(NsvcLookupFragsResult));
	    if (!Rpc_call(rpcc, RPC_NSVC_LOOKUPFRAGS, args, result)) {
		sitemgrDebug(0, "%s: RPC error\n", funcName);
	    }
	    break;
	}

    case CP_INITIALIZEUPDATESTREAMS:
	{
	    if (!Rpc_call(rpcc, RPC_CP_INITIALIZEUPDATESTREAMS, args,
			  &result)) {
		sitemgrDebug(0, "%s: RPC error\n", funcName);
	    }
	    sitemgrDebug(0, "invokeExec(CP_INITIALIZEUPDATESTREAMS): result %d\n",
			 result);
	    break;
	}
	
    case CP_SENDUPDATESTREAM:
	{
	    result = malloc(sizeof(SendUpdateStreamResult));
	    memset(result, 0, sizeof(SendUpdateStreamResult));
	    if (!Rpc_call(rpcc, RPC_CP_SENDUPDATESTREAM, args, result)) {
		sitemgrDebug(0, "%s: RPC error\n", funcName);
	    }
	    sitemgrDebug(0, "invokeExec(CP_SENDUPDATESTREAM): Commit Time %d\n",
			 (*((SendUpdateStreamResult *)result)).committime);
	    break;
	}

    case CANCELCONTRACT:
	{
	    if (!Rpc_call(rpcc, RPC_CANCELCONTRACT, args, &result)) {
		sitemgrDebug(0, "%s: RPC error\n", funcName);
	    }
	    break;
	}

    case SUBCONTRACTBROKER:
	{
	    if (!Rpc_call(rpcc, RPC_SUBCONTRACTBROKER, args, &result)) {
		sitemgrDebug(0, "%s: RPC error\n", funcName);
	    }
	    break;
	}
	
    }

    Rpc_disconnect(rpcc);

    sitemgrDebug(2, "%s: (BackendDone) Acquiring execsLock...", funcName);
    pthread_mutex_lock(&execsLock);
    sitemgrDebug(2, "acquired\n");

    pushDoneExec(execId);

    /* notify waiting threads that a backend is now ready */
    pthread_cond_broadcast(&execAvailableCond);

    sitemgrDebug(2, "%s: Releasing execsLock...", funcName);
    pthread_mutex_unlock(&execsLock);
    sitemgrDebug(2, "released\n");

    return result;
}

/*
 * -------------------------------------------------------------------
 * isLocalFragment
 *
 *   Takes the (logical or store) OID of a fragment and queries the system
 *   catalogs to see if a copy is at this site or not. (logical or store)
 *   is determined by fspec being set to ANY or MASTER.
 * -------------------------------------------------------------------
 */

bool
isLocalFragment(Oid oid, NSVCFragmentSpec fspec)
{
    NSVCProbeArg arg;
    NSVCProbeResult *info;

    arg.action = NSVC_FINDLOCALFRAG;
    arg.oid = (int)oid;
    arg.fspec = fspec;

    info = (NSVCProbeResult *)invokeExec(NSVC_LOCALPROBE, (void *)&arg, -1, "isLocalFragment");

    if (info==NULL) {
	fprintf(stderr, "isLocalFragment: NSVC_LOCALPROBE failed");
	return FALSE;
    }

    return (bool)info->found;
}


/*
 * -------------------------------------------------------------------
 * findPgHosts (for sitemgr)
 *
 *   Find entries in pg_host system catalog for getBidSites()
 *
 *   NB. there is another findPgHosts in rpc/namesvc_be.c for the backend
 * -------------------------------------------------------------------
 */
SiteAddrList *
findPgHosts()
{
    NSVCProbeArg arg;
    NSVCProbeResult *info;

    arg.action = NSVC_FINDPGHOSTS;

    info = (NSVCProbeResult *)invokeExec(NSVC_LOCALPROBE, (void *)&arg, -1, "clnt_findPgHosts");

    if (info==NULL) {
	fprintf(stderr, "clnt_findPgHosts: NSVC_LOCALPROBE failed");
	return NULL;
    }

    return &info->pgsites;
}

/*
 * ----------------------------------------------------------------------
 * SchedRequestSubmit
 *      submit a fragment cache or uncache request to one of the backends
 * -------------------------------------------------------------------
 */
int
SchedRequestSubmit(SchedFragmentArg *arg)
{
    SiteAddr *execAddr;
    void *result = NULL;
    int execId;
    RpcConn *rpcc;

    
    sitemgrDebug(2, "%s: acquiring execsLock...");
    pthread_mutex_lock(&execsLock);
    sitemgrDebug(2, "acquired\n");

    numExecsNeeded++;

    sitemgrDebug(2, "IoSubmitRequest: %d execs assigned, %d needed\n", 
		 numAssignedExecs, numExecsNeeded);

    if (numAssignedExecs < numExecsNeeded) {
	/* need to open a new backend */

	sitemgrDebug(1, "IoSubmitRequest: Starting exec\n");

	if (ExecStart() == SERVICE_REFUSED) {
	    sitemgrDebug(1, "IoSubmitRequest: Cannot start exec\n");
	    return 0;
	}
    }

    /* before sending a query to an exec we must check
       that there is one ready to run a query */
    while (numReadyExecs == 0) { 
	/* wait for some exec to become available */
	sitemgrDebug(1, "IoSubmitRequest: Waiting for ready backend\n");
	pthread_cond_wait(&execAvailableCond, &execsLock);
    } 

    sitemgrDebug(1, "IoSubmitRequest: sending query to backend\n");

    execAddr = popReadyExec(-1, &execId);

    /* notify waiting threads that an exec has been assigned to a query */
    pthread_cond_broadcast(&execAssignedCond);

    sitemgrDebug(2, "IoSubmitRequest: Releasing execsLock...");
    pthread_mutex_unlock(&execsLock);
    sitemgrDebug(2, "IoSubmitRequest: released\n");

    rpcc =
	Rpc_connectExec(execAddr->addr, execAddr->port,
			"SchedRequestSubmit", RPC_ONERR_RETURN);
    if (rpcc == NULL) {
	sitemgrDebug(0, "SchedRequestSubmit: can't create client handle\n");
	return;
    }
    if (!Rpc_call(rpcc, RPC_SCHEDFRAGMENT, arg, NULL)) {
	sitemgrDebug(0, "SchedRequestSubmit: RPC error\n");
    }
    Rpc_disconnect(rpcc);

    return ExecIdGetPid(execId);
}





