/*-------------------------------------------------------------------------
 *
 * tclfuncs.c--
 *
 *    	- C functions to be registered with Tcl, except for subcontract()
 *
 * Copyright (c) 1994, Regents of the University of California
 *
 *
 * IDENTIFICATION
 *    $Header: /usr/local/devel/mariposa/cvs/src/backend/sitemgr/tclfuncs.c,v 1.11 1996/06/15 01:48:11 am Exp $
 *
 *-------------------------------------------------------------------------
 */
#include <math.h>	/* just for atof */

#include "tcl.h"
#include "postgres.h"
#include "mariposa_rpc.h"
#include "syncFrag.h"
#include "sitemgr/sitemgr.h"
#include "sitemgr/bidder.h"
#include <string.h>

/*
 * Command: "movefragment classOID, storeOID, toHostId"
 *
 * Just moves the fragment - no negotiation.
 *
 */
int
movefragment(ClientData clientData,
	     Tcl_Interp *interp,
	     int argc, char *argv[])
{
    SyncFragmentArg arg;
    SyncFragmentResult *result;

    if (argc != 4) {
	return TCL_ERROR;
    }

    arg.fraclassid = atoi(argv[1]);
    arg.frastoreid = atoi(argv[2]);
    arg.receiver.hostid = atoi(argv[3]);

    result = (SyncFragmentResult *)
	invokeExec(SYNCPUTFRAGMENT, (void *)&arg, -1, "movefragment");

    if (result->error != SUCCESS) {
	free(result);
	return TCL_ERROR;
    }

    free(result);
    return TCL_OK;
}

/*
 * Command: "takefragment classOID, storeOID, fromHostId"
 *
 * Just takes the fragment from the site indicated.
 */
int
takefragment(ClientData clientData,
	     Tcl_Interp *interp,
	     int argc, char *argv[])
{
    SyncFragmentArg arg;
    SyncFragmentResult *result;

    if (argc != 4) {
	return TCL_ERROR;
    }

    arg.fraclassid = atoi(argv[1]);
    arg.frastoreid = atoi(argv[2]);
    arg.sender.hostid = atoi(argv[3]);

    result = (SyncFragmentResult *)
	invokeExec(SYNCGETFRAGMENT, (void *)&arg, -1, "takefragment");

    if (result->error != SUCCESS) {
	free(result);
	return TCL_ERROR;
    }

    free(result);
    return TCL_OK;
}

/*
 * Command: fragids <classoid>
 *
 * Returns a list of fragment information for each
 * fragment in the class.  The information is a list-of-lists
 * of the form: {frastoreid fralogicalid fralocation}...
 */
int
fragids(ClientData clientData,
	Tcl_Interp *interp,
	int argc, char *argv[])
{
    Oid classoid;
    FragDataBuffer *result;

    if (argc != 2) {
	return TCL_ERROR;
    }

    classoid = atoi(argv[1]);

    result = (FragDataBuffer *)
	invokeExec(FRAGIDS, (void *)&classoid, -1, "fragids");

    if (result->error != SUCCESS) {
	free(result);
	return TCL_ERROR;
    }
    
    Tcl_AppendResult(interp, result->data.Data_val, NULL);

    free(result);
    return TCL_OK;
}

/*
 * Returns a list of all the user-defined classes
 * in the form {oid relname}...
 */

int
classes(ClientData clientData,
	Tcl_Interp *interp,
	int argc, char *argv[])
{
    FragDataBuffer *result;
    
    if (argc != 1) {
	return TCL_ERROR;
    }

    result = (FragDataBuffer *)
	invokeExec(CLASSES, (void *)NULL, -1, "classes");

    if (result->error != SUCCESS) {
	free(result);
	return TCL_ERROR;
    }
    
    Tcl_AppendResult(interp, result->data.Data_val, NULL);

    free(result);
    return TCL_OK;
}

/*
 * Re-read the bidder file.
 */
int
ReInitBidder(ClientData clientData,
	Tcl_Interp *interp,
	int argc, char *argv[])
{
    char buffer[1000];
    extern char* BidderDir;

    /*
     * Re-read in GetQueryBid procedure from BIDDERDIR
     */

    sprintf(buffer, "%s%s", BidderDir, "/bidder.tcl");
    Tcl_EvalFile(interp, buffer);
    
    return TCL_OK;
    
}

/*
 * Re-read the databroker file.
 */
int
ReInitDataBroker(ClientData clientData,
	Tcl_Interp *interp,
	int argc, char *argv[])
{
    char buffer[1000];
	extern char* DataBrokerDir;

    /*
     * Re-read in DataBroker procedure from DataBrokerDir
     */

    sprintf(buffer, "%s%s", DataBrokerDir, "/databroker.tcl");
    Tcl_EvalFile(interp, buffer);
    
    return TCL_OK;
    
}

int DMT_TraceCmd    _ANSI_ARGS_((ClientData, Tcl_Interp *,int,char **));

void
tclRegisterFuncs()
{
    extern Tcl_Interp *tclInterp;	/* global mariposa interpreter */

    /* install DMT */
    Tcl_CreateCommand(tclInterp, "trace", DMT_TraceCmd, NULL, NULL);   
    Tcl_CreateCommand(tclInterp, "movefragment", movefragment, 
		      NULL, NULL);   
    Tcl_CreateCommand(tclInterp, "takefragment", takefragment, 
		      NULL, NULL);   
    Tcl_CreateCommand(tclInterp, "classes", classes, 
		      NULL, NULL);   
    Tcl_CreateCommand(tclInterp, "fragids", fragids, 
		      NULL, NULL);
    Tcl_CreateCommand(tclInterp, "ReInitBidder", ReInitBidder,
		      NULL, NULL);
    Tcl_CreateCommand(tclInterp, "ReInitDataBroker", ReInitDataBroker,
		      NULL, NULL);
    Tcl_CreateCommand(tclInterp, "subcontract", SubContract,
		      NULL, NULL);
}
