/*
 * jbcomm.h : IPC constants & structs
 *
 * Copyright, supplied AS-IS, and all that.
 */
/* $Header: /usr/local/devel/mariposa/cvs/src/backend/storage/hpcomm.h,v 1.1 1995/04/17 04:34:20 sunita Exp $ */

#define JB_SERVER_PORT	4321
#define JB_SERVER_MACH	"seahorse.cs.Berkeley.EDU"

/*
 * Packet format:
 *
 * offset   description
 *	  +-----------------------------+
 * +00	  | int  message_length		|
 *	  +-----------------------------+
 * +04	  | short message_type		|
 *	  +-----------------------------+
 * +06	  | short unused		|
 *	  +-----------------------------+
 * +08	  | char[] data			|
 *	  +-----------------------------+
 *
 * All numbers are in network order.
 * All ASCII strings are NULL-terminated.  Data which follows an ASCII
 * string begins the byte after the '\0'.
 * "unused" is there to keep the data longword-aligned.  It will likely
 * be used for something eventually...
 */

#define JB_DATA_START	8		/* data starts 8 bytes in */

/*
 * Requests (message_type constants)
 */
#define JB_INIT_REQ		0	/* initialization request */
/* (no data) */
#define JB_OPEN_REQ		1	/* jb_open */
/* +00 flags [note this is backward from procedure args order] */
/* +04 platter_id (ASCII string) */
/* +xx format (ASCII string) */
#define JB_READ_REQ		2	/* jb_read */
/* +00 platter */
/* +04 start block # */
/* +08 #of blocks */
#define JB_WRITE_REQ		3	/* jb_write */
/* +00 platter */
/* +04 start block # */
/* +08 #of blocks */
/* +12 data (byte stream) */
#define JB_CLOSE_REQ		4	/* jb_close */
/* +00 platter */
#define JB_EXIT_REQ		12	/* jb_exit */
/* (no data) */

/*
 * Replies
 *
 * If "error status" is nonzero, the rest of the fields are invalid (and
 * may not exist).  All REPLYs must have the error status field.
 */
#define JB_INIT_REPLY		0	/* initialization reply */
/* +00 error status */
#define JB_OPEN_REPLY		1	/* jb_open */
/* +00 error status */
/* +04 platter */
#define JB_READ_REPLY		2	/* jb_read */
/* +00 error status */
/* +04 data (byte stream; use packet length) */
#define JB_WRITE_REPLY		3	/* jb_write */
/* +00 error status */
#define JB_CLOSE_REPLY		4	/* jb_close */
/* +00 error status */
#define JB_EXIT_REPLY		12	/* jb_exit */
/* +00 error status */

/*
 */
typedef struct request_t {
    char * rdbuf;		/* points to buffer with message */
    int    rdcount;		/* #of bytes in buffer */
    int    from;		/* file descriptor request was rcvd from */
    /* these fields were pulled out of rdbuf for easy access */
    short  type;
} REQUEST;

extern int listen_sock;			/* socket to listen to */
extern int debug;			/* if TRUE, print debugging info */

/*
 * C debugging stuff
 */
#define DBUG(x) if (debug) printf x


#define JB_ENOERR		0	/* no error - must be 0 */

/* problems with arguments */
#define JB_ENOSERVER		1	/* unable to connect to server */
#define JB_EUNKNOWN		2	/* platter name not recognized */
#define JB_EOFFLINE		3	/* platter is currently offline */
#define JB_EBADP		4	/* JBPLATTER pointer is bad */
#define JB_ERANGE		6	/* one of args was out of range */
#define JB_EISOPEN		7	/* platter (side) is already open */
#define JB_EACCES		8	/* JB_RDWR on write-protected platter*/

/* errors during operations */
#define JB_EIO			9	/* unspecified I/O error */
#define JB_ESERVER		10	/* communication failure with server */
#define JB_EBLANK		11	/* tried to read unwritten block */
#define JB_ENOTBLANK		12	/* tried to write written block */
#define JB_EINTERNAL		13	/* internal error; check jbserv.log */

/* errors during debug-only operations */
#define JB_ENODEBUG		19	/* server isn't accepting debug reqs */
#define JB_EOFFSET		20	/* operation illegal b/c non-0 offset*/

/* total #of errors with text msgs (for jb_strerror); last error number +1 */
#define JB_NERR			21

