/*-------------------------------------------------------------------------
 *
 * smgrshmem.c--
 *    routines to calculate the size of share memory required for a
 *    particular storage manager.
 *
 *    this file is shared by the postmaster and the backend.
 *
 * Copyright (c) 1994-5, Regents of the University of California
 *
 *
 * IDENTIFICATION
 *    $Header: /usr/local/devel/mariposa/cvs/src/backend/storage/smgr/smgrshmsz.c,v 1.2 1995/09/05 09:34:34 aoki Exp $
 *
 *-------------------------------------------------------------------------
 */

#include "postgres.h"

#include <math.h>
#include "machine.h"
#include "storage/ipc.h"
#include "storage/smgr.h"	/* where the declarations go */
#include "storage/smgr_mm.h"
#include "storage/hp.h"
#include "storage/shmem.h"
#include "storage/spin.h"
#include "storage/bufmgr.h"

#include "utils/hsearch.h"
#include "utils/memutils.h"

#ifdef MAIN_MEMORY

/*
 *  MMShmemSize() -- Declare amount of shared memory we require.
 *
 *	The shared memory initialization code creates a block of shared
 *	memory exactly big enough to hold all the structures it needs to.
 *	This routine declares how much space the main memory storage
 *	manager will use.
 */
int
MMShmemSize()
{
    int size = 0;
    int nbuckets;
    int nsegs;
    int tmp;

    /*
     *  first compute space occupied by the (dbid,relid,blkno) hash table
     */

    nbuckets = 1 << (int)my_log2((MMNBUFFERS - 1) / DEF_FFACTOR + 1);
    nsegs = 1 << (int)my_log2((nbuckets - 1) / DEF_SEGSIZE + 1);
    
    size += MAXALIGN(my_log2(MMNBUFFERS) * sizeof(void *));
    size += MAXALIGN(sizeof(HHDR));
    size += nsegs * MAXALIGN(DEF_SEGSIZE * sizeof(SEGMENT));
    tmp = (int)ceil((double)MMNBUFFERS/BUCKET_ALLOC_INCR);
    size += tmp * BUCKET_ALLOC_INCR *
	(MAXALIGN(sizeof(BUCKET_INDEX)) +
	 MAXALIGN(sizeof(MMHashEntry)));	/* contains hash key */

    /*
     *  now do the same for the rel hash table
     */

    size += MAXALIGN(my_log2(MMNRELATIONS) * sizeof(void *));
    size += MAXALIGN(sizeof(HHDR));
    size += nsegs * MAXALIGN(DEF_SEGSIZE * sizeof(SEGMENT));
    tmp = (int)ceil((double)MMNRELATIONS/BUCKET_ALLOC_INCR);
    size += tmp * BUCKET_ALLOC_INCR *
	(MAXALIGN(sizeof(BUCKET_INDEX)) +
	 MAXALIGN(sizeof(MMRelHashEntry)));	/* contains hash key */

    /*
     *  finally, add in the memory block we use directly
     */

    size += MAXALIGN(MAXBLCKSZ * MMNBUFFERS);
    size += MAXALIGN(sizeof(*MMCurTop));
    size += MAXALIGN(sizeof(*MMCurRelno));
    size += MAXALIGN(MMNBUFFERS * sizeof(MMCacheTag));

    return (size);
}

#endif /* MAIN_MEMORY */

#ifdef HP_JUKEBOX

/*
 *  HPShmemSize() -- Declare amount of shared memory we require.
 *
 *	The shared memory initialization code creates a block of shared
 *	memory exactly big enough to hold all the structures it needs to.
 *	This routine declares how much space the Hp jukebox cache will
 *	use.
 */

int
HPShmemSize()
{
    int size;
    int nbuckets;
    int nsegs;
    int tmp;

    /* size of cache metadata */
    size =
	MAXALIGN((HPCacheSize + 1) * sizeof(HPCacheItem)) +
	    MAXALIGN(sizeof(HPCacheHeader));

    /* nblock cache */
    size += MAXALIGN(HPNBLKSIZE * sizeof(HPCacheTag));

#ifndef HAS_TEST_AND_SET
    size += MAXALIGN(sizeof(*HPNWaiting));
#endif /* ndef HAS_TEST_AND_SET */

    /* size of hash table */
    nbuckets = 1 << (int)my_log2((HPCacheSize - 1) / DEF_FFACTOR + 1);
    nsegs = 1 << (int)my_log2((nbuckets - 1) / DEF_SEGSIZE + 1);
    size += MAXALIGN(my_log2(HPCacheSize) * sizeof(void *));
    size += MAXALIGN(sizeof(HHDR));
    size += nsegs * MAXALIGN(DEF_SEGSIZE * sizeof(SEGMENT));
    tmp = (int)ceil((double)HPCacheSize/BUCKET_ALLOC_INCR);
    size += tmp * BUCKET_ALLOC_INCR *
	(MAXALIGN(sizeof(BUCKET_INDEX)) +
	 MAXALIGN(sizeof(HPHashEntry)));

    /* count shared memory required for jukebox state */
    size += JBShmemSize();

    size += MAXALIGN(sizeof(*HPNEntries));
    return (size);
}

#endif /* HP_JUKEBOX */
