/*-------------------------------------------------------------------------
 *
 * smgrtype.c--
 *    storage manager type
 *
 * Copyright (c) 1994, Regents of the University of California
 *
 *
 * IDENTIFICATION
 *    $Header: /usr/local/devel/mariposa/cvs/src/backend/storage/smgr/smgrtype.c,v 0.3 1996/06/07 05:00:12 am Exp $
 *
 *-------------------------------------------------------------------------
 */
#include <string.h>
#include "postgres.h"

#include "utils/builtins.h"	/* where the declarations go */
#include "utils/elog.h"
#include "utils/palloc.h"
#include "storage/smgr.h"

typedef struct smgrid {
    char *smgr_name;
} smgrid;

/*
 *  StorageManager[] -- List of defined storage managers.
 *
 *	The weird comma placement is to keep compilers happy no matter
 *	which of these is (or is not) defined.
 */

static smgrid StorageManager[] = {
	{"magnetic disk"},
#ifndef P95

#ifdef HP_JUKEBOX
	{"hp jukebox"},
#endif /* HP_JUKEBOX */

#endif /* P95 */
#ifdef MAIN_MEMORY
#ifndef P95
	{"main memory"},
#else /* P95 */
	{"main memory"}
#endif /* P95 */
#endif /* MAIN_MEMORY */
#ifndef P95

#ifdef FRAGMENT_SMGR
	{"fragmented"},
#endif
#endif /* P95 */
};

static int NStorageManagers = lengthof(StorageManager);

int2
smgrin(char *s)
{
    int i;

    for (i = 0; i < NStorageManagers; i++) {
	if (strcmp(s, StorageManager[i].smgr_name) == 0)
	    return((int2) i);
    }
    elog(WARN, "smgrin: illegal storage manager name %s", s);
#ifdef P95
    return 0;
#endif /* P95 */
}

char *
smgrout(int2 i)
{
    char *s;

    if (i >= NStorageManagers || i < 0)
	elog(WARN, "Illegal storage manager id %d", i);

    s = (char *) palloc(strlen(StorageManager[i].smgr_name) + 1);
    strcpy(s, StorageManager[i].smgr_name);
    return (s);
}

bool
smgreq(int2 a, int2 b)
{
    if (a == b)
	return (true);
    return (false);
}

bool
smgrne(int2 a, int2 b)
{
    if (a == b)
	return (false);
    return (true);
}
