/*-------------------------------------------------------------------------
 *
 * usage_stat.h--
 *    MARIPOSA fragment usage statistics interface definitions
 *
 *
 * Copyright (c) 1994,1995 Regents of the University of California
 * [missing RCS id]
 * $Id
 *
 *-------------------------------------------------------------------------
 */

#ifndef USAGE_STAT_H
#define USAGE_STAT_H

#include "postgres.h"
#include "nodes/pg_list.h"
#include "storage/shmem.h"
#include "storage/spin.h"
#include "utils/hsearch.h"

/*
 * This structure will be used to stored statistics for all fragments
 * that a site knows about, whether local or remote.  Therefore, some
 * of the slots only apply to local fragments and others only apply to
 * remote fragments.
 */
typedef struct _UsageStatEntry {
  Oid fragmentId;
  SHMEM_OFFSET prev;
  SHMEM_OFFSET next;
  int siteId;
  /*
   * Add all new entries below this point.  I am relying on the
   * fact that the &siteId is the point at which I can bzero the
   * structure without losing its identity
   */
  float8 debits;
  float8 credits;
  float8 expectedPrice;
  long numAccesses;
  time_t arrivalTime;
  time_t lastAccessTime;
  NameData dbName;
} UsageStatEntry;

/*
 * This is twisted but it works, so don't ask and don't touch it! -rhp
 */
#define UsageStatEntryDataSize    ((long)sizeof(UsageStatEntry) - \
            ( (long) &(((UsageStatEntry *)0)->siteId) - \
	      (long) &(((UsageStatEntry *)0)->fragmentId)))

#define USAGE_STAT_ENTRY_BZERO_DATA(x)   memset(&(x->siteId), 0, UsageStatEntryDataSize)

typedef struct _UsageStatHotListEntry {
  Oid fragmentId;
  int rank;
  int siteId;
  float8 expectedPrice;
  float8 expectedRevenue;
  long numAccesses;
  time_t lastAccessTime;
  NameData dbName;
} UsageStatHotListEntry;

extern SPINLOCK UsageStatLock;
extern bool NoUsageStats;

extern SHMEM_OFFSET *ShmemUsageStatInitialized;
extern SHMEM_OFFSET *ShmemUsageStatNumStatStructsUsed;
extern SHMEM_OFFSET *ShmemUsageStatFirstEntryOffset;
extern SHMEM_OFFSET *ShmemUsageStatLastEntryOffset;
extern SHMEM_OFFSET *ShmemUsageStatHotListNumEntries;

/* usage_stat.c */
int UsageStatShmemSize(void);
void UsageStatsInitShmem(void);
void UsageStatUpdateEntry(UsageStatEntry *);
void UsageStatUpdateEntries(List *);

/* us_catalog.c */
void UsageStatsInitShmemData(void);
void UsageStatFlushShmemToDisk(void);
void UsageStatUpdateHotList(void);

#endif /* ! USAGE_STAT_H */
