/* ----------------------------------------------------------------------
 *
 * asynchFrag.c--
 *    Asynchronous fragment movement code
 *
 * Copyright (c) 1994, Regents of the University of California
 *
 * IDENTIFICATION
 *    $Header: /usr/local/devel/mariposa/cvs/src/backend/tcop/asynchFrag.c,v 1.6 1995/11/18 01:39:03 jsidell Exp $
 *
 * ----------------------------------------------------------------------
 */

/*
 * XXX - These header files were simply copied from pquery.c and 
 *       should be cleaned up to include only the ones actually 
 *       needed by this file. - rp2y
 */
#include <math.h>
#include <netinet/in.h>

#include "postgres.h"

#include "tcop/tcopdebug.h"

#include "utils/elog.h"
#include "utils/palloc.h"
#include "utils/mcxt.h"
#include "miscadmin.h"
#include "utils/portal.h"

#include "nodes/pg_list.h"
#include "nodes/primnodes.h"
#include "nodes/plannodes.h"
#include "nodes/execnodes.h"
#include "nodes/memnodes.h"

#include "executor/execdefs.h"
#include "executor/execdesc.h"
#include "executor/executor.h"
#include "rpc/rpcutils.h"
#include "tcop/debug.h"
#include "tcop/dest.h"
#include "tcop/pquery.h"
#include "tcop/frag.h"
#include "tcop/broker.h"
#include "tcop/demo.h"
#include "queries.h"

#include "commands/command.h"

#include "brokerRPC.h"

#include "catalog/pg_host.h"

/* -----------------------------------------------------------------------
 * ChooseRandomRemoteSite()
 *
 *      Called by AsyncMoveFragment()
 *
 *      This is a stupid little function that picks a remote site 
 *      in a pseudo-random fashion.
 * -----------------------------------------------------------------------
 */

int
ChooseRandomRemoteSite(int nsites, int localId)
{
    int ret;

    ret = (int)floor(((double)rand() / (double)RAND_MAX) * nsites);
    if (ret == nsites)
        ret -= 2;
    else
        ret--;
    /*
     * Now that we are assured of having an integer between 0 and
     * nsites - 2, we increment the return value by one (so that it
     * is between 1 and localId - 1) or two (so that it is between
     * localId + 1 and nsites).
     */
    if (ret < localId - 1)
        return (ret + 1);
    else
        return (ret + 2);
}

/* -----------------------------------------------------------------------
 * AsyncMoveFragment()
 *
 *      This function will presumably be called by the rule system
 *      to move fragments asynchronously.
 *
 * Side effects:
 *      If using the long protocol, the broker is called to solicit
 *      bid and determine the site that will buy or sell the fragment.
 *      When the broker returns (i.e., calls BrokerDone()), the winner is
 *      notified and the fragment is moved.
 *
 *      If using the short protocol, FragmentTake() or FragmentMove() is
 *      used to handle the move.
 * -----------------------------------------------------------------------
 */
void
AsyncMoveFragment(Oid frastoreid, BidProtocol protocol)
{
    extern pthread_mutex_t brokerDoneLock;
    extern pthread_cond_t brokerDoneCond;
    extern bool brokerDoneFlag;

    Oid fraclassid;
    int site = 0;
    
#ifdef 0
    switch (protocol) {
    case LONG_SELL_FRAGMENT: {  /* Get the fragment from a remote site. */
        extern unsigned short tcpPort;
	BrokerArg bArg;
	RpcConn *rpcc;
	int rExecAddr, rExecPort;
	List *l = lconsi(frastoreid, NULL);

	RExecInit();
	/*
	 * Construct the broker arg
	 */
	bArg.protocol = protocol;
	bArg.nrecv = 1;
	bArg.processingSites.HostList_len = 1;
	bArg.processingSites.HostList_val = (u_int *)calloc(1, sizeof(u_int));
	bArg.processingSites.HostList_val[0] = localSiteHostId;
	/*
	 * The QepFragId isn't really used in buying and selling fragments.
	 * However, the list is accessed directly (i.e., it assumes that its
	 * length is nrecv) by the broker so we have to create the list anyway.
	 */
	bArg.recvQepFragIds.QepFragIdList_len = 1;
	bArg.recvQepFragIds.QepFragIdList_val = (QepFragmentId *)calloc(1, sizeof(QepFragmentId));
	bArg.recvQepFragIds.QepFragIdList_val[0] = 1;
	/*
	 * The broker doesn't use this for buying and selling fragments.
	 */
	bArg.recvPlans.PlanList_len = 0;
	bArg.recvPlans.PlanList_val = NULL;

	bArg.recvRtables.RTableList_len = 1;
	bArg.recvRtables.RTableList_val = (PQA_Int_list *)calloc(1, sizeof(PQA_Int_list));
	bArg.recvRtables.RTableList_val[0] = (PQA_Int_list)l;
	bArg.isSubBid = FALSE;
	bArg.parentContract = 0;
	bArg.callerAddr = getMyAddr();
	bArg.callerPort = tcpPort;
	bArg.callerHostid = localSiteHostId;
	bArg.homeSiteAddr = bArg.callerAddr;
	bArg.homeSitePort = bArg.callerPort;
	bArg.homeSiteHostid = bArg.callerHostid;

	rpcc = Rpc_connectLocalBroker("AsyncMoveFragment", RPC_ONERR_RETURN);
	if (!Rpc_call(rpcc, RPC_BROKER, &bArg, NULL)) {
	    tcopDebug(0, "AsyncMoveFragment: RPC error\n");
	}
	Rpc_disconnect(rpcc);
	tcopDebug(1, "AsyncMoveFragment: Waiting for Broker to finish...\n");

	pthread_mutex_init(&brokerDoneLock, pthread_mutexattr_default);
	pthread_cond_init(&brokerDoneCond, pthread_condattr_default);
	pthread_mutex_lock(&brokerDoneLock);
	brokerDoneFlag = FALSE;

	while (!brokerDoneFlag) {
	    pthread_cond_wait(&brokerDoneCond, &brokerDoneLock);
	}
	tcopDebug(1, "AsyncMoveFragment: Broker finished.\n");

	/*
	 * At this point, the broker has returned so we must wait
	 * for the remote exec process to signal that it's ready.
	 */
	RExecWait(&rExecAddr, &rExecPort, NULL);

	/*
	 * The remote process has signaled us with it's address and 
	 * port.  Now, we just call ExecFragmentTake() to move the data.
	 */
	ExecFragmentTake(rExecAddr, rExecPort, 0, frastoreid);

	/*
	 * Clean up the memory allocated for bArg.
	 */
	free(bArg.processingSites.HostList_val);
	free(bArg.recvQepFragIds.QepFragIdList_val);
	free(bArg.recvRtables.RTableList_val);
	pfree(l);   /* XXX - don't know if we really need to do this */
    }
        break;

    case LONG_BUY_FRAGMENT: {   /* Move the fragment to a remote site. */
        Relation fragdesc;
	HeapScanDesc fragscan;
	HeapTuple htup;
        extern unsigned short tcpPort;
	BrokerArg bArg;
	RpcConn *rpcc;
	int rExecAddr, rExecHostId, rExecPort;
	int foundIt = 0;
	List *l = lconsi(frastoreid, NULL);

	/*
	 * Look up the class oid for the fragment.
	 */
	fragdesc = heap_openr((Name)Name_pg_fragment);
	if (!RelationIsValid(fragdesc))
	    elog(WARN, "cannot open pg_fragment");

	fragscan = heap_beginscan(fragdesc,0,SelfTimeQual,0,NULL);
	do {
	    htup = heap_getnext(fragscan, 0, NULL);
	    if (HeapTupleIsValid(htup)) {
	        Form_pg_fragment frag = (Form_pg_fragment)GETSTRUCT(htup);
		if (frag->frastoreid == frastoreid) {
		    foundIt = 1;
		    fraclassid = frag->fraclassid;
		    break;
		}
	    }
	} while (htup);
	heap_endscan(fragscan);
	heap_close(fragdesc);
	if (!foundIt)
	    elog(WARN, "AsynchMoveFragment: unknown fragment %d", (int)frastoreid);

	RExecInit();
	/*
	 * Construct the broker arg
	 */
	bArg.protocol = protocol;
	bArg.nrecv = 1;
	bArg.processingSites.HostList_len = 1;
	bArg.processingSites.HostList_val = (u_int *)calloc(1, sizeof(u_int));
	bArg.processingSites.HostList_val[0] = localSiteHostId;
	/*
	 * The QepFragId isn't really used in buying and selling fragments.
	 * However, the list is accessed directly (i.e., it assumes that its
	 * length is nrecv) by the broker so we have to create the list anyway.
	 */
	bArg.recvQepFragIds.QepFragIdList_len = 1;
	bArg.recvQepFragIds.QepFragIdList_val = (QepFragmentId *)calloc(1, sizeof(QepFragmentId));
	bArg.recvQepFragIds.QepFragIdList_val[0] = 1;
	/*
	 * The broker doesn't use this for buying and selling fragments.
	 */
	bArg.recvPlans.PlanList_len = 0;
	bArg.recvPlans.PlanList_val = NULL;

	bArg.recvRtables.RTableList_len = 1;
	bArg.recvRtables.RTableList_val = (PQA_Int_list *)calloc(1, sizeof(PQA_Int_list));
	bArg.recvRtables.RTableList_val[0] = (PQA_Int_list)l;
	bArg.isSubBid = FALSE;
	bArg.parentContract = 0;
	bArg.callerAddr = getMyAddr();
	bArg.callerPort = tcpPort;
	bArg.callerHostid = localSiteHostId;
	bArg.homeSiteAddr = bArg.callerAddr;
	bArg.homeSitePort = bArg.callerPort;
	bArg.homeSiteHostid = bArg.callerHostid;

	rpcc = Rpc_connectLocalBroker("AsyncMoveFragment", RPC_ONERR_RETURN);
	if (!Rpc_call(rpcc, RPC_BROKER, &bArg, NULL)) {
	    tcopDebug(0, "AsyncMoveFragment: RPC error\n");
	}
	Rpc_disconnect(rpcc);
	tcopDebug(1, "AsyncMoveFragment: Waiting for Broker to finish...\n");

	pthread_mutex_init(&brokerDoneLock, pthread_mutexattr_default);
	pthread_cond_init(&brokerDoneCond, pthread_condattr_default);
	pthread_mutex_lock(&brokerDoneLock);
	brokerDoneFlag = FALSE;

	while (!brokerDoneFlag) {
	    pthread_cond_wait(&brokerDoneCond, &brokerDoneLock);
	}
	tcopDebug(1, "AsyncMoveFragment: Broker finished.\n");

	/*
	 * At this point, the broker has returned so we must wait
	 * for the remote exec process to signal that it's ready.
	 */
	RExecWait(&rExecAddr, &rExecPort, &rExecHostId);

	/*
	 * The remote process has signaled us with it's address and 
	 * port.  Now, we just call ExecFragmentMove() to move the data.
	 */
	ExecFragmentMove(rExecAddr, rExecPort, fraclassid,
			 frastoreid, rExecHostId, NULL);

	/*
	 * Clean up the memory allocated for bArg.
	 */
	free(bArg.processingSites.HostList_val);
	free(bArg.recvQepFragIds.QepFragIdList_val);
	free(bArg.recvRtables.RTableList_val);
	pfree(l);   /* XXX - don't know if we really need to do this */
    }
        break;

    case SHORT_SELL_FRAGMENT: { /* Get the fragment from a remote site. */
        Relation fragdesc;
	HeapScanDesc fragscan;
	HeapTuple htup;
	int foundIt = 0;

	fragdesc = heap_openr((Name)Name_pg_fragment);
	if (!RelationIsValid(fragdesc))
	    elog(WARN, "cannot open pg_fragment");

	fragscan = heap_beginscan(fragdesc,0,SelfTimeQual,0,NULL);
	do {
	    htup = heap_getnext(fragscan, 0, NULL);
	    if (HeapTupleIsValid(htup)) {
	        Form_pg_fragment frag = (Form_pg_fragment)GETSTRUCT(htup);
		if (frag->frastoreid == frastoreid) {
		    foundIt = 1;
		    fraclassid = frag->fraclassid;
		    site = frag->fralocation;
		    break;
		}
	    }
	} while (htup);
	heap_endscan(fragscan);
	heap_close(fragdesc);

	if (!foundIt)
	    elog(WARN, "AsynchMoveFragment: unknown fragment %d", (int)frastoreid);

	FragmentTake(fraclassid, frastoreid, site);
    }
        break;

    case SHORT_BUY_FRAGMENT: {  /* Move the fragment to a remote site. */
        Relation fragdesc;
	HeapScanDesc fragscan;
	HeapTuple htup;
	int foundIt = 0, nsites = 0;

	fragdesc = heap_openr((Name)Name_pg_fragment);
	if (!RelationIsValid(fragdesc))
	    elog(WARN, "cannot open pg_fragment");

	fragscan = heap_beginscan(fragdesc,0,SelfTimeQual,0,NULL);
	do {
	    htup = heap_getnext(fragscan, 0, NULL);
	    if (HeapTupleIsValid(htup)) {
	        Form_pg_fragment frag = (Form_pg_fragment)GETSTRUCT(htup);
		if (frag->frastoreid == frastoreid) {
		    foundIt = 1;
		    fraclassid = frag->fraclassid;
		    break;
		}
	    }
	} while (htup);
	heap_endscan(fragscan);
	heap_close(fragdesc);

	if (!foundIt)
	    elog(WARN, "AsynchMoveFragment: unknown fragment %d", (int)frastoreid);

	fragdesc = heap_openr((Name)Name_pg_host);
	if (!RelationIsValid(fragdesc))
	    elog(WARN, "cannot open pg_host");

	fragscan = heap_beginscan(fragdesc,0,SelfTimeQual,0,NULL);
	do {
	    htup = heap_getnext(fragscan, 0, NULL);
	    if (HeapTupleIsValid(htup))
	        nsites++;
	} while (htup);
	heap_endscan(fragscan);
	heap_close(fragdesc);

	/*
	 * XXX - For lack of a better way to choose the site that will 
	 *       buy the fragment, use the site id and the number of sites 
	 *       to select a site at random.
	 */

	site = ChooseRandomRemoteSite(nsites, localSiteHostId);
	FragmentMove(fraclassid, frastoreid, site);
    }
        break;

    default:
        elog(WARN, "AsynchMoveFragment: Unknown operation %d", (int)protocol);
        break;
    }
#endif
    
}
