/*-------------------------------------------------------------------------
 *
 * broker.h--
 *    
 *
 * Copyright (c) 1994, Regents of the University of California
 *
 * $Id: broker.h,v 1.16 1996/05/31 17:14:35 jsidell Exp $
 *
 *-------------------------------------------------------------------------
 */


#ifndef BROKER_H
#define BROKER_H

#define DEBUG_BROKER 1
#define DEFAULT_REQUEST_STORAGE 64

#define BROKERDEBUGLEVEL 0

#include "include/postgres.h"
#include "executor/execdesc.h" 		/* Contains def. of QueryDesc */
#include "rpc/rpcutils.h"
#include "mariposa_rpc.h"
#include "sitemgr/sitemgr.h"
#include "catalog/pg_proc.h"		/* for ObjectIdEqualRegProcedure */
#include "utils/plans.h"
#include <values.h>
#include <stdarg.h>

#include "access/heapam.h"

/* #include "nodes/plannodes.h" */
#include "nodes/parsenodes.h"

#include "tcop/coord.h"
#include "tcop/frag.h"
#include "tcop/debug.h"
#include "tcop/demo.h"

#include "utils/elog.h"
#include "utils/rel.h"

#include "queries.h"           /* Contains def. of struct SiteAddrList */
#include "brokerRPC.h"
#include "rpccalls.h"
#include "rpctypes.h"          /* Contains def. of struct SiteAddr & SITEMGRPORT*/

/*
 * Broker divides the plan up into plan chunks.
 * This keeps track of the correspondence between the
 * nodes in a plan chunk and the original plan.
 */
typedef struct brokerPlan {
    Plan *planChunk;
    Plan *originalPlan;
} BrokerPlan;

/*
 * Main broker routine.  Called either from ProcessQueryDesc
 * (for top-level bids), from GetQueryBid (for sub-bids), or
 * from AsyncMoveFragment (for top-level bids).
 */
extern void Broker(QueryDesc *qd, List *plans, int protocol, SiteAddr *homeSiteAddr);

extern void SubcontractWon(int bidCollIndex);

extern void SubcontractLost(int bidCollIndex);

extern List *GetBidSites(Plan *plan, List *rtable);
extern void CopySiteAddr(SiteAddr *from, SiteAddr *to);
extern void CopySiteAddrNode(SiteAddrNode *from, SiteAddrNode *to);
extern int newQepFragId();
extern void CopyProcessingSites(Plan *planChunk, Plan *originalPlan);
extern void CopyBudgets(Plan *planChunk, Plan *originalPlan);
extern void CopyContracts(Plan *from, Plan *to);

#endif
