/*
 * One of these for each bid.
 */
typedef struct Bid {
    ContractId contract;
    SiteAddrNode site;
    Budget bid;
    bool bidderContacted;
    bool bidReceived;
    bool subContracted;
    Plan *subContractedPlan;
} Bid;

/*
 * One of these for each query plan chunk or other
 * piece of work to be bid out.
 */
typedef struct BidInfo {
    int nBidders;
    int nBiddersContacted;
    int nBidsReceived;
    Bid *bids;
    bool okBidReceived;         	/* Whether we've received a bid below the bid curve */
    int bestBidIndex;
    bool biddingDone;			/* TRUE if bidding is finished for this bid collection  */
    pthread_mutex_t bidsLock;   	/* Condition variable to wait until all bids have been  */
    pthread_cond_t bidsCond;    	/* received before returning.				*/
                                        
    union {
	ProcessQueryArg longQuery;
	FragXArg longSell;
	FragXArg longBuy;
    } BidDetails_u;
    
} BidInfo;

/*
 * One of these for each request (query, fragment move, etc.) received by Broker()
 */
typedef struct brokerInfo {
    BidProtocol protocol;		/* What we're bidding on: sell frag, process query...   */
    unsigned int requestId;
    int nBidInfo;
    BidInfo *bidInfo;
    int nChunksLeft;			/* How many query fragments left to be bid on */
    pthread_mutex_t biddingDoneLock; 	/* Wait until all the bidding processes for all the query fragments
					   have finished.  This is signalled after decrementing nChunksLeft
					   in SendRequestsForBids, after all the bids for one query fragment
					   have been received. */
    pthread_cond_t biddingDoneCond;

} BrokerInfo;

/* Called from PostgresMain */
extern void InitBroker(void);

extern void LongProtocol(List *bPlans, int protocol, QueryDesc *qd, SiteAddr *homeSiteAddr);

extern unsigned int GetNextrequestId();

/* Called from remote SiteMgr. */
extern void BrokerReceiveBid(void *arg);
void SubContractBroker(void *arg);
