/*-------------------------------------------------------------------------
 *
 * pquery.c--
 *    POSTGRES process query command code
 *
 * Copyright (c) 1994, Regents of the University of California
 *
 *
 * IDENTIFICATION
 *    $Header: /usr/local/devel/mariposa/cvs/src/backend/tcop/pquery.c,v 0.44 1996/06/15 07:22:29 am Exp $
 *
 *-------------------------------------------------------------------------
 */
#ifndef P95
#include <netinet/in.h>

#endif /* P95 */
#include "postgres.h"

#include "tcop/tcopdebug.h"

#include "utils/elog.h"
#include "utils/palloc.h"
#include "utils/mcxt.h"
#include "miscadmin.h"
#include "utils/portal.h"

#ifndef P95
#include "nodes/nodes.h"
#endif /* P95 */
#include "nodes/pg_list.h"
#include "nodes/primnodes.h"
#include "nodes/plannodes.h"
#include "nodes/execnodes.h"
#include "nodes/memnodes.h"

#ifdef P95
#include "tcop/dest.h"

#endif /* P95 */
#include "executor/execdefs.h"
#include "executor/execdesc.h"
#include "executor/executor.h"
#ifndef P95
#include "executor/nodeSort.h"
#include "rpc/rpcutils.h"
#include "tcop/debug.h"
#include "tcop/dest.h"
#endif /* P95 */
#include "tcop/pquery.h"
#ifndef P95
#include "tcop/frag.h"
#include "tcop/broker.h"
#include "tcop/demo.h"
#include "tcop/tcopprot.h"
#include "parser/parsetree.h"
#include "utils/syscache.h"
#include "queries.h"
#include "databroker.h"
#include "catalog/pg_host.h"
#include "catalog/pg_ledger.h"
#include "access/heapam.h"
#include "optimizer/plancat.h"
#endif /* P95 */

#include "commands/command.h"

#ifndef P95
#include "brokerRPC.h"
#else /* P95 */
static char* CreateOperationTag(int operationType);
#endif /* P95 */

#ifndef P95
/*
 * Forward declarations
 */
QueryDesc *CreateQueryDesc(Query *parsetree, Plan *plantree, char **argv, Oid *typev, int nargs, CommandDest dest);
EState *CreateExecutorState();
static String CreateOperationTag(int operationType);
void ProcessPortal(String portalName, Query *parseTree, Plan *plan, EState *state, AttrInfo *attinfo, CommandDest dest);
void TellSiteMgrLost(Oid logicalid);
void ProcessQueryDesc(QueryDesc *qd, EState *state);
void ProcessQuery(Query *parsetree, Plan *plan, char *argv, Oid *typev, int nargs, CommandDest dest);
void SetUpPlans(QueryDesc *qd, List *plans);
void ProcessRemoteQuery(ProcessQueryArg *queryArg);
void CallDataBroker(QueryDesc *queryDesc);

/* Utilities */
void FillInPorts(Plan *plan, unsigned short port, int hostid);
static void FillInPlanRels(Plan *plan, List *rtable);
void FillInFragStoreids(QueryDesc *qd);
void FillInRTStoreids(List *rtfrags);
static void FillInResultRel(QueryDesc *qd);
static void FillInUsedLocally(QueryDesc *qd);
int getAddrFromHostId(int hostid);
static Plan *AddXIn(Plan *plan);
static Plan *AddTupleBuffer(Plan *plan);
static void PlanCopySite(Plan *child, Plan *parent);
static Plan *AddConnectiveNode(Plan *child, Plan *parent, bool *needBuffer);
static void AddConnectiveNodes(Plan *plan, bool *needBuffer);
static List *CreatePlanList(Plan *plan, Plan *parentNode);
static List *FindPlanLeaves(Plan *plan);
static bool AllNodesHaveProcessingSite(Plan *plan);

BidProtocol defaultProtocol = SHORT_PROCESS_QUERY;

time_t tim;

bool _enable_fragmentation_ = true;

pthread_mutex_t brokerDoneLock;
pthread_cond_t brokerDoneCond;
bool brokerDoneFlag;

#endif /* P95 */
/* ----------------------------------------------------------------
 *	CreateQueryDesc
 * ----------------------------------------------------------------
 */
QueryDesc *
CreateQueryDesc(Query *parsetree,
		Plan *plantree,
#ifndef P95
		char **argv,
		Oid *typev,
		int nargs,
#endif /* P95 */
		CommandDest dest)
{
#ifndef P95
    List *arglist;
    List *typelist;
    int i;
#endif /* P95 */
    QueryDesc *qd = (QueryDesc *)palloc(sizeof(QueryDesc));
    
#ifndef P95
    arglist = typelist = NIL;
    
    for (i = 0; i < nargs; i++) {
	/*
	 * XXX argv is really a vector of Datum.  Using lispInteger works
	 *     but semantically it is bs.   mer 30 Jun 1992
	 */
	arglist = lappend(arglist, ((void *)*argv++));
	typelist = lappend(typelist, ((void *)*typev++));
    }

#endif /* P95 */
    qd->operation = parsetree->commandType; 	/* operation */
#ifndef P95
    qd->requestId = GetNextrequestId();		/* requestId */
#endif /* P95 */
    qd->parsetree = parsetree;			/* parse tree */
    qd->plantree  = plantree;			/* plan */
#ifndef P95
    qd->arglist   = arglist;			/* args to query */
    qd->typelist  = arglist;			/* arg types */
    qd->nargs     = nargs;			/* # of args */
#endif /* P95 */
    qd->dest      = dest;			/* output dest */
    return qd;
}

/* ----------------------------------------------------------------
 *	CreateExecutorState
 *
 *	Note: this may someday take parameters -cim 9/18/89
 * ----------------------------------------------------------------
 */
EState *
CreateExecutorState()
{
    EState		*state;
    extern int		NBuffers;
    long		*refcount;
    
    /* ----------------
     *	create a new executor state
     * ----------------
     */
    state = makeNode(EState);
    
    /* ----------------
     *	initialize the Executor State structure
     * ----------------
     */
    state->es_direction = EXEC_FRWD;
    state->es_range_table = NIL;

    state->es_into_relation_descriptor = NULL;
    state->es_result_relation_info = NULL;
    
    state->es_param_list_info = NULL;
    
    state->es_BaseId = 0;
    state->es_tupleTable = NULL;
    
    state->es_junkFilter = NULL;
    
    refcount = (long *) palloc(NBuffers * sizeof(long));
    memset((char *) refcount, 0, NBuffers * sizeof(long));
    state->es_refcount = (int *) refcount;
    
    /* ----------------
     *	return the executor state structure
     * ----------------
     */
    return state;
}

/* ----------------------------------------------------------------
 *	CreateOperationTag
 *
 *	utility to get a string representation of the
 *	query operation.
 * ----------------------------------------------------------------
 */
#ifndef P95
static String
#else /* P95 */
static char*
#endif /* P95 */
CreateOperationTag(int operationType)
{
#ifndef P95
    String tag;
#else /* P95 */
    char* tag;
#endif /* P95 */
    
    switch (operationType) {
    case CMD_SELECT:
	tag = "SELECT";
	break;
    case CMD_INSERT:
	tag = "INSERT";
	break;
    case CMD_DELETE:
	tag = "DELETE";
	break;
    case CMD_UPDATE:
	tag = "UPDATE";
	break;
#ifndef P95
    case CMD_EXCHOUT:
	tag = "EXCHOUT";
	break;
#endif /* P95 */
    default:
	elog(DEBUG, "CreateOperationTag: unknown operation type %d", 
	     operationType);
	tag = NULL;
	break;
    }
    
    return tag;
}

/* ----------------
 *	ProcessPortal
 * ----------------
 */

void
#ifndef P95
ProcessPortal(String portalName,
#else /* P95 */
ProcessPortal(char* portalName,
#endif /* P95 */
	      Query *parseTree,
	      Plan *plan,
	      EState *state,
#ifndef P95
	      AttrInfo *attinfo, 
#else /* P95 */
	      TupleDesc attinfo, 
#endif /* P95 */
	      CommandDest dest)
{
    Portal		portal;
    MemoryContext 	portalContext;
    
    /* ----------------
     *   convert the current blank portal into the user-specified
     *   portal and initialize the state and query descriptor.
     * ----------------
     */
    
    if (PortalNameIsSpecial(portalName))
	elog(WARN,
	     "The portal name %s is reserved for internal use",
	     portalName);
    
    portal = BlankPortalAssignName(portalName);
    
    PortalSetQuery(portal,
#ifndef P95
		   CreateQueryDesc(parseTree, plan, (char **) NULL,
				   (Oid *) NULL, 0, dest),
#else /* P95 */
		   CreateQueryDesc(parseTree, plan, dest),
		   attinfo,
#endif /* P95 */
		   state,
		   PortalCleanup);
    
    /* ----------------
     *	now create a new blank portal and switch to it. 
     *	Otherwise, the new named portal will be cleaned.
     *
     *  Note: portals will only be supported within a BEGIN...END
     *  block in the near future.  Later, someone will fix it to
     *  do what is possible across transaction boundries. -hirohama
     * ----------------
     */
    portalContext = (MemoryContext)
	PortalGetHeapMemory(GetPortalByName(NULL));
    
    MemoryContextSwitchTo(portalContext);
    
    StartPortalAllocMode(DefaultAllocMode, 0);
}

#ifndef P95
void
TellSiteMgrLost(Oid logicalid) {
    
    RpcConn *rpcc = Rpc_connectSitemgr(localSiteHostId, 0, "TellSiteMgrLost", RPC_ONERR_WARN);
#endif /* P95 */

#ifndef P95
    Rpc_call(rpcc, RPC_TELLLOST, &logicalid, NULL);

}

#endif /* P95 */
/* ----------------------------------------------------------------
 *	ProcessQueryDesc
 *
 *	Read the comments for ProcessQuery() below...
 * ----------------------------------------------------------------
 */
void
#ifndef P95
ProcessQueryDesc(QueryDesc *qd, EState *state)
#else /* P95 */
ProcessQueryDesc(QueryDesc *queryDesc)
#endif /* P95 */
{
    Query 	*parseTree;
    Plan 	*plan;
    int		operation;
#ifndef P95
    String	tag;
    AttrInfo 	*attinfo;
#else /* P95 */
    char*	tag;
    EState 	*state;
    TupleDesc   attinfo;
#endif /* P95 */
    
    bool	isRetrieveIntoPortal;
    bool	isRetrieveIntoRelation;
#ifndef P95
    String	intoName;
#else /* P95 */
    char*	intoName;
#endif /* P95 */
    CommandDest dest;
#ifndef P95
    int i;
    char **argv;
	
#endif /* P95 */
    
#ifndef P95
    /*
     * Fix up range table entries so we can access data fragments
     * used at this site.
     */
    FillInFragStoreids(qd);

#endif /* P95 */
    /* ----------------
     *	get info from the query desc
     * ----------------
     */
#ifndef P95
    parseTree = qd->parsetree;
    plan =	qd->plantree;
#else /* P95 */
    parseTree = queryDesc->parsetree;
    plan =	queryDesc->plantree;
#endif /* P95 */
    
#ifndef P95
    operation = qd->operation;
#else /* P95 */
    operation = queryDesc->operation;
#endif /* P95 */
    tag = 	CreateOperationTag(operation);
#ifndef P95
    dest = 	qd->dest;
#else /* P95 */
    dest = 	queryDesc->dest;
#endif /* P95 */
    
    /* ----------------
     *	initialize portal/into relation status
     * ----------------
     */
    isRetrieveIntoPortal =   false;
    isRetrieveIntoRelation = false;
    
    if (operation == CMD_SELECT) {
	if (parseTree->isPortal) {
	    isRetrieveIntoPortal = true;
	    intoName = parseTree->into;
	    if (parseTree->isBinary) {
		/*
		 * For internal format portals, we change Remote
		 * (externalized form) to RemoteInternal (internalized
		 * form)
		 */
#ifndef P95
		dest = qd->dest = RemoteInternal;
#else /* P95 */
		dest = queryDesc->dest = RemoteInternal;
#endif /* P95 */
	    }
	} else if (parseTree->into != NULL) {
	    /* select into table */
	    isRetrieveIntoRelation = true;
	}

    }
#ifndef P95

#else /* P95 */
    
#endif /* P95 */
    /* ----------------
     *	when performing a retrieve into, we override the normal
     *  communication destination during the processing of the
     *  the query.  This only affects the tuple-output function
     *  - the correct destination will still see BeginCommand()
     *  and EndCommand() messages.
     * ----------------
     */
    if (isRetrieveIntoRelation)
#ifndef P95
    	qd->dest = (int) None;
          
#else /* P95 */
    	queryDesc->dest = (int) None;
    
#endif /* P95 */
    /* ----------------
	 * [Mariposa]
     * If there is a plan to be processed locally, call
     * ExecStart to prepare the plan for execution.  This includes
     * initializing the channels in the ExchangeIn nodes and, if
     * this is a CMD_EXCHOUT query, the outgoing channels.
     *
     * The local plan can be NULL if we're doing a remote INSERT,
     * UPDATE or DELETE.
	 * [Postgres95]
     *	create a default executor state.. 
     * ----------------
     */
#ifndef P95

    if (qd->plantree != NULL) {
	attinfo = ExecutorStart(qd, state);
    } else {
	/*
	 * we need to fake something up so that the routines that rely
	 * on having attinfo don't explode..
	 */
	static AttrInfo dummyAttrInfo = { 0, (AttributeTupleForm *) NULL };

	attinfo = &dummyAttrInfo;
    }
#else /* P95 */
    state = CreateExecutorState();
#endif /* P95 */
    
#ifndef P95
    if (scheduleQueries) {
	informScheduler(AccountNode (qd->plantree));
    } 

    RExecInit();		/* must be before remote execs are started */
    
    
    /*
     * Start processing the remote parts of the query.
	 */
#else /* P95 */
    /* ----------------
     *	call ExecStart to prepare the plan for execution
     * ----------------
	 */
#endif /* P95 */
#ifndef P95
    if (qd->remoteplans != NIL) {
	if (qd->parsetree->commandType == CMD_SELECT ||
	    qd->parsetree->commandType == CMD_INSERT ||
	    qd->parsetree->commandType == CMD_DELETE ||
	    qd->parsetree->commandType == CMD_UPDATE) {
	    SendOffers(qd);
	}
    }
	
#else /* P95 */
    attinfo = ExecutorStart(queryDesc, state);
    
#endif /* P95 */
    /* ----------------
     *   report the query's result type information
     *   back to the front end or to whatever destination
     *   we're dealing with.
     * ----------------
     */
    BeginCommand(NULL,
		 operation,
		 attinfo,
		 isRetrieveIntoRelation,
		 isRetrieveIntoPortal,
		 tag,
		 dest);
    
    /* ----------------
     *  Named portals do not do a "fetch all" initially, so now
     *  we return since ExecMain has been called with EXEC_START
     *  to initialize the query plan.  
     *
     *  Note: ProcessPortal transforms the current "blank" portal
     *        into a named portal and creates a new blank portal so
     *	      everything we allocated in the current "blank" memory
     *	      context will be preserved across queries.  -cim 2/22/91
     * ----------------
     */
    if (isRetrieveIntoPortal) {
	PortalExecutorHeapMemory = NULL;
	
	ProcessPortal(intoName,
		      parseTree,
		      plan,
		      state,
		      attinfo,
		      dest);
	
	EndCommand(tag, dest);
	return;
    }
    
    /* ----------------
     *   Now we get to the important call to ExecutorRun() where we
     *   actually run the plan..
     * ----------------
     */
#ifndef P95

    if (qd->plantree != NULL) {
	
	ExecutorRun(qd, state, EXEC_RUN, 0);
#else /* P95 */
    ExecutorRun(queryDesc, state, EXEC_RUN, 0);
#endif /* P95 */
    
#ifndef P95
	/* ----------------
	 *   now, we close down all the scans and free allocated resources...
	 * with ExecutorEnd()
	 * ----------------
	 */
	ExecutorEnd(qd, state);

	/*
	 * Enter into pg_ledger how much we made on the local plan
	 * and how much we spent on remoteplans.
	 */
	/*MakeLedgerEntries(qd);*/

	/*
	 * Make RPC call to sitemgr to call the databroker Tcl script.
	 */
	CallDataBroker(qd);
	
    }
#else /* P95 */
    /* ----------------
     *   now, we close down all the scans and free allocated resources...
     * with ExecutorEnd()
     * ----------------
     */
    ExecutorEnd(queryDesc, state);
#endif /* P95 */
    
    /* ----------------
     *  Notify the destination of end of processing.
     * ----------------
     */
    EndCommand(tag, dest);
#ifndef P95
    
#endif /* P95 */
}

/* ----------------------------------------------------------------
 *	ProcessQuery
 *
 * [Mariposa]
 *	Execute a plan, the non-parallel version.
 *
 *	Call the Fragmenter, then the Query Broker,
 * 	filling in the range table and plan tree,
 * 	then fill in the appropriate fields in the
 * 	query descriptor.
 * [Postgres95]
 *	Execute a plan, the non-parallel version
 * ----------------------------------------------------------------
 */

void
ProcessQuery(Query *parsetree,
	     Plan *plan,
#ifndef P95
	     char *argv,
#else /* P95 */
	     char *argv[],
#endif /* P95 */
	     Oid *typev,
	     int nargs,
	     CommandDest dest)
{
#ifndef P95
    EState *state;
    List *plans;
    QueryDesc *qd;
    extern bool demo;
    bool needBuffer;
    extern int ShowStats;
    
    qd = CreateQueryDesc(parsetree, plan, (char**) argv, typev, nargs, dest);
#else /* P95 */
    QueryDesc *queryDesc;
    extern int dontExecute; /* from postgres.c */
    extern void print_plan (Plan* p, Query* parsetree); /* from print.c */
#endif /* P95 */

#ifndef P95
    qd->protocol = defaultProtocol;
#else /* P95 */
    queryDesc = CreateQueryDesc(parsetree, plan, dest);
#endif /* P95 */

#ifndef P95
    /*
     * Call the Fragmenter.  First, "fragment" the range table, meaning
     * fill in all the storage fragment information.  Then, send the
     * fragmenter the query descriptor.  It will send back a list of
     * plans:
     * 
     * For SELECT's, there will be one plan in the plan list,
     * since the results of a SELECT query always come back to the
     * home site (even if it's a SELECT INTO... the result table is
     * always local).  The Fragmenter will place a MERGE node at the
     * top of the plan tree if necessary.
     *
     * For UPDATE's and DELETE's, there may be many plan
     * fragments in the list, corresponding to the different places
     * that UPDATE's and DELETE's can happen.  UPDATE's and
     * DELETE's are just Sequential Scan plans with operation set
     * to CMD_UPDATE or CMD_DELETE and the rest of the information
     * necessary to process the query in the plantree.  So, they
     * are fragmented just like a sequential scan for a SELECT
     * query, except the results are not joined together with a
     * MERGE node.
     *
     * INSERT's are a little weird.  For queries of the form
     * "INSERT INTO foo VALUES (1);" there is exactly one node
     * in the plan tree: a RESULT node.  For queries of the
     * form "INSERT INTO foo SELECT x FROM bar;" the plan tree
     * is, not surprisingly, a sequential scan on bar.
     * Operation is CMD_INSERT for both types of plan.
     *
     * Sooooo... INSERT's are handled by fragmenting the plan
     * tree (RESULT nodes aren't fragmented) and then letting
     * the Broker decide where the INSERT's will happen.  It
     * records this information inside the range table fragment
     * information in a list of boolean's called processYN.
     * The nth entry in processYN is TRUE if the nth entry in
     * the storage fragment list is the one that will receive
     * the INSERT.  Otherwise, processYN is FALSE.
     *
     *		JPS 2/4/96
     */
    
    if (demo) {
	demoPrintMsg("DEMO FRAGMENTER START\n");
    }
    
    if (_enable_fragmentation_ == true) {
	FragmentRTable(qd->parsetree->rtable);
	plans = FragmentQuery(qd);
    }

    if (demo) {
	time(&tim);
	demoPrintMsg("DEMO FRAGMENTER END {%s}\n", noCR(ctime(&tim)));
	printf("DEMO FRAGPLAN ");
	printf("DEMO RTABLE\n");
	print_rt_with_fraginfo(parsetree->rtable);
	/*demoPrintPlanTree(qd->plantree, qd->parsetree->rtable, qd->plantree->qepFragId, FALSE);*/
	fflush(stdout);
    }


    /*
     * The Broker fills in the processingSite field in
     * each plan node in the plan tree.  It also fills
     * in the processYN fields inside the fragment information
     * in the range table.  See above comment for the
     * Fragmenter.  	JPS 2/4/96
     */
    
    if (qd->operation == CMD_SELECT ||
	qd->operation == CMD_INSERT ||
	qd->operation == CMD_DELETE ||
	qd->operation == CMD_UPDATE) {

	bool CanRunQuery = TRUE;
	List *lp;
	SiteAddr homeSiteAddr;
	extern unsigned short tcpPort;
	
	if (demo) {
	    demoPrintMsg("DEMO BROKER START\n");
	}

	homeSiteAddr.addr = getMyAddr();
	homeSiteAddr.port = tcpPort;
	homeSiteAddr.hostid = getMyHostId();
	Broker(qd, plans, defaultProtocol, &homeSiteAddr);

	if (demo) {
	    demoPrintMsg("DEMO BROKER END BIDSRECEIVED\n");
	}

	/*--------------------------
	 * If the Broker did not fill in a processing site
	 * for a plan node, it means there was no site that
	 * could process that node under the bid curve.
	 */
	foreach (lp, plans) {
	    Plan *plan = (Plan *)lfirst(lp);
	    if (!AllNodesHaveProcessingSite(plan)) {
		CanRunQuery = FALSE;
		break;
	    }
	}
	
	if (!CanRunQuery) {
	    char *tag = "*ABORT STATE*";
	    elog(NOTICE, "Query could not be performed under bid curve.");
	    EndCommand(tag, dest);
	    return;
	}
	/*--------------------------
	 */

	
	/*
	 * Broker is finished figuring out which sites will process
	 * the query fragments.  It recorded this information
	 * in processingSite in each node of the query plan.  The Broker has
	 * notified the losing sites, but not the winning sites.
	 */    

	/*
	 * Now that the Broker has filled in the processing sites, add
	 * ExchangeIn and TupleBuffer nodes between parts of the plans
	 * that are processed at different sites.
	 */

	foreach (lp, plans) {
	    Plan *plan = (Plan *)lfirst(lp);
	    AddConnectiveNodes(plan, &needBuffer);
	}


	/*-------------------------------------------
	 * Set up the local and remote plans.
	 * The local subplan is the subplan starting at
	 * the root and including all the nodes processed
	 * at the home site.
	 */

	SetUpPlans(qd, plans);
	
    }
    
    /* ----------------
     *	create a default executor state.. 
     * ----------------
     */
    state = CreateExecutorState();
    
    ProcessQueryDesc(qd, state);
    if (ShowStats) {
	fprintf(stdout, "! RequestId %i, qepFragId %i", qd->requestId, plan->qepFragId);
    }
    
#else /* P95 */
    if (dontExecute) {
	/* don't execute it, just show the query plan */
	print_plan(plan, parsetree);
    } else
	ProcessQueryDesc(queryDesc);
#endif /* P95 */
}

#ifndef P95
/* ----------------------------------------------------------------
 *	SetUpPlans--
 *
 *	Fill in the local and remote plan trees in the
 *	query descriptor.
 *
 * ----------------------------------------------------------------
 */
void
SetUpPlans(QueryDesc *qd, List *plans) {

    List *lp;
    
    switch (qd->parsetree->commandType) {

	/*
	 * If we're doing a SELECT, there is one plan tree
	 * in the list of plans.  The top node in the plan tree
	 * must be processed at the home site, so we add an
	 * X-IN node if the top node in the plan is processed
	 * remotely.  We'll save the distributed plan tree (the
	 * entire plan tree with X-IN nodes and remote sub-plans),
	 * put the remote sub-plans into queryDesc->remoteplans,
	 * put the local sub-plan connected to the root into
	 * queryDesc->plantree and then process the query.
	 */
    case CMD_SELECT: {
	qd->plantree = (Plan *)lfirst(plans);
	    
	/*
	 * Add an exchange-in node as the top node if the
	 * top node is processed at a remote site.
	 */
	if (qd->plantree->processingSite.hostid != getMyHostId()) {
	    qd->plantree = (Plan *)AddXIn(qd->plantree);
	    qd->plantree->processingSite.type = T_SiteAddrNode;
	    qd->plantree->processingSite.hostid = getMyHostId();
	    qd->plantree->processingSite.addr = getMyAddr();
	    qd->plantree->processingSite.port = tcpPort;
	    
	    /*
	     * Fill in the qepFragId for the X-IN node so that,
	     * if we're sending out to another site (that is, processing
	     * a remote subplan) the sending qepFragId will match up
	     * with what the receiving site is expecting.
	     *
	     * This is technically incorrect - the qepFragId should
	     * probably be different, since the X-IN node and the
	     * node below it are processed at different sites.
	     */
	    qd->plantree->qepFragId = qd->plantree->lefttree->qepFragId;
	    
	}
    
	if (demo) {
	    printf("DEMO FULLPLAN ");
	    demoPrintPlanTree(qd->plantree, qd->parsetree->rtable, qd->plantree->qepFragId, TRUE);
	    fflush(stdout);
	}

	FillInPorts(qd->plantree, tcpPort, getMyHostId());

	/*
	 * Save the distributed plan.
	 */
	qd->distPlanList = lcons(qd->plantree, NIL);
    
	/*
	 * Construct a list of CoordPlans to be sent out.
	 * CoordPlan's have two pointers: one to the plan
	 * fragment, one to its parent node.  The Coordinator
	 * uses this information to determine where intermediate
	 * results should be sent.
	 *
	 */
	qd->remoteplans = CreatePlanList(qd->plantree, NULL);

	/*
	 * Now make qd->plantree include only the subtree starting at
	 * the root and including any nodes processed locally.  This
	 * is the first entry in the list of plans.
	 */
	qd->plantree = ((CoordPlan *)lfirst(qd->remoteplans))->plan;
	
	/*
	 * The remote plans are in the rest of the list.
	 */
	qd->remoteplans = qd->remoteplans->next;
	    
    } /* end case CMD_SELECT: */
	break;

	/*
	 * If we're doing a CMD_INSERT, there may be multiple plans
	 * in the list of plans.  This happens for queries of the
	 * form "INSERT INTO foo <SUBQUERY>;".  The subquery may be
	 * decomposed into multiple subplans, some of which may be
	 * processed remotely, some locally.  Furthermore, the
	 * result relation, 'foo', may be fragmented with some fragments
	 * remote and others local.  To simplify this situation,
	 * all the subplans are treated as remote plans.  This makes
	 * each subplan conceptually look like:
	 *
	 *		  ^
	 *		  |
	 *		X-OUT
	 *		  |
	 *	     <QUERY PLAN>
	 *		  |
	 *		X-IN
	 *		  ^
	 *		  |
	 *
	 * However, this also makes queries of the form
	 * "INSERT INTO foo VALUES (1);" run REALLY slowly.
	 * Same thing for "INSERT INTO foo <SUBQUERY>;"
	 * if foo and the relations referenced in the
	 * subquery are all local.  So the optimization below
	 * handles the case where there is exactly one receive
	 * site and it is the home site.  Actually, the way
	 * that CMD_EXCHOUT's and getNextTuple() for X-IN's work
	 * should be overhauled.  See Bug #1.  JPS 2/21/96
	 */
	     
    case CMD_INSERT: {

	qd->plantree = NULL;

	qd->distPlanList = plans;

	qd->remoteplans = NIL;
	    
	foreach(lp, plans) {
	    Plan *plan = (Plan *)lfirst(lp);
	    qd->remoteplans = nconc(CreatePlanList(plan, NULL), qd->remoteplans);
	}
	     
	if (length(qd->remoteplans) == 1) {
	    CoordPlan *cp = (CoordPlan *)lfirst(qd->remoteplans);
	    if (cp->plan->processingSite.hostid == getMyHostId()) {
		/*
		 * If there is only one receive site and it is the home site,
		 * just process the plan locally.
		 */
		RangeTblEntry *rte = rt_fetch(qd->parsetree->resultRelation, qd->parsetree->rtable);
		bool processLocally;
		if (length(rte->fragInfo) > 1) {
		    processLocally = FALSE;
		} else {
		    FragInfo *fInfo = (FragInfo *)lfirst(rte->fragInfo);
		    List *lp2 = fInfo->processYN;
		    processLocally = FALSE;
		    foreach(lp, fInfo->storageSites) {
			SiteAddrNode *saddr = (SiteAddrNode *)lfirst(lp);
			bool processYN = (bool) lfirsti(lp2);
			if (processYN && saddr->hostid == cp->plan->processingSite.hostid) {
			    processLocally = TRUE;
			    break;
			}
			lp2 = lp2->next;
		    }
		}
		if (processLocally) {
		    qd->plantree = cp->plan;
		    qd->remoteplans = NIL;
		}
	    }
	}
	
    } /* end case CMD_INSERT: */
	    
	break;

	/*
	 * UPDATE's and DELETE's are also simplified like INSERT's.
	 * This makes UPDATE's and DELETE's on relations that are
	 * stored locally really slow.  For unfragmented, local
	 * relations (wouldn't work for fragmented, local relations -
	 * think about it) we should optimize this.
	 *
	 *			JPS 2/4/96
	 */
    case CMD_UPDATE:
    case CMD_DELETE: {
	    
	qd->plantree = NULL;

	qd->distPlanList = plans;

	qd->remoteplans = NIL;
	    
	foreach(lp, plans) {
	    Plan *plan = (Plan *)lfirst(lp);
	    qd->remoteplans = nconc(CreatePlanList(plan, NULL), qd->remoteplans);
	}
	     
	if (length(qd->remoteplans) == 1) {
	    CoordPlan *cp = (CoordPlan *)lfirst(qd->remoteplans);
	    if (cp->plan->processingSite.hostid == getMyHostId()) {
		qd->plantree = cp->plan;
		qd->remoteplans = NIL;
	    }
	}
	     
	    
    } /* end case CMD_UPDATE, CMD_DELETE: */
	    

    default: {
    }
	    
	
    } /* switch (qd->operation) */
}

/*
 *----------------------------------------------------------------
 *	IndexForSort--
 *
 *	Change Scan->Sort->Scan into IndexScan when possible.
 *
 *	Recursively run through plan tree, looking for sort
 *	nodes.  When we find one, see if it's over a base
 *	relation.  If so, find out which attribute(s) is/are
 *	used for the sort.  Look for an index over the same
 *	attribute(s).  If one exists, substitute an index scan.
 *
 *	
 *----------------------------------------------------------------
 */
void
IndexForSort(QueryDesc *qd, Plan *plan, Plan *parentPlan, int planIndex) {
    
    switch (plan->type) {
    case T_Merge: {
	List *p;
	int i = 0;
	foreach(p, ((Merge *) plan)->mergeplans) {
	    IndexForSort(qd, (Plan *)p, plan, i);
	    i++;
	}
    }
	break;
    case T_Append: {
	List *p;
	int i = 0;
	foreach(p, ((Append *) plan)->unionplans) {
	    IndexForSort(qd, (Plan *)p, plan, i);
	    i++;
	}
    }
	break;

    case T_SeqScan: {
	if (plan->lefttree != NULL) {
	    if (plan->lefttree->type == T_Sort) {
		Sort *sortPlan = (Sort *)plan->lefttree;
		
		/*
		 * If have SeqScan->Sort->SeqScan(base-relation)
		 */
		if (sortPlan->plan.lefttree->type == T_SeqScan &&
		    ((SeqScan *)sortPlan->plan.lefttree)->scanrelid > 0) {
		    SeqScan *ss = (SeqScan *)sortPlan->plan.lefttree;
		    ScanKey sortKeys = FormSortKeys(sortPlan);
		    int nSortKeys = sortPlan->keycount;
		    IdxInfoRetval indexInfo, dummy;
		    bool moreIndexInfo = FALSE;
		    int i, j;
		    bool foundIndex = FALSE;
		    bool foundAttribute;
		    
		    /*
		     * Look through the available indices for this relation
		     */
		    moreIndexInfo = index_info(qd->parsetree->rtable, TRUE, ss->scanrelid, &indexInfo);

		    while (moreIndexInfo) {
			foundIndex = TRUE;

			/*
			 * Determine if the set of attributes in the SORT node is
			 * exactly equal to the set in the index.
			 */
			for (i = 0; i < nSortKeys; i++) {
			    foundAttribute = FALSE;
			    for (j = 0; j < 8 && indexInfo.indexkeys[j] != 0; j++) {
				if (sortKeys[i].sk_attno == indexInfo.indexkeys[j]) {
				    foundAttribute = TRUE;
				    break;
				}
			    }
			    if (!foundAttribute) {
				foundIndex = FALSE;
				break;
			    }
			}

			if (foundIndex) {
			    break;
			}
			
			moreIndexInfo = index_info(qd->parsetree->rtable, FALSE, ss->scanrelid, &indexInfo);
		    }

		    /*
		     * Keep calling index_info until it returns false so that it
		     * can close pg_index and clean up after itself.
		     */
		    while (moreIndexInfo) {
			moreIndexInfo = index_info(qd->parsetree->rtable, FALSE, ss->scanrelid, &dummy);
		    }

		    /*
		     * Found an index that can be used in place of
		     * the sort.  Replace SS->Sort->SS with IndexScan.
		     *
		     * NB - some day, should check for indices matching plan->qual
		     * as well as just indices matching sorts.
		     */
		    if (foundIndex) {
			IndexScan *newIndexNode = makeNode(IndexScan);
			newIndexNode->scan.plan.cost = indexInfo.tuples;
			CopySiteAddrNode(&plan->processingSite, &newIndexNode->scan.plan.processingSite);
			newIndexNode->scan.plan.qepFragId = plan->qepFragId;
			newIndexNode->scan.plan.state = NULL;
			newIndexNode->scan.plan.targetlist = copyObject(ss->plan.targetlist);
			newIndexNode->scan.plan.qual = copyObject(plan->qual);
			newIndexNode->scan.plan.lefttree = NULL;
			newIndexNode->scan.plan.righttree = NULL;
			newIndexNode->scan.scanrelid = ss->scanrelid;
			newIndexNode->scan.scanfragid = ss->scanfragid;
			newIndexNode->scan.scanstate = NULL;
			newIndexNode->scan.availState = ss->availState;
			newIndexNode->indxid = lconsi(indexInfo.relid, NIL);
			newIndexNode->indxqual = lcons(NULL, NIL);
			newIndexNode->indxstate = NULL;

			if (parentPlan != NULL) {
			    switch(parentPlan->type) {
			    case T_Merge: {
				set_nth(((Merge *)parentPlan)->mergeplans, planIndex, (void *)newIndexNode);
			    }
				break;

			    case T_Append: {
				set_nth(((Append *)parentPlan)->unionplans, planIndex, (void *)newIndexNode);
			    }
				break;

			    default: {
				if (planIndex == 0) {
				    parentPlan->lefttree = (Plan *)newIndexNode;
				} else {
				    parentPlan->righttree = (Plan *)newIndexNode;
				}
			    }
				break;
			    }
			} else {
			    qd->plantree = (Plan *)newIndexNode;
			}
			
		    }
		    
		} else {
		    IndexForSort(qd, plan->lefttree->lefttree, plan->lefttree, 0);
		}
	    } else {
		IndexForSort(qd, plan->lefttree, plan, 0);
	    }
	}
    }
	break;
	
    default:
	if (plan->lefttree != NULL) {
	    IndexForSort(qd, plan->lefttree, plan, 0);
	}

	if (plan->righttree != NULL) {
	    IndexForSort(qd, plan->righttree, plan, 1);
	}
	
	break;
    }

    
}

/*
 *----------------------------------------------------------------
 *	Reoptimize--
 *
 *	Right now just calls IndexForSort.
 *
 *----------------------------------------------------------------
 */
void
Reoptimize(QueryDesc *qd) {
    IndexForSort(qd, qd->plantree, NULL, 0);
}

/* ----------------------------------------------------------------
 *	ProcessRemoteQuery--
 *
 *	We are a remote processing site.  Create an executor
 *	state and a query descriptor and then call
 *	ProcessQueryDesc().
 * ----------------------------------------------------------------
 */

void
ProcessRemoteQuery(ProcessQueryArg *queryArg) {
    
    EState *state;
    QueryDesc localQueryDesc;
    Query *parsetree;
    List *rtable = (List *)queryArg->rtable;
    Plan *plan = (Plan *)queryArg->plan;
    List *recvSites = (List *)queryArg->recvSites;
    List *recvQepFragIds = (List *)queryArg->recvQepFragIds;
    int i;
    List *lp;
    bool needBuffer;
    extern int ShowStats;
    
    elog(NOTICE, "PROCESSREMOTEQUERY: RequestID = %i, Plan Fragment = %i", queryArg->requestId, plan->qepFragId);

    if (ShowStats)
	ResetUsage();
    
    /*
     * Create an executor state from the information
     * passed in the ProcessQueryArg.
     */
    state = CreateExecutorState();
    state->es_qepFragId = plan->qepFragId;
    state->es_nRecvSites = length(recvSites);
    state->es_recvSites.SiteAddrList_len = state->es_nRecvSites;
    state->es_recvQepFragIds.QepFragIdList_len = length(recvQepFragIds);
    state->es_partitionMode = queryArg->partitionMode;

    /*
     * Fill in the necessary parts of the parsetree:
     * right now, just the range table, result
     * relation information and commandType.
     */
    parsetree = makeNode(Query);
    parsetree->rtable = rtable;
    parsetree->resultRelation = queryArg->resultRelation;
    parsetree->resultFragment = queryArg->resultFragment;
    parsetree->commandType = queryArg->operation;
    
    /*
     * Create the local query descriptor - the
     * one that this executor will run.  Because
     * of subcontracting, the plan tree may contain
     * sections that are to be processed elsewhere.
     * We need to add in X-IN nodes where they're
     * needed, and create a list of remoteplans
     * if there are any.
     */
    localQueryDesc.parsetree = parsetree;
    localQueryDesc.plantree = (Plan *) queryArg->plan;
    localQueryDesc.protocol = queryArg->protocol;
    localQueryDesc.requestId = queryArg->requestId;
    
    /*
     * Put X-IN and Buffer nodes in the appropriate places.
     */
    AddConnectiveNodes(localQueryDesc.plantree, &needBuffer);

    /*
     * Set up the local and remote plans.
     * The local subplan is the subplan starting at
     * the root and including all the nodes processed
     * at the home site.
     */
    if (queryArg->operation == CMD_SELECT) {
	lp = lcons(localQueryDesc.plantree, NIL);
	SetUpPlans(&localQueryDesc, lp);
    } else {
	localQueryDesc.remoteplans = NIL;
    }
    
	

    /*
     * If there are any receive sites, then we make the operation
     * CMD_EXCHOUT.  Otherwise, this is (one of) the final
     * destination(s) for the query, so the operation is whatever
     * was passed in.  If this is an EXCHOUT, fill in the receive
     * sites and receive qepFragId's.
     */
    if (state->es_nRecvSites > 0) {
	
	localQueryDesc.operation = CMD_EXCHOUT;
	
	/*
	 * This is kind of stupid.  Up until ProcessRemoteQuery, the
	 * recvSites and qepFragIds have been kept in lists.  However,
	 * the code path from here through the executor uses arrays.
	 * So we convert here from lists to arrays.  Eventually, the
	 * code path should contain only lists.
	 */
	state->es_recvSites.SiteAddrList_val = calloc(state->es_nRecvSites, sizeof(SiteAddr));
	state->es_recvQepFragIds.QepFragIdList_val = calloc(state->es_recvQepFragIds.QepFragIdList_len, sizeof(int));
     
	i = 0;
	foreach(lp, recvSites) {
	    state->es_recvSites.SiteAddrList_val[i].addr = ((SiteAddrNode *)(lfirst(lp)))->addr;
	    state->es_recvSites.SiteAddrList_val[i].port = ((SiteAddrNode *)(lfirst(lp)))->port;
	    state->es_recvSites.SiteAddrList_val[i].hostid = ((SiteAddrNode *)(lfirst(lp)))->hostid;
	    i++;
	}

	i = 0;
	foreach(lp, recvQepFragIds) {
	    state->es_recvQepFragIds.QepFragIdList_val[i] = (int)(lfirst(lp));
	    i++;
	}
     
    } else {
	localQueryDesc.operation = queryArg->operation;
    }

    /*
     * Re-optimize the local part of the query plan
     */
    
    if (1) {
	printf("DEMO LOCAPLAN1 ");
	demoPrintPlanTree(localQueryDesc.plantree, localQueryDesc.parsetree->rtable, localQueryDesc.plantree->qepFragId, TRUE);
	fflush(stdout);
    }
    
/*     Reoptimize(&localQueryDesc); */
    
    if (1) {
	printf("DEMO LOCALPLAN2 ");
	demoPrintPlanTree(localQueryDesc.plantree, localQueryDesc.parsetree->rtable, localQueryDesc.plantree->qepFragId, TRUE);
	fflush(stdout);
    }
    
    StartTransactionCommand();
    ProcessQueryDesc(&localQueryDesc, state);
    CommitTransactionCommand();
    if (ShowStats) {
	fprintf(stdout, "! RequestId %i, qepFragId %i", localQueryDesc.requestId, plan->qepFragId);
	ShowUsage();
    }
}

/*
 * ----------------------------------------------------------------
 *	CallDataBroker--
 *
 *     	RPC the sitemgr at this site to call the DataBroker Tcl script.
 * ----------------------------------------------------------------
 */

void
CallDataBroker(QueryDesc *qd) {
    DataBrokerArg dbArg;
    RPCErrorCode result;
    RpcConn *rpcc;
    bool allSysCatalogs = TRUE;
    List *lp;


    /*
     * Don't call data broker for system catalogs
     */
    foreach(lp, qd->parsetree->rtable) {
	RangeTblEntry *rte = (RangeTblEntry *)lfirst(lp);
	if (!issystem(rte->relname->data)) {
	    allSysCatalogs = FALSE;
	    break;
	}
    }
    
    if (!allSysCatalogs) {
	rpcc = Rpc_connectSitemgr(localSiteHostId, localSiteAddr, "CallDataBroker", RPC_ONERR_WARN);
	if (rpcc == NULL) {
	    elog(NOTICE, "CallDataBroker: can't connect to local sitemgr\n");
	    return;
	}
    
	dbArg.plan = (PQA_Plan_tree)lfirst(qd->distPlanList);
	dbArg.rtable = (PQA_Node_list)qd->parsetree->rtable;

	if (!Rpc_call(rpcc, RPC_DATABROKER, &dbArg, &result)) {
	    elog(NOTICE, "CallDataBroker: RPC error\n");
	}
    }
}

/*
 *======================================================================================
 *				      UTILITY ROUTINES
 *======================================================================================
 */

void
FillInPorts(Plan *plan, unsigned short port, int hostid) {

    if (plan->processingSite.hostid == hostid) {
	plan->processingSite.port = port;

	if (plan->type == T_Merge) {
	    List *lp;
	    foreach(lp, ((Merge *)plan)->mergeplans) {
		Plan *p = (Plan *)lfirst(lp);
		FillInPorts(p, port, hostid);
	    }
	
	} else {
	    if (plan->lefttree != NULL) {
		FillInPorts(plan->lefttree, port, hostid);
	    }

	    if (plan->righttree != NULL) {
		FillInPorts(plan->righttree, port, hostid);
	    }
	}
    }
}

/* ----------------------------------------------------------------
 *   	FillInFragStoreids
 *
 *	Precondition: qd->plantree must point to the local plan.
 *
 *	Postcondition: qd->parsetree->rtable is changed
 *	by filling in usedLocally (for both the RangeTblEntry and the
 *	FragInfo inside the RangeTblEntry) and, for fragments accessed
 *	locally, filling in the frastoreid.
 *
 *	Looks at the logical fragment ids in the range table and fills in
 *	the  fragment store ids. Does a single scan of pg_fragment. For
 *	each logical id scanned, checks to see if it is in rtrelids. If so,
 *	it fills in the storeid.
 *
 * ----------------------------------------------------------------
 */
void
FillInFragStoreids(QueryDesc *qd)
{
    List *lp1, *lp2;
    List *rtable = qd->parsetree->rtable;
    Plan *plan = qd->plantree;
    RangeTblEntry *rte;

    if (plan == NULL)
	return;
    
    /*
     * First, fill in usedLocally in RangeTblEntries and FragInfo.
     */
    foreach (lp1, rtable) {
	rte = (RangeTblEntry *) lfirst(lp1);
	rte->usedLocally = false;
	foreach (lp2, rte->fragInfo) {
	    FragInfo *fInfo = (FragInfo *)lfirst(lp2);
	    fInfo->usedLocally = false;
	}
    }

    FillInUsedLocally(qd);

    foreach (lp1, rtable) {
	rte = (RangeTblEntry *) lfirst(lp1);
	if (rte->usedLocally) {
	    FillInRTStoreids(rte->fragInfo);
	}
    }
}

/*
 * ----------------------------------------------------------------
 * FillInRTStoreids--
 *
 * 	Fill in the frastoreid's for fragments that
 *	are used locally.
 * ----------------------------------------------------------------
 */
void
FillInRTStoreids(List *rtfrags)
{
    List *lp;
    FragInfo *fInfo;
    HeapTuple htup;
    Form_pg_fragment frag;
    
    foreach (lp, rtfrags) {
	fInfo = (FragInfo *) lfirst(lp);	
	if (fInfo->usedLocally) {
	    htup = SearchSysCacheTuple(FRALOGICALID,
				       (char *) fInfo->fralogicalid,
				       (char *) NULL,
				       (char *) NULL,
				       (char *) NULL);
	    if (HeapTupleIsValid(htup)) {
		frag = (Form_pg_fragment) GETSTRUCT(htup);
		fInfo->frastoreid = frag->frastoreid;
	    }
	}
    }
}

/*
 * ----------------------------------------------------------------
 * FillInUsedLocally--
 *
 *	Check if each fragment in the range table is accessed.
 *	It could be accessed from the plan tree or as a result
 * 	relation.
 * ----------------------------------------------------------------
 */
void
FillInUsedLocally(QueryDesc *qd)
{
    FillInPlanRels(qd->plantree, qd->parsetree->rtable);
    FillInResultRel(qd);
}

/*
 * ----------------------------------------------------------------
 * FillInResultRel--
 *
 *	Check if fragments are accessed as result relations.
 * ----------------------------------------------------------------
 */
static
void
FillInResultRel(QueryDesc *qd)
{
    if (qd->parsetree->resultRelation > 0) {
	if (qd->parsetree->resultFragment > 0) {
	    RangeTblEntry *rte = (RangeTblEntry *)
		nth(qd->parsetree->resultRelation-1, qd->parsetree->rtable);
	    FragInfo *fInfo = (FragInfo *)
		nth(qd->parsetree->resultFragment-1, rte->fragInfo);
	    rte->usedLocally = true;
	    fInfo->usedLocally = true;
	}
    }
}

/*
 * ----------------------------------------------------------------
 * FillInPlanRels--
 *
 *	Run through the entire plan tree, checking if each
 *	node accesses a local fragment.
 * ----------------------------------------------------------------
 */
static
void
FillInPlanRels(Plan *plan, List *rtable)
{
    List *lp;
    RangeTblEntry *rte;
    FragInfo *fInfo;
    
    switch (plan->type) {
    case T_Merge:
	foreach(lp, ((Merge *)plan)->mergeplans) {
	    FillInPlanRels(lfirst(lp), rtable);
	}
	break;
    case T_Append: {
	Append *ap = (Append *) plan;
	int nplans = length(ap->unionplans);
	int i;

	rte = rt_fetch(ap->unionrelid, rtable);
	for (i = 0; i < nplans; ++i) {
	    rt_store(ap->unionrelid, rtable, nth(i, ap->unionrtentries));
	    FillInPlanRels(nth(i, ap->unionplans), rtable);
	}
	rt_store(ap->unionrelid, rtable, rte);
    }
	break;
    case T_Scan:
    case T_SeqScan:
    case T_IndexScan:
	if (((Scan *)plan)->scanrelid > 0) {
	    rte = (RangeTblEntry *) nth(((Scan *)plan)->scanrelid-1, rtable);
	    rte->usedLocally = true;

	    fInfo = (FragInfo *) nth(((Scan *)plan)->scanfragid-1,
				     rte->fragInfo);
	    fInfo->usedLocally = true;
	}
	break;
    }
    if (plan->lefttree != NULL) {
	FillInPlanRels(plan->lefttree, rtable);
    }
    if (plan->righttree != NULL) {
	FillInPlanRels(plan->righttree, rtable);
    }
}

/*
 *---------------------------------------------------------------------------------
 * getAddrFromHostId--
 *
 *    Looks in pg_host to find matching internet address for hostid. Right now,
 *    just does a sequential scan, looking for the hostid. Improve in the
 *    future? Perhaps not worth it, if we don't have to many hostids. However,
 *    we do have to open the relation, scan it, and close it. Is that a waste
 *    of time?
 *
 *    we should definitely cache the results. getAddrFromHostId is done
 *    many times. WARNING: If you call this from a mariposa exec backend,
 *    make sure you're doing this in the main thread!! -ay 4/95
 *    
 *---------------------------------------------------------------------------------
 */
int
getAddrFromHostId(int hostid)
{
    Relation hostdesc;
    HeapScanDesc hostscan;
    HeapTuple htup;
    int retval;

    if (hostid == getMyHostId()) {
	/* we do this so that SitemgrBackendDone doesn't have to be
	 * run in the main thread. the catalog lookup and will
	 * produce unpredictable results if not run in the main thread.
	 */
	return getMyAddr();
    }

    hostdesc = heap_openr((Name)Name_pg_host);
    if (!RelationIsValid(hostdesc))
        elog(WARN, "cannot open pg_host");
    hostscan = heap_beginscan(hostdesc, 0, SelfTimeQual, 0, NULL);
    do {
        htup = heap_getnext(hostscan, 0, NULL);
        if (HeapTupleIsValid(htup)) {
            Form_pg_host host = (Form_pg_host)GETSTRUCT(htup);
            if (host->hostid == hostid)
                retval = host->address;
        }
    } while (htup);
    heap_endscan(hostscan);
    heap_close(hostdesc);

    return retval;
}

/*
 * ----------------------------------------------------------------
 *	AddXIn--
 *
 *	Input: 	Plan *
 *
 *	Output:	X-in -> Plan *
 * ----------------------------------------------------------------
 */
static
Plan *
AddXIn(Plan *plan)
{
    
    ExchangeIn *node = makeNode(ExchangeIn);
    SiteAddrNode *saddr = makeNode(SiteAddrNode);
    
    node->plan.cost = 0.0;
    node->plan.state = NULL;
    node->plan.qual = NULL;
    node->plan.targetlist = plan->targetlist;
    node->plan.lefttree = plan;
    node->plan.righttree = NULL;

    saddr->addr = getAddrFromHostId(plan->processingSite.hostid);
    saddr->hostid = plan->processingSite.hostid;
    /*
     * Don't know the port yet.
     */
    saddr->port = 0;
    
    node->exchsendingSites = lcons(saddr, NIL);

    /*
     * The sending qepFragId is in the node below the
     * X-in node in the plan tree.
     */
    node->exchsendingQepFragIds = lconsi(plan->qepFragId, NIL);
    
    return((Plan *) node);
}

/*
 * ----------------------------------------------------------------
 *	AddTupleBuffer--
 * ----------------------------------------------------------------
 */
static
Plan *
AddTupleBuffer(Plan *plan)
{
    Plan *newchild = (Plan *) makeNode(TupleBuffer);

    newchild->lefttree = plan;
    newchild->targetlist = plan->targetlist;

    return(newchild);
}

/*
 * ----------------------------------------------------------------
 *	PlanCopySite--
 *
 *	Copies the state from 'parent' to 'child' that indicates
 *	that both should be processed on the same site and as part 
 *	of the same query plan fragment.
 * ----------------------------------------------------------------
 */
static
void
PlanCopySite(Plan *child, Plan *parent)
{
    child->processingSite.type = parent->processingSite.type;
    child->processingSite.hostid = parent->processingSite.hostid;
    child->processingSite.addr = parent->processingSite.addr;
    child->processingSite.port = parent->processingSite.port;
    child->qepFragId = parent->qepFragId;
}

/*
 * ----------------------------------------------------------------
 *	AddConnectiveNode--
 *
 *	This does the actual work for AddConnectiveNodes,
 *	inserting ExchangeIn and TupleBuffer nodes as required.
 * ----------------------------------------------------------------
 */
static
Plan *
AddConnectiveNode(Plan *child, Plan *parent, bool *needBuffer)
{
    AddConnectiveNodes(child, needBuffer);

    /*
     * if we detect that the parent and child are being processed on
     * two different sites, insert an ExchangeIn node to provide the
     * required network plumbing.  The ExchangeIn will always run at
     * the same site as the parent.
     */
    if (child->processingSite.hostid != parent->processingSite.hostid) {
	child = AddXIn(child);
	PlanCopySite(child, parent);
	*needBuffer = true;
    }

    /*
     * when the parent requires mark/restore to operate correctly
     * (currently only NestLoop and MergeJoin) and the child plan
     * contains an unbuffered ExchangeIn, we must insert a TupleBuffer
     * node above the child plan.  The TupleBuffer will always run at
     * the same site as the parent.
     *
     * note that once we insert TupleBuffers below this join, we don't
     * need to insert any more above it.
     */
    if ((IsA(parent,MergeJoin) || IsA(parent,NestLoop)) &&
	*needBuffer == true) {
	child = AddTupleBuffer(child);
	PlanCopySite(child, parent);
	*needBuffer = false;
    }

    return(child);
}

/*
 * ----------------------------------------------------------------
 * 	AddConnectiveNodes--
 *
 *	Add Exchange-in and TupleBuffer nodes where needed.
 *
 *	Exchange-in nodes go between nodes that are executed
 *	at different sites so that the output of the child node
 *	gets sent to the parent node.  The Exchange-in node is
 *	actually part of the parent sub-plan and is processed
 *	at the parent (receiving) site.
 *
 *	NB: The qepFragId's must be filled in before this routine
 *	is called.
 *
 *	The need to buffer is determined as follows.  If we can
 *	trace a path from a node that requires mark/restore to 
 *	an ExchangeIn node without any intermediate Temp or
 *	TupleBuffer nodes, then we must insert a TupleBuffer
 *	node to provide mark/restore.
 * ----------------------------------------------------------------
 */
static
void
AddConnectiveNodes(Plan *plan, bool *needBuffer)
{
    List *lp;
    
    *needBuffer = false;

    /*
     * Merge and Append nodes are treated differently, since they have
     * no left or right child, but have a list of plans whose output
     * they merge together.
     */
    switch (plan->type) {
    case T_Merge:
	foreach (lp, ((Merge *) plan)->mergeplans) {
	    lp->elem = AddConnectiveNode((Plan *) lp->elem, plan,
					 needBuffer);
	}
	break;
    case T_Append:
	foreach (lp, ((Append *) plan)->unionplans) {
	    lp->elem = AddConnectiveNode((Plan *) lp->elem, plan,
					 needBuffer);
	}
	break;
    default:
	if (plan->lefttree != (Plan *) NULL) {
	    plan->lefttree = AddConnectiveNode(plan->lefttree, plan,
					       needBuffer);
	}
	if (plan->righttree != (Plan *) NULL) {
	    plan->righttree = AddConnectiveNode(plan->righttree, plan,
						needBuffer);
	}
	
	/*
	 * if this is a Temp node, its ancestors will never need to
	 * insert a Buffer node -- the materialized temporary (a Sort
	 * temp or a table of some kind) will always support
	 * mark/restore.
	 */
	if (IsA_Temp(plan)) {
	    *needBuffer = false;
	}
	break;
    }
}

/*
 *---------------------------------------------------------------------------------
 * CreatePlanList--
 *
 *	Input: 	A fragmented plan tree, with the processing sites filled in
 *		and exchange-in nodes in the correct places.
 *
 *	Output:	A list of CoordPlan's: plan fragments which will be sent out
 *		to the processing sites.
 *---------------------------------------------------------------------------------
 */
static
List *
CreatePlanList(Plan *plan, Plan *parentNode)
{
    List *planList = NIL;
    List *leaves = FindPlanLeaves(plan);
    List *leaf;
    List *leftChildren = NIL;
    List *rightChildren = NIL;
    Plan *newPlan;
    Plan *pleaf, *leftChild, *rightChild;
    CoordPlan *cPlan = calloc(1, sizeof(CoordPlan));
    
    /*
     * For each leaf node, create the plan list for its children.
     * Then temporarily set its children to NULL and put the resulting
     * plan into the list.
     */
    foreach (leaf, leaves) {
	pleaf = (Plan *) lfirst(leaf);
	leftChild = pleaf->lefttree;
	rightChild = pleaf->righttree;
	leftChildren = lappend(leftChildren, leftChild);
	rightChildren = lappend(rightChildren, rightChild);

	if (leftChild != NULL) {
	    planList = nconc(planList, CreatePlanList(leftChild, pleaf));
	}
	if (rightChild != NULL) {
	    planList = nconc(planList, CreatePlanList(rightChild, pleaf));
	}

	pleaf->lefttree = NULL;
	pleaf->righttree = NULL;
    }

    newPlan = copyObject(plan);
    cPlan->plan = newPlan;
    cPlan->parentNode = parentNode;
    
    planList = lcons(cPlan, planList);

    /* Now restore the leaves' children */
    foreach (leaf, leaves) {
	pleaf = (Plan *) lfirst(leaf);
	pleaf->lefttree = lfirst(leftChildren);
	pleaf->righttree = lfirst(rightChildren);
	leftChildren = lnext(leftChildren);
	rightChildren = lnext(rightChildren);
    }
    
    return planList;
}

/*
 *---------------------------------------------------------------------------------
 * FindPlanLeaves--
 *
 *	Input: 	A plan, with X-in nodes.
 *
 *	Output: A list of pointers to the nodes in the plan that
 *		are the leaves of the plan fragments.  This includes
 *		both X-in nodes and the leaves of the original plan.
 *---------------------------------------------------------------------------------
 */
static
List *
FindPlanLeaves(Plan *plan)
{
    List *leaves = NIL;
    List *lp;

    switch (plan->type) {
    case T_ExchangeIn:
	leaves = lcons(plan, NIL);
	break;
    case T_Merge:
	foreach (lp, ((Merge *) plan)->mergeplans) {
	    leaves = nconc(leaves, FindPlanLeaves((Plan *) lfirst(lp)));
	}
	break;
    case T_Append:
	foreach (lp, ((Append *) plan)->unionplans) {
	    leaves = nconc(leaves, FindPlanLeaves((Plan *) lfirst(lp)));
	}
	break;
    default:
	if (plan->lefttree == NULL && plan->righttree == NULL) {
	    leaves = lappend(leaves, plan);
	} else {
	    if (plan->lefttree != NULL) {
		leaves = nconc(leaves, FindPlanLeaves(plan->lefttree));
	    }
	    if (plan->righttree != NULL) {
		leaves = nconc(leaves, FindPlanLeaves(plan->righttree));
	    }
	}
	break;
    }
    return(leaves);
}

/*
 *---------------------------------------------------------------------------------
 * AllNodesHaveProcessingSite--
 *
 *	Input: 	A plan
 *
 *	Output: TRUE if each node has a valid processing site filled in.
 *		FALSE if any node has a -1, indicating that the Broker could
 *		      not find a processing site for it.
 *
 *---------------------------------------------------------------------------------
 */
static
bool
AllNodesHaveProcessingSite(Plan *plan)
{
    bool retval = true;
    List *lp;

    if (plan != (Plan *) NULL) {
	switch (plan->type) {
	case T_Merge:
	    foreach (lp, ((Merge *) plan)->mergeplans) {
		if (!AllNodesHaveProcessingSite((Plan *) lfirst(lp))) {
		    retval = false;
		}
	    }
	    break;
	case T_Append:
	    foreach (lp, ((Append *) plan)->unionplans) {
		if (!AllNodesHaveProcessingSite((Plan *) lfirst(lp))) {
		    retval = false;
		}
	    }
	    break;
	default:
	    retval = (plan->processingSite.hostid != -1 &&
		      AllNodesHaveProcessingSite(plan->lefttree) &&
		      AllNodesHaveProcessingSite(plan->righttree));
	    break;
	}
    }
    return(retval);
}
#endif /* P95 */
