/*-------------------------------------------------------------------------
 *
 * name.c--
 *    Functions for the built-in type "name".
 *[Postgres95]
 * name replaces char16 and is carefully implemented so that it
 * is a string of length NAMEDATALEN.  DO NOT use hard-coded constants anywhere
 * always use NAMEDATALEN as the symbolic constant!   - jolly 8/21/95
 * 
 *
 * Copyright (c) 1994, Regents of the University of California
 *
 *
 * IDENTIFICATION
 *    $Header: /usr/local/devel/mariposa/cvs/src/backend/utils/adt/name.c,v 0.2 1996/06/07 05:01:09 am Exp $
 *
 *-------------------------------------------------------------------------
 */
#include <string.h>
#include "postgres.h"
#include "utils/builtins.h"	/* where the declarations go */
#ifdef P95
#include "utils/palloc.h"	/* where the declarations go */
#endif /* P95 */

/***************************************************************************** 
 *   USER I/O ROUTINES (none)                                                *
 *****************************************************************************/


#ifdef P95
/*
 *	namein	- converts "..." to internal representation
 *
 *	Note:
 *		Currently if strlen(s) < NAMEDATALEN, the extra chars are nulls
 */
NameData *namein(char *s)
{
    NameData *result;

    if (s == NULL)
	return(NULL);
    result = (NameData*) palloc(NAMEDATALEN);
    /* always keep it null-padded */
    memset(result->data, 0, NAMEDATALEN); 
    (void) strncpy(result->data, s, NAMEDATALEN-1);
    return(result);
}

/*
 *	nameout	- converts internal reprsentation to "..."
 */
char *nameout(NameData *s)
{
    if (s == NULL)
	return "-";
    else
	return pstrdup(s->data);
}


#endif /* P95 */
/***************************************************************************** 
 *   PUBLIC ROUTINES                                                         *
 *****************************************************************************/

#ifdef P95
/*
 *	nameeq	- returns 1 iff arguments are equal
 *	namene	- returns 1 iff arguments are not equal
 *
 *	BUGS:
 *		Assumes that "xy\0\0a" should be equal to "xy\0b".
 *		If not, can do the comparison backwards for efficiency.
 *
 *	namelt	- returns 1 iff a < b
 *	namele	- returns 1 iff a <= b
 *	namegt	- returns 1 iff a < b
 *	namege	- returns 1 iff a <= b
 *
 */
int32 nameeq(NameData *arg1, NameData *arg2)
{
    if (!arg1  || !arg2)
	return 0;
    else
	return (strncmp(arg1->data, arg2->data, NAMEDATALEN) == 0);
}

int32 namene(NameData *arg1, NameData *arg2)
{
    if (arg1 == NULL || arg2 == NULL)
	return((int32) 0);
    return(strncmp(arg1->data, arg2->data, NAMEDATALEN) != 0);
}

int32 namelt(NameData *arg1, NameData *arg2)
{
    if (arg1 == NULL || arg2 == NULL)
	return((int32) 0);
    return((int32) (strncmp(arg1->data, arg2->data, NAMEDATALEN) < 0));
}

int32 namele(NameData *arg1, NameData *arg2)
{
    if (arg1 == NULL || arg2 == NULL)
	return((int32) 0);
    return((int32) (strncmp(arg1->data, arg2->data, NAMEDATALEN) <= 0));
}

int32 namegt(NameData *arg1, NameData *arg2)
{
    if (arg1 == NULL || arg2 == NULL)
	return((int32) 0);
    
    return((int32) (strncmp(arg1->data, arg2->data, NAMEDATALEN) > 0));
}

int32 namege(NameData *arg1, NameData *arg2)
{
    if (arg1 == NULL || arg2 == NULL)
	return((int32) 0);
    
    return((int32) (strncmp(arg1->data, arg2->data, NAMEDATALEN) >= 0));
}


#endif /* P95 */
/* (see char.c for comparison/operation routines) */

int namecpy(Name n1, Name n2)
{
    if (!n1 || !n2)
	return(-1);
    (void) strncpy(n1->data, n2->data, NAMEDATALEN);
    return(0);
}

int namecat(Name n1, Name n2)
{
    return(namestrcat(n1, n2->data)); /* n2 can't be any longer than n1 */
}

int namecmp(Name n1, Name n2)
{
    return(strncmp(n1->data, n2->data, NAMEDATALEN));
}

#ifndef P95
int namestrcpy(Name name, char *str)
#else /* P95 */
int 
namestrcpy(Name name, char *str)
#endif /* P95 */
{
    if (!name || !str)
	return(-1);
    memset(name->data, 0, sizeof(NameData));
    (void) strncpy(name->data, str, NAMEDATALEN);
    return(0);
}

int namestrcat(Name name, char *str)
{
    int i;
    char *p, *q;
    
    if (!name || !str)
	return(-1);
    for (i = 0, p = name->data; i < NAMEDATALEN && *p; ++i, ++p)
	;
    for (q = str; i < NAMEDATALEN; ++i, ++p, ++q) {
	*p = *q;
	if (!*q)
	    break;
    }
    return(0);
}

#ifndef P95
int namestrcmp(Name name, char *str)
#else /* P95 */
int 
namestrcmp(Name name, char *str)
#endif /* P95 */
{
    if (!name && !str)
	return(0);
    if (!name)
	return(-1);	/* NULL < anything */
    if (!str)
	return(1);	/* NULL < anything */
    return(strncmp(name->data, str, NAMEDATALEN));
}

/***************************************************************************** 
 *   PRIVATE ROUTINES                                                        *
 *****************************************************************************/

#ifndef P95
/*
 * Note:
 *	This is the same code as char16eq.
 *	Assumes that "xy\0\0a" should be equal to "xy\0b".
 */
bool NameIsEqual(Name name1, Name name2)
#else /* P95 */
uint32 
NameComputeLength(Name name)
#endif /* P95 */
{
#ifndef P95
    if (! PointerIsValid(name1) || ! PointerIsValid(name2)) {
	return(false);
    }
    return((bool)(strncmp(name1->data, name2->data, NAMEDATALEN) == 0));
}

uint32 NameComputeLength(Name name)
{
#endif /* P95 */
    char	*charP;
    int	length;
#ifndef P95
    
    Assert(NameIsValid(name));
#endif /* P95 */
    
    for (length = 0, charP = name->data;
	 length < NAMEDATALEN && *charP != '\0';
	 length++, charP++) {
	;
    }
    return (uint32)length;
}
