/*-------------------------------------------------------------------------
 *
 * lsyscache.c--
 *    Routines to access information within system caches	
 *
 * Copyright (c) 1994, Regents of the University of California
 *
 *
 * IDENTIFICATION
 *    $Header: /usr/local/devel/mariposa/cvs/src/backend/utils/cache/lsyscache.c,v 0.6 1996/06/07 05:01:25 am Exp $
 *
 * NOTES
 *    Eventually, the index information should go through here, too.
 *    
 *    Most of these routines call SearchSysCacheStruct() and thus simply
 *    (1) allocate some space for the return struct and (2) call it.
 *    
 *-------------------------------------------------------------------------
 */
#include <string.h>
#include "postgres.h"

#include "nodes/pg_list.h"
#include "utils/syscache.h"
#include "utils/lsyscache.h"
#ifndef P95
#include "utils/builtins.h"
#endif /* P95 */
#include "access/tupmacs.h"
#include "utils/rel.h"
#include "utils/palloc.h"
#include "utils/elog.h"
#include "access/attnum.h"
#include "access/heapam.h"
#ifndef P95
#include "access/htup.h"
#endif /* P95 */

#include "catalog/pg_amop.h"
#include "catalog/pg_type.h"

#ifndef P95
/*
 * contains data for lookup of system attributes
 * (get_attnum/name/type/isset)
 */
struct {
    char16 name;
    bool isSet;
    Oid typeId;
} sysAttrs[] = {
    {{"ctid"}, false, 27},
    {{"oid"}, false, 26},
    {{"xmin"}, false, 28},
    {{"cmax"}, false, 29},
    {{"xmax"}, false, 28},
    {{"cmax"}, false, 29},
    {{"globalxmin"}, false, 28},
    {{"globalxmax"}, false, 28},
    {{"chain"}, false, 27},
    {{"anchor"}, false, 27},
    {{"tmax"}, false, 702},
    {{"tmin"}, false, 702},
    {{"vtype"}, false, 18}
};

#endif /* P95 */
/*    		---------- AMOP CACHES ----------			 */

/*    
 * op_class -
 *    
 *    	Return t iff operator 'opno' is in operator class 'opclass'.
 *    
 */
bool
#ifndef P95
op_class(Oid opno, int32 opclass)
#else /* P95 */
op_class(Oid opno, int32 opclass, Oid amopid)
#endif /* P95 */
{
    FormData_pg_amop amoptup;

#ifndef P95
    if (SearchSysCacheStruct(AMOPOPID, (char *) &amoptup,
			     (char *) ObjectIdGetDatum(opclass),
			     (char *) ObjectIdGetDatum(opno),
			     (char *) NULL, (char *) NULL))
#else /* P95 */
    if (SearchSysCacheStruct(AMOPOPID, 
			     (char *) &amoptup,
			     ObjectIdGetDatum(opclass),
			     ObjectIdGetDatum(opno),
			     ObjectIdGetDatum(amopid),
			     0))
#endif /* P95 */
	return(true);
    else
	return(false);
}

/*    		---------- ATTRIBUTE CACHES ----------			 */

/*    
 * get_attname -
 *    
 * [Mariposa]
 *    	Given the relation id and the attribute number (1-based),
 *    	return the "attname" field from the attribute relation.
 *	This had to be changed to use the relation descriptor,
 *	because the syscache does not contain nameserver data
 * 	about remote classes.
 * [Postgres95]
 *    	Given the relation id and the attribute number,
 *    	return the "attname" field from the attribute relation.
 *
 *    
 */
#ifndef P95
Name
#else /* P95 */
char*
#endif /* P95 */
get_attname(Oid relid, AttrNumber attnum)
{
    FormData_pg_attribute att_tup;
#ifndef P95
    Name retval = (Name)NULL;
    Relation rd;
#else /* P95 */
    char *retval;
#endif /* P95 */

#ifndef P95
    if (attnum < 0 && attnum > FirstLowInvalidHeapAttributeNumber) {
	/* this is a system attribute */
	retval = (Name) palloc(sizeof(*retval));
	memmove(retval, sysAttrs[(-1)*attnum+1].name,
	    sizeof(*retval));
#else /* P95 */
    if (SearchSysCacheStruct(ATTNUM, 
			     (char*)&att_tup,
			     ObjectIdGetDatum(relid),
			     UInt16GetDatum(attnum),
			     0,0)) {
	retval = pstrdup(att_tup.attname.data);

#endif /* P95 */
	return(retval);
    }
#ifndef P95
    rd = heap_open(relid);
    if (rd != NULL) {
	Assert(rd->rd_rel->relnatts >= attnum);
	retval = (Name)palloc(sizeof(rd->rd_att[attnum - 1]->attname));
	memmove(retval, &(rd->rd_att[attnum - 1]->attname),
	    sizeof(rd->rd_att[attnum - 1]->attname));
	heap_close(rd);
	return(retval);
    } else {
	heap_close(rd);
        return((Name)NULL);
    }
#else /* P95 */
    else
      return(NULL);
#endif /* P95 */
}

/*    
 * get_attnum -
 *    
 *    	Given the relation id and the attribute name,
 *    	return the "attnum" field from the attribute relation.
 *    
 * [Mariposa]
 *	This had to be changed to use the relation descriptor,
 *	because the syscache does not contain nameserver data
 * 	about remote classes.
 *    
 */
AttrNumber
#ifndef P95
get_attnum(Oid relid, Name attname)
#else /* P95 */
get_attnum(Oid relid, char *attname)
#endif /* P95 */
{
    FormData_pg_attribute att_tup;
#ifndef P95
    Relation rd;
    int i;
#endif /* P95 */

#ifndef P95
    rd = heap_open(relid);
    if (rd != NULL) {
	for (i = 0; i < rd->rd_rel->relnatts; i++) {
	    if (char16eq(attname->data, rd->rd_att[i]->attname.data)) {
		heap_close(rd);
		return (rd->rd_att[i]->attnum);
	    }
	}
    }
    heap_close(rd);

    /* check if this is a system attribute */
    for (i = 0; i < (-1) * FirstLowInvalidHeapAttributeNumber - 1; i++) {
	if (char16eq(sysAttrs[i].name.data, attname->data)) {
	    return ((-1) * i - 1);
	}
    }
    return(InvalidAttrNumber);
#else /* P95 */
    if (SearchSysCacheStruct(ATTNAME, (char *) &att_tup,
			     ObjectIdGetDatum(relid),
			     PointerGetDatum(attname),
			     0,0))
	return(att_tup.attnum);
    else
	return(InvalidAttrNumber);
#endif /* P95 */
}

/*    
 * get_atttype -
 *    
 *    	Given the relation OID and the attribute number with the relation,
 *    	return the attribute type OID.
 * [Mariposa]
 *    
 *	This had to be changed to use the relation descriptor,
 *	because the syscache does not contain nameserver data
 * 	about remote classes.
 *    
 */
Oid
get_atttype(Oid relid, AttrNumber attnum)
{
    AttributeTupleForm att_tup = (AttributeTupleForm)palloc(sizeof(*att_tup));
#ifndef P95
    Relation rd;
#endif /* P95 */

#ifndef P95
    if (attnum < 0 && attnum > FirstLowInvalidHeapAttributeNumber) {
	/* this is a system attribute */
	return(sysAttrs[(-1)*attnum-1].typeId);
    }
    rd = heap_open(relid);
    if (rd != NULL) {
	Assert(rd->rd_rel->relnatts >= attnum);
	return(rd->rd_att[attnum - 1]->atttypid);
    } else {
#else /* P95 */
    if (SearchSysCacheStruct(ATTNUM, 
			     (char *) att_tup,
			     ObjectIdGetDatum(relid),
			     UInt16GetDatum(attnum),
			     0,0))
	return(att_tup->atttypid);
    else
#endif /* P95 */
	return((Oid)NULL);
#ifndef P95
    }
#endif /* P95 */
}

/* This routine uses the attname instead of the attnum because it
 * replaces the routine find_atttype, which is called sometimes when
 * only the attname, not the attno, is available.
 *
 * [Mariposa]
 *	This had to be changed to use the relation descriptor,
 *	because the syscache does not contain nameserver data
 * 	about remote classes. Previously, the attribute was checked
 * 	for a NULL value, but since the relation descriptor
 *  	does not contain the full HeapTuple, only the AttributeTupleForm,
 *	this check was dropped.			01/16/1996 mk
 *    
 */
bool
#ifndef P95
get_attisset(Oid relid, Name attname)
#else /* P95 */
get_attisset(Oid relid, char *attname)
#endif /* P95 */
{
    HeapTuple htup;
    AttrNumber attno;
    AttributeTupleForm att_tup;
#ifndef P95
    int i;
    Relation rd;
#endif /* P95 */

#ifndef P95
    rd = heap_open(relid);
    if (rd == NULL) {
#else /* P95 */
    attno = get_attnum(relid, attname);
     
    htup = SearchSysCacheTuple(ATTNAME, 
			       ObjectIdGetDatum(relid),
			       PointerGetDatum(attname), 
			       0,0);
    if (!HeapTupleIsValid(htup))
#endif /* P95 */
	elog(WARN, "get_attisset: no attribute %.16s in relation %d",
#ifndef P95
	     attname->data, relid);
#else /* P95 */
	     attname, relid);
    if (heap_attisnull(htup, attno))
	return(false);
    else {
	att_tup = (AttributeTupleForm)GETSTRUCT(htup);
	return(att_tup->attisset);
#endif /* P95 */
    }
#ifndef P95
    for (i = 0; i < rd->rd_rel->relnatts; i++) {
	if (char16eq(attname->data, rd->rd_att[i]->attname.data)) {
	    heap_close(rd);
	    return (rd->rd_att[i]->attisset);
	}
    }

    /* check if this is a system attribute */
    for (i = 0; i < (-1) * FirstLowInvalidHeapAttributeNumber - 1; i++) {
	if (char16eq(sysAttrs[i].name.data, attname->data)) {
	    return (sysAttrs[(-1) * i - 1].isSet);
	}
    }
    heap_close(rd);
    elog(WARN, "get_attisset: no attribute %.16s in relation %d",
	 attname->data, relid);
#endif /* P95 */
}

/*    		---------- INDEX CACHE ----------			 */

/*    	watch this space...
 */

/*    		---------- OPERATOR CACHE ----------			 */

/*    
 * get_opcode -
 *    
 *    	Returns the regproc id of the routine used to implement an
 *    	operator given the operator uid.
 *    
 */
RegProcedure
get_opcode(Oid opno)
{
    FormData_pg_operator optup;

    if (SearchSysCacheStruct(OPROID, (char *) &optup,
#ifndef P95
			     (char *) ObjectIdGetDatum(opno),
			     (char *) NULL, (char *) NULL, (char *) NULL))
#else /* P95 */
			     ObjectIdGetDatum(opno),
			     0,0,0))
#endif /* P95 */
	return(optup.oprcode);
    else
	return((RegProcedure)NULL);
}

/*
 * get_opname -
 *    returns the name of the operator with the given opno
 *
 * Note: return the struct so that it gets copied.
 */
#ifndef P95
NameData
#else /* P95 */
char*
#endif /* P95 */
get_opname(Oid opno)
{
    FormData_pg_operator optup;

    if (SearchSysCacheStruct(OPROID, (char *) &optup,
#ifndef P95
			     (char *) ObjectIdGetDatum(opno),
			     (char *) NULL, (char *) NULL, (char *) NULL))
	return (optup.oprname);
    else
#else /* P95 */
			     ObjectIdGetDatum(opno),
			     0,0,0))
	return (pstrdup(optup.oprname.data));
    else {
#endif /* P95 */
	elog(WARN, "can't look up operator %d\n", opno);
#ifdef P95
	return NULL;
    }
#endif /* P95 */
}

/*    
 * op_mergesortable -
 *    
 *    	Returns the left and right sort operators and types corresponding to a
 *    	mergesortable operator, or nil if the operator is not mergesortable.
 *    
 */
bool
op_mergesortable(Oid opno, Oid ltype, Oid rtype, Oid *leftOp, Oid *rightOp) 
{
    FormData_pg_operator optup;

    if (SearchSysCacheStruct(OPROID, (char *) &optup,
#ifndef P95
			     (char *) ObjectIdGetDatum(opno),
			     (char *) NULL, (char *) NULL, (char *) NULL) &&
#else /* P95 */
			     ObjectIdGetDatum(opno),
			     0,0,0) &&
#endif /* P95 */
	optup.oprlsortop &&
	optup.oprrsortop && 
	optup.oprleft == ltype &&
	optup.oprright == rtype) {

	*leftOp = ObjectIdGetDatum(optup.oprlsortop);
	*rightOp = ObjectIdGetDatum(optup.oprrsortop);
	return TRUE;
    } else {
	return FALSE;
    }
}

/*    
 * op_hashjoinable--
 *    
 * Returns the hash operator corresponding to a hashjoinable operator, 
 * or nil if the operator is not hashjoinable.
 *    
 */
Oid
op_hashjoinable(Oid opno, Oid ltype, Oid rtype)
{
    FormData_pg_operator optup;

    if (SearchSysCacheStruct(OPROID, (char *) &optup,
#ifndef P95
			     (char *) ObjectIdGetDatum(opno),
			     (char *) NULL, (char *) NULL, (char *) NULL) &&
#else /* P95 */
			     ObjectIdGetDatum(opno),
			     0,0,0) &&
#endif /* P95 */
	optup.oprcanhash  &&
	optup.oprleft == ltype &&
	optup.oprright == rtype) 
	return(opno);
    else
	return(InvalidOid);
}

/*    
 * get_commutator -
 *    
 *    	Returns the corresponding commutator of an operator.
 *    
 */
Oid
get_commutator(Oid opno)
{
    FormData_pg_operator optup;

    if (SearchSysCacheStruct(OPROID, (char *) &optup,
#ifndef P95
			     (char *) ObjectIdGetDatum(opno),
			     (char *) NULL, (char *) NULL, (char *) NULL))
#else /* P95 */
			     ObjectIdGetDatum(opno),
			     0,0,0))
#endif /* P95 */
	return(optup.oprcom);
    else
	return((Oid)NULL);
}

HeapTuple
get_operator_tuple(Oid opno)
{
    HeapTuple optup;

#ifndef P95
    if ((optup = SearchSysCacheTuple(OPROID, (char *) ObjectIdGetDatum(opno),
				     (char *) NULL, (char *) NULL,
				     (char *) NULL)))
#else /* P95 */
    if ((optup = SearchSysCacheTuple(OPROID, 
				     ObjectIdGetDatum(opno),
				     0,0,0)))
#endif /* P95 */
	return(optup);
    else
	return((HeapTuple)NULL);
}

/*    
 * get_negator -
 *    
 *    	Returns the corresponding negator of an operator.
 *    
 */
Oid
get_negator(Oid opno)
{
    FormData_pg_operator optup;

    if (SearchSysCacheStruct(OPROID, (char *) &optup,
#ifndef P95
			     (char *) ObjectIdGetDatum(opno),
			     (char *) NULL, (char *) NULL, (char *) NULL))
#else /* P95 */
			     ObjectIdGetDatum(opno),
			     0,0,0))
#endif /* P95 */
	return(optup.oprnegate);
    else
	return((Oid)NULL);
}

/*    
 * get_oprrest -
 *    
 *    	Returns procedure id for computing selectivity of an operator.
 *    
 */
RegProcedure
get_oprrest(Oid opno)
{
    FormData_pg_operator optup;

    if (SearchSysCacheStruct(OPROID, (char *) &optup,
#ifndef P95
			     (char *) ObjectIdGetDatum(opno),
			     (char *) NULL, (char *) NULL, (char *) NULL))
#else /* P95 */
			     ObjectIdGetDatum(opno),
			     0,0,0))
#endif /* P95 */
	return(optup.oprrest );
    else
	return((RegProcedure) NULL);
}

/*    
 * get_oprjoin -
 *    
 *    	Returns procedure id for computing selectivity of a join.
 *    
 */
RegProcedure
get_oprjoin(Oid opno)
{
    FormData_pg_operator optup;

    if (SearchSysCacheStruct(OPROID, (char *) &optup,
#ifndef P95
			     (char *) ObjectIdGetDatum(opno),
			     (char *) NULL, (char *) NULL, (char *) NULL))
#else /* P95 */
			     ObjectIdGetDatum(opno),
			     0,0,0))
#endif /* P95 */
	return(optup.oprjoin);
    else
	return((RegProcedure)NULL);
}

/*    		---------- RELATION CACHE ----------			 */

/*    
 * get_relnatts -
 *    
 *    	Returns the number of attributes for a given relation.
 *    
 * [Mariposa]
 *	This had to be changed to use the relation descriptor,
 *	because the syscache does not contain nameserver data
 * 	about remote classes.
 *    
 */
int
get_relnatts(Oid relid)
{
    FormData_pg_class reltup;
#ifndef P95
    Relation rd;
#endif /* P95 */

#ifndef P95
    rd = heap_open(relid);
    if (rd != NULL) {
	heap_close(rd);
	return(rd->rd_rel->relnatts);
    } else {
	heap_close(rd);
#else /* P95 */
    if (SearchSysCacheStruct(RELOID, (char *) &reltup,
			     ObjectIdGetDatum(relid),
			     0,0,0))
	return(reltup.relnatts);
    else
#endif /* P95 */
    	return(InvalidAttrNumber);
#ifndef P95
    }
#endif /* P95 */
}

/*    
 * get_rel_name -
 *    
 *    	Returns the name of a given relation.
 * [Mariposa]
 *	This had to be changed to use the relation descriptor,
 *	because the syscache does not contain nameserver data
 * 	about remote classes.
 *    
 */
#ifndef P95
Name
#else /* P95 */
char*
#endif /* P95 */
get_rel_name(Oid relid)
{
    FormData_pg_class reltup;
#ifndef P95
    Name retval = (Name)NULL;
    Relation rd;
#endif /* P95 */

#ifndef P95
    rd = heap_open(relid);
    if (rd != NULL) {
	retval = (Name)palloc(sizeof(rd->rd_rel->relname));
	memmove(retval, &(rd->rd_rel->relname), sizeof(rd->rd_rel->relname));
	heap_close(rd);
	return (retval);
    } else {
	heap_close(rd);
	return((Name)NULL);
    }
#else /* P95 */
    if ((SearchSysCacheStruct(RELOID, 
			      (char*)&reltup,
			      ObjectIdGetDatum(relid),
			      0,0,0))) {
	return (pstrdup(reltup.relname.data));
    } else
	return(NULL);
#endif /* P95 */
}

/*    		---------- TYPE CACHE ----------			 */

/*    
 * get_typlen -
 *    
 *    	Given the type OID, return the length of the type.
 *    
 */
int16
get_typlen(Oid typid)
{
    TypeTupleFormData typtup;

    if (SearchSysCacheStruct(TYPOID, (char *) &typtup,
#ifndef P95
			     (char *) ObjectIdGetDatum(typid),
			     (char *) NULL, (char *) NULL, (char *) NULL))
#else /* P95 */
			     ObjectIdGetDatum(typid),
			     0,0,0))
#endif /* P95 */
	return(typtup.typlen);
    else
	return((int16)NULL);
}

/*    
 * get_typbyval -
 *    
 *    	Given the type OID, determine whether the type is returned by value or
 *    	not.  Returns 1 if by value, 0 if by reference.
 *    
 */
bool
get_typbyval(Oid typid)
{
    TypeTupleFormData typtup;

    if (SearchSysCacheStruct(TYPOID, (char *) &typtup,
#ifndef P95
			     (char *) ObjectIdGetDatum(typid),
			     (char *) NULL, (char *) NULL, (char *) NULL))
#else /* P95 */
			     ObjectIdGetDatum(typid),
			     0,0,0))
#endif /* P95 */
	return((bool)typtup.typbyval);
    else
	return(false);
}

/*    
 * get_typbyval -
 *    
 *    	Given the type OID, determine whether the type is returned by value or
 *    	not.  Returns 1 if by value, 0 if by reference.
 *    
 */
char
get_typalign(Oid typid)
{
    TypeTupleFormData typtup;

    if (SearchSysCacheStruct(TYPOID, (char *) &typtup,
#ifndef P95
			     (char *) ObjectIdGetDatum(typid),
			     (char *) NULL, (char *) NULL, (char *) NULL))
#else /* P95 */
			     ObjectIdGetDatum(typid),
			     0,0,0))
#endif /* P95 */
	return(typtup.typalign);
    else
	return ('i');
}

/*    
 * get_typdefault - 
 *    
 *    	Given the type OID, return the default value of the ADT.
 *    
 */
struct varlena *
get_typdefault(Oid typid)
{
    struct varlena *typdefault = 
	(struct varlena *)TypeDefaultRetrieve (typid);
    return(typdefault);
}

/*    
 * get_typtype -
 *    
 *    	Given the type OID, find if it is a basic type, a named relation
 *	or the generic type 'relation'.
 *	It returns the null char if the cache lookup fails...
 *    
 */
char
get_typtype(Oid typid)
{
    TypeTupleFormData typtup;

    if (SearchSysCacheStruct(TYPOID, (char *) &typtup,
#ifndef P95
			     (char *) ObjectIdGetDatum(typid),
			     (char *) NULL, (char *) NULL, (char *) NULL)) {
#else /* P95 */
			     ObjectIdGetDatum(typid),
			     0,0,0)) {
#endif /* P95 */
	return(typtup.typtype);
    } else {
	return('\0');
    }
}

