/*-------------------------------------------------------------------------
 *
 * globals.c--
 *    global variable declarations
 *
 * Copyright (c) 1994, Regents of the University of California
 *
 *
 * IDENTIFICATION
 *    $Header: /usr/local/devel/mariposa/cvs/src/backend/utils/init/globals.c,v 0.15 1996/06/07 05:01:42 am Exp $
 *
 * NOTES
 *    Globals used all over the place should be declared here and not
 *    in other modules.  
 *
 *-------------------------------------------------------------------------
 */
#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <sys/file.h>
#include <sys/types.h>
#include <math.h>

#include "postgres.h"
#include "miscadmin.h"		/* where the declarations go */

#ifndef P95
#include "machine.h"		/* for BLCKSZ */
#endif /* P95 */
#include "access/heapam.h"
#include "utils/tqual.h"
#include "storage/sinval.h"
#include "storage/sinvaladt.h"
#include "storage/lmgr.h"
#include "utils/elog.h"

#include "catalog/catname.h"

int		Portfd = -1;
int		Noversion = 0;
#ifdef P95
int             Quiet = 1;
#endif /* P95 */

int		MasterPid;
#ifdef P95
char*           DataDir;
#endif /* P95 */

char		OutputFileName[MAXPGPATH] = "";

BackendId	MyBackendId;
BackendTag	MyBackendTag;

#ifndef P95
int		MyHostId;
int 		MyNameServerHostId;
#else /* P95 */
char            *UserName = NULL;
char            *DatabaseName = NULL;
char 		*DatabasePath = NULL;
#endif /* P95 */

#ifndef P95
Name		UserName;
Name            DatabaseName;
String		DatabasePath;

#endif /* P95 */
bool		MyDatabaseIdIsInitialized = false;
Oid		MyDatabaseId = InvalidOid;
bool		TransactionInitWasProcessed = false;

bool		IsUnderPostmaster = false;
bool		IsPostmaster = false;
#ifndef P95
bool		IsMariposaExec = false;
#endif /* P95 */

#ifndef P95
int             Quiet = 1;
#endif /* P95 */
short		DebugLvl = 0;

#ifndef P95
Size		LocalBlockSize = BLCKSZ;

bool            scheduleQueries = false;

static NameData   UserNameData;
static NameData   DatabaseNameData;
static NameData   pg_access_namedata = { "pg_access" };
static NameData   pg_aggregate_namedata =  { "pg_aggregate" };
static NameData   pg_am_namedata = { "pg_am" };
static NameData   pg_amop_namedata = { "pg_amop" };
static NameData   pg_amproc_namedata = { "pg_amproc" };
static NameData   pg_attribute_namedata = { "pg_attribute" };
static NameData   pg_contract_namedata = { "pg_contract" };
static NameData   pg_database_namedata = { "pg_database" };
static NameData   pg_defaults_namedata = { "pg_defaults" };
static NameData   pg_demon_namedata = { "pg_demon" };
static NameData   pg_fragment_namedata = { "pg_fragment" };
static NameData   pg_group_namedata = { "pg_group" };
static NameData   pg_has_namedata = { "pg_has" };
static NameData   pg_host_namedata = { "pg_host" };
static NameData   pg_index_namedata = { "pg_index" };
static NameData   pg_inheritproc_namedata = { "pg_inheritproc" };
static NameData   pg_inherits_namedata = { "pg_inherits" };
static NameData   pg_ipl_namedata = { "pg_ipl" };
static NameData   pg_language_namedata = { "pg_language" };
static NameData   pg_large_object_namedata = { "pg_large_object" };
static NameData   pg_ledger_namedata = { "pg_ledger" };
static NameData   pg_listener_namedata = { "pg_listener" };
static NameData   pg_log_namedata = { "pg_log" };
static NameData   pg_magic_namedata = { "pg_magic" };
static NameData   pg_nsvcattr_namedata = { "pg_nsvcattr" };
static NameData   pg_nsvcclass_namedata = { "pg_nsvcclass" };
static NameData   pg_nsvcfrag_namedata = { "pg_nsvcfrag" };
static NameData   pg_naming_namedata = { "pg_naming" };
static NameData   pg_opclass_namedata = { "pg_opclass" };
static NameData   pg_operator_namedata = { "pg_operator" };
static NameData   pg_parg_namedata = { "pg_parg" };
#ifdef HP_JUKEBOX
static NameData   pg_platter_namedata = { "pg_platter" };
static NameData   pg_plmap_namedata = { "pg_plmap" };
static NameData   pg_plrel_namedata = { "pg_plrel"};
#endif
static NameData   pg_proc_namedata = { "pg_proc" };
static NameData   pg_class_namedata = { "pg_class" }; 
static NameData   pg_rewrite_namedata = { "pg_rewrite" };
static NameData   pg_server_namedata = { "pg_server" };
static NameData   pg_statistic_namedata = { "pg_statistic" };
static NameData   pg_time_namedata = { "pg_time" };
static NameData   pg_type_namedata = { "pg_type" };
static NameData   pg_user_namedata = { "pg_user" };
static NameData   pg_variable_namedata = { "pg_variable" };
static NameData   pg_version_namedata = { "pg_version" };

Name            Name_pg_access;
Name            Name_pg_aggregate;
Name            Name_pg_am;
Name            Name_pg_amop;
Name            Name_pg_amproc;
Name            Name_pg_attribute;
Name            Name_pg_contract;
Name            Name_pg_database;
Name            Name_pg_defaults;
Name            Name_pg_demon;
Name            Name_pg_fragment;
Name            Name_pg_group;
Name            Name_pg_has;
Name		Name_pg_host;
Name            Name_pg_index;
Name            Name_pg_inheritproc;
Name            Name_pg_inherits;
Name            Name_pg_ipl;
Name            Name_pg_language;
Name            Name_pg_large_object;
Name	        Name_pg_ledger;
Name            Name_pg_listener;
Name            Name_pg_log;
Name            Name_pg_magic;
Name            Name_pg_nsvcattr;
Name            Name_pg_nsvcclass;
Name            Name_pg_nsvcfrag;
Name            Name_pg_naming;
Name            Name_pg_opclass;
Name            Name_pg_operator;
Name            Name_pg_parg;
#ifdef HP_JUKEBOX
Name            Name_pg_platter;
Name            Name_pg_plmap;
Name            Name_pg_plrel;
#endif
Name            Name_pg_proc;
Name            Name_pg_class;
Name            Name_pg_rewrite;
Name            Name_pg_server;
Name            Name_pg_statistic;
Name            Name_pg_time;
Name            Name_pg_type;
Name            Name_pg_user;
Name            Name_pg_variable;
Name            Name_pg_version;

/* the following names are synonyms for Name_pg_*,
   unfortunately, both sets of names are used throughout the code */

Name	AccessRelationName;
Name	AggregateRelationName;
Name	AccessMethodRelationName;
Name	AccessMethodOperatorRelationName;
Name	AccessMethodProcedureRelationName;
Name	AttributeRelationName;
Name	ContractRelationName;
Name	DatabaseRelationName;
Name	DefaultsRelationName;
Name	DemonRelationName;
Name    FragRelationName;
Name	GroupRelationName;
Name	HasRelationName;
Name	HostRelationName;
Name	IndexRelationName;
Name	InheritProcedureRelationName;
Name	InheritsRelationName;
Name	InheritancePrecidenceListRelationName;
Name	LanguageRelationName;
Name	LedgerRelationName;
Name	ListenerRelationName;
Name    LogRelationName;
Name	MagicRelationName;
Name	OperatorClassRelationName;
Name	OperatorRelationName;
Name	ProcedureArgumentRelationName;
Name	ProcedureRelationName;
Name	Prs2PlansRelationName;
Name	Prs2RuleRelationName;
Name	Prs2StubRelationName;
Name	RelationRelationName;
Name	RewriteRelationName;
Name	ServerRelationName;
Name	StatisticRelationName;
Name    TimeRelationName;
Name	TypeRelationName;
Name	UserRelationName;
Name	VariableRelationName;
Name	VersionRelationName;

Name IndexedCatalogNames[] = {
    &pg_attribute_namedata,
    &pg_proc_namedata,
    &pg_type_namedata,
    &pg_naming_namedata,
    &pg_class_namedata,
    (Name) NULL
#else /* P95 */
char *IndexedCatalogNames[] = {
    AttributeRelationName,
    ProcedureRelationName,
    TypeRelationName,
    RelationRelationName,
    0
#endif /* P95 */
};


/* ----------------
 * we just do a linear search now so there's no requirement that the list
 * be ordered.  The list is so small it shouldn't make much difference.
 * make sure the list is null-terminated
 *              - jolly 8/19/95
 *                                  
 * OLD COMMENT
 *	WARNING  WARNING  WARNING  WARNING  WARNING  WARNING
 *
 *	keep SharedSystemRelationNames[] in SORTED order!  A binary search
 *	is done on it in catalog.c!
 *
 *	XXX this is a serious hack which should be fixed -cim 1/26/90
 * ----------------
 */
#ifndef P95
Name SharedSystemRelationNames[17] = {
        &pg_access_namedata,
        &pg_contract_namedata,
	&pg_database_namedata,
	&pg_defaults_namedata,
	&pg_demon_namedata,
	&pg_group_namedata,
        &pg_has_namedata,
	&pg_host_namedata,
	&pg_log_namedata,
	&pg_magic_namedata,
	&pg_nsvcattr_namedata,
	&pg_nsvcclass_namedata,
	&pg_nsvcfrag_namedata,
	&pg_server_namedata,
	&pg_time_namedata,
	&pg_user_namedata,
	&pg_variable_namedata
#else /* P95 */
char *SharedSystemRelationNames[] = {
    DatabaseRelationName, 
    DefaultsRelationName,
    DemonRelationName,
    GroupRelationName,
    HostsRelationName,
    LogRelationName,
    MagicRelationName,
    ServerRelationName,
    TimeRelationName,
    UserRelationName,
    VariableRelationName,
    0
#endif /* P95 */
};

/* set up global variables, pointers, etc. */
void InitGlobals()
{
#ifndef P95
    UserName = &UserNameData;
    DatabaseName = &DatabaseNameData;
    DatabasePath = malloc(MAXPGPATH);
    
    Name_pg_access = &pg_access_namedata;
    Name_pg_aggregate = &pg_aggregate_namedata;
    Name_pg_am = &pg_am_namedata;
    Name_pg_amop = &pg_amop_namedata;
    Name_pg_amproc = &pg_amproc_namedata;
    Name_pg_attribute = &pg_attribute_namedata;
    Name_pg_contract = &pg_contract_namedata;
    Name_pg_database = &pg_database_namedata;
    Name_pg_defaults = &pg_defaults_namedata;
    Name_pg_demon = &pg_demon_namedata;
    Name_pg_fragment = &pg_fragment_namedata;
    Name_pg_group = &pg_group_namedata;
    Name_pg_has = &pg_has_namedata;
    Name_pg_host = &pg_host_namedata;
    Name_pg_index = &pg_index_namedata;
    Name_pg_inheritproc = &pg_inheritproc_namedata;
    Name_pg_inherits = &pg_inherits_namedata;
    Name_pg_ipl = &pg_ipl_namedata;
    Name_pg_language = &pg_language_namedata;
    Name_pg_large_object = &pg_large_object_namedata;
    Name_pg_ledger = &pg_ledger_namedata;
    Name_pg_listener = &pg_listener_namedata;
    Name_pg_log = &pg_log_namedata;
    Name_pg_magic = &pg_magic_namedata;
    Name_pg_nsvcattr = &pg_nsvcattr_namedata;
    Name_pg_nsvcclass = &pg_nsvcclass_namedata;
    Name_pg_nsvcfrag = &pg_nsvcfrag_namedata;
    Name_pg_naming = &pg_naming_namedata;
    Name_pg_opclass = &pg_opclass_namedata;
    Name_pg_operator = &pg_operator_namedata;
    Name_pg_parg = &pg_parg_namedata;
#ifdef HP_JUKEBOX
    Name_pg_platter = &pg_platter_namedata;
    Name_pg_plmap = &pg_plmap_namedata;
    Name_pg_plrel = &pg_plrel_namedata;
#endif
    Name_pg_proc = &pg_proc_namedata;
    Name_pg_class = &pg_class_namedata;
    Name_pg_rewrite = &pg_rewrite_namedata;
    Name_pg_server = &pg_server_namedata;
    Name_pg_statistic = &pg_statistic_namedata;
    Name_pg_time = &pg_time_namedata;
    Name_pg_type = &pg_type_namedata;
    Name_pg_user = &pg_user_namedata;
    Name_pg_variable = &pg_variable_namedata;
    Name_pg_version = &pg_version_namedata;
    
    AccessRelationName = Name_pg_access;
    AggregateRelationName = Name_pg_aggregate;
    AccessMethodRelationName = Name_pg_am;
    AccessMethodOperatorRelationName = Name_pg_amop;
    AccessMethodProcedureRelationName = Name_pg_proc;
    AttributeRelationName = Name_pg_attribute;
    ContractRelationName = Name_pg_contract;
    DatabaseRelationName = Name_pg_database;
    DefaultsRelationName = Name_pg_defaults;
    DemonRelationName = Name_pg_demon;
    FragRelationName = Name_pg_fragment;
    GroupRelationName = Name_pg_group;
    HasRelationName = Name_pg_has;
    HostRelationName = Name_pg_host;
    IndexRelationName = Name_pg_index;
    InheritProcedureRelationName = Name_pg_inheritproc;
    InheritsRelationName = Name_pg_inherits;
    InheritancePrecidenceListRelationName = Name_pg_ipl;
    LanguageRelationName = Name_pg_language;
    LedgerRelationName = Name_pg_ledger;
    ListenerRelationName = Name_pg_listener;
    LogRelationName = Name_pg_log;
    MagicRelationName = Name_pg_magic;
    OperatorClassRelationName = Name_pg_opclass;
    OperatorRelationName = Name_pg_operator;
    ProcedureArgumentRelationName = Name_pg_parg;
    ProcedureRelationName = Name_pg_proc;
    RelationRelationName = Name_pg_class;
    RewriteRelationName = Name_pg_rewrite;
    ServerRelationName = Name_pg_server;
    StatisticRelationName = Name_pg_statistic;
    TimeRelationName = Name_pg_time;
    TypeRelationName = Name_pg_type;
    UserRelationName = Name_pg_user;
    VariableRelationName = Name_pg_variable;
    VersionRelationName = Name_pg_version;

#endif /* P95 */
    MasterPid = getpid();
#ifdef P95
    DataDir = GetPGData();
#endif /* P95 */
}
#ifdef P95

#endif /* P95 */

