/*-------------------------------------------------------------------------
 *
 * ledger.c--
 *    
 *
 * Copyright (c) 1994, Regents of the University of California
 *
 *
 * IDENTIFICATION
 *    $Header: /usr/local/devel/mariposa/cvs/src/backend/utils/ledger/ledger.c,v 1.3 1996/06/15 07:22:31 am Exp $
 *
 *-------------------------------------------------------------------------
 */

#include "utils/ledger.h"

void
LedgerInsert(Form_pg_ledger newEntry) {
    Relation ledgerRel;
    Oid newOid;
    HeapTuple ledgerTuple;
    Datum values[Natts_pg_ledger];
    char nulls[Natts_pg_ledger];
    int i;

    /*
     * No nulls - indicated in heap_formtuple by a space
     */
    for (i = 0; i < Natts_pg_ledger; i++) {
	nulls[i] = ' ';
    }

    values[Anum_pg_ledger_procsite - 1] = Int32GetDatum(newEntry->procsite);
    values[Anum_pg_ledger_bidamount - 1] = Float64GetDatum(&newEntry->bidamount);
    values[Anum_pg_ledger_recdamount - 1] = Float64GetDatum(&newEntry->recdamount);
    values[Anum_pg_ledger_bidwon - 1] = CharGetDatum(newEntry->bidwon);
    /*values[Anum_pg_ledger_planstring - 1] = PointerGetDatum(newEntry->planstring);*/
	values[Anum_pg_ledger_planstring - 1] =  0;
    
    ledgerRel = heap_openr(Name_pg_ledger);
    
    ledgerTuple = heap_formtuple(Natts_pg_ledger, ledgerRel->rd_att, values, nulls);

    newOid = heap_insert(ledgerRel, ledgerTuple);

    heap_close(ledgerRel);
    
}

/*
 * Enter into pg_ledger the payments and receipts from
 * the queries in qd.
 */
void
MakeLedgerEntries(QueryDesc *qd) {

    FormData_pg_ledger newEntry;
    char planStr[1024];
    int nodeNum = 0;
    
    /*
     * Enter the money made for the local plan,
     * if any.
     */
    if (qd->plantree != NULL) {
	newEntry.procsite = qd->plantree->processingSite.hostid;
	if (qd->protocol == SHORT_PROCESS_QUERY) {
	    newEntry.bidamount = 0.0;
	    newEntry.bidwon = TRUE;
	} else {
	    newEntry.bidamount = qd->plantree->budget.cost;
	    newEntry.bidwon = TRUE;
	}
	newEntry.recdamount = qd->plantree->budget.cost;

	planStr[0] = '\0';
	planString(planStr, qd->plantree, qd->parsetree->rtable, &nodeNum, FALSE, TRUE);
	/*newEntry.planstring = textin(planStr)*/;
	LedgerInsert(&newEntry);
    }
    


    /*
     * And enter the money spent on remote plans, if any
     */

    if (qd->remoteplans != NIL) {
	List *lp;

	foreach(lp, qd->remoteplans) {
	    Plan *p = ((CoordPlan *)lfirst(lp))->plan;
	    newEntry.procsite = p->processingSite.hostid;
	    newEntry.bidamount = 0.0;
	    newEntry.bidwon = TRUE;
	    newEntry.recdamount = -1.0 * p->budget.cost;
	    planStr[0] = '\0';
	    planString(planStr, p, qd->parsetree->rtable, &nodeNum, FALSE, TRUE);
	    /*newEntry.planstring = textin(planStr);*/
	    LedgerInsert(&newEntry);
	}
	
    }
    
}

