/*-------------------------------------------------------------------------
 *
 * plans.c--
 *    
 *
 * Copyright (c) 1994, Regents of the University of California
 *
 *
 * IDENTIFICATION
 *    $Header: /usr/local/devel/mariposa/cvs/src/backend/utils/plans/plans.c,v 1.1 1996/05/31 17:15:19 jsidell Exp $
 *
 *-------------------------------------------------------------------------
 */

#include "utils/plans.h"

/*
 *-------------------------------------------------------------------------
 * CopyBudgetIntoPlan--
 *
 *	Copies the budget passed in to the root of the plan
 *	tree passed in.  Then recursively sets all the other
 *	budgets to 0.  If a bid is for a subtree of a plan,
 *	the bid for the subtree is kept in its root, and the
 *	bids for all the other nodes in the subtree are 0.
 *
 *-------------------------------------------------------------------------
 */
void
CopyBudgetIntoPlan(Budget *budget, Plan *plan)
{
    List *lp;

    /*
     * Default bid response is BID.  If any node
     * in a sub-tree is REFUSETOBID, then the
     * bid for the entire tree will be REFUSETOBID.
     * The broker adds up all the bids for a subplan
     * during subcontracting.
     */
    static Budget b = { BID, 0.0, 0.0, 0.0, 0.0 };
    
    if (plan != (Plan *) NULL) {
	plan->budget.response = budget->response;
	plan->budget.cost = budget->cost;
	plan->budget.delay = budget->delay;
	plan->budget.staleness = budget->staleness;
	plan->budget.accuracy = budget->accuracy;
	
	switch (plan->type) {
	case T_Merge:
	    foreach(lp, ((Merge *) plan)->mergeplans) {
		CopyBudgetIntoPlan(&b, (Plan *) lfirst(lp));
	    }
	    break;
	case T_Append:
	    foreach(lp, ((Append *) plan)->unionplans) {
		CopyBudgetIntoPlan(&b, (Plan *) lfirst(lp));
	    }
	    break;
	default:
	    CopyBudgetIntoPlan(&b, plan->lefttree);
	    CopyBudgetIntoPlan(&b, plan->righttree);
	}
    }
}
