/*-------------------------------------------------------------------------
 *
 * planstring.c--
 *    
 *
 * Copyright (c) 1994, Regents of the University of California
 *
 *
 * IDENTIFICATION
 *    $Header: /usr/local/devel/mariposa/cvs/src/backend/utils/plans/planstring.c,v 1.2 1996/06/14 20:24:17 jsidell Exp $
 *
 *-------------------------------------------------------------------------
 */

#include "utils/planstring.h"

char *fragInfoString(Plan *p, List *rtable);
void deleteLeadingAndTrailingSpaces(char **ps);
char *tclList(char *s);
void tclUnlist(char **ps);
int PopTypeStr(char **ps, char *typeStr);
int StringToType(char *typeStr, int *type);
int lindex(char *listStr, int n, char **resultStr);
int llength(char *listStr, int *resultStr);
Plan *FindPlanNodeNumber(Plan *plan, int rootNodeNum, int *currentNodeNumber);


static Tcl_Interp* tclInterp = NULL;
 
/*
 *-------------------------------------------------------------------------
 * StringToPlan--	Take the string representation of a plan, as created
 *			by planString(), and turn it back into a Plan.
 *
 * Input: 	plan string
 *
 * Output:	Plan
 *
 * Returns:	TCL_RESULT
 *-------------------------------------------------------------------------
 */
int 
StringToPlan(char *planString, Plan **p) {
    char typeStr[100];
    int result;
    int type;
    char *ps;

    fprintf(stderr, "Inside StringToPlan...\n");

    if (planString == NULL) {
	*p = NULL;
	return TCL_OK;
    }
    
    deleteLeadingAndTrailingSpaces(&planString);
    
    if (strlen(planString) == 0) {
	*p = NULL;
	return TCL_OK;
	
    }
    
    ps = (char *)malloc(strlen(planString) * sizeof(char));
    strcpy(ps, planString);
    
    tclUnlist(&ps);
    deleteLeadingAndTrailingSpaces(&ps);
    
    result = PopTypeStr(&ps, typeStr);
    if (result != TCL_OK) {
	return result;
    }
    
    result = StringToType(typeStr, &type);
    if (result != TCL_OK) {
	return result;
    }
    
    fprintf(stderr, "switch type...\n");
    switch (type) {
	
	/*
	 * Merge join node: MERGEJOIN NODENUM {LEFT-TREE} {RIGHT-TREE}
	 */
    case T_NestLoop:
 {
	Plan *lefttree;
	Plan *righttree;
	char *lefttreeStr; 
	char *righttreeStr;
	NestLoop *p2;
	
	result = lindex(tclList(ps), 1, &lefttreeStr);
	if (result != TCL_OK) {
	    return result;
	}
	
	result = lindex(tclList(ps), 2, &righttreeStr);
	if (result != TCL_OK) {
	    return result;
	}

	p2 = makeNode(NestLoop);
	

	result = StringToPlan(lefttreeStr, &lefttree);
	if (result != TCL_OK) {
	    return result;
	}

	result = StringToPlan(righttreeStr, &righttree);
	if (result != TCL_OK) {
	    return result;
	}

	((Plan *)p2)->lefttree = lefttree;
	((Plan *)p2)->righttree = righttree;
	
	*p = (Plan *)p2;
	
    }
	break;
 
    case T_MergeJoin: {
	Plan *lefttree;
	Plan *righttree;
	char *lefttreeStr; 
	char *righttreeStr;
	MergeJoin *p2;
	
	result = lindex(tclList(ps), 1, &lefttreeStr);
	if (result != TCL_OK) {
	    return result;
	}
	
	result = lindex(tclList(ps), 2, &righttreeStr);
	if (result != TCL_OK) {
	    return result;
	}

	p2 = makeNode(MergeJoin);
	

	result = StringToPlan(lefttreeStr, &lefttree);
	if (result != TCL_OK) {
	    return result;
	}

	result = StringToPlan(righttreeStr, &righttree);
	if (result != TCL_OK) {
	    return result;
	}

	((Plan *)p2)->lefttree = lefttree;
	((Plan *)p2)->righttree = righttree;
	
	*p = (Plan *)p2;
	
    }
	break;

	/*
	 * Sequential Scan node:
	 * 	If on a base relation: SEQSCAN NODENUM RTABLE-INDEX FRAG-INDEX {LEFT-TREE}
	 *	If on a temp relation: SEQSCAN NODENUM -1 {LEFT-TREE}
	 */
    case T_SeqScan: {
	Plan *lefttree;
	char *lefttreeStr;
	int scanrelid;
	int scanfragid;
	char *scanStr;
	char *fragStr;
	SeqScan *p2;
	
	p2 = makeNode(SeqScan);
	
	result = lindex(tclList(ps), 1, &scanStr);
	
	scanrelid = atoi(scanStr);

	if (scanrelid == -1) {
	    result = lindex(tclList(ps), 2, &lefttreeStr);
	    if (result != TCL_OK) {
		return result;
	    }
	    scanfragid = -1;
	} else {
	    result = lindex(tclList(ps), 2, &fragStr);
	    if (result != TCL_OK) {
		return result;
	    }
	    scanfragid = atoi(fragStr);
	    result = lindex(tclList(ps), 3, &lefttreeStr);
	    if (result != TCL_OK) {
		return result;
	    }
	}

	result = StringToPlan(lefttreeStr, &lefttree);
	if (result != TCL_OK) {
	    return result;
	}

	/*
	 * Tcl lindex's lists starting at 0,
	 * rtable stuff starts at 1.
	 */
	p2->scanrelid = scanrelid + 1;
	p2->scanfragid = scanfragid + 1;
	
	((Plan *)p2)->lefttree = lefttree;
	((Plan *)p2)->righttree = NULL;
	
	*p = (Plan *)p2;
	
    }
	break;

	/*
	 * Sort node: SORT NODENUM {LEFT-TREE}
	 */
    case T_Sort: {
	Plan *lefttree;
	char *lefttreeStr;
	Sort *p2;
	
	p2 = makeNode(Sort);
	
	result = lindex(tclList(ps), 1, &lefttreeStr);
	if (result != TCL_OK) {
	    return result;
	}

	result = StringToPlan(lefttreeStr, &lefttree);
	if (result != TCL_OK) {
	    return result;
	}
	
	((Plan *)p2)->lefttree = lefttree;
	((Plan *)p2)->righttree = NULL;
	
	*p = (Plan *)p2;
	
    }
	break;

	/*
	 * Merge node: MERGE NODENUM { {CHILD-TREE} {CHILD-TREE} {CHILD-TREE}... }
	 */
    case T_Merge: {
	Plan *child;
	char *childStr;
	int nChildren;
	int i;
	Merge *p2;
	
	result = llength(tclList(ps), &nChildren);
	
	if (result != TCL_OK) {
	    return result;
	}
	
	p2 = makeNode(Merge);
	
	p2->mergeplans = NIL;
	
	for (i = 1; i < nChildren; i++) {
	    result = lindex(tclList(ps), i, &childStr);
	    if (result != TCL_OK) {
		return result;
	    }
	    result = StringToPlan(childStr, &child);
	    if (result != TCL_OK) {
		return result;
	    }
	    p2->mergeplans = lcons(child, p2->mergeplans);
	}

	*p = (Plan *)p2;
    }
	break;
	
	/*
	 * X-in node: XIN NODENUM{LEFT-TREE}
	 */
    case T_ExchangeIn: {
	Plan *lefttree;
	char *lefttreeStr;
	ExchangeIn *p2;
	
	p2 = makeNode(ExchangeIn);	
	
	result = lindex(tclList(ps), 1, &lefttreeStr);
	if (result != TCL_OK) {
	    return result;
	}

	result = StringToPlan(lefttreeStr, &lefttree);
	if (result != TCL_OK) {
	    return result;
	}
	
	((Plan *)p2)->lefttree = lefttree;
	((Plan *)p2)->righttree = NULL;
	
	*p = (Plan *)p2;
	
    }
	break;
	
    }

    /*
     * Fill in this site for the processing site for every
     * node in the plan.  The Broker will fill in the other
     * processing sites.
     */
    (*p)->processingSite.type = T_SiteAddrNode;
    (*p)->processingSite.hostid = getMyHostId();
    (*p)->processingSite.port = 0;
    (*p)->processingSite.addr = getMyAddr();
    
    return TCL_OK;
    
}



/*
 *-------------------------------------------------------------------------
 * planString-- Create a string representation of a plan tree to be passed
 *		to the Tcl bidder script.
 *
 * Input: 	Plan tree, range table, whether or not to print processing sites.
 *
 * Output: 	String representation of plan tree in 'ps'.  The string is a Tcl
 *		list of the form: { NODE NODENUM {LEFT-TREE} {RIGHT-TREE} } for
 *		everything except MERGE nodes, which have no left/right
 *		tree, but have a list of plan trees whose results are
 *		being merged.  NODENUM is generated on the fly and is used
 *		during subcontracting to identify the subplan that is being
 *		subcontracted.
 *
 *		NODE is a string representing the node type:
 *
 *		MERGEJOIN, SCAN, MERGE...
 *
 *	NB - nodeNum is passed in as a pointer because the routine is
 *	     recursive and multithreaded.  nodeNum has to be incremented
 *	     with each recursive call and maintain its value when the
 *	     recursive call returns, but a static variable would just
 *	     keep increasing forever...
 *-------------------------------------------------------------------------
 */
 
void
planString(char *ps, Plan *plan, List *rtable, int *nodeNum, bool printNodeNum, bool printRelName) {
    
    if (plan == NULL) {
	(*nodeNum) = (*nodeNum) - 1;
	return;
    }
    
    switch (plan->type) {

	/*
	 * Merge join node: { MERGEJOIN NODENUM {LEFT-TREE} {RIGHT-TREE} }
	 */
    case T_MergeJoin: {
	char temp[100];

	strcat(ps, "{ MERGEJOIN ");
	if (printNodeNum) {
	    sprintf(temp, " %i ", *nodeNum);
	    strcat(ps, temp);
	}

	(*nodeNum) = (*nodeNum) + 1;
	planString(ps, plan->lefttree, rtable, nodeNum, printNodeNum, printRelName);
	(*nodeNum) = (*nodeNum) + 1;
	planString(ps, plan->righttree, rtable, nodeNum, printNodeNum, printRelName);
	strcat(ps, "} ");
    }
	break;


	/*
	 * Nested-loop join node: { NESTEDLOOP NODENUM {LEFT-TREE} {RIGHT-TREE} }
	 */
    case T_NestLoop: {
	char temp[100];

	strcat(ps, "{ NESTEDLOOP ");
	if (printNodeNum) {
	    sprintf(temp, " %i ", *nodeNum);
	    strcat(ps, temp);
	}
	
	(*nodeNum) = (*nodeNum) + 1;
	planString(ps, plan->lefttree, rtable, nodeNum, printNodeNum, printRelName);
	(*nodeNum) = (*nodeNum) + 1;
	planString(ps, plan->righttree, rtable, nodeNum, printNodeNum, printRelName);
	strcat(ps, "} ");
    }
	break;

	/*
	 * Sequential Scan node:
	 * 	If on a base relation: { SEQSCAN NODENUM RTABLE-INDEX FRAG-INDEX {LEFT-TREE} }
	 *	If on a temp relation: { SEQSCAN NODENUM -1 {LEFT-TREE} }
	 */
    case T_SeqScan: {
	int scanrelid = ((Scan *)plan)->scanrelid;
	int scanfragid = ((Scan *)plan)->scanfragid;
	char temp[100];
	
	/*
	 * Scan on a storage fragment.
	 */
	if (scanrelid > 0) {

	    	if (printNodeNum) {
		    if (printRelName) {
			RangeTblEntry *rte = rt_fetch(scanrelid, rtable);
			char *relName = rte->relname->data;
			
			sprintf(temp, "{ SEQSCAN %i %s ", *nodeNum, relName);
		    } else {
			sprintf(temp, "{ SEQSCAN %i %i %i ", *nodeNum, (scanrelid-1), (scanfragid-1));
		    }
		} else {
		    if (printRelName) {
			RangeTblEntry *rte = rt_fetch(scanrelid, rtable);
			char *relName = rte->relname->data;
			sprintf(temp, "{ SEQSCAN %s ", relName);
		    } else {
			sprintf(temp, "{ SEQSCAN %i %i ", (scanrelid-1), (scanfragid-1));
		    }
		    
		}
		
	    /*
	     * Scan on a temporary relation.
	     */
	} else {
	    if (printNodeNum) {
		if (printRelName) {
		} else {
		    sprintf(temp, "{ SEQSCAN %i -1 -1 ", *nodeNum);
		}
	    } else {
		if (printRelName) {
		} else {
		    sprintf(temp, "{ SEQSCAN -1 -1 ");
		}
		    
	    }
	}

	strcat(ps, temp);
	
	(*nodeNum) = (*nodeNum) + 1;
	planString(ps, plan->lefttree, rtable, nodeNum, printNodeNum, printRelName);
	

	strcat(ps, "} ");
	
    }
	break;

	/*
	 * Sort node: { SORT NODENUM {LEFT-TREE} }
	 */
    case T_Sort: {
	char temp[100];

	strcat(ps, "{ SORT ");
	
	if (printNodeNum) {
	    sprintf(temp, " %i ", *nodeNum);
	    strcat(ps, temp);
	}
	
	(*nodeNum) = (*nodeNum) + 1;
	planString(ps, plan->lefttree, rtable, nodeNum, printNodeNum, printRelName);
	strcat(ps, "} ");
    }
	break;

	/*
	 * Merge node: { MERGE NODENUM { {CHILD-TREE} {CHILD-TREE} {CHILD-TREE}...} }
	 */
    case T_Merge: {
	List *lp;
	char temp[100];
    
	strcat(ps, "{ MERGE ");
	if (printNodeNum) {
	    sprintf(temp, " %i { ", *nodeNum);
	    strcat(ps, temp);
	}
	foreach(lp, ((Merge *)plan)->mergeplans) {
	    Plan *mp = (Plan *)lp->elem;
	    (*nodeNum) = (*nodeNum) + 1;
	    planString(ps, mp, rtable, nodeNum, printNodeNum, printRelName);
	}
	
	strcat(ps, "} } ");
    }
	break;

	/*
	 * X-in node: {XIN NODENUM {LEFT-TREE} }
	 */
    case T_ExchangeIn: {
	char temp[100];
	strcat(ps, "{ XIN ");
	if (printNodeNum) {
	    sprintf(temp, " %i ", *nodeNum);
	    strcat(ps, temp);
	}
	(*nodeNum) = (*nodeNum) + 1;
	planString(ps, plan->lefttree, rtable, nodeNum, printNodeNum, printRelName);
	strcat(ps, "} ");
    }
	break;
	
	/*
	 * Agg node: {AGG NODENUM {LEFT-TREE} }
	 */
    case T_Agg: {
	char temp[100];
	strcat(ps, "{ AGG ");
	if (printNodeNum) {
	    sprintf(temp, " %i ", *nodeNum);
	    strcat(ps, temp);
	}
	(*nodeNum) = (*nodeNum) + 1;
	planString(ps, plan->lefttree, rtable, nodeNum, printNodeNum, printRelName);
	strcat(ps, "} ");
    }
	break;

	/*
	 * Group-By node: {GROUPBY NODENUM {LEFT-TREE} }
	 */
    case T_Group: {
	char temp[100];
	strcat(ps, "{ GROUPBY ");
	if (printNodeNum) {
	    sprintf(temp, " %i ", *nodeNum);
	    strcat(ps, temp);
	}
	(*nodeNum) = (*nodeNum) + 1;
	planString(ps, plan->lefttree, rtable, nodeNum, printNodeNum, printRelName);
	strcat(ps, "} ");
    }
	break;

	
	/*
	 * Some other type of node: {UNKNOWN NODENUM {LEFT-TREE} {RIGHT-TREE} }
	 */
    default: {
	char temp[100];
	strcat(ps, "{ UNKNOWN ");
	if (printNodeNum) {
	    sprintf(temp, " %i ", *nodeNum);
	    strcat(ps, temp);
	}
	(*nodeNum) = (*nodeNum) + 1;
	planString(ps, plan->lefttree, rtable, nodeNum, printNodeNum, printRelName);
	(*nodeNum) = (*nodeNum) + 1;
	planString(ps, plan->righttree, rtable, nodeNum, printNodeNum, printRelName);
	strcat(ps, "} ");
    }
	break;

    }

}



/*
 *-------------------------------------------------------------------------
 * rtableString--
 *
 * Input: 	List of range table entries
 *
 * Output: 	String representation of range table.  The string is a Tcl
 *		list of the form:
 *			{ rangeTableEntry rangeTableEntry... }
 *
 *		where each rangeTableEntry is a list:
 *			{ relname refname relid fragInfo }
 *
 *		and fragInfo is a list in which each entry is:
 *
 *			{ fralogicalid frastoreid frapages fratuples storagesites }
 *
 *		and (finally) storagesites is a list in which each entry is:
 *			{ port address hostid }
 *
 *
 *     		So, for instance, you might have (indentation added for clarity):
 *
 *		{ {Emp1 emp 12345678
 *			{11111111 22222222 8 8096
 *				{ {3042 102948372 1} }
 *			}
 *		} }
 *-------------------------------------------------------------------------
 */

char *
rtableString(List *rtable) {
    List *lp, *lp2, *lp3;
    char *rs;		/* Result string */
    char *current;	/* Points to current position in rs */
    int nchars;
    
    /*
     * For want of a better idea, I'm allocating 1024 characters
     * per range table entry.
     */
    rs = (char *)malloc(1024 * length(rtable) * sizeof(char));
    current = rs;

    nchars = sprintf(current, "{ ");
    current += nchars;
    
    foreach(lp, rtable) {
	RangeTblEntry *rte = (RangeTblEntry *)lfirst(lp);

	nchars = sprintf(current, "{ \"%s\" \"%s\" %i { ", rte->relname->data, rte->refname, rte->relid);
	current += nchars;

	foreach(lp2, rte->fragInfo) {
	    FragInfo *fInfo = (FragInfo *)lfirst(lp2);

	    nchars = sprintf(current, "{ %i %i %i %i { ",
			     fInfo->fralogicalid, fInfo->frastoreid, fInfo->frapages, fInfo->fratuples);
	    current += nchars;

	    foreach(lp3, fInfo->storageSites) {
		SiteAddrNode *s = (SiteAddrNode *)lfirst(lp3);
		nchars = sprintf(current, "{ %i %i %i } ", s->port, s->addr, s->hostid);
		current += nchars;
	    }

	    nchars = sprintf(current, "} } ");
	    current += nchars;
	}

	nchars = sprintf(current, "} } ");
	current += nchars;
	
    }
    
    nchars = sprintf(current, "} ");

    return rs;
}


/*
 * issystem	- returns non-zero iff relname is a system catalog
 *
 *	We now make a new requirement where system catalog relns must begin
 *	with pg_ while user relns are forbidden to do so.  Make the test
 * 	trivial and instantaneous.
 *
 *	XXX this is way bogus. -- pma
 *	YYY copied from /catalog/catalog.c - jps
 */
bool
issys(char relname[])
{
    if (relname[0] && relname[1] && relname[2])
	return (relname[0] == 'p' && 
		relname[1] == 'g' && 
		relname[2] == '_');
    else
	return FALSE;
}

/*
 *-------------------------------------------------------------------------
 * databrokerString--
 *
 * Returns a list of the form:
 *
 *	{fragment-info fragment-info...}
 *
 * where each "fragment-info" is:
 *
 *	{fraclassid fralogicalid frastoreid frapages fratuples {location-hostid's}}
 *
 * FragmentTake needs to be changed to accept the logicalid instead of the
 * storeid, since we don't know what the storeid is.  Then we can include
 * all of the storage locations and give the data broker a choice of where
 * to buy the fragment from.  Right now, it has to buy it from the processing
 * site.
 *
 * Just for kicks, I decided not to use recursion, but to perform a DFS
 * of the plan tree by using a stack.  When a scan node is encountered,
 * a "fragment-info" entry is created.
 *-------------------------------------------------------------------------
 */

char *
databrokerString(Plan *plan, List *rtable) {
    List *stack = NIL;
    char rs[1024];		/* Return string */

    rs[0] = '\0';	/* Start with a null-terminated, zero-length string */

    strcpy(rs, "{ ");	/* Opening bracket of list of fragment-info's */
    
    stack = lappend(stack, (void *)plan);	/* PUSH */
    
    while (stack != NIL) {
	Plan *p = (Plan *)listPop(&stack);	/* POP */

	switch (p->type) {
	case T_Merge: {
	    List *lp;
	    foreach(lp, ((Merge *)p)->mergeplans) {
		Plan *mp = (Plan *)lp->elem;
		stack = lappend(stack, (void *)mp);
	    }

	    
	}
	    break;

	case T_Append: {
	    List *lp;
	    foreach(lp, ((Append *) p)->unionplans) {
		Plan *ap = (Plan *)lp->elem;
		stack = lappend(stack, (void *)ap);
	    }
	    
	}
	    break;

	case T_Scan:
	case T_SeqScan:
	case T_IndexScan: {
	    /*
	     * If this is a scan over a base relation
	     */
	    if (((SeqScan *)p)->scanrelid != 0) {
		RangeTblEntry *rte = rt_fetch(((SeqScan *)p)->scanrelid, rtable);
		if (!issys(rte->relname->data)) {
		    char *rs2 = fragInfoString(p, rtable);
		    strcat(rs, rs2);
		    free(rs2);
		}
	    } else {
		stack = lappend(stack, p->lefttree);
	    }
	    
	}
	    break;

	default: {
	    if (p->lefttree != NULL) {
		stack = lappend(stack, p->lefttree);
	    }

	    if (p->righttree != NULL) {
		stack = lappend(stack, p->righttree);
	    }
	}
	    
	}
	
    }

    strcat(rs, "}");	/* Closing bracket of list of fragment-info's */
    
    return rs;
}

/*
 *-------------------------------------------------------------------------
 * fragInfoString--
 *
 * See comments above for databrokerString
 *-------------------------------------------------------------------------
 */
char *
fragInfoString(Plan *p, List *rtable) {
    char *rs = (char *)malloc(1024 * sizeof(char));
    int scanrelid = ((SeqScan *)p)->scanrelid;
    int scanfragid = ((SeqScan *)p)->scanfragid;
    RangeTblEntry *rte = rt_fetch(scanrelid, rtable);
    FragInfo *fInfo = nth(scanfragid - 1, rte->fragInfo);
    char temp[1000];
    List *lp;
    
    rs[0] = '\0';

    strcpy(rs, "{ ");	/* Opening bracket of fragInfoString */

    /*
     * Include classid, logicalid, storeid, nPages, nTuples
     */
    sprintf(temp, "%i %i %i %i %i %i ", rte->relid, fInfo->fralogicalid, fInfo->frastoreid, fInfo->frapages, fInfo->fratuples, p->processingSite.hostid);

    strcat(rs, temp);

    /*
     * Add storage site hostid's, processing site first.
     */

    strcat(rs, "{ ");	/* Opening bracket of storage sites */

    sprintf(temp, "%i ", p->processingSite.hostid);

    foreach(lp, fInfo->storageSites) {
	SiteAddrNode *sAddr = (SiteAddrNode *)lfirst(lp);
	if (sAddr->hostid != p->processingSite.hostid) {
	    sprintf(temp, "%i ", sAddr->hostid);
	}
    }

    strcat(rs, temp);
    
    strcat(rs, "} ");	/* Closing bracket of storage sites */
    
    strcat(rs, "} ");	/* Closing bracket of fragInfoString */

    return rs;
}

    
/*
 *-------------------------------------------------------------------------
 * bidString--
 *
 * Return a string representation of a Tcl list representing
 * a bid.  {response, cost,delay, staleness, accuracy}
 *-------------------------------------------------------------------------
 */
char *
bidString(Budget *bid)
{
    char bidStr[1024];
    char temp[100];

    bidStr[0] = '\0';
    
    if (bid->response == BID) {
	strcat(bidStr, "1");
    } else {
	strcat(bidStr, "0");
    }
    
    sprintf(temp, " %f %f %f %f", bid->cost, bid->delay, bid->staleness, bid->accuracy);

    strcat(bidStr, temp);
    
    return bidStr;
    
}

/*
 *-------------------------------------------------------------------------
 * deleteLeadingAndTrailingSpaces--	Well, duh.
 *
 *-------------------------------------------------------------------------
 */
void
deleteLeadingAndTrailingSpaces(char **ps) {

    int l;

    if (*ps == NULL) {
	return;
    }
    
    /*
     * Delete leading...
     */
    while ((*ps)[0] == ' ') {
	(*ps)++;
    }

    /*
     * ...and trailing spaces
     */
    l = strlen(*ps) - 1;
    while ((*ps)[l] == ' ') {
	l--;
    }

    (*ps)[++l] = '\0';
}




/*
 *-------------------------------------------------------------------------
 * PopTypeStr--		Return the string from the plan string that
 *			corresponds to plan->type.  This is MERGE, SEQSCAN,
 *			etc.  Pop the string off the front, modifying
 *			the string passed in.
 *
 * Input:	A string with no leading or trailing spaces.  The first
 *		token in the string should be the type.
 *
 * Output:      The type string.
 *
 * Returns:	TCL_RESULT
 *-------------------------------------------------------------------------
 */
int
PopTypeStr(char **ps, char *typeStr) {

    char *spacePos;
    char space = ' ';

    spacePos = strchr(*ps, (int)space);
    spacePos[0] = '\0';
    
    strcpy(typeStr, *ps);

    spacePos[0] = ' ';

    *ps = spacePos + 1;
    
    return TCL_OK;
}

/*
 *-------------------------------------------------------------------------
 * StringToType--
 *
 * Input: 	String representation of plan->type
 *
 * Output: 	enumerated type
 *
 * Returns:	TCL_RESULT
 *-------------------------------------------------------------------------
 */
int
StringToType(char *typeStr, int *type) {
    
    if (!strcmp(typeStr, "MERGEJOIN")) {
	*type = T_MergeJoin;
	return TCL_OK;
    }
    
    if (!strcmp(typeStr, "NESTEDLOOP")) {
	*type = T_NestLoop;
	return TCL_OK;
    }
    
    if (!strcmp(typeStr, "SEQSCAN")) {
	*type = T_SeqScan;
	return TCL_OK;
    }
    
    if (!strcmp(typeStr, "SORT")) {
	*type = T_Sort;
	return TCL_OK;
    }
    
    if (!strcmp(typeStr, "MERGE")) {
	*type = T_Merge;
	return TCL_OK;
    }
    
    if (!strcmp(typeStr, "XIN")) {
	*type = T_ExchangeIn;
	return TCL_OK;
    }

    return TCL_ERROR;
    
}

/*
 *-------------------------------------------------------------------------
 * tclList--
 *
 *	Input: a string.
 *
 *	Output: The string with braces around it.
 *-------------------------------------------------------------------------
 */
char *
tclList(char *s) {
    
    char *resultStr = (char *)malloc((strlen(s) + 4) * sizeof(char));
    
    sprintf(resultStr, "{ %s }", s);
    
    return resultStr;
}

/*
 *-------------------------------------------------------------------------
 * tclUnlist--
 *
 * Take the leading and trailing braces off a Tcl list
 *-------------------------------------------------------------------------
 */
void
tclUnlist(char **ps) {

    int l;

    if (*ps == NULL) {
	return;
    }
    
    if ((*ps)[0] == '{') {
	(*ps)++;
    }

    l = strlen(*ps) - 1;
    if ((*ps)[l] == '}') {
	l--;
    }

    (*ps)[++l] = '\0';
}

/*
 *-------------------------------------------------------------------------
 * lindex--	invoke tcl lindex command.
 *
 * Input:	input string listStr, index n
 *
 * Output:	result string, the nth element of listStr
 *
 * Returns:	TCL_RESULT
 *
 *-------------------------------------------------------------------------
 */
int
lindex(char *listStr, int n, char **resultStr) {
    int result;
    char cmd[4096];

    if (tclInterp == NULL) {
	tclInterp = Tcl_CreateInterp();
    }
    
    sprintf(cmd, "lindex %s %i", listStr, n);

    result = Tcl_Eval(tclInterp, cmd);

    *resultStr = tclInterp->result;

    return result;
     
}

/*
 *-------------------------------------------------------------------------
 * llength--	invokes the Tcl llength function
 *
 * Input:	input string listStr
 *
 * Output:	result string, length of listStr
 *
 * Returns:	TCL_RESULT
 *
 *-------------------------------------------------------------------------
 */

int
llength(char *listStr, int *resultInt) {
    int result;
    char cmd[4096];

    if (tclInterp == NULL) {
	tclInterp = Tcl_CreateInterp();
    }
    
    sprintf(cmd, "llength %s", listStr);

    result = Tcl_Eval(tclInterp, cmd);

    if (result == TCL_OK) {
	*resultInt = atoi(tclInterp->result);
    }

    return result;
     
}

/*
 *-------------------------------------------------------------------------
 * FindPlanNodeNumber-- Find the node in the plan tree that corresponds
 * 			to rootNodeNum
 *
 *	
 *-------------------------------------------------------------------------
 */
Plan *
FindPlanNodeNumber(Plan *plan, int nodeNum, int *currentNodeNumber) {

    if (plan == NULL) {
	(*currentNodeNumber) = (*currentNodeNumber) - 1;
	return NULL;
    }

    if (*currentNodeNumber == nodeNum) {
	return plan;
    }
    
    if (plan->type == T_Merge) {
	List *lp;

	foreach(lp, ((Merge *)plan)->mergeplans) {
	    Plan *mp = (Plan *)lfirst(lp);
	     Plan *p;
	    (*currentNodeNumber) = (*currentNodeNumber) + 1;
	    p = FindPlanNodeNumber(mp, nodeNum, currentNodeNumber);
	    if (p != NULL) {
		return p;
	    }
	}

    } else {
	Plan *p;

	(*currentNodeNumber) = (*currentNodeNumber) + 1;
	p = FindPlanNodeNumber(plan->lefttree, nodeNum, currentNodeNumber);
	if (p != NULL) {
	    return p;
	}
	(*currentNodeNumber) = (*currentNodeNumber) + 1;
	p = FindPlanNodeNumber(plan->righttree, nodeNum, currentNodeNumber);
	if (p != NULL) {
	    return p;
	}
    }
    
    return NULL;
}
