/*-------------------------------------------------------------------------
 *
 * rel.h--
 *    POSTGRES relation descriptor definitions.
 *
 *
 * Copyright (c) 1994, Regents of the University of California
 *
 * $Id: rel.h,v 0.10 1996/06/07 05:00:52 am Exp $
 *
 *-------------------------------------------------------------------------
 */
#ifndef	REL_H
#define REL_H

#include "postgres.h"

#include "storage/fd.h"
#include "access/strat.h"	
#include "access/tupdesc.h"

#include "catalog/pg_am.h"
#include "catalog/pg_operator.h"
#include "catalog/pg_class.h"
#ifndef P95
#include "catalog/pg_fragment.h"
#endif /* P95 */

#include "rewrite/prs2lock.h"
#ifndef P95
#include "nodes/pg_list.h"
#endif /* P95 */

typedef struct RelationData {
    File		rd_fd; 		/* open file descriptor */
    int                 rd_nblocks;	/* number of blocks in rel */
    uint16		rd_refcnt; 	/* reference count */
    bool		rd_islocal; 	/* uses the local buffer mgr */
    bool		rd_isnailed; 	/* rel is nailed in cache */
#ifndef P95
    Size		rd_blocksz;	/* block size for this relation */
#endif /* P95 */
    Form_pg_am 		rd_am; 		/* AM tuple */
    Form_pg_class	rd_rel;		/* RELATION tuple */
#ifndef P95
    Oid			rd_id;		/* relation's object id */
    Pointer		rd_lockinfo; 	/* lock manager LockInfo state */
    FragPartInfo	rd_fraginfo;	/* fragment partition info */
    bool		rd_iscopied;	/* true if engaged in copy contract */
    Oid			rd_copysidef;	/* reloid of copy side-file */
    bool		rd_isnsvcdata;	/* data provided by nameserver */
    TupleDesc		rd_att;		/* array of AttributeTupleForm */
#else /* P95 */
    Oid			rd_id;		/* relations's object id */
    Pointer		lockInfo; 	/* ptr. to misc. info. */
    TupleDesc           rd_att;		/* tuple desciptor */
#endif /* P95 */
    RuleLock		*rd_rules;	/* rewrite rules */
#ifndef P95
    List                *rd_fragrels;	/* XXX for 'fragment' smgr */
    IndexStrategy       rd_istrat;	/* array of index strategies */
    RegProcedure	*rd_support;	/* array of amproc procedures */
#else /* P95 */
    IndexStrategy       rd_istrat;    
    RegProcedure*       rd_support;
#endif /* P95 */
} RelationData;

typedef RelationData	*Relation;

/* ----------------
 *	RelationPtr is used in the executor to support index scans
 *	where we have to keep track of several index relations in an
 *	array.  -cim 9/10/89
 * ----------------
 */
typedef Relation	*RelationPtr;

#define InvalidRelation	((Relation)NULL)

typedef char	ArchiveMode;

/*
 * RelationIsValid --
 *	True iff relation descriptor is valid.
 */
#define	RelationIsValid(relation) PointerIsValid(relation)

/*
 * RelationGetSystemPort --
 *	Returns system port of a relation.
 *
 * Note:
 *	Assumes relation descriptor is valid.
 */
#define RelationGetSystemPort(relation) ((relation)->rd_fd)

/*
 * RelationGetLockInfo --
 *      Returns the lock information structure in the reldesc
 *
 */
#ifndef P95
#define RelationGetLockInfo(relation) ((relation)->rd_lockinfo)
#else /* P95 */
#define RelationGetLockInfo(relation) ((relation)->lockInfo)
#endif /* P95 */

/*
 * RelationHasReferenceCountZero --
 *	True iff relation reference count is zero.
 *
 * Note:
 *	Assumes relation descriptor is valid.
 */
#define RelationHasReferenceCountZero(relation) \
	((bool)((relation)->rd_refcnt == 0))

/*
 * RelationSetReferenceCount --
 *	Sets relation reference count.
 */
#define RelationSetReferenceCount(relation,count) ((relation)->rd_refcnt = count)

/*
 * RelationIncrementReferenceCount --
 *	Increments relation reference count.
 */
#define RelationIncrementReferenceCount(relation) ((relation)->rd_refcnt += 1);

/*
 * RelationDecrementReferenceCount --
 *	Decrements relation reference count.
 */
#define RelationDecrementReferenceCount(relation) ((relation)->rd_refcnt -= 1)

/*
 * RelationGetAccessMethodTupleForm --
 *	Returns access method attribute values for a relation.
 *
 * Note:
 *	Assumes relation descriptor is valid.
 */
#define RelationGetAccessMethodTupleForm(relation) ((relation)->rd_am)

/*
 * RelationGetRelationTupleForm --
 *	Returns relation attribute values for a relation.
 *
 * Note:
 *	Assumes relation descriptor is valid.
 */
#define RelationGetRelationTupleForm(relation) ((relation)->rd_rel)

#ifndef P95
/*
 * RelationGetTupleDescriptor --
 *	Returns tuple descriptor for a relation.
 *
 * Note:
 *	Assumes relation descriptor is valid.
 */
#define RelationGetTupleDescriptor(relation) ((relation)->rd_att)
#endif /* P95 */

/* 
 * RelationGetRelationId --
 *
 *  returns the object id of the relation
 *
 */
#define RelationGetRelationId(relation) ((relation)->rd_id)

/*
 * RelationGetFile --
 *
 *    Returns the open File decscriptor
 */
#define RelationGetFile(relation) ((relation)->rd_fd)


/*
 * RelationGetRelationName --
 *
 *    Returns a Relation Name
 */
#define RelationGetRelationName(relation) (&(relation)->rd_rel->relname)

/*
 *    Returns a the number of attributes.
 */
#define RelationGetNumberOfAttributes(relation) ((relation)->rd_rel->relnatts)

#ifdef P95
/*
 * RelationGetTupleDescriptor --
 *	Returns tuple descriptor for a relation.
 *
 * Note:
 *	Assumes relation descriptor is valid.
 */
#define RelationGetTupleDescriptor(relation) ((relation)->rd_att)

#endif /* P95 */
extern IndexStrategy RelationGetIndexStrategy(Relation relation);

extern void RelationSetIndexSupport(Relation relation, IndexStrategy strategy,
			     RegProcedure *support);
#ifndef P95

#define RelationGetSmgr(relation) ((relation)->rd_rel->relsmgr)
#endif /* P95 */
#endif	/* REL_H */
