/*-------------------------------------------------------------------------
 *
 * lselect.c--
 *    leftist tree selection algorithm (linked priority queue--Knuth, Vol.3,
 *    pp.150-52)
 *
 * Copyright (c) 1994, Regents of the University of California
 *
 *
 * IDENTIFICATION
 *    $Header: /usr/local/devel/mariposa/cvs/src/backend/utils/sort/lselect.c,v 0.7 1996/06/07 05:01:54 am Exp $
 *
 *-------------------------------------------------------------------------
 */
#include <string.h>
#include <stdio.h>

#include "c.h"

#include "storage/buf.h"
#include "access/skey.h"
#include "access/heapam.h"
#include "access/htup.h"
#include "utils/rel.h"

#ifndef P95
#include "nodes/execnodes.h"
#include "nodes/plannodes.h"

#endif /* P95 */
#include "utils/psort.h"
#include "utils/lselect.h"

#ifdef P95
extern	Relation	SortRdesc;		/* later static */ 
#endif /* P95 */

#ifdef P95
/*
 *	PUTTUP		- writes the next tuple
 *	ENDRUN		- mark end of run
 *	GETLEN		- reads the length of the next tuple
 *	ALLOCTUP	- returns space for the new tuple
 *	SETTUPLEN	- stores the length into the tuple
 *	GETTUP		- reads the tuple
 *
 *	Note:
 *		LEN field must be a short; FP is a stream
 */

#endif /* P95 */
#define	PUTTUP(TUP, FP)	fwrite((char *)TUP, (TUP)->t_len, 1, FP)
#ifndef P95
    /* writes the next tuple */
#else /* P95 */
#define	ENDRUN(FP)	fwrite((char *)&shortzero, sizeof (shortzero), 1, FP)
#define	GETLEN(LEN, FP)	fread(&(LEN), sizeof (shortzero), 1, FP)
#define	ALLOCTUP(LEN)	((HeapTuple)malloc((unsigned)LEN))
#define	GETTUP(TUP, LEN, FP)\
	fread((char *)(TUP) + sizeof (shortzero), 1, (LEN) - sizeof (shortzero), FP)
#define	SETTUPLEN(TUP, LEN)	(TUP)->t_len = LEN
#endif /* P95 */

/*
 *	USEMEM		- record use of memory
 *	FREEMEM		- record freeing of memory
 * [Postgres95]
 *	FULLMEM		- 1 iff a tuple will fit
 */
#ifndef P95
#define	USEMEM(AMT)	*(context->sortMem) -= (AMT)
#define	FREEMEM(AMT)	*(context->sortMem) += (AMT)
#endif /* P95 */

#ifdef P95
#define	USEMEM(AMT)	SortMemory -= (AMT)
#define	FREEMEM(AMT)	SortMemory += (AMT)
#define	LACKMEM()	(SortMemory <= BLCKSZ)		/* not accurate */

#endif /* P95 */
/*
 *	lmerge		- merges two leftist trees into one
 *
 *	Note:
 *		Enforcing the rule that pt->lt_dist >= qt->lt_dist may
 *		simplifify much of the code.  Removing recursion will not
 *		speed up code significantly.
 */
struct leftist *
#ifndef P95
lmerge(struct leftist *pt, struct leftist *qt, LeftistContext context)
#else /* P95 */
lmerge(struct leftist *pt, struct leftist *qt)
#endif /* P95 */
{
    register struct	leftist	*root, *majorLeftist, *minorLeftist;
    int		dist;
    
#ifndef P95
    if (tuplecmp(pt->lt_tuple, qt->lt_tuple, context)) {
#else /* P95 */
    if (tuplecmp(pt->lt_tuple, qt->lt_tuple)) {
#endif /* P95 */
	root = pt;
	majorLeftist = qt;
    } else {
	root = qt;
	majorLeftist = pt;
    }
    if (root->lt_left == NULL)
	root->lt_left = majorLeftist;
    else {
	if ((minorLeftist = root->lt_right) != NULL)
#ifndef P95
	    majorLeftist = lmerge(majorLeftist, minorLeftist, context);
#else /* P95 */
	    majorLeftist = lmerge(majorLeftist, minorLeftist);
#endif /* P95 */
	if ((dist = root->lt_left->lt_dist) < majorLeftist->lt_dist) {
	    root->lt_dist = 1 + dist;
	    root->lt_right = root->lt_left;
	    root->lt_left = majorLeftist;
	} else {
	    root->lt_dist = 1 + majorLeftist->lt_dist;
	    root->lt_right = majorLeftist;
	}
    }
    return(root);
}

static struct leftist *
#ifndef P95
linsert(struct leftist *root, struct leftist *new1, LeftistContext context)
#else /* P95 */
linsert(struct leftist *root, struct leftist *new1)
#endif /* P95 */
{
    register struct	leftist	*left, *right;
    
#ifndef P95
    if (! tuplecmp(root->lt_tuple, new1->lt_tuple, context)) {
#else /* P95 */
    if (! tuplecmp(root->lt_tuple, new1->lt_tuple)) {
#endif /* P95 */
	new1->lt_left = root;
	return(new1);
    }
    left = root->lt_left;
    right = root->lt_right;
    if (right == NULL) {
	if (left == NULL)
	    root->lt_left = new1;
	else {
	    root->lt_right = new1;
	    root->lt_dist = 2;
	}
	return(root);
    }
#ifndef P95
    right = linsert(right, new1, context);
#else /* P95 */
    right = linsert(right, new1);
#endif /* P95 */
    if (right->lt_dist < left->lt_dist) {
	root->lt_dist = 1 + left->lt_dist;
	root->lt_left = right;
	root->lt_right = left;
    } else {
	root->lt_dist = 1 + right->lt_dist;
	root->lt_right = right;
    }
    return(root);
}

/*
 *	gettuple	- returns tuple at top of tree (Tuples)
 *
 *	Returns:
 *		tuple at top of tree, NULL if failed ALLOC()
 *		*devnum is set to the devnum of tuple returned
 *		*treep is set to the new tree
 *
 *	Note:
 *		*treep must not be NULL
 *		NULL is currently never returned BUG
 */
HeapTuple
gettuple(struct leftist **treep,
#ifndef P95
	 short *devnum,		/* device from which tuple came */
	 LeftistContext context)
#else /* P95 */
	 short *devnum)		/* device from which tuple came */
#endif /* P95 */
{
    register struct	leftist	 *tp;
    HeapTuple	tup;
    
    tp = *treep;
    tup = tp->lt_tuple;
    *devnum = tp->lt_devnum;
    if (tp->lt_dist == 1)				/* lt_left == NULL */
	*treep = tp->lt_left;
    else
#ifndef P95
	*treep = lmerge(tp->lt_left, tp->lt_right, context);
#else /* P95 */
	*treep = lmerge(tp->lt_left, tp->lt_right);
#endif /* P95 */
    
    FREEMEM(sizeof (struct leftist));
    FREE(tp);
    return(tup);
}

/*
 *	puttuple	- inserts new tuple into tree
 *
 *	Returns:
 *		NULL iff failed ALLOC()
 *
 *	Note:
 *		Currently never returns NULL BUG
 */
#ifndef P95
void
puttuple(struct leftist **treep,
	 HeapTuple newtuple,
	 short devnum,
	 LeftistContext context)
#else /* P95 */
int
puttuple(struct leftist **treep, HeapTuple newtuple, int devnum)
#endif /* P95 */
{
    register struct	leftist	*new1;
    register struct	leftist	*tp;
    
    new1 = (struct leftist *) malloc((unsigned) sizeof (struct leftist));
    USEMEM(sizeof (struct leftist));
    new1->lt_dist = 1;
    new1->lt_devnum = devnum;
    new1->lt_tuple = newtuple;
    new1->lt_left = NULL;
    new1->lt_right = NULL;
    if ((tp = *treep) == NULL)
	*treep = new1;
    else
#ifndef P95
	*treep = linsert(tp, new1, context);
    return;
#else /* P95 */
	*treep = linsert(tp, new1);
    return(1);
#endif /* P95 */
}


/*
 * dumptuples	- stores all the tuples in tree into file
 */
void
#ifndef P95
dumptuples(FILE *file,
	   struct leftist **treep,
	   LeftistContext context)
#else /* P95 */
dumptuples(FILE *file)
#endif /* P95 */
{
    register struct	leftist	*tp;
    register struct	leftist	*newp;
    HeapTuple	tup;
    
#ifndef P95
    tp = *treep;
#else /* P95 */
    tp = Tuples;
#endif /* P95 */
    while (tp != NULL) {
	tup = tp->lt_tuple;
	if (tp->lt_dist == 1)			/* lt_right == NULL */
	    newp = tp->lt_left;
	else
#ifndef P95
	    newp = lmerge(tp->lt_left, tp->lt_right, context);
#else /* P95 */
	    newp = lmerge(tp->lt_left, tp->lt_right);
#endif /* P95 */
	FREEMEM(sizeof (struct leftist));
	FREE(tp);
	PUTTUP(tup, file);
	FREEMEM(tup->t_len);
	FREE(tup);
	tp = newp;
    }
#ifndef P95
    *treep = NULL;
#else /* P95 */
    Tuples = NULL;
#endif /* P95 */
}

/*
 *	tuplecmp	- Compares two tuples with respect CmpList
 *
 *	Returns:
 *		1 if left < right ;0 otherwise
 *	Assumtions:
 */
int
#ifndef P95
tuplecmp(HeapTuple ltup, HeapTuple rtup, LeftistContext context)
#else /* P95 */
tuplecmp(HeapTuple ltup, HeapTuple rtup)
#endif /* P95 */
{
    register char	*lattr, *rattr;
    int		nkey = 0;
    extern	int	Nkeys;
    extern	ScanKey	Key;
    int		result = 0;
    bool		isnull;
    
    if (ltup == (HeapTuple)NULL)
	return(0);
    if (rtup == (HeapTuple)NULL)
	return(1);
#ifndef P95
    while (nkey < context->nKeys && !result) {
#else /* P95 */
    while (nkey < Nkeys && !result) {
#endif /* P95 */
	lattr = heap_getattr(ltup, InvalidBuffer,
#ifndef P95
			     context->scanKeys[nkey].sk_attno,
			     context->tupDesc, &isnull);
#else /* P95 */
			     Key[nkey].sk_attno, 
			     RelationGetTupleDescriptor(SortRdesc),
			     &isnull);
#endif /* P95 */
	if (isnull)
	    return(0);
	rattr = heap_getattr(rtup, InvalidBuffer,
#ifndef P95
			     context->scanKeys[nkey].sk_attno,
			     context->tupDesc,
#else /* P95 */
			     Key[nkey].sk_attno, 
			     RelationGetTupleDescriptor(SortRdesc),
#endif /* P95 */
			     &isnull);
	if (isnull)
	    return(1);
#ifndef P95
	if (context->scanKeys[nkey].sk_flags & SK_COMMUTE) {
	    if (!(result =
		  (long) (*context->scanKeys[nkey].sk_func) (rattr, lattr)))
		result =
		    -(long) (*context->scanKeys[nkey].sk_func) (lattr, rattr);
	} else if (!(result =
		     (long) (*context->scanKeys[nkey].sk_func) (lattr, rattr)))
	    result =
		-(long) (*context->scanKeys[nkey].sk_func) (rattr, lattr);
#else /* P95 */
	if (Key[nkey].sk_flags & SK_COMMUTE) {
	    if (!(result = (long) (*Key[nkey].sk_func) (rattr, lattr)))
		result = -(long) (*Key[nkey].sk_func) (lattr, rattr);
	} else if (!(result = (long) (*Key[nkey].sk_func) (lattr, rattr)))
	    result = -(long) (*Key[nkey].sk_func) (rattr, lattr);
#endif /* P95 */
	nkey++;
    }
    return (result == 1);
}

#ifdef	EBUG
void
#ifndef P95
checktree(struct leftist *tree, LeftistContext context)
#else /* P95 */
checktree(struct leftist *tree)
#endif /* P95 */
{
    int		lnodes;
    int		rnodes;
    
    if (tree == NULL) {
	puts("Null tree.");
	return;
    }
#ifndef P95
    lnodes = checktreer(tree->lt_left, 1, context);
    rnodes = checktreer(tree->lt_right, 1, context);
#else /* P95 */
    lnodes = checktreer(tree->lt_left, 1);
    rnodes = checktreer(tree->lt_right, 1);
#endif /* P95 */
    if (lnodes < 0) {
	lnodes = -lnodes;
	puts("0:\tBad left side.");
    }
    if (rnodes < 0) {
	rnodes = -rnodes;
	puts("0:\tBad right side.");
    }
    if (lnodes == 0) {
	if (rnodes != 0)
	    puts("0:\tLeft and right reversed.");
	if (tree->lt_dist != 1)
	    puts("0:\tDistance incorrect.");
    } else if (rnodes == 0) {
	if (tree->lt_dist != 1)
	    puts("0:\tDistance incorrect.");
    } else if (tree->lt_left->lt_dist < tree->lt_right->lt_dist) {
	puts("0:\tLeft and right reversed.");
	if (tree->lt_dist != 1 + tree->lt_left->lt_dist)
	    puts("0:\tDistance incorrect.");
    } else if (tree->lt_dist != 1+ tree->lt_right->lt_dist)
	puts("0:\tDistance incorrect.");
    if (lnodes > 0)
#ifndef P95
	if (tuplecmp(tree->lt_left->lt_tuple, tree->lt_tuple, context))
#else /* P95 */
	if (tuplecmp(tree->lt_left->lt_tuple, tree->lt_tuple))
#endif /* P95 */
	    printf("%d:\tLeft child < parent.\n");
    if (rnodes > 0)
#ifndef P95
	if (tuplecmp(tree->lt_right->lt_tuple, tree->lt_tuple, context))
#else /* P95 */
	if (tuplecmp(tree->lt_right->lt_tuple, tree->lt_tuple))
#endif /* P95 */
	    printf("%d:\tRight child < parent.\n");
    printf("Tree has %d nodes\n", 1 + lnodes + rnodes);
}

int
#ifndef P95
checktreer(struct leftist *tree, int level, LeftistContext context)
#else /* P95 */
checktreer(struct leftist *tree, int level)
#endif /* P95 */
{
    int	lnodes, rnodes;
    int	error = 0;
    
    if (tree == NULL)
	return(0);
#ifndef P95
    lnodes = checktreer(tree->lt_left, level + 1, context);
    rnodes = checktreer(tree->lt_right, level + 1, context);
#else /* P95 */
    lnodes = checktreer(tree->lt_left, level + 1);
    rnodes = checktreer(tree->lt_right, level + 1);
#endif /* P95 */
    if (lnodes < 0) {
	error = 1;
	lnodes = -lnodes;
	printf("%d:\tBad left side.\n", level);
    }
    if (rnodes < 0) {
	error = 1;
	rnodes = -rnodes;
	printf("%d:\tBad right side.\n", level);
    }
    if (lnodes == 0) {
	if (rnodes != 0) {
	    error = 1;
	    printf("%d:\tLeft and right reversed.\n", level);
	}
	if (tree->lt_dist != 1) {
	    error = 1;
	    printf("%d:\tDistance incorrect.\n", level);
	}
    } else if (rnodes == 0) {
	if (tree->lt_dist != 1) {
	    error = 1;
	    printf("%d:\tDistance incorrect.\n", level);
	}			
    } else if (tree->lt_left->lt_dist < tree->lt_right->lt_dist) {
	error = 1;
	printf("%d:\tLeft and right reversed.\n", level);
	if (tree->lt_dist != 1 + tree->lt_left->lt_dist)
	    printf("%d:\tDistance incorrect.\n", level);
    } else if (tree->lt_dist != 1+ tree->lt_right->lt_dist) {
	error = 1;
	printf("%d:\tDistance incorrect.\n", level);
    }
    if (lnodes > 0)
#ifndef P95
	if (tuplecmp(tree->lt_left->lt_tuple, tree->lt_tuple, context)) {
#else /* P95 */
	if (tuplecmp(tree->lt_left->lt_tuple, tree->lt_tuple)) {
#endif /* P95 */
	    error = 1;
	    printf("%d:\tLeft child < parent.\n");
	}
    if (rnodes > 0)
#ifndef P95
	if (tuplecmp(tree->lt_right->lt_tuple, tree->lt_tuple, context)) {
#else /* P95 */
	if (tuplecmp(tree->lt_right->lt_tuple, tree->lt_tuple)) {
#endif /* P95 */
	    error = 1;
	    printf("%d:\tRight child < parent.\n");
	}
    if (error)
	return(-1 + -lnodes + -rnodes);
    return(1 + lnodes + rnodes);
}
#endif
