/*-------------------------------------------------------------------------
 *
 * syscache.h--
 *    System catalog cache definitions.
 *
 *
 * Copyright (c) 1994, Regents of the University of California
 *
 * $Id: syscache.h,v 0.5 1996/06/07 05:00:54 am Exp $
 *
 *-------------------------------------------------------------------------
 */
#ifndef	SYSCACHE_H
#define SYSCACHE_H

/*#define CACHEDEBUG*/ 	/* turns DEBUG elogs on */

#include "postgres.h"
#include "access/htup.h"
#include "nodes/pg_list.h"

/*
 *	Declarations for util/syscache.c.
 *
 *	SysCache identifiers.
 *
 *      The order of these must match the order
 *      they are entered into the structure cacheinfo[] in syscache.c 
 *      The best thing to do is to add yours at the END, because some
 *      code assumes that certain caches are at certain places in this
 *      array.
 */

#define	AMOPOPID	0
#define	AMOPSTRATEGY	1
#define	ATTNAME		2
#define	ATTNUM		3
#define	INDEXRELID	4
#define	LANNAME		5
#define	OPRNAME		6
#define	OPROID		7
#define	PRONAME		8
#define	PROOID		9
#define	RELNAME		10
#define	RELOID		11
#define	TYPNAME		12
#define	TYPOID 		13
#define	AMNAME		14
#define	CLANAME		15
#define INDRELIDKEY	16
#define INHRELID	17
#define	RULOID		18
#define AGGNAME		19
#define LISTENREL       20
#define USENAME		21
#define USESYSID	22
#define GRONAME		23
#define GROSYSID	24
#define	REWRITENAME	25
#define PROSRC          26
#ifndef P95
#define FRALOGICALID	27
#endif /* P95 */

/* ----------------
 *	struct cachedesc:	information needed for a call to InitSysCache()
 * ----------------
 */
struct cachedesc {
#ifndef P95
    Name      *name;	       /* this is Name * so that we can initialize it */
#else /* P95 */
    char     *name;	       /* this is Name * so that we can initialize it */
#endif /* P95 */
    int	      nkeys;
    int	      key[4];
    int	      size;            /* sizeof(appropriate struct) */
#ifndef P95
    Name      *indname;	       /* index relation for this cache, if exists */
#else /* P95 */
    char      *indname;	       /* index relation for this cache, if exists */
#endif /* P95 */
    HeapTuple (*iScanFunc)();  /* function to handle index scans */
};

extern void zerocaches(void);
extern void InitCatalogCache(void);
#ifndef P95
extern HeapTuple SearchSysCacheTuple(int cacheId, char *key1, char *key2,
				     char *key3, char *key4);
#else /* P95 */
extern HeapTuple SearchSysCacheTuple(int cacheId, Datum key1, Datum key2,
				     Datum key3, Datum key4);
#endif /* P95 */
extern int32 SearchSysCacheStruct(int cacheId, char *returnStruct,
#ifndef P95
			char *key1, char *key2, char *key3, char *key4);
#else /* P95 */
			Datum key1, Datum key2, Datum key3, Datum key4);
#endif /* P95 */
extern void *SearchSysCacheGetAttribute(int cacheId,
#ifndef P95
			AttrNumber attributeNumber, char *key1,
			char *key2, char *key3, char *key4);
#else /* P95 */
					AttrNumber attributeNumber, 
					Datum key1,
					Datum key2, 
					Datum key3, 
					Datum key4);
#endif /* P95 */
extern void *TypeDefaultRetrieve(Oid typId);

#endif	/* SYSCACHE_H */
