#!/bin/sh
#-------------------------------------------------------------------------
#
# initmp.sh--
#    initialize mariposa sites on the same machine
#
# Copyright (c) 1994, Regents of the University of California
#
#
# This script initializes databases on each machine specified in SITES. The
# location of the databases are assumed to be in $PGDATA-hostid
# It accepts the following options:
#	-h	hostname to use for the pg_host IP addresses
#	-n	no prompt
#	-s	skip initdb (for reinit)
#	
# The following environment variables can be set to change the default
# settings:
#
# HOSTS		- the mariposa hosts (eg. '1 2 3 4')
#
# $Id: initmp.sh,v 1.4 1996/06/14 20:07:40 am Exp $
#
#-------------------------------------------------------------------------

######################################################################
# default settings

[ -z "$HOSTS" ] && HOSTS='1 2 3 4'
[ -z "$PGDATA" ] && PGDATA=_fUnKy_DATADIR_sTuFf_
FILESDIR=$PGDATA/files
BINDIR=_fUnKy_BINDIR_sTuFf_
PATH=$BINDIR:$PATH

noprompt=0
skipinitdb=0

while [ -n "$1" ]; do
	case $1 in
	-h)	ourhostname=$2
		shift;;
	-n)	noprompt=1;;
	-s)	skipinitdb=1;;
	esac
	shift;
done
if [ -z "$ourhostname" ]; then
  ourhostname=`hostname`
fi
ouraddress=`mp_hostaddr $ourhostname`

######################################################################
# prompt for confirmation
if [ $skipinitdb -eq 0 ]; then
	echo "Initializing Mariposa Sites";
else
	echo "Re-initializing Mariposa Sites";
	for h in $HOSTS; do
		if [ ! -f $PGDATA-$h/base/template1/PG_VERSION ]; then
			echo "Error: Initialize $PGDATA-$h first!";	
			exit 1
		fi
	done
fi
echo ""
echo "hosts = $HOSTS"
echo "PGDATA = $PGDATA-##"
echo "host name = $ourhostname"
echo "host address = $ouraddress"
echo ""
if [ $noprompt -eq 0 ]; then
	echo -n 'proceed (y/n)? '
	read input
	if [ $input != "y" ]; then 
		exit 1
	fi
	echo ""
fi

######################################################################
# make sure they are all empty
if [ $skipinitdb -eq 0 ]; then
	echo "Checking each PGDATA directory to make sure they are empty..."
	for h in $HOSTS; do
		file="$PGDATA-$h"
		if [ ! -e $file ]; then
			echo mkdir $file
			mkdir $file
		fi
		if [ `ls $file | wc -l` -ne 0 ]; then
			if [ $noprompt -eq 0 ]; then
				echo -n "$file is not empty, remove? "
				read input
				if [ $input != "y" ]; then 
					exit 1
				fi
			fi
			echo "removing $file/* ...";
			/bin/rm -r $file/*
		fi
	done
fi

######################################################################
# link the source bki files into the local PGDATA directories
if [ $skipinitdb -eq 0 ]; then
	echo "Linking bki files in PGDATA..."
	curdir=`pwd`
	for h in $HOSTS; do
		file="$PGDATA-$h"
		cd $file
		ln -s $FILESDIR .
		cd $curdir
	done
fi		

######################################################################
# now do initdb on them
if [ $skipinitdb -eq 0 ]; then
	echo "Initing the data directories...";
else
	echo "Re-initing the data directories...";
fi
localpgdata=$PGDATA
for h1 in $HOSTS; do
  PGDATA="$localpgdata-$h1"
  export PGDATA	
  if [ $skipinitdb -eq 0 ]; then
    echo "initdb in $PGDATA (hostid=$h1)...";
    initdb -h $h1 >/dev/null
    echo "Updating pg_host in $PGDATA... "
    for h2 in $HOSTS; do
      if [ $h2 -ne $h1 ]; then
        echo "INSERT INTO pg_host VALUES ($h2,$ouraddress);" |
          postgres template1 >/dev/null 2>&1
      fi
    done
  else
    echo "In $PGDATA (hostid=$h1)..."
    echo "destroydb $USER..."
    echo "DESTROYDB $USER;" | postgres template1 >/dev/null 2>&1
    echo "createdb $USER..."
    echo "CREATEDB $USER;" | postgres template1 >/dev/null 2>&1
  fi
done

echo "initmp successfully completed."
