#!/bin/sh

if [ -d ./obj ]; then
	cd ./obj
fi

[ -z "$HOSTS" ] && HOSTS='1 2 3 4'
[ -z "$PMFLAGS" ] && PMFLAGS='-d 3'
[ -z "$PGPORT" ] && PGPORT=5432

if [ -z "$PGDATA" ]; then
  echo "$0: PGDATA must be set"
  exit 1
fi

localpgdata=$PGDATA
pgport=$PGPORT
pm_pids=""

############################################################################
# Re-init the database

echo =============== re-initing mariposa database... =================
initmp -s -n


############################################################################
# Starting postmasters:

echo =============== starting postmasters... =================
for h in $HOSTS; do
	PGPORT=$pgport
	PGDATA=$localpgdata-$h
	export PGPORT PGDATA
	echo "Starting postmaster (PGDATA=$PGDATA, PGPORT=$PGPORT)"
	postmaster $PMFLAGS > PMLOG$$.$h 2>&1 &
	if [ -z "$pm_pids" ]; then
		PGPORT1=$PGPORT;
		PGDATA1=$PGDATA;
	fi
	pm_pids="$pm_pids $!"
	pgport=`echo $pgport + 1 | bc`;
done

# make sure the postmasters are ready
echo 'Wait a little while for the postmaster to get ready.'
sleep 10

#echo =============== starting broker... =================
#PGPORT=$pgport
#PGDATA=$localpgdata-1
#postgres -r > BROKERLOG &
#pm_pids="$pm_pids $!"


############################################################################
# Running queries:

PGPORT=$PGPORT1
PGDATA=$PGDATA1
export PGPORT PGDATA

echo =============== creating new mariposa database... =================
monitor < create.sql
if [ $? -ne 0 ]; then
     echo the creation script has an error
     exit 1
fi

echo =============== running mariposa queries... =================
monitor < queries.sql
if [ $? -ne 0 ]; then
     echo the queries script has an error
     exit 1
fi


############################################################################
# Clean up postmasters:

echo "Killing postmasters $pm_pids."
kill $pm_pids
